@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: (ARCH :use CL) -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 (defpackage "ARCH"
   (:use "CL"))
 
@@ -643,7 +639,7 @@
   register_cstack
   condemn-area
   metering-control
-  excise-library
+  restore-soft-stack-limit
   egc-control
   lisp-bug
   NewThread
@@ -749,9 +745,7 @@
     ccl::%macro-code% ccl::%init-misc ccl::%builtin-functions%
     ccl::*total-gc-microseconds* ccl::*total-bytes-freed*
     ccl::resolve-slep-address ccl::%excised-code% ccl::%new-gcable-ptr
-    ccl::*current-cursor* ccl::*cs-segment-size* ccl::*vs-segment-size*
-    ccl::*ts-segment-size* ccl::*cs-hard-overflow-size* ccl::*cs-soft-overflow-size*
-    ccl::cs-overflow-callback))
+))
 
 (defmacro nrs-offset (name)
   (let* ((pos (position name *ppc-nilreg-relative-symbols* :test #'eq)))
