@@ -1,22 +1,18 @@
 ;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 
@@ -849,7 +845,6 @@
                   token sym
                   containing-env (nx-new-lexical-environment defenv))
             (unless info
-              (setq info (assq sym *nx-globally-inline*))
               (if (cdr (setq info (assq sym *nx-globally-inline*)))
                 (setq lambda-form (%cdr info)
                       token sym
@@ -985,7 +980,7 @@
   (when afunc
     (setq
      tags (if toplevel *nx-tags* (afunc-tags afunc))
-     taginfo (assq tagname tags)
+     taginfo (assoc tagname tags)
      parent (afunc-parent afunc))
     (if taginfo
       (values taginfo nil)
@@ -1784,6 +1779,16 @@ Or something. Right? ~s ~s" var varbits))
       (when (atom form) (go DONE))
       (unless (symbolp (setq sym (%car form)))
         (go DONE))
+      (when (eq sym 'the)
+	(destructuring-bind (typespec thing) (cdr form)
+	  (multiple-value-bind (newform win) (nx-transform thing environment)
+	    (when win
+	      (setq changed t)
+	      (if (and (self-evaluating-p newform)
+		       (typep newform typespec))
+		(setq form newform)
+		(setq form `(the ,typespec ,newform)))
+	      (go DONE)))))
       (when (nx-quoted-form-p form)
         (when (self-evaluating-p (%cadr form))
           (setq form (%cadr form)))
