@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 (defnx1 nx1-the the (&whole call typespec form &environment env)
@@ -113,7 +109,7 @@
     (nx1-form (%car args))
     (make-acode (%nx1-operator progn) (nx1-formlist args))))
 
-(defnx1 nx1-unaryop ((%word-to-int) (uvsize) (integer-point-h) (integer-point-v) (%reference-external-entry-point))
+(defnx1 nx1-unaryop ((%word-to-int) (uvsize)  (%reference-external-entry-point))
         (arg)
   (make-acode
    (%nx1-default-operator) (nx1-form arg)))
@@ -282,7 +278,7 @@
 (defnx1 nx1-%null-ptr-p ((%null-ptr-p)) (ptr)
   (nx1-form `(%ptr-eql ,ptr (%int-to-ptr 0))))
 
-(defnx1 nx1-binop ((uvref) (%ilsl) (%ilsr) (%iasr)
+(defnx1 nx1-binop ( (%ilsl) (%ilsr) (%iasr)
                    (cons) (%temp-cons))
         (arg1 arg2)
   (make-acode (%nx1-default-operator) (nx1-form arg1) (nx1-form arg2)))
@@ -589,6 +585,8 @@
 
 (defnx1 nx1-num= ((=-2) (/=-2)) (&whole whole &environment env num1 num2 )
   (let* ((op *nx-sfname*)
+	 (2-fixnums (and (nx-form-typep num1 'fixnum env)
+			 (nx-form-typep num2 'fixnum env)))
 	 (2-u32s (and (nx-form-typep num1 '(unsigned-byte 32) env)
 		      (nx-form-typep num2 '(unsigned-byte 32) env)))
          (2-rats (and (nx-form-typep num1 'rational env)
@@ -604,7 +602,7 @@
                         (or sfloat-2 (if (typep num2 'fixnum) (setq num2 (coerce num2 'short-float))))
                         (if sfloat-2 (if (typep num1 'fixnum) (setq num1 (coerce num1 'short-float)))))))
          )
-    (if 2-u32s
+    (if (and 2-u32s (not 2-fixnums))
       (make-acode
        (%nx1-operator %u32<>)
        (make-acode
@@ -632,7 +630,7 @@
 	  (nx1-treat-as-call whole))))))
              
 
-(defnx1 nx1-uvset ((uvset) (%misc-set)) (vector index value)
+(defnx1 nx1-uvset ((uvset) (%misc-set)) (vector index value &environment env)
   (make-acode (%nx1-operator uvset)
               (nx1-form vector)
               (nx1-form index)
@@ -695,14 +693,12 @@
       (progn
         (make-acode (%nx1-operator or) (nx1-formlist (%cdr whole)))))))
 
-(defnx1 nx1-aref ((aref)) (&whole whole &environment env arr &optional (dim0 nil dim0-p)
-                                  &rest other-dims)
-   (if (and dim0-p (null other-dims))
-       (let* ((simple-vector-p (nx-form-typep arr 'simple-vector env))
+(defun nx1-1d-vref (env arr dim0 &optional uvref-p)
+  (let* ((simple-vector-p (nx-form-typep arr 'simple-vector env))
               (string-p (unless simple-vector-p 
                           (if (nx-form-typep arr 'string env)
                               (or (nx-form-typep arr 'simple-string env)
-                                  (return-from nx1-aref (nx1-form `(char ,arr ,dim0)))))))
+                                  (return-from nx1-1d-vref (nx1-form `(char ,arr ,dim0)))))))
               (simple-1d-array-p (unless (or simple-vector-p string-p) 
                                    (nx-form-typep arr '(simple-array * (*)) env)))
               
@@ -717,9 +713,18 @@
                               (string-p (%nx1-operator %sbchar))
                               (simple-vector-p 
                                (if (nx-inhibit-safety-checking env) (%nx1-operator %svref) (%nx1-operator svref)))
-                                   (t (%nx1-operator %aref1)))))
-                    (make-acode op (nx1-form arr) (nx1-form dim0)))))
-            (nx1-treat-as-call whole)))
+			      (uvref-p (%nx1-operator uvref))
+			      (t (%nx1-operator %aref1)))))
+                    (make-acode op (nx1-form arr) (nx1-form dim0))))))
+  
+(defnx1 nx1-aref ((aref)) (&whole whole &environment env arr &optional (dim0 nil dim0-p)
+                                  &rest other-dims)
+   (if (and dim0-p (null other-dims))
+     (nx1-1d-vref env arr dim0)
+     (nx1-treat-as-call whole)))
+
+(defnx1 nx1-uvref ((uvref)) (&environment env arr dim0)
+  (nx1-1d-vref env arr dim0 t))
 
 (defnx1 nx1-%aref2 ((%aref2)) (&whole whole &environment env arr i j)
   ;; For now, we only care about the (simple-array double-float (* *)) case.
@@ -740,17 +745,12 @@
 		    (nx1-form (if (typep dim1 'fixnum) dim1))))
 	(nx1-treat-as-call whole))))
 
-(defnx1 nx1-aset ((aset)) (&whole whole 
-                                  arr newval 
-                                  &optional (dim0 nil dim0-p)
-                                  &environment env
-                                  &rest other-dims)
-   (if (and dim0-p (null other-dims))
-       (let* ((simple-vector-p (nx-form-typep arr 'simple-vector env))
+(defun nx1-1d-vset (arr newval dim0 env &optional uvset-p)
+  (let* ((simple-vector-p (nx-form-typep arr 'simple-vector env))
               (string-p (unless simple-vector-p 
                           (if (nx-form-typep arr 'string env)
                               (or (nx-form-typep arr 'simple-string env)
-                                  (return-from nx1-aset (nx1-form `(set-char ,arr ,newval ,dim0)))))))
+                                  (return-from nx1-1d-vset (nx1-form `(set-char ,arr ,newval ,dim0)))))))
               (simple-1d-array-p (unless (or simple-vector-p string-p) 
                                    (nx-form-typep arr '(simple-array * (*)) env)))
               (element-type (if simple-1d-array-p (array-ctype-element-type (specifier-type (nx-form-type arr env)))))
@@ -764,6 +764,7 @@
              (let* ((op (cond (simple-1d-array-p (%nx1-operator uvset))
                               (string-p (%nx1-operator %set-sbchar))
                               (simple-vector-p (if (nx-inhibit-safety-checking env) (%nx1-operator %svset) (%nx1-operator svset)))
+			      (uvset-p (%nx1-operator uvset))
                               (t (%nx1-operator aset1)))))
                (if op
                    (make-acode
@@ -771,7 +772,15 @@
                     (nx1-form arr)
                     (nx1-form newval)
                     (nx1-form dim0))
-                   (nx1-form `(,(if string-p 'set-schar '%aset1) ,arr ,newval ,dim0))))))
+                   (nx1-form `(,(if string-p 'set-schar '%aset1) ,arr ,newval ,dim0)))))))
+
+(defnx1 nx1-aset ((aset)) (&whole whole 
+                                  arr newval 
+                                  &optional (dim0 nil dim0-p)
+                                  &environment env
+                                  &rest other-dims)
+   (if (and dim0-p (null other-dims))
+       (nx1-1d-vset arr newval dim0 env)
        (nx1-treat-as-call whole)))
             
 (defnx1 nx1-%aset2 ((%aset2)) (&whole whole &environment env arr i j new)
@@ -1088,7 +1097,7 @@
     (dolist (form args)
       (when (atom form)
         (if (or (symbolp form) (integerp form))
-          (if (assq form newtags)
+          (if (assoc form newtags)
             (nx-error "Duplicate tag in TAGBODY: ~S." form)
             (%temp-push (%temp-list form nil counter catchvar nil nil) newtags))
           (nx-error "Illegal form in TAGBODY: ~S." form))))
@@ -1120,7 +1129,7 @@
           (dolist (tag (reverse newtags))
             (when (%cadr tag)
               (%temp-push  
-               (nx1-form `(if (eq ,(var-name indexvar) ,(%cadr tag)) (go ,(%car tag))))
+               (nx1-form `(if (eql ,(var-name indexvar) ,(%cadr tag)) (go ,(%car tag))))
                body)))
           (make-acode
            (%nx1-operator let*)
@@ -1181,7 +1190,7 @@
     :signed-halfword :unsigned-halfword :signed-byte :unsigned-byte))
 
 
-(defnx1 nx1-ff-call ((eabi-ff-call) (ff-call)) (address-expression &rest arg-specs-and-result-spec)
+(defnx1 nx1-ff-call ((%ff-call)) (address-expression &rest arg-specs-and-result-spec)
    (nx1-ff-call-internal
     address-expression arg-specs-and-result-spec
     (ecase (backend-name *target-backend*)
@@ -1204,25 +1213,24 @@
       (error "odd number of arg-specs"))
     (loop
       (when (null arg-specs) (return))
-      (let ((arg-keyword (pop arg-specs))
-            (value (pop arg-specs)))
-        (if (memq arg-keyword *arg-spec-keywords*)
+      (let* ((arg-keyword (pop arg-specs))
+	     (value (pop arg-specs)))
+        (if (or (memq arg-keyword *arg-spec-keywords*)
+		(and (or (eql operator (%nx1-operator darwin-syscall))
+			 (eql operator (%nx1-operator poweropen-ff-call)))
+		     (typep arg-keyword 'unsigned-byte)))
           (progn 
             (push arg-keyword specs)
             (push value vals))
           (error "Unknown argument spec: ~s" arg-keyword))))
-    (setq result-spec (if (null result-spec) 
-                        :void 
-                        (if (eq result-spec :void) 
-                          result-spec
-                          (if (memq result-spec *arg-spec-keywords*)
-                            result-spec
-                            (error "Unknown result spec: ~s" result-spec)))))
+    (unless (or (eq result-spec :void)
+		(memq result-spec *arg-spec-keywords*))
+      (error "Unknown result spec: ~s" result-spec))
     (make-acode operator
-                (nx1-form address-expression)
-                (nreverse specs)
-                (mapcar #'nx1-form (nreverse vals))
-                result-spec)))
+		(nx1-form address-expression)
+		(nreverse specs)
+		(mapcar #'nx1-form (nreverse vals))
+		result-spec)))
   
 (defnx1 nx1-block block (blockname &body forms)
   (let* ((*nx-blocks* *nx-blocks*)
@@ -1475,7 +1483,7 @@
          body
          *nx-new-p2decls*)))))
 
-(defnx1 nx1-put-xxx ((%put-ptr) (%put-long) (%put-full-long) (%put-point) (%put-ostype) (%put-word) (%put-byte)) 
+(defnx1 nx1-put-xxx ((%put-ptr) (%put-long) (%put-full-long)  (%put-ostype) (%put-word) (%put-byte)) 
         (ptr newval &optional (offset 0) &aux (op *nx-sfname*))
   (make-acode
    (%nx1-operator immediate-put-xxx)
@@ -1523,7 +1531,7 @@
     (make-acode (%nx1-operator %macptrptr%) (nx1-form ptrform))
     (nx1-form offset))))
 
-(defnx1 nx1-get-xxx ((%get-long) (%get-point) (%get-full-long)  (%get-signed-long)
+(defnx1 nx1-get-xxx ((%get-long)  (%get-full-long)  (%get-signed-long)
                      (%get-fixnum) 
                      (%get-word) (%get-unsigned-word)
                      (%get-byte) (%get-unsigned-byte)
@@ -1533,7 +1541,7 @@
   (ptrform &optional (offset 0))
   (let* ((sfname *nx-sfname*)
          (flagbits (case sfname
-                     ((%get-long %get-full-long %get-point %get-signed-long) 1)
+                     ((%get-long %get-full-long  %get-signed-long) 1)
                      (%get-fixnum 33)
 		     
                      ((%get-word %get-unsigned-word) 2)
@@ -1567,6 +1575,34 @@
     (make-acode (%nx1-operator %macptrptr%) (nx1-form ptrform))
     (nx1-form offset))))
 
+(defnx1 nx1-%get-float ((%get-single-float)
+			(%get-double-float)) (ptrform &optional (offset 0))
+  (make-acode
+   (%nx1-operator typed-form)
+   (if (eq *nx-sfname* '%get-single-float)
+     'single-float
+     'double-float)
+   (make-acode
+    (%nx1-default-operator)
+    (make-acode (%nx1-operator %macptrptr%) (nx1-form ptrform))
+    (nx1-form offset))))
+
+(defnx1 nx1-%set-float ((%set-single-float)
+			(%set-double-float)) (ptrform offset &optional (newval nil newval-p))
+  (unless newval-p
+    (setq newval offset
+	  offset 0))
+  (make-acode
+   (%nx1-operator typed-form)
+   (if (eq *nx-sfname* '%set-single-float)
+     'single-float
+     'double-float)
+   (make-acode
+    (%nx1-default-operator)
+    (make-acode (%nx1-operator %macptrptr%) (nx1-form ptrform))
+    (nx1-form offset)
+    (nx1-form newval))))
+
 (defnx1 nx1-let let (pairs &body forms &environment old-env)
   (let* ((vars nil)
          (vals nil)
@@ -1741,6 +1777,7 @@
                                   (dpb (length bindings) $lfbits-numreq 0))))
 
 (defnx1 nx1-sparc-lap-function (sparc-lap-function) (name bindings &body body)
+  (declare (ftype (function (t t t t)) %define-sparc-lap-function))
   (require "SPARC-LAP")
   (setf (afunc-lfun *nx-current-function*) 
         (%define-sparc-lap-function name `((let ,bindings ,@body))
@@ -1748,10 +1785,6 @@
 
 
 
-
-
-
-
 (defun nx1-env-body (body old-env)
   (do* ((form (nx1-progn-body body))
         (env *nx-lexical-environment* (lexenv.parent-env env)))
@@ -1832,6 +1865,39 @@
               (nx1-form base)
               (nx1-form offset)))
 
+(defnx1 nx1-ash (ash) (&whole call &environment env num amt)
+  (cond ((eq amt 0) (nx1-form num))
+        ((and (fixnump amt)
+              (< amt 0))
+	 (if (nx-form-typep num 'fixnum env)
+	   (make-acode (%nx1-operator %iasr)
+		       (make-acode (%nx1-operator fixnum)
+				   (- amt))
+		       (nx1-form num))
+	   (if (nx-form-typep num '(unsigned-byte 32) env)
+	     (make-acode (%nx1-operator u32-shift-right)
+			 (nx1-form num)
+			 (make-acode (%nx1-operator fixnum)
+				     (- amt)))
+	     (nx1-treat-as-call call))))
+	((and (fixnump amt)
+	      (< amt 32)
+	      (nx-form-typep num '(unsigned-byte 32) env)
+	      (nx-trust-declarations env)
+	      (subtypep *nx-form-type* '(unsigned-byte 32)))
+	 (make-acode (%nx1-operator u32-shift-left)
+		     (nx1-form num)
+		     (nx1-form amt)))
+        ((and (fixnump amt)
+              (<= 0 amt 27)
+              (or (nx-form-typep num `(signed-byte ,(- 28 amt)) env)
+                  (and (nx-form-typep num 'fixnum env)
+                       (nx-trust-declarations env)
+                       (subtypep *nx-form-type* 'fixnum))))
+         (nx1-form `(%ilsl ,amt ,num)))
+        (t (nx1-treat-as-call call))))
+
+    
 ; This idea went nowhere
 #+later
 (defnx1 nx1-%lisp-lowbyte-ref (%lisp-lowbyte-ref) (base offset)
