@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 ; Optimizers.lisp - compiler optimizers
 
 (eval-when (eval compile)
@@ -76,11 +72,11 @@
                      (logxor . fold-constant-subforms) logcount logorc2 listp consp expt
                      logorc1 logtest lognand logeqv lognor lognot logandc2 logandc1
                      numerator denominator ldb-test byte-position byte-size isqrt gcd
-                     floor mod boole max min ldb dpb mask-field deposit-field
+                     floor mod truncate rem round boole max min ldb dpb mask-field deposit-field
                      length aref svref char schar bit sbit getf identity list-length
                      car cdr cadr cddr nth nthcdr last load-byte deposit-byte byte-mask
                      member search count position assoc rassoc integer-length
-		         float not null char-int expt))
+		         float not null char-int expt abs))
 
 (defun %binop-cassoc (call)
   (unless (and (cddr call) (null (cdr (%cddr call))))
@@ -441,7 +437,7 @@
           `(let* ((,limit ,n) (,i 0))
              ,@decls
              (declare (fixnum ,limit)
-                      (type (integer 0 (,upper)) ,i)
+                      (type (integer 0 ,(if (<= upper 0) 0 `(,upper))) ,i)
                       (unsettable ,i))
              (block nil
                (tagbody
@@ -563,8 +559,13 @@
       `(progn ,@body))))
 
 
-(defun target-element-type-subtype (subtype)
-  (element-type-subtype subtype))
+(defun target-element-type-subtype (typespec)
+  (let* ((ctype (ignore-errors (specifier-type typespec))))
+    (if (or (null ctype) (typep ctype 'unknown-ctype))
+      (progn
+        (nx1-whine :unknown-type-declaration typespec)
+        nil)
+      (element-type-subtype typespec))))
 
 (define-compiler-macro make-array (&whole call &environment env dims &rest keys)
   (if (constant-keywords-p keys)
@@ -594,34 +595,56 @@
                  (not (constantp element-type))
                  (null (setq element-type (target-element-type-subtype (eval element-type)))))
              (comp-make-array-1 dims keys))
-            (t ;Should do more here
+            ((and (typep element-type 'fixnum) 
+                  (nx-form-typep dims 'fixnum env) 
+                  (null (or adjustable fill-pointer initial-contents 
+                            initial-contents-p))) 
+             (if 
+               (or (null initial-element-p) 
+                   (cond ((eql element-type arch::subtag-double-float-vector) 
+                          (eql initial-element 0.0d0)) 
+                         ((eql element-type arch::subtag-single-float-vector) 
+                          (eql initial-element 0.0s0)) 
+                         ((or (eql element-type arch::subtag-simple-base-string) 
+                              (eql element-type arch::subtag-simple-general-string)) 
+                          (eql initial-element #\Null))
+                         (t (eql initial-element 0))))
+               `(%alloc-misc ,dims ,element-type) 
+               `(%alloc-misc ,dims ,element-type ,initial-element))) 
+	     (t ;Should do more here
              (comp-make-uarray dims keys element-type))))
     call))
 
 (defun comp-make-displaced-array (dims keys)
   (let* ((call-list (make-list 4 :initial-element nil))
+	 (dims-var (make-symbol "DIMS"))
          (let-list (comp-nuke-keys keys
                                    '((:displaced-to 0)
                                      (:fill-pointer 1)
                                      (:adjustable 2)
                                      (:displaced-index-offset 3))
-                                   call-list)))
+                                   call-list
+				   `((,dims-var ,dims)))))
+
     `(let ,let-list
-       (%make-displaced-array ,dims ,@call-list))))
+       (%make-displaced-array ,dims-var ,@call-list))))
 
 (defun comp-make-uarray (dims keys subtype)
   (let* ((call-list (make-list 6))
+	 (dims-var (make-symbol "DIMS"))
          (let-list (comp-nuke-keys keys
                                    '((:adjustable 0)
                                      (:fill-pointer 1)
                                      (:initial-element 2 3)
                                      (:initial-contents 4 5))
-                                   call-list)))
+                                   call-list
+				   `((,dims-var ,dims)))))
     `(let ,let-list
-       (make-uarray-1 ,subtype ,dims ,@call-list nil nil))))
+       (make-uarray-1 ,subtype ,dims-var ,@call-list nil nil))))
 
 (defun comp-make-array-1 (dims keys)
   (let* ((call-list (make-list 10 :initial-element nil))
+	 (dims-var (make-symbol "DIMS"))
          (let-list (comp-nuke-keys keys                                   
                                    '((:element-type 0 1)
                                      (:displaced-to 2)
@@ -630,13 +653,14 @@
                                      (:fill-pointer 5)
                                      (:initial-element 6 7)
                                      (:initial-contents 8 9))
-                                   call-list)))
+                                   call-list
+				   `((,dims-var ,dims)))))
     `(let ,let-list
-       (make-array-1 ,dims ,@call-list nil))))
+       (make-array-1 ,dims-var ,@call-list nil))))
 
-(defun comp-nuke-keys (keys key-list call-list)
+(defun comp-nuke-keys (keys key-list call-list &optional required-bindings)
   ; side effects call list, returns a let-list
-  (let ((let-list nil))
+  (let ((let-list (reverse required-bindings)))
     (do ((lst keys (cddr lst)))
         ((null lst) nil)
       (let* ((key (car lst))
@@ -672,10 +696,12 @@
   (if more
     call
     (let* ((temp-var (gensym))
-           (elt-var (gensym)))
-       `(let* ((,temp-var ,lst))
+           (elt-var (gensym))
+           (fn-var (gensym)))
+       `(let* ((,fn-var ,fn)
+               (,temp-var ,lst))
           (dolist (,elt-var ,temp-var ,temp-var)
-            (funcall ,fn ,elt-var))
+            (funcall ,fn-var ,elt-var))
           ))))
 
 (define-compiler-macro mapcar (&whole call fn lst &rest more)
@@ -683,13 +709,15 @@
     call
     (let* ((temp-var (gensym))
            (result-var (gensym))
-           (elt-var (gensym)))
+           (elt-var (gensym))
+           (fn-var (gensym)))
       `(let* ((,temp-var (cons nil nil))
-              (,result-var ,temp-var))
+              (,result-var ,temp-var)
+              (,fn-var ,fn))
          (declare (dynamic-extent ,temp-var)
                   (type cons ,temp-var ,result-var))
          (dolist (,elt-var ,lst (cdr ,result-var))
-           (setq ,temp-var (setf (cdr ,temp-var) (list (funcall ,fn ,elt-var)))))))))
+           (setq ,temp-var (setf (cdr ,temp-var) (list (funcall ,fn-var ,elt-var)))))))))
 
 (define-compiler-macro member (&whole call &environment env item list &rest keys)
   (or (eq-eql-call item list keys 'memq 'memeql env)
@@ -742,8 +770,9 @@
 ;but I don't think this can be done just with optimizers... For now, at least
 ;try to get it to become (%car (<typecheck> foo)).
 (define-compiler-macro require-type (&whole call &environment env arg type)
-  (cond ((and (quoted-form-p type))
-         (setq type (%cadr type))
+  (cond ((and (quoted-form-p type)
+	      (setq type (%cadr type))
+	      (not (typep (specifier-type type) 'unknown-ctype)))	 
          (cond ((nx-form-typep arg type env) arg)
                ((eq type 'simple-vector)
                 `(the simple-vector (require-simple-vector ,arg)))
@@ -843,7 +872,7 @@
                      (,sequence-var ,sequence)
                      (,test-var ,(or test test-not))
                      (,key-var ,key))
-                 (declare (dynamic-extent ,sequence-var ,test-var ,key-var))
+                 (declare (dynamic-extent ,item-var ,sequence-var ,test-var ,key-var))
                  (,find-function ,item-var ,sequence-var ,test-var ,key-var)))))
         call))
       call))
@@ -975,41 +1004,7 @@
       `(the ,typespec ,form)
       call)))
 
-#| ; there were 2 of these - damn
-(define-compiler-macro typep  (&whole call &environment env thing type &aux typep typesym)
-  (if (quoted-form-p type)
-    (cond ((symbolp (setq typesym (%cadr type)))
-           (cond ((and (setq typep (type-predicate typesym))
-                       (symbolp typep))
-                  `(,typep ,thing))
-                 ((structure-class-p typesym)
-                  `(structure-typep ,thing ,type))
-                 ((find-class typesym nil env)
-                  ; or should second arg be t???
-                  `(class-cell-typep ,thing (load-time-value (find-class-cell ',typesym t))))
-                 ((info-type-builtin typesym) ; bootstrap troubles here?
-                  `(builtin-typep ,thing (load-time-value (find-builtin-cell ',typesym))))
-                 (t call)))
-          ((listp typesym)
-           (cond 
-            ((info-type-builtin typesym)  ; byte types
-             `(builtin-typep ,thing (load-time-value (find-builtin-cell ',typesym))))
-            (t
-             (case (car typesym)
-               (satisfies `(funcall ',(cadr typesym) ,thing))
-               (eql `(eql ,thing ',(cadr typesym)))
-               (member `(not ( null (member ,thing ',(cdr typesym)))))
-               (not `(not (typep ,thing ',(cadr typesym))))
-               ((or and)
-                (let ((thing-sym (gensym)))
-                  `(let ((,thing-sym ,thing))
-                     (,(car typesym)
-                      ,@(mapcar #'(lambda (type-spec) `(typep ,thing-sym ',type-spec))
-                                (cdr typesym))))))
-               (t call))))) 
-          (t call))
-    call))
-|#
+
 
 (define-compiler-macro zerop (arg &environment env)
   (let* ((z (if (nx-form-typep arg 'float env)
@@ -1195,6 +1190,12 @@
   (if (nx-form-typep n1 'fixnum env)
     `(%ilognot ,n1)
     w))
+
+(define-compiler-macro logtest (&whole w &environment env n1 n2)
+  (if (and (nx-form-typep n1 'fixnum env)
+           (nx-form-typep n2 'fixnum env))
+    `(not (eql 0 (logand ,n1 ,n2)))
+    w))
   
 
 (defmacro defsynonym (from to)
@@ -1287,7 +1288,8 @@
              (t nil)))))
         (t nil)))
 
-(define-compiler-macro typep  (&whole call &environment env thing type)
+(define-compiler-macro typep  (&whole call &environment env thing type &optional e)
+  (declare (ignore e))
   (if (quoted-form-p type)
     (or (optimize-typep thing (%cadr type) env)
         call)
@@ -1296,8 +1298,8 @@
 
 
 (define-compiler-macro find-class (&whole call type &optional (errorp t) env)
-  (declare (ignore env))
-  (if (and (quoted-form-p type)(not *dont-find-class-optimize*))
+  ;(declare (ignore env))
+  (if (and (quoted-form-p type)(not *dont-find-class-optimize*)(not env))
       `(class-cell-find-class (load-time-value (find-class-cell ,type t)) ,errorp)
     call))
 
@@ -1665,32 +1667,6 @@
           call)))
     call))
 
-(define-compiler-macro make-uarray-1 (&whole call
-                                             &environment env
-                                             subtype dims adjustable fill-pointer
-                                             initial-element initial-element-p
-                                             initial-contents initial-contents-p
-                                             temporary 
-                                             size)
-  (if (and (fixnump subtype)
-           (nx-form-typep dims 'fixnum env)
-           (null (or adjustable fill-pointer 
-                     initial-contents initial-contents-p
-                     temporary size)))
-    (if (or (null initial-element-p)
-            (and (eql subtype arch::subtag-double-float-vector) (eql initial-element 0.0))
-            (and (not (eql subtype arch::subtag-double-float-vector))
-                 (not (eql subtype arch::subtag-simple-base-string))
-                 (not (eql subtype arch::subtag-simple-general-string))
-                 (eql initial-element 0))
-            (and (or (eql subtype arch::subtag-simple-base-string)
-                     (eql subtype arch::subtag-simple-general-string))
-                 (eql initial-element #\null)))
-      `(%alloc-misc ,dims ,subtype)
-      `(%alloc-misc ,dims ,subtype ,initial-element))
-    call))
-
-
 (define-compiler-macro sbit (&environment env &whole call v &optional sub0 &rest others)
   (if (and sub0 (null others))
     `(%typed-miscref #.arch::subtag-bit-vector ,v ,sub0)
