@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 (cl:eval-when (:compile-toplevel :load-toplevel :execute)
@@ -77,6 +73,8 @@
   template                              ; The vinsn-template of which this is an instance
   variable-parts                        ; vector of result-vregs, arguments, temps, local-labels
   annotation
+  (gprs-set 0)
+  (fprs-set 0)
 )
 
 (defloadvar *vinsn-freelist* (make-dll-node-freelist))
@@ -281,6 +279,17 @@
          (pool.data *vinsn-varparts*) v)
    nil))
 
+(defun elide-vinsn (vinsn)
+  (let* ((nvp (vinsn-template-nvp (vinsn-template vinsn)))
+	 (vp (vinsn-variable-parts vinsn)))
+    (dotimes (i nvp)
+      (let* ((v (svref vp i)))
+	(when (typep v 'lreg)
+	  (setf (lreg-defs v) (delete vinsn (lreg-defs v)))
+	  (setf (lreg-refs v) (delete vinsn (lreg-refs v))))))
+    (free-varparts-vector vp)
+    (remove-dll-node vinsn)))
+    
 (defun encode-vinsn-attributes (attribute-list)
   (flet ((attribute-weight (k)
            (let* ((pos (position k *known-vinsn-attributes*)))
@@ -522,6 +531,7 @@
          (nvp (vinsn-template-nvp template))
          (vp (alloc-varparts-vector))
          (*available-backend-node-temps* *available-backend-node-temps*)
+	 (*available-backend-fp-temps* *available-backend-fp-temps*)
          (*available-backend-imm-temps* *available-backend-imm-temps*)
          (*available-backend-crf-temps* *available-backend-crf-temps*))
     (declare (fixnum nvp ntemps nsupp)
@@ -540,7 +550,13 @@
           (temps temp-specs (cdr temps)))
          ((null temps) vp)
       (declare (fixnum i))
-      (let* ((lreg (allocate-temporary-vreg (car temps))))      
+      (let* ((lreg (allocate-temporary-vreg (car temps)))
+	     (class (hard-regspec-class lreg))
+	     (value (hard-regspec-value lreg)))
+	(when value
+	  (case class
+	    (#.hard-reg-class-gpr (note-vinsn-sets-gpr vinsn value))
+	    (#.hard-reg-class-fpr (note-vinsn-sets-fpr vinsn value))))
         (setf (svref vp i) lreg)
         (pushnew vinsn (lreg-defs lreg))
         (pushnew vinsn (lreg-refs lreg))))))
@@ -591,6 +607,48 @@
 (defun %emit-vinsn (vlist name vinsn-table &rest vregs)
   (append-dll-node (select-vinsn name vinsn-table vregs) vlist))
 
+(defun varpart-matches-reg (varpart-value class regval spec)
+  (setq spec (if (atom spec) spec (car spec)))
+  (and
+   (or
+    (and (eq class hard-reg-class-fpr)
+	 (memq spec '(:single-float :double-float)))
+    (and (eq class hard-reg-class-gpr)
+	 (memq spec '(:u32 :s32 :u16 :s16 :u8 :s8 :lisp :address :imm))))
+   (eq (hard-regspec-value varpart-value) regval)))
+
+(defun vinsn-sets-reg-p (element reg)
+  (if (typep element 'vinsn)
+    (if (vinsn-attribute-p element :call)
+      t
+      (let* ((class (hard-regspec-class reg))
+	     (value (hard-regspec-value reg)))
+	(if (eq class hard-reg-class-gpr)
+	  (logbitp value (vinsn-gprs-set element))
+	  (if (eq class hard-reg-class-fpr)
+	    (logbitp value (vinsn-fprs-set element))))))))
+
+;;; Return bitmasks of all GPRs and all FPRs set in the vinsns between
+;;; START and END, exclusive.  Any :call vinsn implicitly clobbers
+;;; all registers.
+(defun regs-set-in-vinsn-sequence (start end)
+  (let* ((gprs-set 0)
+	 (fprs-set 0))
+    (do* ((element (vinsn-succ start) (vinsn-succ element)))
+	 ((eq element end) (values gprs-set fprs-set))n
+      (if (typep element 'vinsn)
+	(if (vinsn-attribute-p element :call)
+	  (return (values #xffffffff #xffffffff))
+	  (setq gprs-set (logior (vinsn-gprs-set element))
+		fprs-set (logior (vinsn-fprs-set element))))))))
+      
+;;; Return T if any vinsn between START and END (exclusive) sets REG.
+(defun vinsn-sequence-sets-reg-p (start end reg)
+  (do* ((element (vinsn-succ start) (vinsn-succ element)))
+       ((eq element end))
+    (if (vinsn-sets-reg-p element reg)
+      (return t))))
+	
 
 ;;; Flow-graph nodes (FGNs)
 
@@ -694,9 +752,10 @@
       ((nil)
        (if (eq prevtype :label)
 	 (let* ((lab (dll-node-succ current)))
+	   (when (vinsn-label-p lab)
 	   (insert-dll-node-after
 	    (select-vinsn "JUMP" *backend-vinsns* (list lab))
-	    current)))))))
+	      current))))))))
 
 
 ;;; Unless the header is empty, remove the last vinsn and all preceding
