@@ -1,29 +1,33 @@
 ;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;(in-package "CCL")
 
 (eval-when (:compile-toplevel :execute)
   (require "PPC-ARCH")
-  (require "PPC-LAPMACROS"))
+  (require "PPC-LAPMACROS")
+
+  ;; Set RES to 1 if (u< x y), to 0 otherwise.
+  (defppclapmacro sltu (res x y)
+    `(progn
+      (subfc ,res ,x ,y)
+      (subfe ,res ,res ,res)
+      (neg ,res ,res)))
+  )
 
 ;;; %BIGNUM-REF needs to access bignums as obviously as possible, and it needs
 ;;; to be able to return 32 bits somewhere no one looks for real objects.
@@ -43,8 +47,259 @@
                                         ; called us with 2 args, but that's horrible style.)
   (ba .SPvalues))
 
+;;; Set the 0th element of DEST (a bignum or some other 32-bit ivector)
+;;; to the Ith element of the bignum SRC.
+(defppclapfunction %ref-digit ((bignum arg_x) (i arg_y) (dest arg_z))
+  (la imm1 arch::misc-data-offset i)
+  (lwzx imm0 bignum imm1)
+  (stw imm0 arch::misc-data-offset dest)
+  (blr))
+
+;;; BIGNUM[I] := DIGIT[0]
+(defppclapfunction %set-digit ((bignum arg_x) (i arg_y) (digit arg_z))
+  (la imm1 arch::misc-data-offset i)
+  (lwz imm0 arch::misc-data-offset digit)
+  (stwx imm0 bignum imm1)
+  (blr))
+
+;;; Return 0 if the 0th digit in X is 0.
+(defppclapfunction %digit-zerop ((x arg_z))
+  (lwz imm0 arch::misc-data-offset x)
+  (cntlzw imm0 imm0)
+  (srwi imm0 imm0 5)
+  (rlwimi imm0 imm0 4 27 27)
+  (add arg_z rnil imm0)
+  (blr))
+
+;;; store the sign of bignum (0 or -1) in the one-word bignum "digit".
+(defppclapfunction %bignum-sign-digit ((bignum arg_y) (digit arg_z))
+  (vector-length imm0 bignum imm0)
+  (la imm0 (- arch::misc-data-offset 4) imm0) ; Reference last (most significant) digit
+  (lwzx imm0 bignum imm0)
+  (srawi imm0 imm0 31)			;propagate sign bit
+  (stw imm0 arch::misc-data-offset digit)
+  (blr))
+
+;;; Return the sign of bignum (0 or -1) as a fixnum
+(defppclapfunction %bignum-sign ((bignum arg_z))
+  (vector-length imm0 bignum imm0)
+  (la imm0 (- arch::misc-data-offset 4) imm0) ; Reference last (most significant) digit
+  (lwzx imm0 bignum imm0)
+  (srawi imm0 imm0 31)			;propagate sign bit
+  (box-fixnum arg_z imm0)
+  (blr))
+
+;;; Count the sign bits in the most significant digit of bignum; return fixnum count.
+(defppclapfunction %bignum-sign-bits ((bignum arg_z))
+  (vector-length imm0 bignum imm0)
+  (la imm0 (- arch::misc-data-offset 4) imm0) ; Reference last (most significant) digit
+  (lwzx imm0 bignum imm0)
+  (cmpwi imm0 0)
+  (not imm0 imm0)
+  (blt @wasneg)
+  (not imm0 imm0)
+  @wasneg
+  (cntlzw imm0 imm0)
+  (box-fixnum arg_z imm0)
+  (blr))
+
+(defppclapfunction %digit-0-or-plusp ((bignum arg_y) (idx arg_z))
+  (la imm0 arch::misc-data-offset idx)
+  (lwzx imm0 bignum imm0)
+  (xoris imm0 imm0 #x8000)		; invert sign bit
+  (srwi imm0 imm0 31)
+  (bit0->boolean arg_z imm0 imm0)	; return T if sign bit was clear before inversion
+  (blr))
+
+;;; For oddp, evenp
+(defppclapfunction %bignum-oddp ((bignum arg_z))
+  (lwz imm0 arch::misc-data-offset bignum)
+  (clrlwi imm0 imm0 31)
+  (bit0->boolean arg_z imm0 imm0)
+  (blr))
+  
+(defppclapfunction bignum-plusp ((bignum arg_z))
+  (vector-length imm0 bignum imm0)
+  (la imm0 (- arch::misc-data-offset 4) imm0) ; Reference last (most significant) digit
+  (lwzx imm0 bignum imm0)
+  (xoris imm0 imm0 #x8000)		; invert sign bit
+  (srwi imm0 imm0 31)
+  (bit0->boolean arg_z imm0 imm0)	; return T if sign bit was clear before inversion
+  (blr))
+
+(defppclapfunction %fixnum-to-bignum-set ((bignum arg_y) (fixnum arg_z))
+  (unbox-fixnum imm0 fixnum)
+  (stw imm0 arch::misc-data-offset bignum)
+  (blr))
+
+(defppclapfunction bignum-minusp ((bignum arg_z))
+  (vector-length imm0 bignum imm0)
+  (la imm0 (- arch::misc-data-offset 4) imm0) ; Reference last (most significant) digit
+  (lwzx imm0 bignum imm0)
+  (srwi imm0 imm0 31)
+  (rlwimi imm0 imm0 4 27 27)
+  (add arg_z rnil imm0)			; return T if sign bit was clear before inversion
+  (blr))
 
 
+;;; Add the digits A[I] and B[J], and the incoming carry C (a fixnum).
+;;; Store the result in R[K], and return the outgoing carry.
+;;; If I is NIL, A is a fixnum.  If J is NIL, B is a fixnum.
+(defppclapfunction %add-with-carry ((r 12) (k 8) (c 4) (a 0) (i arg_x) (b arg_y) (j arg_z))
+  (cmpw cr1 j rnil)
+  (cmpw cr0 i rnil)
+  (lwz temp0 a vsp)
+  (unbox-fixnum imm1 temp0)
+  (unbox-fixnum imm2 b)
+  (beq cr0 @got-a)
+  (la imm1 arch::misc-data-offset i)
+  (lwzx imm1 temp0 imm1)
+  @got-a
+  (beq cr1 @got-b)
+  (la imm2 arch::misc-data-offset j)
+  (lwzx imm2 b imm2)
+  @got-b
+  (lwz temp0 c vsp)
+  (unbox-fixnum imm0 temp0)
+  (addic imm0 imm0 -1)
+  (lwz temp1 r vsp)
+  (lwz temp0 k vsp)
+  (la vsp 16 vsp)
+  (adde imm0 imm1 imm2)
+  (la imm2 arch::misc-data-offset temp0)
+  (stwx imm0 temp1 imm2)
+  (addze imm0 rzero)
+  (box-fixnum arg_z imm0)
+  (blr))
+
+;;; Store the result of A[I] - B[J] - borrow into R[K], returning the borrow.
+;;; If I is NIL, A is a fixnum; likewise for J and B.
+(defppclapfunction %subtract-with-borrow ((r 12) (k 8) (borrow 4) (a 0) (i arg_x) (b arg_y) (j arg_z))
+  (cmpw cr0 i rnil)
+  (cmpw cr1 j rnil)
+  (lwz temp0 a vsp)
+  (unbox-fixnum imm2 b)
+  (unbox-fixnum imm1 temp0)
+  (beq cr1 @got-b)
+  (la imm2 arch::misc-data-offset j)
+  (lwzx imm2 b imm2)
+  @got-b
+  (beq cr0 @got-a)
+  (la imm1 arch::misc-data-offset i)
+  (lwzx imm1 temp0 imm1)
+  @got-a
+  (lwz temp0 borrow vsp)
+  (unbox-fixnum imm0 temp0)
+  (addic imm0 imm0 -1)
+  (lwz temp0 r vsp)
+  (lwz temp1 k vsp)
+  (la vsp 16 vsp)
+  (subfe imm0 imm2 imm1)
+  (la imm1 arch::misc-data-offset temp1)
+  (stwx imm0 temp0 imm1)
+  (addze imm0 rzero)
+  (box-fixnum arg_z imm0)
+  (blr))
+
+;; multiply i'th digit of x by y and add to result starting at digit i
+(defppclapfunction %multiply-and-add-harder-loop-2
+    ((x-ptr 4) (y-ptr 0) (resptr arg_x)(residx arg_y) (count arg_z))  
+  (let ((tem imm0)
+        (y imm1)
+        (prod-h imm2)
+        (prod-l imm3)
+        (x imm4)
+        (xptr temp2)
+        (yidx temp1)
+        (yptr temp0))
+    (lwz xptr x-ptr vsp)
+    (la tem arch::misc-data-offset residx)
+    (lwzx x xptr tem)
+    (lwz yptr y-ptr vsp)
+    (li yidx 0) ; init yidx 0 
+    (addc prod-h rzero rzero) ; init carry 0, mumble 0
+    @loop
+    (subi count count '1)
+    (cmpwi count 0)
+    (la tem arch::misc-data-offset yidx)   ; get yidx
+    (lwzx y yptr tem) 
+    (mullw prod-l x y)
+    (addc prod-l prod-l prod-h)
+    (mulhwu prod-h x y)
+    (addze prod-h prod-h)
+    (la tem arch::misc-data-offset residx)
+    (lwzx y resptr tem)    
+    (addc prod-l prod-l y)
+    (addze prod-h prod-h)
+    (stwx prod-l resptr tem)    
+    (addi residx residx '1)
+    (addi yidx yidx '1)
+    (bgt @loop)
+    (la tem arch::misc-data-offset residx)
+    (stwx prod-h resptr tem)
+    (la vsp 8 vsp)
+    (blr)))
+
+
+
+;;; Multiply X[I] by the unboxed value of the (non-negative) fixnum Y;
+;;; add the incoming carry from CARRY[0] to the 64-bit product.  Store
+;;; the low word of the 64-bit sum in R[0] and the high word in
+;;; CARRY[0].
+
+(defppclapfunction %multiply-and-add ((r 4) (carry 0) (x arg_y) (i arg_x) (y arg_z))
+  (unbox-fixnum imm0 arg_z)
+  (la imm1 arch::misc-data-offset i)
+  (lwzx imm1 x imm1)
+  (mulhwu imm2 imm0 imm1)
+  (mullw imm1 imm0 imm1)
+  (lwz temp0 carry vsp)
+  (lwz imm0 arch::misc-data-offset temp0)
+  (addc imm1 imm1 imm0)
+  (addze imm2 imm2)
+  (stw imm2 arch::misc-data-offset temp0)
+  (lwz arg_z r vsp)
+  (la vsp 8 vsp)
+  (stw imm1 arch::misc-data-offset arg_z)
+  (blr))
+  
+(defppclapfunction %floor ((q 4) (r 0) (num-high arg_x) (num-low arg_y) (denom-arg arg_z))
+  (let ((rem imm0)
+	(rem-low imm1)
+	(quo imm2)
+	(temp imm3)
+	(denom imm4))
+    (lwz denom arch::misc-data-offset denom)
+    (lwz rem arch::misc-data-offset num-high)
+    (lwz rem-low arch::misc-data-offset num-low)
+    (mr temp denom)
+    (sltu quo rem denom)
+    (subi temp temp quo)
+    (and temp temp denom)
+    (sub rem temp rem)
+    (li temp0 '32)
+    @loop
+    (subi temp0 temp0 '1)
+    (cmpwi temp0 0)
+    (slwi rem rem 1)
+    (srwi temp rem-low 31)
+    (or rem rem temp)
+    (slwi rem-low rem-low 1)
+    (sltu rem rem denom)
+    (slwi quo quo 1)
+    (or quo quo temp)
+    (subi temp temp 1)
+    (and temp temp denom)
+    (sub rem rem temp)
+    (bne @loop)
+    (not quo quo)
+    (lwz temp0 q vsp)
+    (stw quo arch::misc-data-offset temp0)
+    (lwz arg_z r vsp)
+    (la vsp 8 vsp)
+    (stw rem arch::misc-data-offset arg_z)
+    (blr)))
+    
 (defppclapfunction %bignum-ref-hi ((bignum arg_y) (i arg_z))
   (la imm1 arch::misc-data-offset i)
   (lhzx imm0 bignum imm1)
@@ -59,12 +314,6 @@
   (la vsp 4 vsp)
   (blr))
 
-;;; %ADD-WITH-CARRY -- Internal.
-;;;
-;;; This should be in assembler, and should not cons intermediate results.  It
-;;; returns a 32bit digit (split in half) and a carry resulting from adding 
-;;; together the a, b, and an incoming carry.
-;;;
 
 
 
@@ -97,7 +346,7 @@
 ;;; We really do:  a - b - 1 + borrow, where borrow is either 0 or 1.
 ;;; 
 
-(defppclapfunction %subtract-with-borrow ((a-h 4) (a-l 0) (b-h arg_x) (b-l
+(defppclapfunction %subtract-with-borrow-1 ((a-h 4) (a-l 0) (b-h arg_x) (b-l
 arg_y) (borrow-in arg_z))
   (let ((a imm0)
         (b imm1)
@@ -136,35 +385,8 @@ arg_y) (borrow-in arg_z))
     (set-nargs 2)
     (ba .spvalues)))
 
-;;; %MULTIPLY -- Internal.
-;;;
-;;; This multiplies two digit-size (32-bit) numbers, returning a 64-bit result
-;;; split into two 32-bit quantities.
-;;; Or, as fate would have it, into 4 16-bit quantities
-;;;
 
 
-; given 2 fixnums, returns product as 4 16 bit dohickies
-(defppclapfunction %multiply-signed-fixnums ((x arg_y)(y arg_z))
-  (let ((x-un imm0)
-        (y-un imm1)
-        (res-h imm2)
-        (res-l imm3))
-    (unbox-fixnum x-un x)
-    (unbox-fixnum y-un y)
-    (mullw res-l x-un y-un)
-    (mulhw res-h x-un y-un)  ; vs mulhwu
-    (digit-h temp0 res-h)
-    (digit-l temp1 res-h)
-    (digit-h temp2 res-l)
-    (digit-l temp3 res-l)
-    (vpush temp0)
-    (vpush temp1)
-    (vpush temp2)
-    (vpush temp3)
-    (set-nargs 4)
-    (la temp0 16 vsp)
-    (ba .SPvalues)))
 
 ;;; %MULTIPLY-AND-ADD  --  Internal.
 ;;;
@@ -180,12 +402,12 @@ arg_y) (borrow-in arg_z))
 ;;; [slh] I assume that the returned carry "digit" can only be 0, 1 or 2
 
 
-(defppclapfunction %multiply-and-add ((x-high 8)
-				      (x-low 4)
-				      (y-high 0)
-				      (y-low arg_x)
-				      (carry-in-high arg_y)
-				      (carry-in-low arg_z))
+(defppclapfunction %multiply-and-add-1 ((x-high 8)
+					(x-low 4)
+					(y-high 0)
+					(y-low arg_x)
+					(carry-in-high arg_y)
+					(carry-in-low arg_z))
   (let ((x imm0)
 	(y imm1)
 	(carry-in imm2)
@@ -213,83 +435,30 @@ arg_y) (borrow-in arg_z))
     (la temp0 16 vsp)
     (ba .SPvalues)))
 
-    
-; return carry-hi carry-lo (= halves of prod-h) - no don't just store it at len
-(defppclapfunction %multiply-and-add-loop ((Bignum 0)(res arg_x)(len arg_y) (x-box arg_z))
-  (let ((x imm0)
-        (idx imm1)
-        (big temp0)
-        ;(res temp1)
-        (count temp2)
-        (prod-h imm2)
-        (prod-l imm3)
-        (y imm4))
-    (unbox-fixnum x x-box)
-    (li idx arch::misc-data-offset)
-    (li count 0)
-    (lwz big bignum vsp)
-    (li prod-h 0) ; init de carry
-    @loop
-    (lwzx y big idx)               ; get digit
-    (mullw prod-l x y)             ; times x to prod-l
-    (addc prod-l prod-l prod-h)    ; add last prod-h with carry out
-    (mulhwu prod-h x y)            ; high times x to prod-h    
-    (adde prod-h prod-h rzero)     ; add carry out to prod-h    
-    (stwx prod-l res idx)    
-    (addi count count '1)
-    (cmpw count len)
-    (addi idx idx '1)
-    (blt @loop)
-    (stwx prod-h res idx)
-    (la vsp 4 vsp)
-    (blr)))
 
-
-;; multiply i'th digit of x by y and add to result starting at digit i
-(defppclapfunction %multiply-and-add-harder-loop-2 ((x-ptr 4) (y-ptr 0)
-                                                    (resptr arg_x)(residx arg_y) (count arg_z))  
-  (let ((tem imm0)
-        (y imm1)
-        (prod-h imm2)
-        (prod-l imm3)
-        (x imm4)
-        (xptr temp2)
-        (yidx temp1)
-        (yptr temp0))
-    (lwz xptr x-ptr vsp)
-    (addi tem residx arch::misc-data-offset)
-    (lwzx x xptr tem)
-    (lwz yptr y-ptr vsp)
-    (li yidx 0) ; init yidx 0 
-    (addc prod-h rzero rzero) ; init carry 0, mumble 0
-    @loop
-    (addi tem yidx arch::misc-data-offset)   ; get yidx
-    (lwzx y yptr tem) 
-    (mullw prod-l x y)
-    (addc prod-l prod-l prod-h)
-    (mulhwu prod-h x y) ; do we know for sure that this doesn't clobber carry?
-    (adde prod-h prod-h rzero)
-    (addi tem residx arch::misc-data-offset)
-    (lwzx y resptr tem)    
-    (addc prod-l prod-l y)
-    (adde prod-h prod-h rzero)
-    (stwx prod-l resptr tem)    
-    (subic. count count '1)
-    (addi residx residx '1)
-    (addi yidx yidx '1)
-    (bgt @loop)
-    (addi tem residx arch::misc-data-offset)
-    (stwx prod-h resptr tem)
-    (la vsp 8 vsp)
-    (blr)))
-
-
-
-(defppclapfunction %logcount ((high arg_y) (low arg_z))
+(defppclapfunction %logcount-complement ((bignum arg_y) (idx arg_z))
   (let ((arg imm0)
         (shift imm1)
         (temp imm2))
-    (compose-digit arg high low)
+    (la arg arch::misc-data-offset idx)
+    (lwzx arg bignum arg)
+    (not. shift arg)
+    (li arg_z 0)
+    (if ne
+      (progn
+        @loop
+        (la temp -1 shift)
+        (and. shift shift temp)
+        (la arg_z '1 arg_z)
+        (bne @loop)))
+    (blr)))
+
+(defppclapfunction %logcount ((bignum arg_y) (idx arg_z))
+  (let ((arg imm0)
+        (shift imm1)
+        (temp imm2))
+    (la arg arch::misc-data-offset idx)
+    (lwzx arg bignum arg)
     (mr. shift arg)
     (li arg_z 0)
     (if ne
@@ -509,82 +678,33 @@ arg_y) (borrow-in arg_z))
     (la vsp 8 vsp)
     (blr)))
 
-(defppclapfunction bignum-subtract-loop ((a 16)(len-a 12)(sa 8)
-                                         (b 4) (len-b 0) (sb arg_x)
-                                         (result arg_y) (length arg_z))
-  (let ((idx imm0)
-        (count temp1)
-        (x imm2)
-        (y imm3)
-        (aptr temp0)
-        (tlen-a temp2)
-        (bptr temp3)
-        (sign-a imm1)
-        (sign-b imm4)
-        (tlen-b arg_x))
-    (li idx arch::misc-data-offset)
-    (li count 0)
-    (lwz sign-a sa vsp)
-    (unbox-fixnum sign-a sign-a)
-    (lwz aptr a vsp)
-    (lwz bptr b vsp)
-    (lwz tlen-a len-a vsp)    
-    (unbox-fixnum sign-b sb)  ; get arg_x to imm reg
-    (lwz tlen-b len-b vsp)  ; tlen-b is arg_x
-    ;(unbox-fixnum sign-b sb)
-    ; initialize carry 1
-    (li x -1)
-    (addic x x 1)
-    ;(addc x rzero rzero)  ; or zero?
-    ; this could be faster - count down len-a and len-b and use more cr's
-    @loop    
-    (cmpw count tlen-a)
-    (mr x sign-a)
-    (bge @a1)
-    (lwzx x aptr idx)
-    @a1
-    (cmpw count tlen-b)
-    (mr y sign-b)
-    ;(unbox-fixnum y sign-b) ; p.o.s clobbers carry
-    (bge @b1)    
-    (lwzx y bptr idx)
-    @b1
-    (subfe x y x)
-    (stwx x result idx)
-    (addi count count '1)
-    (cmpw count length)
-    (addi idx idx '1)
-    (blt @loop)
-    ; return carry
-    (li x 0)
-    (addze x  rzero)
-    (box-fixnum arg_z x)
-    (la vsp 20 vsp)
-    (blr)))
 
-(defppclapfunction bignum-compare-loop ((a arg_x) (b arg_y) (len arg_z))
-  (let ((x imm0)
-        (y imm1)
-        (idx imm2))
-    (addi idx len (- arch::misc-data-offset 4))
-    @loop
-    (lwzx x a idx)
-    (lwzx y b idx)
-    (cmpl :cr1 x y)
-    (subic. len len '1)
-    (subi idx idx '1)
-    (bgt :cr1 @gt)
-    (blt :cr1 @lt)    
-    (bgt :cr0 @loop)
-    (li arg_z 0)   ; was all =
-    (blr)
-    @gt
-    (li arg_z '1)
-    (blr)
-    @lt
-    (li arg_z '-1)
-    (blr)))
+(defppclapfunction %compare-digits ((a arg_x) (b arg_y) (idx arg_z))
+  (la imm0 arch::misc-data-offset idx)
+  (lwzx imm1 a imm0)
+  (lwzx imm0 b imm0)
+  (cmplw imm1 imm0)
+  (li arg_z '0)
+  (beqlr)
+  (li arg_z '1)
+  (bgtlr)
+  (li arg_z '-1)
+  (blr))
 
+;;; Compare A[I] with A[J].
+(defppclapfunction %compare-bignum-digits ((a arg_x) (i arg_y) (j arg_z))
+  (la imm1 arch::misc-data-offset i)
+  (lwzx imm1 a imm1)
+  (la imm0 arch::misc-data-offset j)
+  (lwzx imm0 a imm0)
+  (cmplw imm1 imm0)
+  (li arg_z '0)
+  (beqlr)
+  (li arg_z '1)
+  (bgtlr)
+  (li arg_z '-1)
+  (blr))
+  
 ;; returns number of bits in digit-hi,digit-lo that are sign bits
 ;; 32 - digits-sign-bits is integer-length
 
@@ -615,64 +735,46 @@ arg_y) (borrow-in arg_z))
   (addi arg_z rnil arch::t-offset)
   (blr))
 
-(defppclapfunction bignum-not-loop ((count arg_x) (s1 arg_Y) (dest arg_z))
-  ;(lwz imm0 count vsp)
-  (addi imm1 rzero arch::misc-data-offset)
-  @loop
-  (lwzx imm2 s1 imm1)
-  (not imm2 imm2)
-  (subic. count count 4)
-  (stwx imm2 dest imm1)
-  (addi imm1 imm1 4)
-  (bgt @loop)
-  @out  
+;;; dest[idx] <- (lognot src[idx])
+(defppclapfunction %bignum-lognot ((idx arg_x) (src arg_y) (dest arg_z))
+  (la imm1 arch::misc-data-offset idx)
+  (lwzx imm0 src imm1)
+  (not imm0 imm0)
+  (stwx imm0 dest imm1)
   (blr))
 
-(defppclapfunction bignum-and-loop ((count 0) (s1 arg_x) (s2 arg_y) (dest arg_z))
-  (lwz imm0 count vsp)
-  (addi imm1 rzero arch::misc-data-offset)
-  @loop
-  (lwzx imm2 s1 imm1)
-  (lwzx imm3 s2 imm1)
-  (and imm2 imm3 imm2)
-  (subic. imm0 imm0 4)
-  (stwx imm2 dest imm1)
-  (addi imm1 imm1 4)
-  (bgt @loop)
-  @out
-  (la vsp 4 vsp)
+;;; dest[idx] <- (logand x[idx] y[idx])
+(defppclapfunction %bignum-logand ((idx 0) (x arg_x) (y arg_y) (dest arg_z))
+  (vpop temp0)
+  (la imm1 arch::misc-data-offset temp0)
+  (lwzx imm0 x imm1)
+  (lwzx imm2 y imm1)
+  (and imm0 imm0 imm2)
+  (stwx imm0 dest imm1)
   (blr))
 
-(defppclapfunction bignum-andc2-loop ((count 0) (s1 arg_x) (s2 arg_y) (dest arg_z))
-  (lwz imm0 count vsp)
-  (addi imm1 rzero arch::misc-data-offset)
-  @loop
-  (lwzx imm2 s1 imm1)
-  (lwzx imm3 s2 imm1)
-  (andc imm2 imm2 imm3)
-  (subic. imm0 imm0 4)
-  (stwx imm2 dest imm1)
-  (addi imm1 imm1 4)
-  (bgt @loop)
-  @out
-  (la vsp 4 vsp)
+;;; dest[idx] <- (logandc2 x[idx] y[idx])
+(defppclapfunction %bignum-logandc2 ((idx 0) (x arg_x) (y arg_y) (dest arg_z))
+  (vpop temp0)
+  (la imm1 arch::misc-data-offset temp0)
+  (lwzx imm0 x imm1)
+  (lwzx imm2 y imm1)
+  (andc imm0 imm0 imm2)
+  (stwx imm0 dest imm1)
   (blr))
 
-(defppclapfunction bignum-andc1-loop ((count 0) (s1 arg_x) (s2 arg_y) (dest arg_z))
-  (lwz imm0 count vsp)
-  (addi imm1 rzero arch::misc-data-offset)
-  @loop
-  (lwzx imm2 s1 imm1)
-  (lwzx imm3 s2 imm1)
-  (andc imm2 imm3 imm2)
-  (subic. imm0 imm0 4)
-  (stwx imm2 dest imm1)
-  (addi imm1 imm1 4)
-  (bgt @loop)
-  @out
-  (la vsp 4 vsp)
+;;; dest[idx] <- (logandc1 x[idx] y[idx])
+(defppclapfunction %bignum-logandc1 ((idx 0) (x arg_x) (y arg_y) (dest arg_z))
+  (vpop temp0)
+  (la imm1 arch::misc-data-offset temp0)
+  (lwzx imm0 x imm1)
+  (lwzx imm2 y imm1)
+  (andc imm0 imm2 imm0)
+  (stwx imm0 dest imm1)
   (blr))
 
+
+
 (defppclapfunction digit-lognot-move ((index arg_x) (source arg_y) (dest arg_z))
   (let ((scaled-index imm1))
     (vref32 imm0 source index scaled-index) ; imm1 has c(index) + data-offset
@@ -729,23 +831,28 @@ arg_y) (borrow-in arg_z))
     (stw w1 arch::misc-data-offset dest)
     (blr)))
 
-(defppclapfunction bignum-ior-loop ((count 0) (s1 arg_x) (s2 arg_y) (dest arg_z))
-  (lwz imm0 count vsp)
-  ;(cmpw imm0 rzero)
-  (addi imm1 rzero arch::misc-data-offset)
-  ;(beq @out)
-  @loop
-  (lwzx imm2 s1 imm1)
-  (lwzx imm3 s2 imm1)
-  (or imm2 imm3 imm2)
-  (subic. imm0 imm0 4)
-  (stwx imm2 dest imm1)
-  (addi imm1 imm1 4)
-  (bgt @loop)
-  @out
-  (la vsp 4 vsp)
+;;; dest[idx] <- (logior x[idx] y[idx])
+(defppclapfunction %bignum-logior ((idx 0) (x arg_x) (y arg_y) (dest arg_z))
+  (vpop temp0)
+  (la imm1 arch::misc-data-offset temp0)
+  (lwzx imm0 x imm1)
+  (lwzx imm2 y imm1)
+  (or imm0 imm0 imm2)
+  (stwx imm0 dest imm1)
   (blr))
 
+;;; dest[idx] <- (logxor x[idx] y[idx])
+(defppclapfunction %bignum-logxor ((idx 0) (x arg_x) (y arg_y) (dest arg_z))
+  (vpop temp0)
+  (la imm1 arch::misc-data-offset temp0)
+  (lwzx imm0 x imm1)
+  (lwzx imm2 y imm1)
+  (xor imm0 imm0 imm2)
+  (stwx imm0 dest imm1)
+  (blr))
+
+
+
 (defppclapfunction bignum-xor-loop ((count 0) (s1 arg_x) (s2 arg_y) (dest arg_z))
   (lwz imm0 count vsp)
   (addi imm1 rzero arch::misc-data-offset)
@@ -1018,6 +1125,429 @@ logical means unsigned
     (add arg_z nbits ndigits)
     (blr)))
 
+
+(defppclapfunction %bignum-trim-leading-zeros ((bignum arg_x)
+					       (start arg_y)
+					       (len arg_z))
+  (add imm1 start len)
+  (la imm1 (- arch::misc-data-offset 4) imm1)
+  @loop
+  (cmpwi cr0 len '1)
+  (lwzx imm0 bignum imm1)
+  (cmpwi cr1 imm0 0)
+  (la imm1 -4 imm1)
+  (bnelr cr1)
+  (la len '-1 len)
+  (bne @loop)
+  (blr))
+  
+;;; Set length of bignum to new-len (zeroing out any trailing words between
+;;; the old length and the new.
+(defppclapfunction %shrink-bignum ((new-len arg_y) (bignum arg_z))
+  (let ((old-len temp0)
+	(old-idx imm0)
+	(new-idx imm2)
+	(header imm1))
+    (getvheader header bignum)
+    (header-length old-len header)
+    (cmpw old-len new-len)
+    (la old-idx arch::misc-data-offset old-len)
+    (la new-idx arch::misc-data-offset new-len)
+    (beqlr)
+    @loop
+    (subi old-idx old-idx 4)
+    (cmpw old-idx new-idx)
+    (stwx ppc::rzero bignum old-idx)
+    (bne @loop)
+    (slwi header new-len (- arch::num-subtag-bits arch::fixnumshift))
+    (ori header header arch::subtag-bignum)
+    (stw header arch::misc-header-offset bignum)
+    (blr)))
     
+;;; Especially when large operands are involved, the GNU Multiple Precision
+;;; library's algorithm's are often faster than OpenMCL's.  GMP's MPN
+;;; library defines operations on "limb vectors", which are basically
+;;; just sequences of 32-bit digits (least-significant digit first), which
+;;; is just about exactly the same way that OpenMCL stores bignums.
+;;; We might want to (eventually) link some or all of GMP into OpenMCL;
+;;; in the meantime, it seems that we get some performance benefit from
+;;; using GMP representation and algorithms in some mixture of LAP and Lisp.
+;;; To approximate the "limb vector" representation, we copy operands to
+;;; (and results from) stack-allocated macptrs.  Since those macptrs are
+;;; word-aligned, we can use fixnums to represent word-aligned pointers.
+;;; Obviously, it costs a little to copy back and forth like this; we
+;;; only win when operands are fairly large, and when we can replace an
+;;; N^2 algorithm with something cheaper.
+
+;;; Macptr MUST be word-aligned (low 2 bits must be 0).  Extract
+;;; such an address, return it as a fixnum.
+(defppclapfunction macptr->fixnum ((ptr arg_z))
+  (macptr-ptr arg_z ptr)
+  (blr))
+
+;;; Copy the limb SRC points to to where DEST points.
+(defppclapfunction copy-limb ((src arg_y) (dest arg_z))
+  (lwz imm0 0 src)
+  (stw imm0 0 dest)
+  (blr))
+
+;;; Return T iff LIMB contains 0.
+(defppclapfunction limb-zerop ((limb arg_z))
+  (lwz imm0 0 limb)
+  (cntlzw imm0 imm0)
+  (srwi imm0 imm0 5)
+  (bit0->boolean arg_z imm0 imm0)
+  (blr))
+
+;;; Return -1,0,1 according to whether the contents of Y are
+;;; <,=,> the contents of Z.
+(defppclapfunction compare-limbs ((y arg_y) (z arg_z))
+  (lwz imm1 0 z)
+  (lwz imm0 0 y)
+  (cmplw imm0 imm1)
+  (li arg_z 0)
+  (beqlr)
+  (li arg_z '1)
+  (bgtlr)
+  (li arg_z '-1)
+  (blr))
+
+;;; Add a fixnum to the limb LIMB points to.  Ignore overflow.
+(defppclapfunction add-fixnum-to-limb ((fixnum arg_y) (limb arg_z))
+  (unbox-fixnum imm0 fixnum)
+  (lwz imm1 0 limb)
+  (add imm1 imm1 imm0)
+  (stw imm1 0 limb)
+  (blr))
+
+;;; Store a fixnum value where LIMB points.
+(defppclapfunction copy-fixnum-to-limb ((fixnum arg_y) (limb arg_z))
+  (unbox-fixnum imm0 fixnum)
+  (stwu imm0 0 limb)
+  (blr))
+
+;;; Increment a "LIMB VECTOR" (bignum) by a small amount.  The caller
+;;; knows that carries will only propagate for a word or two.
+(defppclapfunction mpn-incr-u ((limb arg_y) (fixby arg_z))
+  (let ((by imm0)
+	(sum imm1))
+    (unbox-fixnum by fixby)
+    @loop
+    (lwz sum 0 limb)
+    (add sum sum by)
+    (cmplw sum by)
+    (stw sum 0 limb)
+    (li by 1)
+    (la limb 4 limb)
+    (blt @loop)
+    (blr)))
+
+;;; Store XP-YP at WP; return carry (0 or 1).
+;;; wp, xp, yp: word-aligned, unboxed ptrs (fixnums)
+;;; size: boxed fixnum
+;;; returns boxed carry 
+(defppclapfunction mpn-sub-n ((wp 0) (xp arg_x) (yp arg_y) (size arg_z))
+  (vpop imm0)
+  (subi size size '1)
+  (cmpwi size 0)
+  (lwz imm3 0 xp)
+  (lwz imm4 0 yp)
+  (sub imm1 xp imm0)			; imm1 = xp-wp
+  (sub imm2 yp imm0)			; imm2 = yp-wp
+  (addi imm1 imm1 4)			; imm1 = xp-wp+4
+  (addi imm2 imm2 4)			; imm2 = yp-wp+4
+  (subfc imm3 imm4 imm3)
+  (stw imm3 0 imm0)			; wp[0]
+  (beq @done)
+  @top
+  (subi size size '1)
+  (cmpwi size 0)
+  (lwzx imm3 imm1 imm0)			; imm3 = xp[i]
+  (lwzx imm4 imm2 imm0)			; imm4 = xp[i]
+  (subfe imm3 imm4 imm3)
+  (stwu imm3 4 imm0)
+  (bne @top)
+  @done
+  (subfe imm0 rzero rzero)
+  (subfic imm0 imm0 0)
+  (box-fixnum arg_z imm0)
+  (blr))
+
+;;; Store XP+YP at WP; return carry (0 or 1).
+;;; wp, xp, yp = word-aligned, unboxed macptrs (fixnums).
+;;; size = boxed fixnum
+;;; result = boxed carry
+(defppclapfunction mpn-add-n ((wp 0) (xp arg_x) (yp arg_y) (size arg_z))
+  (vpop imm0)
+  (subi size size '1)
+  (cmpwi size 0)
+  (lwz imm3 0 xp)
+  (lwz imm4 0 yp)
+  (sub imm1 xp imm0)			; imm1 = xp-wp
+  (sub imm2 yp imm0)			; imm2 = yp-wp
+  (addi imm1 imm1 4)			; imm1 = xp-wp+4
+  (addi imm2 imm2 4)			; imm2 = yp-wp+4
+  (addc imm3 imm3 imm4)
+  (stw imm3 0 imm0)			; wp[0]
+  (beq @done)
+  @top
+  (subi size size '1)
+  (cmpwi size 0)
+  (lwzx imm3 imm1 imm0)			; imm3 = xp[i]
+  (lwzx imm4 imm2 imm0)			; imm4 = xp[i]
+  (adde imm3 imm4 imm3)
+  (stwu imm3 4 imm0)
+  (bne @top)
+  @done
+  (addze imm0 rzero)
+  (box-fixnum arg_z imm0)
+  (blr))
+
+;;; Add the single limb LIMB to S1P (propagating carry.)  Store the
+;;; result at RP.  RP and S1P may be the same place, so check for
+;;; that and do nothing after carry stops propagating.  Return carry.
+(defppclapfunction mpn-add-1 ((rp-offset 0) (s1p arg_x) (size arg_y) (limb arg_z))
+  (let ((rp temp0))
+    (vpop rp)
+    (subi size size '1)
+    (cmpwi cr2 size 0)
+    (cmpw cr1 rp s1p)			;a common case
+    (subi rp rp 4)
+    (subi s1p s1p 4)
+    (lwz imm0 0 limb)
+    (lwzu imm1 4 s1p)
+    (addc imm1 imm1 imm0)
+    (addze. imm0 rzero)
+    (stwu imm1 4 rp)
+    (beq cr2 @done)
+    @top
+    (beq cr0 @finish)			; branch if  no more carry
+    (subi size size '1)
+    (cmpwi cr2 size 0)
+    (lwzu imm1 4 s1p)
+    (addc imm1 imm1 imm0)
+    (addze. imm0 rzero)
+    (stwu imm1 4 rp)
+    (bne cr2 @top)
+    (box-fixnum arg_z imm0)
+    (blr)
+    @finish
+    (beq cr1 @done)
+    @loop
+    (subi size size '1)
+    (cmpwi cr2 size 0)
+    (lwzu imm1 4 s1p)
+    (stwu imm1 4 rp)
+    (bne cr2 @loop)
+    @done
+    (box-fixnum arg_z imm0)
+    (blr)))
+;;; Multiply the limb vector S1 by the single limb at LIMBPTR, storing
+;;; the result at RES.  Store the "carry out" (high word of last 64-bit
+;;; partial product) at the limb RESULT.
+;;; res, s1, limbptr, result:
+;;;   unboxed, word-aligned ptrs (fixnums).  size: boxed fixnum
+;;; It'd be hard to transliterate the GMP code here; the GMP version
+;;; uses lots more immediate registers than we can easily use in LAP
+;;; (and is much more aggressively pipelined).
+(defppclapfunction mpn-mul-1 ((res-offset 4)
+			      (s1-offset 0)
+			      (size arg_x)
+			      (limbptr arg_y)
+			      (result arg_z))
+  (let ((limb imm0)
+	(resptr temp0)
+	(s1 temp1)
+	(src imm1)
+	(prod-low imm2)
+	(prod-high imm3)
+	(carry imm4))
+    (lwz resptr res-offset vsp)
+    (lwz s1 s1-offset vsp)
+    (la vsp 8 vsp)
+    (la resptr -4 resptr)		; pre-decrement
+    (la s1 -4 s1)
+    (addic carry carry 0)
+    (li carry 0)
+    (lwz limb 0 limbptr)
+    @loop
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu src 4 s1)
+    (mulhwu prod-high src limb)
+    (mullw prod-low src limb)
+    (addc prod-low prod-low carry)
+    (addze carry prod-high)
+    (stwu prod-low 4 resptr)
+    (bne @loop)
+    (stw carry 0 result)
+    (blr)))
+
+;;; multiply s1*limb and add result to res
+;;; res, s1, limbptr, result:
+;;;   unboxed, word-aligned ptrs (fixnums).
+;;; size: boxed fixnum
+;;; limbptr: source "limb".
+;;; result: carry out (high word of product).
+(defppclapfunction mpn-addmul-1 ((res-offset 4)
+				 (s1-offset 0)
+				 (size arg_x)
+				 (limbptr arg_y)
+				 (result arg_z))
+  (let ((limb imm0)
+	(resptr temp0)
+	(s1 temp1)
+	(src imm1)
+	(prod-low imm2)
+	(prod-high imm3)
+	(carry imm4)
+	(prev imm4))
+    (lwz resptr res-offset vsp)
+    (lwz s1 s1-offset vsp)
+    (la vsp 8 vsp)
+    (la resptr -4 resptr)		; pre-decrement
+    (la s1 -4 s1)
+    (addic carry carry 0)
+    (li carry 0)
+    (lwz limb 0 limbptr)
+    @loop
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu src 4 s1)
+    (mulhwu prod-high src limb)
+    (mullw prod-low src limb)
+    (addc prod-low prod-low carry)
+    (addze prod-high prod-high)
+    (lwz prev 4 resptr)
+    (addc prev prev prod-low)
+    (stwu prev 4 resptr)
+    (addze carry prod-high)
+    (bne @loop)
+    (stw carry 0 result)
+    (blr)))  
+
+;;; Multiply the UN-word limb vector at UP and the VN-word limb vector
+;;; at VP, store the result at RP.
+(defppclapfunction mpn-mul-basecase ((rp-offset 4)
+				     (up-offset 0)
+				     (un arg_x)
+				     (vp arg_y)
+				     (vn arg_z))
+  (let ((resptr temp0)
+	(s1 temp1)
+	(up temp2)
+	(rp temp3)
+	(size nargs)
+	(limb imm0)
+	(src imm1)
+	(prod-low imm2)
+	(prod-high imm3)
+	(prev imm4)
+	(carry imm4))
+    (lwz resptr rp-offset vsp)
+    (la rp -4 resptr)
+    (lwz up up-offset vsp)
+    (la s1 -4 up)
+    (la vsp 8 vsp)
+    (mr size un)
+    (lwz limb 0 vp)
+    (subi vn vn '1)
+    (cmpwi cr2 vn 0)
+    (li carry 0)
+    @mul-1-loop
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu src 4 s1)
+    (mulhwu prod-high src limb)
+    (mullw prod-low src limb)
+    (addc prod-low prod-low carry)
+    (addze carry prod-high)
+    (stwu prod-low 4 rp)
+    (bne @mul-1-loop)
+    (stw carry 4 rp)
+    @again
+    (beq cr2 @done)
+    (subi vn vn '1)
+    (cmpwi cr2 vn 0)
+    (mr rp resptr)
+    (la resptr 4 resptr)
+    (la s1 -4 up)
+    (lwzu limb 4 vp)
+    (mr size un)
+    (addic carry carry 0)
+    (li carry 0)
+    @addmul-1-loop
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu src 4 s1)
+    (mulhwu prod-high src limb)
+    (mullw prod-low src limb)
+    (addc prod-low prod-low carry)
+    (addze prod-high prod-high)
+    (lwz prev 4 rp)
+    (addc prev prev prod-low)
+    (stwu prev 4 rp)
+    (addze carry prod-high)
+    (bne @addmul-1-loop)
+    (stw carry 4 rp)
+    (b @again)
+    @done
+    (mr arg_z rnil)
+    (blr)))
+
+;;; left-shift src by 1 bit, storing result at res.  Return
+;;; the bit that was shifted out.
+(defppclapfunction mpn-lshift-1 ((resptr arg_x) (s1ptr arg_y) (size-arg arg_z))
+  (let ((size temp0)
+	(last-bit imm0)
+	(prev imm1)
+	(curr imm2)
+	(sleft imm3)
+	(sright imm4))
+    (subi size size-arg '1)
+    (cmpwi size 0)
+    (add resptr resptr size-arg)
+    (add s1ptr s1ptr size-arg)
+    (lwzu prev -4 s1ptr)
+    (srwi last-bit prev 31)
+    (box-fixnum arg_z last-bit)
+    (beq @end1)
+    @loop
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu curr -4 s1ptr)
+    (slwi sleft prev 1)
+    (srwi sright curr 31)
+    (or sright sright sleft)
+    (stwu sright -4 resptr)
+    (beq @end2)
+    (subi size size '1)
+    (cmpwi size 0)
+    (lwzu prev -4 s1ptr)
+    (slwi sleft curr 1)
+    (srwi sright prev 31)
+    (or sright sright sleft)
+    (stwu sright -4 resptr)
+    (bne @loop)
+    @end1
+    (slwi sleft prev 1)
+    (stwu sleft -4 resptr)
+    (blr)
+    @end2
+    (slwi sleft curr 1)
+    (stwu sleft -4 resptr)
+    (blr)))
+
+;;; Do a 32x32=64 unsigned multiply of the words at X and Y.  Store
+;;; result (low word first) at RESULT.
+(defppclapfunction umulppm ((x arg_x) (y arg_y) (result arg_z))
+  (lwz imm0 0 x)
+  (lwz imm1 0 y)
+  (mullw imm2 imm0 imm1)
+  (mulhwu imm3 imm0 imm1)
+  (stw imm2 0 result)
+  (stw imm3 4 result)
+  (blr))
+
     
 ; End of ppc-bignum.lisp
