@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 #+allow-in-package
 (in-package "CCL")
@@ -616,7 +612,7 @@
 (defun fp-condition-from-fpscr (status-bits control-bits)
   (cond 
    ((and (logbitp (- 23 ppc::fpscr-vx-bit) status-bits)
-         (logbitp (- 31 ppc::fpscr-ve-bit) control-bits)) 'invalid-operation)
+         (logbitp (- 31 ppc::fpscr-ve-bit) control-bits)) 'floating-point-invalid-operation)
    ((and (logbitp (- 23 ppc::fpscr-ox-bit) status-bits)
          (logbitp (- 31 ppc::fpscr-oe-bit) control-bits)) 'floating-point-overflow)
    ((and (logbitp (- 23 ppc::fpscr-ux-bit) status-bits)
@@ -624,7 +620,7 @@
    ((and (logbitp (- 23 ppc::fpscr-zx-bit) status-bits)
          (logbitp (- 31 ppc::fpscr-ze-bit) control-bits)) 'division-by-zero)
    ((and (logbitp (- 23 ppc::fpscr-xx-bit) status-bits)
-         (logbitp (- 31 ppc::fpscr-xe-bit) control-bits)) 'inexact-result)))
+         (logbitp (- 31 ppc::fpscr-xe-bit) control-bits)) 'floating-point-inexact)))
 
 ; This assumes that the FEX and one of {VX OX UX ZX XX} is set.
 ; Ignore 
