@@ -1,22 +1,18 @@
 ;;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 #+allow-in-package
 (in-package "CCL")
@@ -502,53 +498,43 @@
                 4
                 (if (= subtag arch::subtag-double-float-vector)
                   6
-                  1)))))
+                  0)))))
          (total-bits (ash element-count element-bit-shift)))
     (ash (+ 7 total-bits) -3)))
 
 (defun element-type-subtype (type)
   "Convert element type specifier to internal array subtype code"
-  (cond ((or (eq type t)(null type)) arch::subtag-simple-vector)
-        ((memq type '(base-char character)) 
-         arch::subtag-simple-base-string)
-        ((eq type 'bit) arch::subtag-bit-vector)
-        ((memq type '(long-float  double-float))
-         arch::subtag-double-float-vector)
-        ((memq type '(short-float single-float))
-         arch::subtag-single-float-vector)
-        ((and (consp type)
-              (if (eq (%car type) 'signed-byte)
-                (let ((size (car (%cdr type))))
-                  (and (null (%cddr type))
-                       (fixnump size)
-                       (%i< 0 size)
-                       (cond ((%i<= size 8) arch::subtag-s8-vector)
-                             ((%i<= size 16) arch::subtag-s16-vector)
-                             ((%i<= size 32) arch::subtag-s32-vector)
-                             (t arch::subtag-simple-vector))))
-                (if (eq (%car type) 'unsigned-byte)
-                  (let ((size (car (%cdr type))))
-                    (and (null (%cddr type))
-                         (fixnump size)
-                         (%i< 0 size)
-                         (cond ((%i<= size 8) arch::subtag-u8-vector)
-                               ((%i<= size 16) arch::subtag-u16-vector)
-                               ((%i<= size 32) arch::subtag-u32-vector)
-                               (t arch::subtag-simple-vector))))))))
-        ((subtypep type 'bit) arch::subtag-bit-vector)
-        ((subtypep type '(signed-byte 8)) arch::subtag-s8-vector)
-        ((subtypep type '(unsigned-byte 8)) arch::subtag-u8-vector)
-        ((subtypep type '(signed-byte 16)) arch::subtag-s16-vector)
-        ((subtypep type '(unsigned-byte 16)) arch::subtag-u16-vector)
-        ((subtypep type '(signed-byte 32)) arch::subtag-s32-vector)
-        ((subtypep type '(unsigned-byte 32)) arch::subtag-u32-vector)
-        ((subtypep type 'double-float) arch::subtag-double-float-vector)
-        ((subtypep type 'short-float) arch::subtag-single-float-vector)
-        ((subtypep type 'base-char) arch::subtag-simple-base-string)
-        ((subtypep type 'character) arch::subtag-simple-base-string)
-        ((subtypep type nil) nil)
-        ((not (type-specifier-p type)) nil)
-        (t arch::subtag-simple-vector)))
+  (ctype-subtype (specifier-type type)))
+
+(defun ctype-subtype (ctype)
+  (typecase ctype
+    (class-ctype
+     (if (or (eq (class-ctype-class ctype) *character-class*)
+             (eq (class-ctype-class ctype) *standard-char-class*))
+       arch::subtag-simple-base-string
+       arch::subtag-simple-vector))
+    (numeric-ctype
+     (case (numeric-ctype-class ctype)
+       (integer
+        (let* ((low (numeric-ctype-low ctype))
+               (high (numeric-ctype-high ctype)))
+          (cond ((or (null low) (null high)) arch::subtag-simple-vector)
+                ((and (= low 0) (= high 1) arch::subtag-bit-vector))
+                ((and (>= low 0) (<= high 255)) arch::subtag-u8-vector)
+                ((and (>= low 0) (<= high 65535)) arch::subtag-u16-vector)
+                ((and (>= low 0) (<= high #xffffffff) arch::subtag-u32-vector))
+                ((and (>= low -128) (<= high 127)) arch::subtag-s8-vector)
+                ((and (>= low -32768) (<= high 32767) arch::subtag-s16-vector))
+                ((and (>= low (ash -1 31)) (<= high (1- (ash 1 31))))
+                 arch::subtag-s32-vector)
+                (t arch::subtag-simple-vector))))
+       (float
+        (case (numeric-ctype-format ctype)
+          (double-float arch::subtag-double-float-vector)
+          (short-float arch::subtag-single-float-vector)
+          (t arch::subtag-simple-vector)))
+       (t arch::subtag-simple-vector)))
+    (t arch::subtag-simple-vector)))
 
 (defun %set-simple-array-p (array)
   (setf (%svref array  arch::arrayh.flags-cell)
