@@ -1,22 +1,18 @@
 ;;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 ;;;
 ;;; level-0;l0-float.lisp
 
@@ -674,7 +670,8 @@
 
 
 (eval-when (:execute :compile-toplevel)
-  (defconstant half-pi (asin 1.0d0))  ;Not CL.  ; what kind of half-pi?
+  (defconstant double-float-half-pi (asin 1.0d0))
+  (defconstant single-float-half-pi (asin 1.0e0))
 )
 
 
@@ -685,15 +682,14 @@
       (if (and (<= -1.0d0 x)
                (<= x 1.0d0))
         (%double-float-acos! x (%make-sfloat))
-        (- half-pi (asin x))))
+        (- double-float-half-pi (asin x))))
     (with-stack-short-floats ((sx x))
       (locally
           (declare (type short-float sx))
         (if (and (<= -1.0s0 sx)
                  (<= sx 1.0s0))
           (%single-float-acos! sx (%make-sfloat))
-          (with-stack-short-floats ((shalf-pi half-pi))
-            (- shalf-pi (asin sx))))))))
+	  (- single-float-half-pi (asin sx)))))))
 
 
 (defun fsqrt (x)
@@ -717,7 +713,7 @@
       (if (plusp (float-sign x))
         y
         (float-sign y pi))
-      (float-sign y half-pi))
+      (float-sign y double-float-half-pi))
     (%double-float-atan2! y x (or result (%make-dfloat)))))
 
 (defun %sf-atan2 (y x &optional result)
@@ -726,7 +722,7 @@
       (if (plusp (float-sign x))
         y
         (float-sign y pi))
-      (float-sign y half-pi))
+      (float-sign y single-float-half-pi))
     (%single-float-atan2! y x (or result (%make-sfloat)))))
 
 ; end of l0-float.lisp
