@@ -1,22 +1,18 @@
 ;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 
@@ -247,6 +243,8 @@
             (or cell (%car (push (cons key (%unbound-marker-8)) (nhash.locked-additions hash))))
             cell)))))
 
+;;; Don't rehash at all, unless some key is address-based (directly or
+;;; indirectly.)
 (defun %needs-rehashing-p (hash)
   (unless (eql (get-fwdnum) (nhash.fixnum hash))
     (let ((flags (nhash.vector.flags (nhash.vector hash))))
@@ -255,9 +253,11 @@
         ; GC is tracking key movement
         (logbitp $nhash_key_moved_bit flags)
         ; GC is not tracking key movement
-        (or (logbitp $nhash_key_moved_bit flags)
-            ; key_moved_bit clear means only rehash on full gc or growzone
-            (not (eql (gc-count) (nhash.gc-count hash))))))))
+        (or (logbitp $nhash_key_moved_bit flags) ; Not sure what this means, but rarely true ...
+            ;; Rehash if some key is somehow address-based and a GC occurred since last rehash
+            (and (not (eql (gc-count) (nhash.gc-count hash))) 
+                 (or (logbitp $nhash_component_address_bit flags)
+                     (logbitp $nhash_ephemeral_bit flags))))))))
 
 (defun %does-not-need-rehashing (hash)
   (get-fwdnum hash)
