@@ -1,22 +1,18 @@
 ;;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;;;
 ;;; level-0;l0-numbers.lisp
@@ -28,25 +24,7 @@
   (require "NUMBER-MACROS")
   (require "NUMBER-CASE-MACRO")
 
-  (defmacro fixnum-to-bignum-set (fix big)
-    (arch::once-only ((fixv fix))
-		    `(%bignum-set ,big 0 (%ilogand (%iasr 16 ,fixv) #xffff) (%ilogand ,fixv #xffff))))
-  
-  (defmacro with-small-bignum-buffers (specs &body body)
-    (arch::collect ((binds)
-		   (inits)
-		   (names))
-		  (dolist (spec specs)
-		    (let ((name (first spec)))
-		      (binds `(,name (%alloc-misc 1 arch::subtag-bignum)))
-		      (names name)
-		      (let ((init (second spec)))
-			(when init
-			  (inits `(fixnum-to-bignum-set ,init ,name))))))
-		  `(let* ,(binds)
-		    (declare (dynamic-extent ,@(names)))
-		    ,@(inits)
-		    ,@body)))
+
 
   (defvar *dfloat-dops* '((* . %double-float*-2!)(/ . %double-float/-2!)
 			  (+ . %double-float+-2!)(- . %double-float--2!)))
@@ -87,73 +65,73 @@
   (defmacro two-arg-+/- (name op big-op)
     `(defun ,name (x y)     
       (number-case x
-       (fixnum (number-case y
-			    (fixnum (,op (the fixnum x) (the fixnum y)))
-			    (double-float (rat-dfloat ,op x y))
-			    (short-float (rat-sfloat ,op x y))
-			    (bignum (with-small-bignum-buffers ((bx x))
-				      (,big-op bx y)))
-			    (complex (complex (,op x (%realpart y))
-					      ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))
-			    (ratio (let* ((dy (%denominator y)) 
-					  (n (,op (* x dy) (%numerator y))))
-				     (%make-ratio n dy)))))
-       (double-float (number-case y
-				  (double-float (,op (the double-float x) (the double-float y)))
-				  (short-float (with-stack-double-floats ((dy y))
-						 (,op (the double-float x) (the double-float dy))))
-				  (rational (dfloat-rat ,op x y))
-				  (complex (complex (,op x (%realpart y)) 
-						    ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))))
-       (short-float (number-case y                                
-				 (short-float (,op (the short-float x) (the short-float y)))
-				 (double-float (with-stack-double-floats ((dx x))
-						 (,op (the double-float dx) (the double-float y))))
-				 (rational (sfloat-rat ,op x y))
-				 (complex (complex (,op x (%realpart y))
-						   ,(if (eq op '-) `(- (%imagpart y)) `(%imagpart y))))))
-       (bignum (number-case y
-			    (bignum (,big-op x y))
-			    (fixnum (with-small-bignum-buffers ((by y))
-				      (,big-op x by)))
-			    (double-float (rat-dfloat ,op x y))
-			    (short-float (rat-sfloat ,op x y))
-			    (complex (complex (,op x (realpart y)) 
-					      ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))
-			    (ratio
-			     (let* ((dy (%denominator y))
-				    (n (,op (* x dy) (%numerator y))))
-			       (%make-ratio n dy)))))
-       (complex (number-case y
-			     (complex (canonical-complex (,op (%realpart x) (%realpart y))
-							 (,op (%imagpart x) (%imagpart y))))
-			     ((rational float) (complex (,op (%realpart x) y) (%imagpart x)))))
-       (ratio (number-case y
-			   (ratio
-			    (let* ((nx (%numerator x))
-				   (dx (%denominator x))
-				   (ny (%numerator y))
-				   (dy (%denominator y))
-				   (g1 (gcd dx dy)))
-			      (if (eql g1 1)
-				  (%make-ratio (,op (* nx dy) (* dx ny)) (* dx dy))
-				  (let* ((t1 (,op (* nx (truncate dy g1)) (* (truncate dx g1) ny)))
-					 (g2 (gcd t1 g1))
-					 (t2 (truncate dx g1)))
-				    (cond ((eql t1 0) 0)
-					  ((eql g2 1) (%make-ratio t1 (* t2 dy)))
-					  (t
-					   (let* ((nn (truncate t1 g2))
-						  (t3 (truncate dy g2))
-						  (nd (if (eql t2 1) t3 (* t2 t3))))
-					     (if (eql nd 1) nn (%make-ratio nn nd)))))))))
-			   (integer
-			    (let* ((dx (%denominator x)) (n (,op (%numerator x) (* y dx))))
-			      (%make-ratio n dx)))
-			   (double-float (rat-dfloat ,op x y))
-			   (short-float (rat-sfloat ,op x y))
-			   (complex (complex (,op x (%realpart y)) 
-					     ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y)))))))))
+	(fixnum (number-case y
+		  (fixnum (,op (the fixnum x) (the fixnum y)))
+		  (double-float (rat-dfloat ,op x y))
+		  (short-float (rat-sfloat ,op x y))
+		  (bignum (with-small-bignum-buffers ((bx x))
+			    (,big-op bx y)))
+		  (complex (complex (,op x (%realpart y))
+				    ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))
+		  (ratio (let* ((dy (%denominator y)) 
+				(n (,op (* x dy) (%numerator y))))
+			   (%make-ratio n dy)))))
+	(double-float (number-case y
+			(double-float (,op (the double-float x) (the double-float y)))
+			(short-float (with-stack-double-floats ((dy y))
+				       (,op (the double-float x) (the double-float dy))))
+			(rational (dfloat-rat ,op x y))
+			(complex (complex (,op x (%realpart y)) 
+					  ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))))
+	(short-float (number-case y                                
+		       (short-float (,op (the short-float x) (the short-float y)))
+		       (double-float (with-stack-double-floats ((dx x))
+				       (,op (the double-float dx) (the double-float y))))
+		       (rational (sfloat-rat ,op x y))
+		       (complex (complex (,op x (%realpart y))
+					 ,(if (eq op '-) `(- (%imagpart y)) `(%imagpart y))))))
+	(bignum (number-case y
+		  (bignum (,big-op x y))
+		  (fixnum (with-small-bignum-buffers ((by y))
+			    (,big-op x by)))
+		  (double-float (rat-dfloat ,op x y))
+		  (short-float (rat-sfloat ,op x y))
+		  (complex (complex (,op x (realpart y)) 
+				    ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y))))
+		  (ratio
+		   (let* ((dy (%denominator y))
+			  (n (,op (* x dy) (%numerator y))))
+		     (%make-ratio n dy)))))
+	(complex (number-case y
+		   (complex (canonical-complex (,op (%realpart x) (%realpart y))
+					       (,op (%imagpart x) (%imagpart y))))
+		   ((rational float) (complex (,op (%realpart x) y) (%imagpart x)))))
+	(ratio (number-case y
+		 (ratio
+		  (let* ((nx (%numerator x))
+			 (dx (%denominator x))
+			 (ny (%numerator y))
+			 (dy (%denominator y))
+			 (g1 (gcd dx dy)))
+		    (if (eql g1 1)
+		      (%make-ratio (,op (* nx dy) (* dx ny)) (* dx dy))
+		      (let* ((t1 (,op (* nx (truncate dy g1)) (* (truncate dx g1) ny)))
+			     (g2 (gcd t1 g1))
+			     (t2 (truncate dx g1)))
+			(cond ((eql t1 0) 0)
+			      ((eql g2 1) (%make-ratio t1 (* t2 dy)))
+			      (t
+			       (let* ((nn (truncate t1 g2))
+				      (t3 (truncate dy g2))
+				      (nd (if (eql t2 1) t3 (* t2 t3))))
+				 (if (eql nd 1) nn (%make-ratio nn nd)))))))))
+		 (integer
+		  (let* ((dx (%denominator x)) (n (,op (%numerator x) (* y dx))))
+		    (%make-ratio n dx)))
+		 (double-float (rat-dfloat ,op x y))
+		 (short-float (rat-sfloat ,op x y))
+		 (complex (complex (,op x (%realpart y)) 
+				   ,(if (eq op '-)`(- (%imagpart y)) `(%imagpart y)))))))))
 
   (declaim (inline  %make-complex %make-ratio))
   (declaim (inline canonical-complex))
@@ -230,7 +208,7 @@
 (defun plusp (number)
   (number-case number
     (fixnum (%i> number 0))
-    (bignum (bignum-plus-p number))
+    (bignum (bignum-plusp number))
     (short-float (%short-float-plusp number))
     (double-float (%double-float-plusp number))
     (ratio (plusp (%numerator number)))))
@@ -320,7 +298,7 @@
               (fixnum (< (the fixnum x) (the fixnum y)))
               (double-float (eq -1 (fixnum-dfloat-compare x y)))
               (short-float (eq -1 (fixnum-sfloat-compare x y)))
-              (bignum (bignum-plus-p y))
+              (bignum (bignum-plusp y))
               (ratio (< x (ceiling (%numerator y)(%denominator y))))))
     (double-float (number-case y ; x
                     (double-float (< (the double-float x)(the double-float y))) ;x
@@ -338,7 +316,7 @@
                     (ratio (< (rational x) y))))
     (bignum (number-case y 
               (bignum (EQ -1 (bignum-compare x y)))
-              (fixnum (not (bignum-plus-p x)))
+              (fixnum (not (bignum-plusp x)))
               (ratio (< x (ceiling (%numerator y)(%denominator y))))
               (double-float (eq -1 (bignum-dfloat-compare x y)))
               (short-float (eq -1 (bignum-sfloat-compare x y)))))
@@ -358,7 +336,7 @@
   (number-case x
     (fixnum (number-case y
               (fixnum (> (the fixnum x) (the fixnum y)))
-              (bignum (not (bignum-plus-p y)))
+              (bignum (not (bignum-plusp y)))
               (double-float (eq 1 (fixnum-dfloat-compare x y)))
               (short-float (eq 1 (fixnum-sfloat-compare x y)))
               ; or (> (* x denom) num) ?
@@ -378,7 +356,7 @@
                     (bignum (eq -1 (bignum-sfloat-compare y x)))
                     (ratio (> (rational x) y))))
     (bignum (number-case y
-              (fixnum (bignum-plus-p x))
+              (fixnum (bignum-plusp x))
               (bignum (eq 1 (bignum-compare x y)))
               ; or (> (* x demon) num)
               (ratio (> x (floor (%numerator y) (%denominator y))))
@@ -557,13 +535,6 @@
     x
     (truncate x y)))
 
-
-
-(defun %fixnum-*-2 (y z)
-  (or (%%fixnum-*-2 y z)
-      ; let this deal with making a 1 or 2 digit bignum
-      (multiply-fixnums y z)))
-
 (defun *-2 (x y)
   ;(declare (optimize (speed 3)(safety 0)))
   (flet ((integer*ratio (x y)
@@ -602,7 +573,7 @@
                 (complex (complex*real y x))))
       (fixnum (number-case y
                 (bignum (multiply-bignum-and-fixnum y x))
-                (fixnum (%fixnum-*-2 (the fixnum x) (the fixnum y)))
+                (fixnum (multiply-fixnums (the fixnum x) (the fixnum y)))
                 (short-float (sfloat-rat * y x))
                 (double-float (dfloat-rat * y x))
                 (ratio (integer*ratio x y))
@@ -636,7 +607,7 @@
   (declare (ignore res))
   (number-case x      
       (fixnum (number-case y
-                (fixnum (%fixnum-*-2 (the fixnum x) (the fixnum y)))
+                (fixnum (* (the fixnum x) (the fixnum y)))
                 (t (multiply-bignum-and-fixnum y x))))
       (bignum (number-case y
                 (fixnum (multiply-bignum-and-fixnum x y))
@@ -1142,7 +1113,7 @@
                       (short-float
                        (with-stack-short-floats ((ddiv divisor)
                                                       (f2))
-                         (%unary-truncate (%double-float/-2! number ddiv f2))))
+                         (%unary-truncate (%short-float/-2! number ddiv f2))))
                       (double-float
                        (with-stack-double-floats ((n2 number)
 						      (f2))
