@@ -1,22 +1,18 @@
 ;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (eval-when (:compile-toplevel :execute)
 
@@ -62,7 +58,7 @@
     (incf (the fixnum (faslstate.faslecnt s))))
   val)
 
-(defun %fasl-read-buffer (s)
+(defun %simple-fasl-read-buffer (s)
   (let* ((fd (faslstate.faslfd s))
          (buffer (faslstate.iobuffer s))
          (bufptr (%get-ptr buffer)))
@@ -77,7 +73,7 @@
         (error "Fix this: look at errno, EOF")))))
 
  
-(defun %fasl-read-byte (s)
+(defun %simple-fasl-read-byte (s)
   (loop
     (let* ((buffer (faslstate.iobuffer s))
            (bufptr (%get-ptr buffer)))
@@ -158,7 +154,7 @@
         (setq size (%fasl-read-long s))))
     size))
 
-(defun %fasl-read-n-bytes (s ivector byte-offset n)
+(defun %simple-fasl-read-n-bytes (s ivector byte-offset n)
   (declare (fixnum byte-offset n))
   (do* ()
        ((= n 0))
@@ -254,7 +250,10 @@
 
 
 (defun pkg-arg (thing &optional deleted-ok)
-  (let* ((xthing (if (symbolp thing) (symbol-name thing) thing)))
+  (let* ((xthing (if (or (symbolp thing)
+                         (typep thing 'character))
+                   (string thing)
+                   thing)))
     (let* ((typecode (typecode xthing)))
         (declare (fixnum typecode))
         (cond ((= typecode arch::subtag-package)
@@ -276,6 +275,8 @@
 (defun %fasl-listX (s dotp)
   (let* ((len (%fasl-read-word s)))
     (declare (fixnum len))
+    (when (= len 0)
+      (setq len (%fasl-read-long s)))
     (let* ((val (%epushval s (cons nil nil)))
            (tail val))
       (declare (type cons val tail))
@@ -506,6 +507,7 @@
     ; (format t "~& source-file = ~s" source-file)
     (setq *loading-file-source-file* source-file)))
 
+#-openmcl
 (deffaslop $fasl-library-pointer (s)
   (setf (faslstate.faslval s)
         (pfsl-shared-library-offset s)))
@@ -522,7 +524,7 @@
 
 ;;; The loader itself
 
-(defun %fasl-set-file-pos (s new)
+(defun %simple-fasl-set-file-pos (s new)
   (let* ((fd (faslstate.faslfd s))
          (posoffset (fd-tell fd)))
     (if (>= (decf posoffset new) 0)
@@ -531,16 +533,84 @@
           (progn
             (setf (faslstate.bufcount s) posoffset)
             (incf (%get-long (faslstate.iobuffer s)) count)
-            (return-from %fasl-set-file-pos nil)))))
+            (return-from %simple-fasl-set-file-pos nil)))))
     (progn
       (setf (faslstate.bufcount s) 0)
       (fd-lseek fd new #$SEEK_SET))))
 
-(defun %fasl-get-file-pos (s)
+(defun %simple-fasl-get-file-pos (s)
   (- (fd-tell (faslstate.faslfd s)) (faslstate.bufcount s)))
 
 (defparameter *%fasload-verbose* t)
 
+;; the default fasl file opener sets up the fasl state and checks the header
+(defun %simple-fasl-open (string s)
+  (let* ((ok nil)
+	 (fd (fd-open string #$O_RDONLY))
+	 (err 0))
+    (declare (fixnum fd))
+    (if (>= fd 0)
+	(if (< (fd-lseek fd 0 #$SEEK_END) 4)
+	    (setq err $xnotfasl)
+	  (progn
+	    (setq err 0)
+	    (setf (faslstate.bufcount s) 0
+		  (faslstate.faslfd s) fd)
+	    (fd-lseek fd 0 #$SEEK_SET)
+	    (multiple-value-setq (ok err) (%fasl-check-header s))))
+      (setq err fd))
+    (unless (eql err 0) (setf (faslstate.faslerr s) err))
+    ok))
+
+;; once the fasl state is set up, this checks the fasl header
+;; and returns (values ok err)
+(defun %fasl-check-header (s)
+  (let* ((signature (%fasl-read-word s)))
+    (declare (fixnum signature))
+    (if (= signature $fasl-file-id)
+	(values t 0)
+      (if (= signature $fasl-file-id1)
+	  (progn
+	    (%fasl-set-file-pos s (%fasl-read-long s))
+	    (values t 0))
+	(values nil $xnotfasl)))))
+
+(defun %simple-fasl-close (s)
+  (let* ((fd (faslstate.faslfd s)))
+    (when fd (fd-close fd))))
+
+(defun %simple-fasl-init-buffer (s)
+  (declare (ignore s))
+  nil)
+
+(defvar *fasl-api* nil)
+(setf *fasl-api* (%istruct 'faslapi
+			   #'%simple-fasl-open
+			   #'%simple-fasl-close
+			   #'%simple-fasl-init-buffer
+			   #'%simple-fasl-set-file-pos
+			   #'%simple-fasl-get-file-pos
+			   #'%simple-fasl-read-buffer
+			   #'%simple-fasl-read-byte
+			   #'%simple-fasl-read-n-bytes))
+
+(defun %fasl-open (string s)
+  (funcall (faslapi.fasl-open *fasl-api*) string s))
+(defun %fasl-close (s)
+  (funcall (faslapi.fasl-close *fasl-api*) s))
+(defun %fasl-init-buffer (s)
+  (funcall (faslapi.fasl-init-buffer *fasl-api*) s))
+(defun %fasl-set-file-pos (s new)
+  (funcall (faslapi.fasl-set-file-pos *fasl-api*) s new))
+(defun %fasl-get-file-pos (s)
+  (funcall (faslapi.fasl-get-file-pos *fasl-api*) s))
+(defun %fasl-read-buffer (s)
+  (funcall (faslapi.fasl-read-buffer *fasl-api*) s))
+(defun %fasl-read-byte (s)
+  (funcall (faslapi.fasl-read-byte *fasl-api*) s))
+(defun %fasl-read-n-bytes (s ivector byte-offset n)
+  (funcall (faslapi.fasl-read-n-bytes *fasl-api*) s ivector byte-offset n))
+
 (defun %fasload (string &optional (table *fasl-dispatch-table*)
                         start-faslops-function
                         stop-faslops-function)
@@ -570,76 +640,58 @@
     (setf (faslstate.faslversion s) 0)
     (%stack-block ((buffer (+ 4 $fasl-buf-len)))
       (setf (faslstate.iobuffer s) buffer)
+      (%fasl-init-buffer s)
       (let* ((old %parse-string%))
         (setq %parse-string% nil)    ;  mark as in use
         (setf (faslstate.oldfaslstr s) old
               (faslstate.faslstr s) (or old (make-string 255 :element-type 'base-char)))
-        
-        (flet ((%fasl-open (s)
-                 (let* ((ok nil)
-                        (fd (fd-open string #$O_RDONLY))
-                        (err 0))
-                   (declare (fixnum fd))
-                   (if (>= fd 0)
-                     (if (< (fd-lseek fd 0 #$SEEK_END) 4)
-                       (setq err $xnotfasl)
-                       (progn
-                         (setq err 0)
-                         (setf (faslstate.bufcount s) 0
-                               (faslstate.faslfd s) fd)
-                         (fd-lseek fd 0 #$SEEK_SET)
-                         (let* ((signature (%fasl-read-word s)))
-                           (declare (fixnum signature))
-                           (if (= signature $fasl-file-id)
-                             (setq ok t)
-                             (if (= signature $fasl-file-id1)
-                               (progn
-                                 (%fasl-set-file-pos s (%fasl-read-long s))
-                                 (setq ok t))
-                               (setq err $xnotfasl))))))
-                     (setq err fd))
-                   (unless (eql err 0) (setf (faslstate.faslerr s) err))
-                   ok)))
-          (unwind-protect
-            (when (%fasl-open s)
-              (let* ((nblocks (%fasl-read-word s))
-                     (*pfsl-library-base* nil)
-                     (*pfsl-library* nil))
-                (declare (fixnum nblocks))
-                (declare (special *pfsl-library-base* *pfsl-library*))
-                (unless (= nblocks 0)
-                  (let* ((pos (%fasl-get-file-pos s)))
-                    (dotimes (i nblocks)
-                      (%fasl-set-file-pos s pos)
-                      (%fasl-set-file-pos s (%fasl-read-long s))
-                      (incf pos 8)
-                      (when start-faslops-function (funcall start-faslops-function s))
-                      (let* ((version (%fasl-read-word s)))
-                        (declare (fixnum version))
-                        (if (or (> version (+ #xff00 $fasl-vers))
-                                (< version (+ #xff00 $fasl-min-vers)))
+	(unwind-protect
+          (when (%fasl-open string s)
+	    (let* ((nblocks (%fasl-read-word s))
+		   (*pfsl-library-base* nil)
+		   (*pfsl-library* nil))
+	      (declare (fixnum nblocks))
+	      (declare (special *pfsl-library-base* *pfsl-library*))
+	      (unless (= nblocks 0)
+		(let* ((pos (%fasl-get-file-pos s)))
+		  (dotimes (i nblocks)
+		    (%fasl-set-file-pos s pos)
+		    (%fasl-set-file-pos s (%fasl-read-long s))
+		    (incf pos 8)
+		    (when start-faslops-function (funcall start-faslops-function s))
+		    (let* ((version (%fasl-read-word s)))
+		      (declare (fixnum version))
+		      (if (or (> version (+ #xff00 $fasl-vers))
+			      (< version (+ #xff00 $fasl-min-vers)))
                           (%err-disp (if (>= version #xff00) $xfaslvers $xnotfasl))
-                          (progn
-                            (setf (faslstate.faslversion s) version)
-                            (%fasl-read-word s) 
-                            (%fasl-read-word s)       ; Ignore kernel version stuff
-                            (setf (faslstate.faslevec s) nil
-                                  (faslstate.faslecnt s) 0)
-                            (do* ((op (%fasl-read-byte s) (%fasl-read-byte s)))
-                                 ((= op $faslend))
-                              (declare (fixnum op))
-                              (%fasl-dispatch s op))
-                            (when stop-faslops-function (funcall stop-faslops-function s))
-                            ))))))))
-            (setq %parse-string% (faslstate.oldfaslstr s))
-            (let* ((fd (faslstate.faslfd s)))
-              (when fd (fd-close fd))))
-          (let* ((err (faslstate.faslerr s)))
-            (if err
-              (progn
-                (%string-to-stderr ";!!Error loading ") (pdbg string)
-                (values nil err))
-              (values t nil))))))))
+			(progn
+			  (setf (faslstate.faslversion s) version)
+			  (%fasl-read-word s) 
+			  (%fasl-read-word s)       ; Ignore kernel version stuff
+			  (setf (faslstate.faslevec s) nil
+				(faslstate.faslecnt s) 0)
+			  (do* ((op (%fasl-read-byte s) (%fasl-read-byte s)))
+			       ((= op $faslend))
+			    (declare (fixnum op))
+			    (%fasl-dispatch s op))
+			  (when stop-faslops-function (funcall stop-faslops-function s))
+			  ))))))))
+	  (setq %parse-string% (faslstate.oldfaslstr s))
+	  (%fasl-close s))
+	(let* ((err (faslstate.faslerr s)))
+	  (if err
+	      (progn
+		(when *%fasload-verbose*
+		  (let* ((herald ";!!Error loading ")
+			 (hlen (length herald))
+			 (len (length string))
+			 (msg (make-string (+ hlen len))))
+		    (declare (dynamic-extent msg))
+		    (%copy-ivector-to-ivector herald 0 msg 0 hlen)
+		    (%copy-ivector-to-ivector string 0 msg hlen len)
+		    (bug msg))
+		  (values nil err)))
+	    (values t nil)))))))
 
 
 (defun %new-package-hashtable (size)
