@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;; L1-aprims.lisp
 
@@ -192,16 +188,7 @@
        (setf (gethash sym %setf-function-names%) (construct-setf-function-name sym))))
 
 
-(defun string (thing)
-  (if (stringp thing)
-    thing
-    (if (symbolp thing)
-      (symbol-name thing)
-      (if (characterp thing)
-        (make-string 1 
-                     :element-type (if (typep thing 'base-char) 'base-char 'character)
-                     :initial-element thing)
-        (report-bad-arg thing '(or string symbol character))))))
+
                      
 
 (defconstant *setf-package* (or (find-package "SETF") (make-package "SETF" :use nil :external-size 1)))
@@ -348,46 +335,38 @@
   (seq-dispatch seq (list-reverse seq) (vector-reverse seq)))
 )
 
-; The length in bytes of the packed equivalent of a fat string.
-; this guy does not count fat chars that make no sense in script. actually does now
-; And  %pstr-pointer will truncate fat chars that make no sense in script.
-(defun byte-length (string &optional  start end)
-  (when (or start end)(chkbounds string start end))
-  (if (not start) (setq start 0))
-  (if (not end)(setq end (length string)))
-  (let ((len (- end start)))
-    (cond 
-     ((base-string-p string)
-      len)
-     (t 
-        (let () ;(table (get-char-byte-table script))) ; just is script fat??
-          (cond 
-           (nil (not table) len)
-           (t (multiple-value-bind (str offset)(array-data-and-offset string)                
-                (let* ((start (%i+ offset start))
-                       (end (%i+ offset end))
-                       (j 0))
-                  (until (eq start end)
-                    (let ((c (%scharcode str start)))
-                      (if (and (%i> c #xff)) #|(eq (aref table (%ilsr 8 c)) 1))|#
-                        (setq j (%i+ j 2))
-                        (setq j (%i+ j 1))))
-                    (setq start (%i+ start 1)))
-                  j)))))))))
+(defun check-sequence-bounds (seq start end &optional length)
+  (if (not length) 
+    (setq length (length seq)))
+  (if (not end)
+    (setq end length)
+    (unless (typep end 'fixnum)
+      (report-bad-arg end 'fixnum)))
+  (unless (typep start 'fixnum)
+    (report-bad-arg start 'fixnum))
+  (cond ((> end length)
+         (report-bad-arg end `(integer 0 (,length))))
+	((< start 0)
+	 (report-bad-arg start `(integer 0)))
+        ((> start end)
+         (report-bad-arg start `(integer 0 ,end)))
+        (t end)))
+  
 
-
-
-
-
-
-; used by with-cstr (used by help-manager.lisp)
-(defun %cstr-segment-pointer (string pointer start end)
-  (setf (%get-byte pointer (%put-string-segment-contents pointer string start end)) 0)
-  nil)
-
+(defun byte-length (string &optional  (start 0) end)
+  (setq end (check-sequence-bounds string start end))
+  (- end start))
 
 
 
+(defun make-cstring (string)
+  (let* ((len (length string)))
+    (declare (fixnum len))
+    (let* ((s (malloc (the fixnum (1+ len)))))
+      (setf (%get-byte s len) 0)
+      (multiple-value-bind (data offset) (array-data-and-offset string)
+	(%copy-ivector-to-ptr data offset s 0 len)
+	s))))
 
 
 (defun extended-string-p (thing)
@@ -487,10 +466,12 @@
 
 
 ; appears to be unused
-(defun upgraded-array-element-type (type)
+(defun upgraded-array-element-type (type &optional env)
+  (declare (ignore env))
   (element-subtype-type (element-type-subtype type)))
 
-(defun upgraded-complex-part-type (type)
+(defun upgraded-complex-part-type (type &optional env)
+  (declare (ignore env))
   (declare (ignore type))               ; Ok, ok.  So (upgraded-complex-part-type 'bogus) is 'REAL. So ?
   'real)
 
@@ -644,9 +625,7 @@
      (unless (and (typep idx 'fixnum) (>= (the fixnum idx) 0))
        (report-bad-arg idx 'unsigned-byte))
      (locally 
-       (declare (fixnum idx))
-       (if (and (array-has-fill-pointer-p sequence)
-                (>= idx (the fixnum  (fill-pointer sequence))))
+       (if (>= idx (length sequence))
          (%err-disp $XACCESSNTH idx sequence)
          (aref sequence idx))))))
 
@@ -667,8 +646,7 @@
        (report-bad-arg idx 'unsigned-byte))
      (locally 
        (declare (fixnum idx))
-       (if (and (array-has-fill-pointer-p sequence)
-                (>= idx (the fixnum (fill-pointer sequence))))
+       (if (>= idx (length sequence))
          (%err-disp $XACCESSNTH idx sequence)
          (setf (aref sequence idx) value))))))
 
@@ -1197,10 +1175,6 @@
                     (car (push (cons type nil) *def-accessor-types*)))))
       (setf (cdr cell) (if (vectorp names) names (%list-to-uvector nil names))))))
 
-; Real definition in lib;misc.lisp
-(defun (setf documentation) (string thing doc-type)
-  (declare (ignore thing doc-type))
-  string)
 
 
 (defpackage "OS"
