@@ -1,27 +1,24 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;;; This file contains the error/condition system.  Functions that
 ;;; signal/handle errors are defined later.
 
+(in-package "CCL")
 
 ;;;***********************************
 ;;; Error System
@@ -101,11 +98,31 @@
                                      :expected-type (%type-error-type expected)))))
 )
 
-  
+
+(define-condition sequence-index-type-error (type-error)
+  ((sequence :initarg :sequence))
+  (:report (lambda (c s)
+             (format s "~s is not a valid sequence index for ~s"
+                     (type-error-datum c)
+                     (slot-value c 'sequence)))))
+
+
+;;; This is admittedly sleazy; ANSI CL requires TYPE-ERRORs to be
+;;; signalled in cases where a type-specifier is not of an appropriate
+;;; subtype.  The sleazy part is whether it's right to overload TYPE-ERROR
+;;; like this.
+
+(define-condition invalid-subtype-error (type-error)
+  ()
+  (:report (lambda (c s)
+             (format s "The type specifier ~S is not determinably a subtype of the type ~S"
+                     (type-error-datum c)
+                     (type-error-expected-type c)))))
 
 (define-condition simple-type-error (simple-condition type-error))
 
 
+
 (define-condition program-error (error))
 (define-condition simple-program-error (simple-condition program-error)
   ((context :initarg :context :reader simple-program-error-context :initform nil)))
@@ -173,6 +190,12 @@
 
 (define-condition control-error (error))
 
+(define-condition cant-throw-error (control-error)
+  ((tag :initarg :tag))
+  (:report (lambda (c s)
+             (format s "Can't throw to tag ~s" (slot-value c 'tag)))))
+
+
 (define-condition package-error (error)
   ((package :initarg :package :reader package-error-package)))
 (define-condition no-such-package (package-error)
@@ -221,11 +244,34 @@
                    conflicting-sym
                    using-package
                    package)))))))
-  
+
+(define-condition export-requires-import (package-error)
+  ((to-be-imported :initarg :to-be-imported))
+  (:report
+   (lambda (c s)
+     (let* ((p (package-error-package c)))
+       (format s "The following symbols need to be imported to ~S before they can be exported ~& from that package:~%~s:" p (slot-value c 'to-be-imported) p)))))
+
+
 (define-condition package-name-conflict-error (package-error simple-error) ())
 
+(define-condition package-is-used-by (package-error)
+  ((using-packages :initarg :using-packages))
+  (:report (lambda (c s)
+             (format s "~S is used by ~S" (package-error-package c)
+                     (slot-value c 'using-packages)))))
+
+(define-condition symbol-name-not-accessible (package-error)
+  ((symbol-name :initarg :symbol-name))
+  (:report (lambda (c s)
+             (format s "No aymbol named ~S is accessible in package ~s"
+                     (slot-value c 'symbol-name)
+                     (package-error-package c)))))
+
 (define-condition stream-error (error)
   ((stream :initarg :stream :reader stream-error-stream)))
+(define-condition parse-error (stream-error) ())
+(define-condition reader-error (parse-error) ())
 (define-condition end-of-file (stream-error) ()
   (:report (lambda (c s)
              (format s "Unexpected end of file on ~s" (stream-error-stream c)))))
@@ -241,13 +287,19 @@
              (declare (ignore c))
              (format s "Cannot modify a read-only buffer"))))
 
-(define-condition file-error (simple-condition error)
+(define-condition file-error (error)
   ((pathname :initarg :pathname :reader file-error-pathname)
    (error-type :initarg :error-type :initform "File error on file ~S"))
   (:report (lambda (c s)
-              (apply #'format s (slot-value c 'error-type) 
-                     (file-error-pathname c)
-                     (simple-condition-format-arguments c)))))
+              (format s (slot-value c 'error-type) 
+                     (file-error-pathname c)))))
+
+(define-condition simple-file-error (simple-condition file-error)
+  ()
+  (:report (lambda (c s)
+	     (apply #'format s (slot-value c 'error-type) 
+		    (file-error-pathname c)
+		    (simple-condition-format-arguments c)))))
 
 
 (define-condition namestring-parse-error (error)
@@ -280,6 +332,18 @@
                                  (undefined-function-call-name c)
                                  (undefined-function-call-arguments c)))))
 
+(define-condition call-special-operator-or-macro (undefined-function-call)
+  ()
+  (:report (lambda (c s) (format s "Special operator or global macro-function ~s can't be FUNCALLed or APPLYed" (undefined-function-call-name c)))))
+
+  
+(define-condition unbound-slot (cell-error)
+  ((instance :initarg :instance :accessor unbound-slot-instance))
+  (:report (lambda (c s) (format s "Slot ~s is unbound in ~s"
+                                 (cell-error-name c)
+                                 (unbound-slot-instance c)))))
+  
+
 (define-condition arithmetic-error (error) 
   ((operation :initform nil :initarg :operation :reader arithmetic-error-operation)
    (operands :initform nil :initarg :operands :reader arithmetic-error-operands))
@@ -291,11 +355,9 @@
 (define-condition division-by-zero (arithmetic-error))
   
 (define-condition floating-point-underflow (arithmetic-error))
-                  
-                  
 (define-condition floating-point-overflow (arithmetic-error))
-(define-condition inexact-result (arithmetic-error))
-(define-condition invalid-operation (arithmetic-error))
+(define-condition floating-point-inexact (arithmetic-error))
+(define-condition floating-point-invalid-operation (arithmetic-error))
 
 (defun restartp (thing) 
   (istruct-typep thing 'restart))
@@ -762,11 +824,14 @@
         (cons $xdivzro 'division-by-zero)
         (cons $xflovfl 'floating-point-overflow)
         (cons $xfunbnd 'undefined-function)
-	(cons $xbadkeys 'simple-program-error)))
+	(cons $xbadkeys 'simple-program-error)
+        (cons $xnotfun 'call-special-operator-or-macro)
+        (cons $xaccessnth 'sequence-index-type-error)
+        ))
 
 ; do we like this?
 (defparameter *simple-error-types*
-  (vector nil 'simple-program-error 'file-error))
+  (vector nil 'simple-program-error 'simple-file-error))
 
 (defconstant $pgm-err #x10000)
 
