@@ -2,22 +2,18 @@
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
 ;;;   Portions copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;; L1-files.lisp - Object oriented file stuff
 
@@ -88,6 +84,24 @@
       (signal-file-error $xillwild name))
     (namestring-unquote name)))
 
+;; Reverse of above, take native namestring and make a Lisp pathname.
+(defun native-to-pathname (name)
+  (pathname (%path-std-quotes name nil "*")))
+
+(defun native-to-directory-pathname (name)
+  (make-directory-pathname  :device nil :directory (%path-std-quotes name nil "*")))
+
+;;; Make a pathname which names the specified directory; use
+;;; explict :NAME, :TYPE, and :VERSION components of NIL.
+(defun make-directory-pathname (&key host device directory)
+  (make-pathname :host host
+		 :device device
+		 :directory directory
+                 :name nil
+                 :type nil
+                 :version nil))
+		   
+
 (defun namestring-unquote (name)
   (let ((esc *pathname-escape-character*))
     (if (position esc name)
@@ -122,6 +136,7 @@
     ;; %UNIX-FILE-KIND would do so.
     (when kind
       (when (and realpath
+		 (> (length native) 0)
 		 (eq (aref native (1- (length native))) #\/))
 	;; If started with a directory, return a directory (%realpath
 	;; always strips off the trailing /).
@@ -131,7 +146,7 @@
 	    (setq realpath (%str-cat realpath "/")))
 	  (setq realpath nil)))
       (when realpath
-	(pathname realpath)))))
+	(native-to-pathname realpath)))))
 
 (defun cwd (path)  
   (multiple-value-bind (realpath kind) (%probe-file-x (native-translated-namestring path))
@@ -144,13 +159,16 @@
 	(error "~S is not a directory pathname." path))
       (error "Invalid pathname : ~s." path))))
 
-(defun create-file (path &key
+(defun create-file (path &key (if-exists :error) (create-directory t))
+  (native-to-pathname (%create-file path :if-exists if-exists
+				      :create-directory create-directory)))
+(defun %create-file (path &key
 			 (if-exists :error)
 			 (create-directory t))
   (when create-directory
     (create-directory path))
   (when (directory-pathname-p path)
-    (return-from create-file (probe-file-x path)))
+    (return-from %create-file (probe-file-x path)))
   (assert (or (eql if-exists :overwrite) (not (probe-file path))) ()
 	  "~s ~s not implemented yet" :if-exists if-exists)
   (let* ((unix-name (native-translated-namestring path))
@@ -800,13 +818,13 @@
 
 
 (defun mac-default-directory ()
-  (let* ((dname (current-directory-name))
-	 (len (length dname)))
+  (let* ((native-name (current-directory-name))
+	 (len (length native-name)))
     (declare (fixnum len))
     (when (and (> len 1)
-	       (not (eq #\/ (schar dname (1- len)))))
-      (setq dname (%str-cat dname "/")))
-    (pathname dname)))
+	       (not (eq #\/ (schar native-name (1- len)))))
+      (setq native-name (%str-cat native-name "/")))
+    (native-to-pathname native-name)))
 
 
 ; I thought I wanted to call this from elsewhere but perhaps not
@@ -1048,11 +1066,12 @@
                            &aux path default-dir)
   (declare (ignore device))
   (when case (setq case (require-type case pathname-case-type)))
-  (when (null host-p)
+  (if (null host-p)
     (let ((defaulted-defaults (if defaults-p defaults *default-pathname-defaults*)))
       (setq host (if defaulted-defaults
 		   (pathname-host defaulted-defaults)
-		   :unspecific))))
+		   :unspecific)))
+    (unless host (setq host :unspecific)))
   (if directory-p 
     (setq directory (%std-directory-component directory host)))
   (if defaults
@@ -1419,6 +1438,7 @@
          (if (listp name)
            (mapcar #'(lambda (name) (%reverse-component-case name case))  name)
            name))
+        #+advanced-studlification-feature
         ((eq case :studly) (string-studlify name))
 	((eq case :logical)
 	 (if (every #'(lambda (ch) (not (lower-case-p ch))) name)
@@ -1537,27 +1557,6 @@
       (when (eq (%schar string i) char)
         (return i)))))
 
-; need to keep these until world is recompiled with different fcomp-standard-source
-
-(defvar *logical-directory-alist* nil)
-#|
-  `(("CCL" . ":")                       ; reset in l1-boot
-    ("HOME" . ":")                      ; ditto.
-))
-
-    ("L1" . "ccl;level-1:")
-    ("L1F" . "ccl;L1-fasls:")
-    ("LIB" . "ccl;lib:")
-    ("BIN" . "ccl;bin:"))
-    ("CODE" . "ccl;lib:")
-    ("FASLS"   . "ccl;bin:")
-    ("COMPILER" . "ccl;compiler:")
-    ("EXAMPLES" . "ccl;examples:")
-    ("LIBRARY" . "ccl;library:")
-    ("INTERFACES" . "library;interfaces:")
-    ))
-|# ; why do we crash if this is nil
-
 
 (defun file-write-date (path)
   (%file-write-date (native-translated-namestring path)))
@@ -1638,11 +1637,12 @@
 
 (defun load (file-name &key (verbose *load-verbose*)
                        (print *load-print*)
-                       (if-does-not-exist :error))
+                       (if-does-not-exist :error)
+		       (external-format :default))
   "Extension: :PRINT :SOURCE means print source as well as value"
   (loop
     (restart-case
-      (return (%load file-name verbose print if-does-not-exist))
+      (return (%load file-name verbose print if-does-not-exist external-format))
       (retry-load ()
                   :report (lambda (stream) (format stream "Retry loading ~s" file-name)))
       (skip-load ()
@@ -1657,7 +1657,7 @@
                          :if-does-not-exist if-does-not-exist))))))
 
 
-(defun %load (file-name verbose print if-does-not-exist)
+(defun %load (file-name verbose print if-does-not-exist external-format)
   (let ((*load-pathname* file-name)
         (*load-truename* file-name)
         (source-file file-name)
@@ -1710,11 +1710,13 @@
                                             source-file *load-pathname*))
                           (return-from 
                             %load
-                            (%load source-file verbose print if-does-not-exist ))))
+                            (%load source-file verbose print if-does-not-exist  external-format))))
                        ;;duplicated code
                        (attempt-load file-name)))))
                 (t 
-                 (with-open-file (stream file-name :element-type 'base-char)
+                 (with-open-file (stream file-name
+					 :element-type 'base-char
+					 :external-format external-format)
                    (load-from-stream stream print))))))))
   file-name)
 
