@@ -1,22 +1,18 @@
 ; -*- Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;; L1-io.lisp
 
@@ -1346,8 +1342,10 @@ printed using \"#:\" syntax.  NIL means no prefix is printed.")
   (let* ((class (ccl::struct-def object)) ;;guaranteed non-NIL if this function is called
          (pf (structure-print-function class)))
     (cond (pf
-           (funcall pf 
-                    object stream (backtranslate-level level)))
+	   (if (consp pf)
+	     (funcall (%car pf) object stream)
+	     (funcall pf 
+		      object stream (backtranslate-level level))))
           ((and (not *print-structure*) (not *print-readably*))
            (print-unreadable-object (object stream :identity t)
             (write-a-symbol (ccl::sd-name class) stream)))
@@ -1450,7 +1448,7 @@ printed using \"#:\" syntax.  NIL means no prefix is printed.")
 (defun function-is-current-definition? (function)
   (let ((name (function-name function)))
     (and name
-         (function-spec-p name)
+         (symbolp name)
          (eq function (fboundp name)))))
 
 ;; outputs to stream or returns a string.  Barf!
@@ -1682,6 +1680,7 @@ printed using \"#:\" syntax.  NIL means no prefix is printed.")
   (print object stream)
   nil)                                  ; pprint returns nil
 
+
 (defun read-sequence (seq stream &key (start 0) end)
   (setq end (check-sequence-bounds seq start end))
   (locally (declare (fixnum start end))
@@ -1695,11 +1694,14 @@ printed using \"#:\" syntax.  NIL means no prefix is printed.")
 			     (the fixnum (- end start)))))
        (multiple-value-bind (vector offset) (array-data-and-offset seq)
 	 (declare (fixnum offset))
-	 (stream-read-vector
-	  stream
-	  vector
-	  (the fixnum (+ offset start))
-	  (the fixnum (+ offset end))))))))
+	 (-
+	  (stream-read-vector
+	   stream
+	   vector
+	   (the fixnum (+ offset start))
+	   (the fixnum (+ offset end)))
+	  offset))))))
+
 
 (defun write-sequence (seq stream &key (start 0) end)
   (setq end (check-sequence-bounds seq start end))
@@ -1714,6 +1716,74 @@ printed using \"#:\" syntax.  NIL means no prefix is printed.")
 	(the fixnum (+ offset start))
 	(the fixnum (+ offset end))))))
   seq)
-	
-  
 
+(defpackage "GRAY"
+  (:use)
+  (:import-from "CCL"
+                "FUNDAMENTAL-STREAM"
+                "FUNDAMENTAL-INPUT-STREAM"
+                "FUNDAMENTAL-OUTPUT-STREAM"
+                "FUNDAMENTAL-CHARACTER-STREAM"
+                "FUNDAMENTAL-CHARACTER-INPUT-STREAM"
+                "FUNDAMENTAL-CHARACTER-OUTPUT-STREAM"
+                "FUNDAMENTAL-BINARY-STREAM"
+                "FUNDAMENTAL-BINARY-INPUT-STREAM"
+                "FUNDAMENTAL-BINARY-OUTPUT-STREAM"
+
+                "STREAM-READ-CHAR"
+                "STREAM-UNREAD-CHAR"
+                "STREAM-READ-CHAR-NO-HANG"
+                "STREAM-PEEK-CHAR"
+                "STREAM-LISTEN"
+                "STREAM-READ-LINE"
+                "STREAM-CLEAR-INPUT"
+
+                "STREAM-WRITE-CHAR"
+                "STREAM-LINE-COLUMN"
+                "STREAM-START-LINE-P"
+                "STREAM-WRITE-STRING"
+                "STREAM-TERPRI"
+                "STREAM-FRESH-LINE"
+                "STREAM-FORCE-OUTPUT"
+                "STREAM-FINISH-OUTPUT"
+                "STREAM-CLEAR-OUTPUT"
+                "STREAM-ADVANCE-TO-COLUMN"
+
+                "STREAM-READ-BYTE"
+                "STREAM-WRITE-BYTE"
+		)
+  (:export
+   "FUNDAMENTAL-STREAM"
+   "FUNDAMENTAL-INPUT-STREAM"
+   "FUNDAMENTAL-OUTPUT-STREAM"
+   "FUNDAMENTAL-CHARACTER-STREAM"
+   "FUNDAMENTAL-CHARACTER-INPUT-STREAM"
+   "FUNDAMENTAL-CHARACTER-OUTPUT-STREAM"
+   "FUNDAMENTAL-BINARY-STREAM"
+   "FUNDAMENTAL-BINARY-INPUT-STREAM"
+   "FUNDAMENTAL-BINARY-OUTPUT-STREAM"
+
+   "STREAM-READ-CHAR"
+   "STREAM-UNREAD-CHAR"
+   "STREAM-READ-CHAR-NO-HANG"
+   "STREAM-PEEK-CHAR"
+   "STREAM-LISTEN"
+   "STREAM-READ-LINE"
+   "STREAM-CLEAR-INPUT"
+
+   "STREAM-WRITE-CHAR"
+   "STREAM-LINE-COLUMN"
+   "STREAM-START-LINE-P"
+   "STREAM-WRITE-STRING"
+   "STREAM-TERPRI"
+   "STREAM-FRESH-LINE"
+   "STREAM-FORCE-OUTPUT"
+   "STREAM-FINISH-OUTPUT"
+   "STREAM-CLEAR-OUTPUT"
+   "STREAM-ADVANCE-TO-COLUMN"
+
+   "STREAM-READ-BYTE"
+   "STREAM-WRITE-BYTE"
+   ))
+                
+                
