@@ -2,22 +2,18 @@
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
 ;;;   Portions copyright (c) 2001 Clozure Associates.
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 
@@ -95,402 +91,6 @@
         (rplacd trans (cons new (cdr trans)))
         (cdr trans)))))
 
-#|
-(defun maybe-diddle-case (thing diddle-p)
-  (if (and diddle-p (not (or (symbolp thing) (integerp thing))))
-      (labels ((check-for (pred in)
-		 (typecase in
-		   (pattern
-		    (dolist (piece (pattern-pieces in))
-		      (when (typecase piece
-			      (simple-string
-			       (check-for pred piece))
-			      (cons
-			       (case (car in)
-				 (:character-set
-				  (check-for pred (cdr in))))))
-			(return t))))
-		   (list
-		    (dolist (x in)
-		      (when (check-for pred x)
-			(return t))))
-		   (simple-base-string
-		    (dotimes (i (length in))
-		      (when (funcall pred (schar in i))
-			(return t))))
-		   (t nil)))
-	       (diddle-with (fun thing)
-		 (typecase thing
-		   (pattern
-		    (make-pattern
-		     (mapcar #'(lambda (piece)
-				 (typecase piece
-				   (simple-base-string
-				    (funcall fun piece))
-				   (cons
-				    (case (car piece)
-				      (:character-set
-				       (cons :character-set
-					     (funcall fun (cdr piece))))
-				      (t
-				       piece)))
-				   (t
-				    piece)))
-			     (pattern-pieces thing))))
-		   (list
-		    (mapcar fun thing))
-		   (simple-base-string 
-		    (funcall fun thing))
-		   (t
-		    thing))))
-	(let ((any-uppers (check-for #'upper-case-p thing))
-	      (any-lowers (check-for #'lower-case-p thing)))
-	  (cond ((and any-uppers any-lowers)
-		 ;; Mixed case, stays the same.
-		 thing)
-		(any-uppers
-		 ;; All uppercase, becomes all lower case.
-		 (diddle-with #'(lambda (x) (if (stringp x)
-						(string-downcase x)
-						x)) thing))
-		(any-lowers
-		 ;; All lowercase, becomes all upper case.
-		 (diddle-with #'(lambda (x) (if (stringp x)
-						(string-upcase x)
-						x)) thing))
-		(t
-		 ;; No letters?  I guess just leave it.
-		 thing))))
-      thing))
-
-(defun substitute-into (pattern subs diddle-case)
-  (declare (type pattern pattern)
-	   (type list subs)
-	   (values (or simple-base-string pattern)))
-  (let ((in-wildcard nil)
-	(pieces nil)
-	(strings nil))
-    (dolist (piece (pattern-pieces pattern))
-      (cond ((simple-string-p piece)
-	     (push piece strings)
-	     (setf in-wildcard nil))
-	    (in-wildcard)
-	    (t
-	     (setf in-wildcard t)
-	     (unless subs
-	       (error "Not enough wildcards in FROM pattern to match ~
-		       TO pattern:~%  ~S"
-		      pattern))
-	     (let ((sub (pop subs)))
-	       (typecase sub
-		 (pattern
-		  (when strings
-		    (push (apply #'concatenate 'simple-string
-				 (nreverse strings))
-			  pieces))
-		  (dolist (piece (pattern-pieces sub))
-		    (push piece pieces)))
-		 (simple-string
-		  (push sub strings))
-		 (t
-		  (error "Can't substitute this into the middle of a word:~
-			  ~%  ~S"
-			 sub)))))))
-
-    (when strings
-      (push (apply #'concatenate 'simple-string (nreverse strings))
-	    pieces))
-    (values
-     (maybe-diddle-case
-      (if (and pieces (simple-string-p (car pieces)) (null (cdr pieces)))
-	  (car pieces)
-	  (make-pattern (nreverse pieces)))
-      diddle-case)
-     subs)))
-
-(defun didnt-match-error (source from)
-  (error "Pathname components from Source and From args to TRANSLATE-PATHNAME~@
-	  did not match:~%  ~S ~S"
-	 source from))
-
-(defun compute-directory-substitutions (orig-source orig-from)
-  (let ((source orig-source)
-	(from orig-from))
-    (let* ((subs ()))
-      (loop
-	(unless source
-	  (unless (every #'(lambda (x) (eq x :wild-inferiors)) from)
-	    (didnt-match-error orig-source orig-from))
-	  (push () subs)
-	  (return))
-	(unless from (didnt-match-error orig-source orig-from))
-	(let ((from-part (pop from))
-	      (source-part (pop source)))
-	  (typecase from-part
-	    (pattern
-	     (typecase source-part
-	       (pattern
-		(if (pattern= from-part source-part)
-		    (push source-part subs)
-		    (didnt-match-error orig-source orig-from)))
-	       (simple-string
-		(multiple-value-bind
-		    (won new-subs)
-		    (pattern-matches from-part source-part)
-		  (if won
-		      (dolist (sub new-subs)
-			(push sub subs))
-		      (didnt-match-error orig-source orig-from))))
-	       (t
-		(didnt-match-error orig-source orig-from))))
-	    ((member :wild)
-	     (push source-part subs))
-	    ((member :wild-inferiors)
-	     (let ((remaining-source (cons source-part source)))
-	       (let* ((res ()))
-		 (loop
-		   (when (directory-components-match remaining-source from)
-		     (return))
-		   (unless remaining-source
-		     (didnt-match-error orig-source orig-from))
-		   (push (pop remaining-source) res))
-		 (push (nreverse res) subs)
-		 (setq source remaining-source))))
-	    (simple-string
-	     (unless (and (simple-string-p source-part)
-			  (string= from-part source-part))
-	       (didnt-match-error orig-source orig-from)))
-	    (t
-	     (didnt-match-error orig-source orig-from)))))
-      (nreverse subs))))
-
-(defun translate-component (source from to diddle-case) 
-  (typecase to
-    (pattern
-     (typecase from
-       (pattern
-	(typecase source
-	  (pattern
-	   (if (pattern= from source)
-	       source
-	       (didnt-match-error source from)))
-	  (simple-string
-	   (multiple-value-bind
-	       (won subs)
-	       (pattern-matches from source)
-	     (if won
-	       (values (substitute-into to subs diddle-case))
-	       (didnt-match-error source from))))
-	  (t
-	   (maybe-diddle-case source diddle-case))))
-       ((member :wild)
-	(values (substitute-into to (list source) diddle-case)))
-       (t
-	(if (components-match source from)
-	  (maybe-diddle-case source diddle-case)
-	  (didnt-match-error source from)))))
-    ((member nil :wild)
-     (maybe-diddle-case source diddle-case))
-    (t
-     (if (components-match source from)
-       to
-       (didnt-match-error source from)))))
-
-(defun translate-directories (source from to diddle-case)
-  (if (not (and ;;source
-	        to
-		from))
-    (or to
-	(mapcar #'(lambda (x) (maybe-diddle-case x diddle-case)) source))
-    (let* ((res ()))
-      (push (first (or source from)) res)
-      (let ((subs-left (compute-directory-substitutions (rest source)
-							(rest from))))
-	(dolist (to-part (rest to))
-	  (typecase to-part
-	    ((member :wild)
-	     (assert subs-left)
-	     (let ((match (pop subs-left)))
-	       (when (listp match)
-		 (error ":WILD-INFERIORS not paired in from and to ~
-			   patterns:~%  ~S ~S" from to))
-	       (maybe-diddle-case match diddle-case)))
-	    ((member :wild-inferiors)
-	     (let ((match (pop subs-left)))
-	       (unless (listp match)
-		 (error ":WILD-INFERIORS not paired in from and to ~
-			   patterns:~%  ~S ~S" from to))
-	       (dolist (x match)
-		 (push (maybe-diddle-case x diddle-case) res))))
-	    (pattern
-	     (multiple-value-bind
-		 (new new-subs-left)
-		 (substitute-into to-part subs-left diddle-case)
-	       (setf subs-left new-subs-left)
-	       new))
-	    (t (push to-part res)))))
-      (nreverse res))))
-
-(defun translate-pathname (source from-wildname to-wildname &key)
-  "Use the source pathname to translate the from-wildname's wild and
-   unspecified elements into a completed to-pathname based on the to-wildname."
-  (declare (type path-designator source from-wildname to-wildname))
-  (let* ((source (pathname source)))
-    (let* ((from (pathname from-wildname)))
-      (let* ((to (pathname to-wildname)))
-	  (let* ((source-host (pathname-host source))
-		 (to-host (pathname-host to))
-		 (diddle-case nil))
-	    (macrolet ((frob (field &optional (op 'translate-component))
-			 `(let ((result (,op (,field source)
-					     (,field from)
-					     (,field to)
-					     diddle-case)))
-			    (if (eq result :error)
-				(error "~S doesn't match ~S" source from)
-				result))))
-	      (cons-pathname
-	       (frob pathname-directory translate-directories)
-	       (frob %pathname-name)
-	       (frob %pathname-type)
-	       (or to-host source-host)
-	       (frob pathname-version))))))))
-
-(defun pattern= (pattern1 pattern2)
-  (declare (type pattern pattern1 pattern2))
-  (let ((pieces1 (pattern-pieces pattern1))
-	(pieces2 (pattern-pieces pattern2)))
-    (and (= (length pieces1) (length pieces2))
-	 (every #'(lambda (piece1 piece2)
-		    (typecase piece1
-		      (simple-string
-		       (and (simple-string-p piece2)
-			    (string= piece1 piece2)))
-		      (cons
-		       (and (consp piece2)
-			    (eq (car piece1) (car piece2))
-			    (string= (cdr piece1) (cdr piece2))))
-		      (t
-		       (eq piece1 piece2))))
-		pieces1
-		pieces2))))
-
-(defun pattern-matches (pattern string)
-  (declare (type pattern pattern)
-	   (type simple-string string))
-  (let ((len (length string)))
-    (labels ((maybe-prepend (subs cur-sub chars)
-	       (if cur-sub
-		   (let* ((len (length chars))
-			  (new (make-string len :element-type 'base-char))
-			  (index len))
-		     (dolist (char chars)
-		       (setf (schar new (decf index)) char))
-		     (cons new subs))
-		   subs))
-	     (matches (pieces start subs cur-sub chars)
-	       (if (null pieces)
-		   (if (= start len)
-		       (values t (maybe-prepend subs cur-sub chars))
-		       (values nil nil))
-		   (let ((piece (car pieces)))
-		     (etypecase piece
-		       (simple-string
-			(let ((end (+ start (length piece))))
-			  (and (<= end len)
-			       (string= piece string
-					:start2 start :end2 end)
-			       (matches (cdr pieces) end
-					(maybe-prepend subs cur-sub chars)
-					nil nil))))
-		       (list
-			(ecase (car piece)
-			  (:character-set
-			   (and (< start len)
-				(let ((char (schar string start)))
-				  (if (find char (cdr piece) :test #'char=)
-				      (matches (cdr pieces) (1+ start) subs t
-					       (cons char chars))))))))
-		       ((member :single-char-wild)
-			(and (< start len)
-			     (matches (cdr pieces) (1+ start) subs t
-				      (cons (schar string start) chars))))
-		       ((member :multi-char-wild)
-			(multiple-value-bind
-			    (won new-subs)
-			    (matches (cdr pieces) start subs t chars)
-			  (if won
-			      (values t new-subs)
-			      (and (< start len)
-				   (matches pieces (1+ start) subs t
-					    (cons (schar string start)
-						  chars)))))))))))
-      (multiple-value-bind
-	  (won subs)
-	  (matches (pattern-pieces pattern) 0 nil nil nil)
-	(values won (reverse subs))))))
-
-(defun components-match (thing wild)
-  (declare (type (or pattern symbol simple-string integer) thing wild))
-  (or (eq thing wild)
-      (eq wild :wild)
-      (typecase thing
-	(simple-base-string
-	 ;; String is matched by itself, a matching pattern or :WILD.
-	 (typecase wild
-	   (pattern 
-	    (values (pattern-matches wild thing)))
-	   (simple-base-string
-	    (string= thing wild))))
-	(pattern
-	 ;; A pattern is only matched by an identical pattern.
-	 (and (pattern-p wild) (pattern= thing wild)))
-	(integer
-	 ;; an integer (version number) is matched by :WILD or the same
-	 ;; integer.  This branch will actually always be NIL as long is the
-	 ;; version is a fixnum.
-	 (eql thing wild)))))
-
-(defun directory-components-match (thing wild)
-  (or (eq thing wild)
-      (eq wild :wild)
-      (and (consp wild)
-	   (or (and (= (length wild) 2)
-		    (eq (car wild) :absolute)
-		    (null thing)
-		    (eq (cadr wild) :wild-inferiors))
-	       (let ((wild1 (first wild)))
-		 (if (eq wild1 :wild-inferiors)
-		     (let ((wild-subdirs (rest wild)))
-		       (or (null wild-subdirs)
-			   (loop
-			       (when (directory-components-match thing wild-subdirs)
-				 (return t))
-			       (pop thing)
-			     (unless thing (return nil)))))
-		     (and (consp thing)
-			  (components-match (first thing) wild1)
-			  (directory-components-match (rest thing)
-						      (rest wild)))))))))
-
-
-(defun pathname-match-p (in-pathname in-wildname)
-  "Pathname matches the wildname template?"
-  (declare (type path-designator in-pathname))
-  (let* ((pathname (pathname in-pathname)))
-    (let* ((wildname (pathname in-wildname)))
-      (macrolet ((frob (field &optional (op 'components-match ))
-		   `(or (null (,field wildname))
-			(,op (,field pathname) (,field wildname)))))
-	(and (or (null (pathname-host wildname))
-		 (equal (pathname-host wildname) (pathname-host pathname)))
-	     (frob pathname-directory directory-components-match)
-	     (frob pathname-name)
-	     (frob pathname-type)
-	     (frob pathname-version))))))
-
-|#
-
 (defun %component-match-p (name wild) 
   (if (or (eq name :unspecific)(and (stringp name) (or  (string= name "*")(string= name "**"))))
     (setq name nil))  
@@ -996,15 +596,15 @@
 (defun ccl-directory ()
   (let* ((dirpath (getenv "CCL_DEFAULT_DIRECTORY")))
     (if dirpath
-      (make-pathname :defaults nil :directory dirpath)      
+      (native-to-directory-pathname dirpath)
       (mac-default-directory))))
 
 
 (defun user-homedir-pathname (&optional host)
   (declare (ignore host))  
-  (let* ((dir (get-user-home-dir (getuid))))
-    (if dir
-	(make-pathname :defaults nil :directory dir))))
+  (let* ((native (get-user-home-dir (getuid))))
+    (if native
+      (native-to-directory-pathname native))))
 
 (progn
 (setf (logical-pathname-translations "home")
@@ -1088,7 +688,7 @@
 
 (defparameter *module-search-path* (list
                                     (cons-pathname '(:absolute "bin") nil nil "ccl")
-                                    *user-homedir-pathname*
+                                    (cons-pathname  nil nil nil "home")
                                     (cons-pathname  nil nil nil "ccl")
                                     (cons-pathname '(:absolute "lib") nil nil "ccl")
 				    (cons-pathname '(:absolute "library") nil nil "ccl")
