@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;; L1-processes.lisp
@@ -62,13 +58,22 @@
 
 
 #-proxy-scheduler
-(defstruct (process-io-request (:include dll-node))
+(defstruct (process-io-request (:include dll-node)
+			       (:constructor make-process-io-request (fd out-p)))
   fd					; file descriptor
-  process				; what process is waiting
   out-p					; t if output, else input
-  error					; set if error status was true
+  ready					; non-nil if i/o possible
   )
 
+#-proxy-scheduler
+(defmacro %cons-process-io-request (fd out-p)
+  `(%gvector arch::subtag-struct
+    '(process-io-request dll-node)
+    nil
+    nil
+    ,fd
+    ,out-p
+    nil))
 
 (defun startup-shutdown-processes ()
   (dolist (p *all-processes*)
@@ -187,9 +192,9 @@
                           (priority #+proxy-scheduler 100 #-proxy-scheduler 0)
                           run-reasons
                           arrest-reasons
-                          (stack-size *default-process-stackseg-size*)
-                          (vstack-size stack-size)
-                          (tstack-size stack-size)
+                          (stack-size *default-control-stack-size*)
+                          (vstack-size *default-value-stack-size*)
+                          (tstack-size *default-temp-stack-size*)
                           background-p)
   (declare (ignore flavor))
   (let* ((sg (unless simple-p (or stack-group 
@@ -241,6 +246,9 @@
 
 (defglobal *event-processor* *initial-process*)
 
+(defglobal *interactive-abort-process* *initial-process*)
+
+
 
 (defun processp (p)
   (istruct-typep p 'process))
@@ -481,11 +489,11 @@
      (progn
        (process-interrupt *initial-process*
 			  #'(lambda (process)
-					; Avoid function entry event processing
+			      ;; Avoid function entry event processing
 			      (declare (optimize (speed 3) (safety 0)))
                               (without-interrupts
                                (let-globally ((*in-scheduler* t))
-					     (%activate-process process t t))))
+				 (%activate-process process t t))))
 			  *current-process*)
        (%activate-process *initial-process* t))
      (maybe-finish-process-kill process kill))))
@@ -524,8 +532,7 @@
                (rplacd splice cell)
                (rplacd h splice)
                (return))))
-         (setq *active-processes* (cdr handle)
-               *active-processes-tail* *active-processes*))))))
+         (setq *active-processes* (cdr handle)))))))
 
 (defun deactivate-process (p)
   (without-interrupts
@@ -546,58 +553,60 @@
 ; This is an attempt to debug the new %activate-process code.
 (defparameter *using-stack-groups* t)   ; set true if you use explicit stack groups
 
+(defloadvar *pending-io-requests* (make-dll-header))
+
 ; P is an active (runnable) process.
 ; Don't call this unless you've (let-globally ((*in-scheduler* t)) ...)
 (defun %activate-process (p &optional dont-set-in-scheduler funcall-stack-group)
-  (without-interrupts
-   (let* ((current *current-process*)
-	  (sg (process.stack-group p)))
-     (unless (eq p current)
-       (if (%stack-group-exhausted-p sg)
-	   (progn
-	     (process-disable p)
-	     (error "Exhausted stack group for ~s" p))
-	   (let* ((now (get-tick-count))
-		  (nexttick (%tick-sum now (process.quantum p))))
-	     (setf (process.whostate p) "Running")
-	     (setf (process.nexttick p) nexttick)
-	     (let ((sg *current-stack-group*))
-	       (unless (eq sg (process.stack-group current))
-		 (cond ((stack-group-process sg)
-			(when *process-debug-msgs*
-			  (warn "Can't set stack group of ~s to ~s" current sg)))
-		       ((and *using-stack-groups*
-			     (neq current *initial-process*)
-			     (neq current *event-processor*))
-			(setf (process.stack-group current) sg)))))
-	     (setq *current-process* p)
-	     (unless (eq p current)
-	       (unless (process-background-p p)
-		 (setq *idle* nil))
-	       (let* ((ticks (%tick-difference now (process.last-run-time current)))
-		      (total (process.total-run-time current))
-		      (lo-total (cdr total))
-		      (new-lo-total (%tick-sum lo-total ticks)))
-		 (declare (cons total) (fixnum ticks lo-total new-lo-total))
-		 (setf (cdr total) new-lo-total)
-		 (when (and (> lo-total 0) (< new-lo-total 0))
-		   (setf (car total) (the fixnum (1+ (the fixnum (car total)))))))
-	       (setf (process.last-run-time p) now
-		     (process.last-run-time current) now)
-	       (unless dont-set-in-scheduler
-		 (setq *in-scheduler* nil))
-	       (unless (eq *scheduler-value*
-			   (if funcall-stack-group
-			       (funcall sg *scheduler-value*)
-			       (stack-group-resume sg *scheduler-value*)))
-					; I'm being resumed by other than myself in another process.
-					; The *current-process* must have become exhausted and resumed its resumer.
-					; Need to make *current-process* match *current-stack-group*.
-		 (when *using-stack-groups*
-		   (let ((process (stack-group-process *current-stack-group*)))
-		     (when process
-		       (setq *current-process* process)))))
-	       nil)))))))
+  (let* ((current *current-process*)
+	 (sg (process.stack-group p)))
+    (unless (eq p current)
+      (if (%stack-group-exhausted-p sg)
+	(progn
+	  (process-disable p)
+	  (error "Exhausted stack group for ~s" p))
+	(let* ((now (get-tick-count))
+	       (nexttick (%tick-sum now (process.quantum p))))
+	  (setf (process.whostate p) "Running")
+	  (setf (process.nexttick p) nexttick)
+	  (let ((sg *current-stack-group*))
+	    (unless (eq sg (process.stack-group current))
+	      (cond ((stack-group-process sg)
+		     (when *process-debug-msgs*
+		       (warn "Can't set stack group of ~s to ~s" current sg)))
+		    ((and *using-stack-groups*
+			  (neq current *initial-process*)
+			  (neq current *event-processor*))
+		     (setf (process.stack-group current) sg)))))
+	  (setq *current-process* p)
+	  (unless (eq p current)
+	    (unless (process-background-p p)
+	      (setq *idle* nil))
+	    (let* ((ticks (%tick-difference now (process.last-run-time current)))
+		   (total (process.total-run-time current))
+		   (lo-total (cdr total))
+		   (new-lo-total (%tick-sum lo-total ticks)))
+	      (declare (cons total) (fixnum ticks lo-total new-lo-total))
+	      (setf (cdr total) new-lo-total)
+	      (when (and (> lo-total 0) (< new-lo-total 0))
+		(setf (car total) (the fixnum (1+ (the fixnum (car total)))))))
+	    (setf (process.last-run-time p) now
+		  (process.last-run-time current) now)
+	    (unless dont-set-in-scheduler
+	      (setq *in-scheduler* nil))
+	    (unless (eq *scheduler-value*
+			(if funcall-stack-group
+			  (funcall sg *scheduler-value*)
+			  (stack-group-resume sg *scheduler-value*)))
+	      ;; I'm being resumed by other than myself in another
+	      ;; process.  The *current-process* must have become
+	      ;; exhausted and resumed its resumer.  Need to make
+	      ;; *current-process* match *current-stack-group*.
+	       (when *using-stack-groups*
+		 (let ((process (stack-group-process *current-stack-group*)))
+		   (when process
+		     (setq *current-process* process)))))
+	     nil))))))
 
 (defvar *in-process-wait-function* nil)
 
@@ -613,21 +622,25 @@
 	  (t (catch '%process-wait-p
 	       (let* ((*in-process-wait-function* process)
 		      (old-debugger-hook *debugger-hook*)
-		      (*debugger-hook* #'(lambda (condition hook)
-					   (declare (ignore hook))
-					   (if (eq process *current-process*)
-					     (let ((*in-process-wait-function* nil))
-					       (if old-debugger-hook
-						 (funcall old-debugger-hook condition old-debugger-hook)
-						 (error condition)))
-					     (process-interrupt process
-								#'(lambda (process wait-function wait-args)
-					; Hope to signal the error in context
-								    (%process-wait-p process wait-function wait-args)
-					; Didn't signal in context, so signal out of context
-								    (error "Error inside process-wait function"))
-								process wf args))
-					   (throw '%process-wait-p nil))))
+		      (*debugger-hook*
+		       #'(lambda (condition hook)
+			   (declare (ignore hook))
+			   (if (eq process *current-process*)
+			     (let ((*in-process-wait-function* nil))
+			       (if old-debugger-hook
+				 (funcall old-debugger-hook condition old-debugger-hook)
+				 (error condition)))
+			     (process-interrupt
+			      process
+			      #'(lambda (process wait-function wait-args)
+				  ;; Hope to signal the error in
+				  ;; context
+				  (%process-wait-p process wait-function wait-args)
+				  ;; Didn't signal in context, so
+				  ;; signal out of context
+				  (error "Error inside process-wait function"))
+			      process wf args))
+			   (throw '%process-wait-p nil))))
 		 (declare (dynamic-extent *debugger-hook*))
 		 (when (apply wf args)
 		   (when *new-processes* 
@@ -653,28 +666,62 @@
 (defloadvar *in-scheduler* nil)
 (defloadvar *select-timeout-ticks* 1)
 
+(defun enqueue-io-request (req)
+  (insert-dll-node-after req *pending-io-requests*))
+
+(defun check-pending-io-requests (&optional (timeout *select-timeout-ticks*))
+  (let* ((header *pending-io-requests*))
+    (unless (eq (dll-node-succ header) header)
+      (let* ((maxfd -1))
+	(declare (fixnum maxfd))
+	(rlet ((tv :timeval))
+	  (ticks-to-timeval (or timeout 0) tv)
+	  (%stack-block ((infds *fd-set-size*)
+			 (outfds *fd-set-size*))
+	    (fd-zero infds)
+	    (fd-zero outfds)
+	    (do-dll-nodes (req header)
+	      (unless (process-io-request-ready req)
+		(let* ((fd (process-io-request-fd req)))
+		  (declare (fixnum fd))
+		  (if (> fd maxfd)
+		    (setq maxfd fd))
+		  (if (process-io-request-out-p req)
+		    (fd-set fd outfds)
+		    (fd-set fd infds)))))
+	    (when (> maxfd -1)
+	      (let* ((res (#_select (1+ maxfd) infds outfds (%null-ptr) tv)))
+		(when (> res 0)
+		  (do-dll-nodes (req header)
+		    (unless (process-io-request-ready req)
+		      (let* ((fd (process-io-request-fd req)))
+			(when
+			  (if (process-io-request-out-p req)
+			    (fd-is-set fd outfds)
+			    (fd-is-set fd infds))
+			  (setf (process-io-request-ready req) t)
+			  (remove-dll-node req))))))))))))))
+      
+
 (defun scheduler ()
   #+proxy-scheduler
   (warn "Scheduler called!")
   #-proxy-scheduler
   (let-globally ((*in-scheduler* t))
-		(if (null *current-process*)
-		    (bug "*current-process* is nil in scheduler"))
-		;; give first preference to the guy who is processing events or who doesnt
-		;; want event processing to happen quite yet.
-		(loop
-		    (let ((active-processes-tail *active-processes-tail*))
-		      (if active-processes-tail
-			  (setq *active-processes-tail* nil)
-			  (setq active-processes-tail *active-processes*))
-		      (if *active-processes*
-			  (progn
-			    (dolist (p active-processes-tail)
-			      (unless (%process-blocked-p p)
-				(when (%process-wait-p p)
-				  (return-from scheduler (%activate-process p))))))
-			  (progn
-			    (clear-process-polling)))))))
+    (if (null *current-process*)
+      (bug "*current-process* is nil in scheduler"))
+    (loop
+	(check-pending-io-requests 0)
+	(let ((active-processes-tail *active-processes-tail*))
+	  (if active-processes-tail
+	    (setq *active-processes-tail* nil)
+	    (setq active-processes-tail *active-processes*))
+	  (dolist (p active-processes-tail)
+	    (unless (%process-blocked-p p)
+	      (when (%process-wait-p p)
+		(return-from scheduler (%activate-process p)))))
+	  (check-pending-io-requests))
+	(clear-process-polling))))
 
 
 
@@ -995,6 +1042,7 @@
          (progn
            (setf (process.wait-function p) nil
                  (process.wait-argument-list p) nil)
+
 	   (stack-group-interrupt
 	    sg nil
 	    #'(lambda (function args p wait wait-args whostate)
@@ -1218,4 +1266,5 @@ and returns nil; else returns true."
     (string (find id *all-processes*
 		  :key #'(lambda (p)
 			   (process.name p))
-		  :test #'equal))))
\ No newline at end of file
+		  :test #'equal))))
+
