@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;;; READ and related functions.
@@ -31,31 +27,17 @@
 
 (defparameter *name-char-alist*
   '(("Null" . #\000) ("Nul" . #\000)
-    ("Home" . #\001)
-    ("Enter" . #\003)
-    ("End" . #\004)
-    ("Help" . #\005)
     ("Bell"  . #\007) ; ^G , used by Franz (and others with bells.)
     ("Delete" . #\010) ("Backspace" . #\010)("BS" . #\010)
     ("Tab" . #\011)
+    ("Newline" .  #\012) ; arguably, the more "standard" name.
     ("Linefeed" . #\012) ("LF" . #\012)
     ("PageUp" . #\013)
     ("Page" . #\014)("PageDown" . #\014)("Formfeed" . #\014) ("FF" . #\014)
-    ("Newline" .  #\012) 
     ("Return" . #\015) ("CR" . #\015)
-    ("CommandMark" . #\021)
-    ("CheckMark" . #\022)
-    ("DiamondMark" . #\023)
-    ("AppleMark" . #\024)
     ("ESC" .  #\033) ("Escape" . #\033) ("Clear" .  #\033)
        ("Altmode" .  #\033) ("ALT" .  #\033)
-    ("BackArrow" . #\034) ("Backward-arrow" . #\034)
-    ("ForwardArrow" . #\035) ("Forward-arrow" . #\035)
-    ("UpArrow" . #\036) ("Up-arrow" . #\036)
-    ("DownArrow" . #\037) ("Down-arrow" . #\037)
     ("Space" . #\040)
-    ("Dot" . #\245)
-    ("altCheckMark" . #\303)
     ("DEL" . #\177)("ForwardDelete" . #\177) ("Rubout" . #\177)
     ))
 
@@ -84,7 +66,7 @@
            (if (and (= namelen 2) (eq (char name 0) #\^))
              (code-char (the fixnum (logxor (the fixnum (char-code (char-upcase (char name 1)))) #x40)))
              (let* ((n 0))
-               (dotimes (i namelen (code-char n))
+               (dotimes (i namelen (code-char (logand n (1- char-code-limit))))
                  (let* ((code (the fixnum (- (the fixnum (char-code (char name i)))
                                              (char-code #\0)))))
                    (declare (fixnum code))
@@ -722,6 +704,10 @@ c)" t)
               (declare (ignore ignore))
               `(quote ,(read stream t nil t)))))
 
+(defparameter *alternate-line-terminator*
+    #+darwinppc-target #\Return
+    #-darwinppc-target nil)
+	     
 (set-macro-character
  #\;
  (nfunction |;-reader|
@@ -730,7 +716,8 @@ c)" t)
               (let* ((ch nil))
                 (loop 
                     (if (or (null (setq ch (tyi stream)))   ; EOF
-                            (eq ch #\linefeed))
+                            (eq ch #\NewLine)
+			    (eq ch *alternate-line-terminator*))
                         (return (values))))))))
 
 (defun read-string (stream termch)
@@ -752,7 +739,9 @@ c)" t)
  #\)
  #'(lambda (stream ch)
      (if *ignore-extra-close-parenthesis*
-       (warn "Ignoring extra \"~c\" on ~s ." ch stream)
+       (let* ((pos (if (typep stream 'file-stream)
+                     (file-position stream))))
+         (warn "Ignoring extra \"~c\" ~@[near position ~d.~] on ~s ." ch pos stream))
        (%err-disp $XUMRPR)))          ; is this a READER-ERROR ? 
 )
 
@@ -866,7 +855,8 @@ c)" t)
                        (%casify-token tb (unless (atom escapes) escapes))
                        (or explicit-package
                            (not nondots)
-                           (%token-to-number tb (%validate-radix *read-base*)))))
+                           (and (not escapes)
+                                (%token-to-number tb (%validate-radix *read-base*))))))
                (%err-disp $XBADSYM)
                (make-symbol (%string-from-token tb)))))
          (progn
