@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ; l1-readloop-lds.lisp
 
@@ -148,6 +144,28 @@
     (dotimes (i (length r) (terpri))
       (format t "~&~d. ~a" i (svref r i)))))
 
+;;; From Marco Baringer 2003/03/18
+(define-toplevel-command :break set (n frame value) "Set <n>th item of frame <frame> to <value>"
+  (let* ((frame-sp (nth-raw-frame frame *break-frame* *current-stack-group*)))
+    (if frame-sp
+        (toplevel-print (list (set-nth-value-in-frame frame-sp n *current-stack-group* value)))
+        (format *debug-io* "No frame with number ~D~%" frame))))
+
+;;;; this is the one i've found the most usefull since from within
+;;;; the inspector you can change the data on the stack
+(define-toplevel-command :break iv (n frame) "Inspect the <n>th item of frame <frame>"
+  (let* ((frame-sp (nth-raw-frame frame *break-frame* *current-stack-group*)))
+    (if frame-sp
+        (inspect (nth-value-in-frame frame-sp n *current-stack-group*))
+        (format *debug-io* "No frame with number ~D~%" frame))))
+
+(define-toplevel-command :break d (n frame) "Describe the <n>th item of frame number <frame>"
+  (let* ((frame-sp (nth-raw-frame frame *break-frame* *current-stack-group*)))
+    (if frame-sp
+        (describe (nth-value-in-frame frame-sp n *current-stack-group*))
+        (format *debug-io* "No frame with number ~D~%" frame))))
+
+
 (define-toplevel-command :global ? () "help"
   (dolist (g *active-toplevel-commands*)
     (dolist (c (cdr g))
@@ -171,6 +189,11 @@
    (print-call-history :start-frame *break-frame*
                        :detailed-p n))
 
+(define-toplevel-command :break v (n frame-number) "Return value <n> in frame <frame-number>"
+  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* *current-stack-group*)))
+    (if frame-sp
+      (toplevel-print (list (nth-value-in-frame frame-sp n *current-stack-group*))))))
+
 (defun %use-toplevel-commands (group-name)
   ;; Push the whole group
   (pushnew (assoc group-name *defined-toplevel-commands*)
@@ -216,7 +239,7 @@
 					  :output-stream output-stream
 					  :prompt-function prompt-function))
                 (if (eq form *eof-value*)
-		  (if (interactive-stream-p *stdin*)
+		  (if (eof-transient-p (stream-device *stdin* :input))
 		    (progn
 		      (stream-clear-input *terminal-io*)
 		      (abort-break))
@@ -254,8 +277,7 @@
                (form))
           (catch '%re-read            
             (if (eq (setq form (read listener nil *eof-value*)) *eof-value*)
-              (progn
-                (return form))
+              (return form)
 	      (progn
 		(let ((ch)) ;Trim whitespace
 		  (while (and (listen listener)
@@ -301,6 +323,27 @@
     (setq *last-break-level* *break-level*))
       (force-output *terminal-io*))
 
+
+;;; Fairly crude default error-handlingbehavior, and a fairly crude mechanism
+;;; for customizing it.
+
+(defvar *app-error-handler-mode* :quit
+  "one of :quit, :quit-quietly, :listener might be useful.")
+
+(defmethod application-error ((a application) condition error-pointer)
+  (case *app-error-handler-mode*
+    (:listener   (break-loop-handle-error condition error-pointer))
+    (:quit-quietly (quit -1))
+    (:quit  (format t "~&Fatal error in ~s : ~a"
+                    (pathname-name (car *command-line-argument-list*))
+                    condition)
+                    (quit -1))))
+
+(defun make-application-error-handler (app mode)
+  (declare (ignore app))
+  (setq *app-error-handler-mode* mode))
+
+
 ; You may want to do this anyway even if your application
 ; does not otherwise wish to be a "lisp-development-system"
 (defmethod application-error ((a lisp-development-system) condition error-pointer)
