@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (in-package "CCL")
 
@@ -129,6 +125,7 @@
 ; - two or more processes busy: cmd-. & opt-cmd-. gives dialog
 ; (a busy process is a non-idling listener, or any other that's not event-processor)
 
+#+notyet
 (defun process-to-abort (what)
   (let ((l (mapcan #'(lambda (x)
                        (unless (or (%stack-group-exhausted-p (process-stack-group x))
@@ -178,7 +175,7 @@
       (when (eq (restart-name restart) name)                 
 	(if res (return-from find-restart-2 restart)(setq res restart))))))
 |#
-(defun quit ()
+(defun quit (&optional (exit-status 0))
   (let ((cancelled-p (list nil)))
     (declare (type cons cancelled-p))
     (process-interrupt *initial-process*
@@ -187,7 +184,9 @@
                              (progn
                                (prepare-to-quit  0)
                                (%set-toplevel #'(lambda () 
-                                                  (%set-toplevel nil)
+                                                  (%set-toplevel
+						   #'(lambda ()
+						       (#_exit exit-status)))
                                                   (ignore-errors (prepare-to-quit 1))
                                                   (throw :toplevel nil)))
                                (throw :toplevel nil))
@@ -261,8 +260,7 @@
 (defclass application ()
     ((command-line-arguments
       :initform
-      (list *standard-help-argument*)
-      :initarg :command-line-arguments)))
+      (list *standard-help-argument*))))
        
 			     
 (defun %usage-exit (banner exit-status other-args)
@@ -380,12 +378,9 @@
 
 
 (defclass lisp-development-system (application) 
-  ())
-
-(defparameter *application*
-  (make-instance 'lisp-development-system
-		 :command-line-arguments
-		 (list *standard-help-argument*
+  ((command-line-arguments
+    :initform
+    (list *standard-help-argument*
 		       (make-command-line-argument
 			:option-char #\n
 			:long-name "no-init"
@@ -405,7 +400,16 @@
 			:help-string "load <file>"
 			:may-take-operand t
 			:allow-multiple t)
-		       )))
+		       (make-command-line-argument
+			:option-char #\T
+			:long-name "set-lisp-heap-gc-threshold"
+			:help-string "set lisp-heap-gc-threshold to <n>"
+			:keyword :gc-threshold
+			:may-take-operand t
+			:allow-multiple nil)))))
+
+(defparameter *application*
+  (make-instance 'lisp-development-system))
 
 (defvar *load-lisp-init-file* t)
 (defvar *lisp-startup-parameters* ())
@@ -421,7 +425,7 @@
     (setq *load-lisp-init-file* (not (assoc :noinit options))
 	  *lisp-startup-parameters*
 	  (mapcan #'(lambda (x)
-		      (and (member (car x) '(:load :eval)) (list x)))
+		      (and (member (car x) '(:load :eval :gc-threshold)) (list x)))
 		  options))))
 	
 
