@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (in-package "CCL")
 
@@ -37,10 +33,10 @@
 	    SOCKET-OS-FD
 	    REMOTE-HOST
 	    REMOTE-PORT
-	    ;;remote-filename
+	    REMOTE-FILENAME
 	    LOCAL-HOST
 	    LOCAL-PORT
-	    ;;local-filename
+	    LOCAL-FILENAME
 	    SOCKET-ADDRESS-FAMILY
 	    SOCKET-CONNECT
 	    SOCKET-FORMAT
@@ -51,6 +47,66 @@
 	    SOCKET-ERROR-SITUATION
 	    WITH-OPEN-SOCKET)))
 
+(defpackage "OPENMCL-SOCKET"
+  (:use "CL")
+  (:import-from "CCL"
+		"MAKE-SOCKET"
+		"ACCEPT-CONNECTION"
+		"DOTTED-TO-IPADDR"
+		"IPADDR-TO-DOTTED"
+		"IPADDR-TO-HOSTNAME"
+		"LOOKUP-HOSTNAME"
+		"LOOKUP-PORT"
+		;;with-pending-connect
+		"RECEIVE-FROM"
+		"SEND-TO"
+		"SHUTDOWN"
+		;;socket-control
+		"SOCKET-OS-FD"
+		"REMOTE-HOST"
+		"REMOTE-PORT"
+		"REMOTE-FILENAME"
+		"LOCAL-HOST"
+		"LOCAL-PORT"
+		"LOCAL-FILENAME"
+		"SOCKET-ADDRESS-FAMILY"
+		"SOCKET-CONNECT"
+		"SOCKET-FORMAT"
+		"SOCKET-TYPE"
+		"SOCKET-ERROR"
+		"SOCKET-ERROR-CODE"
+		"SOCKET-ERROR-IDENTIFIER"
+		"SOCKET-ERROR-SITUATION"
+		"WITH-OPEN-SOCKET")
+  (:export  "MAKE-SOCKET"
+	    "ACCEPT-CONNECTION"
+	    "DOTTED-TO-IPADDR"
+	    "IPADDR-TO-DOTTED"
+	    "IPADDR-TO-HOSTNAME"
+	    "LOOKUP-HOSTNAME"
+	    "LOOKUP-PORT"
+	    ;;with-pending-connect
+	    "RECEIVE-FROM"
+	    "SEND-TO"
+	    "SHUTDOWN"
+	    ;;socket-control
+	    "SOCKET-OS-FD"
+	    "REMOTE-HOST"
+	    "REMOTE-PORT"
+	    "REMOTE-FILENAME"
+	    "LOCAL-HOST"
+	    "LOCAL-PORT"
+	    "LOCAL-FILENAME"
+	    "SOCKET-ADDRESS-FAMILY"
+	    "SOCKET-CONNECT"
+	    "SOCKET-FORMAT"
+	    "SOCKET-TYPE"
+	    "SOCKET-ERROR"
+	    "SOCKET-ERROR-CODE"
+	    "SOCKET-ERROR-IDENTIFIER"
+	    "SOCKET-ERROR-SITUATION"
+	    "WITH-OPEN-SOCKET"))
+
 (eval-when (:compile-toplevel :execute)
   #+linuxppc-target
   (require "LINUX-SYSCALLS")
@@ -88,7 +144,14 @@
     (socket-error where res)
     res))
 
-(defun socket-error (where errno)
+(defun %hstrerror (h_errno)
+  (with-macptrs ((p (#_hstrerror (abs h_errno))))
+    (if p
+      (%get-cstring p)
+      (format nil "Nameserver error ~d" (abs h_errno)))))
+    
+  
+(defun socket-error (where errno &optional nameserver-p)
   (when (< errno 0)
     (setq errno (- errno)))
   (error (make-condition 'socket-error
@@ -98,7 +161,11 @@
 			 ;; TODO: this is a constant arg, there is a way to put this
 			 ;; in the class definition, just need to remember how...
 			 :format-control "~a (error #~d) in ~a"
-			 :format-arguments (list (%strerror errno) errno where))))
+			 :format-arguments (list
+                                            (if nameserver-p
+                                              (%hstrerror errno)
+                                              (%strerror errno))
+                                            errno where))))
 
 ;; If true, this will try to allow other processes to run while
 ;; socket io is happening.
@@ -123,11 +190,22 @@
 
 (defmethod SOCKET-ADDRESS-FAMILY ((socket ip-socket)) :internet)
 
+(defclass file-socket (socket)
+  ())
+
+(defmethod SOCKET-ADDRESS-FAMILY ((socket file-socket)) :file)
+
 (defclass tcp-socket (ip-socket)
   ())
 
 (defmethod SOCKET-TYPE ((socket tcp-socket)) :stream)
 
+(defclass stream-file-socket (file-socket)
+  ())
+
+(defmethod SOCKET-TYPE ((socket stream-file-socket)) :stream)
+
+
 ;; An active TCP socket is an honest-to-goodness stream.
 (defclass tcp-stream (tcp-socket fd-stream
 				 buffered-binary-io-stream-mixin
@@ -152,6 +230,18 @@
   (assert (and in-p out-p) () "Non-bidirectional tcp stream?")
   'tcp-stream)
 
+;; A FILE-SOCKET-STREAM is also honest. To goodness.
+(defclass file-socket-stream (stream-file-socket
+                              fd-stream
+                              buffered-binary-io-stream-mixin
+                              buffered-character-io-stream-mixin)
+  ())
+
+(defmethod select-stream-class ((class file-socket-stream) in-p out-p char-p)
+  (declare (ignore char-p)) ; TODO: is there any real reason to care about this?
+  (assert (and in-p out-p) () "Non-bidirectional tcp stream?")
+  'file-socket-stream)
+
 (defclass unconnected-socket (socket)
   ((device :initarg :device :accessor socket-device)
    (keys :initarg :keys :reader socket-keys)))
@@ -170,6 +260,19 @@
 
 (defmethod SOCKET-CONNECT ((stream listener-socket)) :passive)
 
+(defclass file-listener-socket (stream-file-socket unconnected-socket) ())
+
+(defmethod SOCKET-CONNECT ((stream file-listener-socket)) :passive)
+
+;;; A FILE-LISTENER-SOCKET should try to delete the filesystem
+;;; entity when closing.
+
+(defmethod close :before ((s file-listener-socket) &key abort)
+  (declare (ignore abort))
+  (let* ((path (local-socket-filename (socket-device s))))
+    (when path (%delete-file path))))
+
+
 ;; A udp socket just sends and receives packets.
 (defclass udp-socket (ip-socket unconnected-socket) ())
 
@@ -187,9 +290,28 @@
 	      (namelen :signed))
 	     (setf (pref namelen :signed) (record-length :sockaddr_in))
 	     (socket-call "getsockname" (c_getsockname fd sockaddr namelen))
-	     (ecase type
-	       (:host (#_ntohl (pref sockaddr :sockaddr_in.sin_addr.s_addr)))
-	       (:port (#_ntohs (pref sockaddr :sockaddr_in.sin_port)))))))
+	     (when (= #$AF_INET (pref sockaddr :sockaddr_in.sin_family))
+	       (ecase type
+		 (:host (#_ntohl (pref sockaddr :sockaddr_in.sin_addr.s_addr)))
+		 (:port (#_ntohs (pref sockaddr :sockaddr_in.sin_port))))))))
+
+(defun path-from-unix-address (addr)
+  (when (= #$AF_LOCAL (pref addr :sockaddr_un.sun_family))
+    #+darwinppc-target
+    (%str-from-ptr (pref addr :sockaddr_un.sun_path)
+		   (- (pref addr :sockaddr_un.sun_len) 2))
+    #-darwinppc-target
+    (%get-cstring (pref addr :sockaddr_un.sun_path))))
+
+(defun local-socket-filename (fd)
+  (and fd
+       (rlet ((addr :sockaddr_un)
+              (namelen :signed))
+         (setf (pref namelen :signed) (record-length :sockaddr_un))
+         (socket-call "getsockname" (c_getsockname fd addr namelen))
+	 (path-from-unix-address addr))))
+
+         
 
 (defun remote-socket-info (fd type)
   (and fd
@@ -200,9 +322,20 @@
 	       (cond ((eql err (- #$ENOTCONN)) nil)
 		     ((< err 0) (socket-error "getpeername" err))
 		     (t
-		      (ecase type
-			(:host (#_ntohl (pref sockaddr :sockaddr_in.sin_addr.s_addr)))
-			(:port (#_ntohs (pref sockaddr :sockaddr_in.sin_port))))))))))
+		      (when (= #$AF_INET (pref sockaddr :sockaddr_in.sin_family))
+			(ecase type
+			  (:host (#_ntohl (pref sockaddr :sockaddr_in.sin_addr.s_addr)))
+			  (:port (#_ntohs (pref sockaddr :sockaddr_in.sin_port)))))))))))
+
+(defun remote-socket-filename (fd)
+  (and fd
+       (rlet ((addr :sockaddr_un)
+              (namelen :signed))
+         (setf (pref namelen :signed) (record-length :sockaddr_un))
+         (let* ((err (c_getsockname fd addr namelen)))
+           (cond ((eql err (- #$ENOTCONN)) nil)
+                 ((< err 0) (socket-error "getpeername" err))
+                 (t (path-from-unix-address addr)))))))
 
 (defmethod LOCAL-PORT ((socket socket))
   (local-socket-info (socket-device socket) :port))
@@ -210,34 +343,39 @@
 (defmethod LOCAL-HOST ((socket socket))
   (local-socket-info (socket-device socket) :host))
 
+(defmethod LOCAL-FILENAME ((socket socket))
+  (local-socket-filename (socket-device socket)))
+
 ;; Returns NIL if socket is not connected
 (defmethod REMOTE-HOST ((socket socket))
   (remote-socket-info (socket-device socket) :host))
 
 (defmethod REMOTE-PORT ((socket socket))
   (remote-socket-info (socket-device socket) :port))
-    
 
+(defmethod REMOTE-FILENAME ((socket socket))
+  (remote-socket-filename (socket-device socket)))
+  
 (defun set-socket-options (fd &key keepalive
 			           reuse-address
 				   nodelay
 				   broadcast
 				   linger
+				   address-family
 				   local-port
 				   local-host
+				   local-filename
 				   type
+				   connect
 				   out-of-band-inline
 				   &allow-other-keys)
   ;; see man socket(7) tcp(7) ip(7)
+  (if (null address-family)
+    (setq address-family :internet))
   (when keepalive
     (int-setsockopt fd #$SOL_SOCKET #$SO_KEEPALIVE 1))
   (when reuse-address
     (int-setsockopt fd #$SOL_SOCKET #$SO_REUSEADDR 1))
-  (when nodelay
-    (int-setsockopt fd
-		    #+linuxppc-target #$SOL_TCP
-		    #+darwinppc-target #$IPPROTO_TCP
-		    #$TCP_NODELAY 1))
   (when broadcast
     (int-setsockopt fd #$SOL_SOCKET #$SO_BROADCAST 1))
   (when out-of-band-inline
@@ -248,34 +386,33 @@
 	    (%get-long plinger 4) linger)
       (socket-call "setsockopt"
 		   (c_setsockopt fd #$SOL_SOCKET #$SO_LINGER plinger 8))))
-  (when (or local-port local-host)
-    (let* ((proto (if (eq type :stream) "tcp" "udp"))
-	   (port-n (if local-port (port-as-inet-port local-port proto) 0))
-	   (host-n (if local-host (host-as-inet-host local-host) #$INADDR_ANY)))
-      (rlet ((sockaddr :sockaddr_in))
-	(setf (pref sockaddr :sockaddr_in.sin_family) #$AF_INET
-	      (pref sockaddr :sockaddr_in.sin_port) port-n
-	      (pref sockaddr :sockaddr_in.sin_addr.s_addr) host-n)
-	(socket-call "bind" (c_bind fd sockaddr (record-length :sockaddr_in))))))
+  (when (eq address-family :internet)
+    (when nodelay
+      (int-setsockopt fd
+		      #+linuxppc-target #$SOL_TCP
+		      #+darwinppc-target #$IPPROTO_TCP
+		      #$TCP_NODELAY 1))
+    (when (or local-port local-host)
+      (let* ((proto (if (eq type :stream) "tcp" "udp"))
+	     (port-n (if local-port (port-as-inet-port local-port proto) 0))
+	     (host-n (if local-host (host-as-inet-host local-host) #$INADDR_ANY)))
+	;; Darwin includes the SIN_ZERO field of the sockaddr_in when
+	;; comparing the requested address to the addresses of configured
+	;; interfaces (as if the zeros were somehow part of either address.)
+	;; "rletz" zeros out the stack-allocated structure, so those zeros
+	;; will be 0.
+	(rletz ((sockaddr :sockaddr_in))
+	       (setf (pref sockaddr :sockaddr_in.sin_family) #$AF_INET
+		     (pref sockaddr :sockaddr_in.sin_port) port-n
+		     (pref sockaddr :sockaddr_in.sin_addr.s_addr) host-n)
+	       (socket-call "bind" (c_bind fd sockaddr (record-length :sockaddr_in)))))))
+  (when (and (eq address-family :file)
+	     (eq connect :passive)
+	     local-filename)
+    (bind-unix-socket fd local-filename))    
   (when *multiprocessing-socket-io*
     (socket-call "fcntl" (fd-set-flag fd #$O_NONBLOCK))))
 
-(defun MAKE-SOCKET (&rest keys
-		    &key address-family
-		    ;; List all keys here just for error checking...
-		    ;; &allow-other-keys
-		    type connect remote-host remote-port eol format
-		    keepalive reuse-address nodelay broadcast linger
-		    local-port local-host backlog class out-of-band-inline)
-  (declare (dynamic-extent keys))
-  (declare (ignore type connect remote-host remote-port eol format
-		   keepalive reuse-address nodelay broadcast linger
-		   local-port local-host backlog class out-of-band-inline))
-  (ecase address-family
-    ;; ((:file) (apply #'make-file-socket keys))
-    ((nil :internet) (apply #'make-ip-socket keys))))
-
-
 ;; I hope the inline declaration makes the &rest/apply's go away...
 (declaim (inline make-ip-socket))
 (defun make-ip-socket (&rest keys &key type &allow-other-keys)
@@ -284,6 +421,32 @@
     ((nil :stream) (apply #'make-tcp-socket keys))
     ((:datagram) (apply #'make-udp-socket keys))))
 
+(declaim (inline make-file-socket))
+(defun make-file-socket (&rest keys &key type &allow-other-keys)
+  (declare (dynamic-extent keys))
+  (ecase type
+    ((nil :stream) (apply #'make-stream-file-socket keys))
+    (:datagram (apply #'make-datagram-file-socket keys))))
+
+(defun MAKE-SOCKET (&rest keys
+		    &key address-family
+		    ;; List all keys here just for error checking...
+		    ;; &allow-other-keys
+		    type connect remote-host remote-port eol format
+		    keepalive reuse-address nodelay broadcast linger
+		    local-port local-host backlog class out-of-band-inline
+		    local-filename remote-filename)
+  (declare (dynamic-extent keys))
+  (declare (ignore type connect remote-host remote-port eol format
+		   keepalive reuse-address nodelay broadcast linger
+		   local-port local-host backlog class out-of-band-inline
+		   local-filename remote-filename))
+  (ecase address-family
+    ((:file) (apply #'make-file-socket keys))
+    ((nil :internet) (apply #'make-ip-socket keys))))
+
+
+
 (defun make-udp-socket (&rest keys &aux (fd -1))
   (unwind-protect
     (let (socket)
@@ -313,18 +476,41 @@
     (unless (< fd 0)
       (fd-close fd))))
 
-(defun inet-connect (fd host-n port-n)
-  (let ((err (rlet ((sockaddr :sockaddr_in))
-	       (setf (pref sockaddr :sockaddr_in.sin_family) #$AF_INET
-		     (pref sockaddr :sockaddr_in.sin_port) port-n
-		     (pref sockaddr :sockaddr_in.sin_addr.s_addr) host-n)
-	       (c_connect fd sockaddr (record-length :sockaddr_in)))))
+(defun make-stream-file-socket (&rest keys &key connect &allow-other-keys &aux (fd -1))
+  (unwind-protect
+    (let (socket)
+      (setq fd (socket-call "socket" (c_socket #$PF_LOCAL #$SOCK_STREAM 0)))
+      (apply #'set-socket-options fd keys)
+      (setq socket
+	    (ecase connect
+	      ((nil :active) (apply #'make-file-stream-socket fd keys))
+	      ((:passive) (apply #'make-file-listener-socket fd keys))))
+      (setq fd -1)
+      socket)
+    (unless (< fd 0)
+      (fd-close fd))))
+
+(defun %socket-connect (fd addr addrlen)
+  (let* ((err (c_connect fd addr addrlen)))
     (declare (fixnum err))
     (when (eql err (- #$EINPROGRESS))
       (process-output-wait fd)
       (setq err (- (int-getsockopt fd #$SOL_SOCKET #$SO_ERROR))))
     (unless (eql err 0) (socket-error "connect" err))))
-
+    
+(defun inet-connect (fd host-n port-n)
+  (rlet ((sockaddr :sockaddr_in))
+    (setf (pref sockaddr :sockaddr_in.sin_family) #$AF_INET
+          (pref sockaddr :sockaddr_in.sin_port) port-n
+          (pref sockaddr :sockaddr_in.sin_addr.s_addr) host-n)
+    (%socket-connect fd sockaddr (record-length :sockaddr_in))))
+               
+(defun file-socket-connect (fd remote-filename)
+  (rletz ((sockaddr :sockaddr_un))
+    (init-unix-sockaddr sockaddr remote-filename)
+    (%socket-connect fd sockaddr (record-length :sockaddr_un))))
+         
+  
 (defun make-tcp-stream-socket (fd &key remote-host
 				  remote-port
 				  eol
@@ -336,6 +522,15 @@
 		(port-as-inet-port remote-port "tcp"))
   (make-tcp-stream fd :format format :eol eol :class class))
 
+(defun make-file-stream-socket (fd &key remote-filename
+                                   eol
+                                   format
+                                   (class 'file-socket-stream)
+                                   &allow-other-keys)
+  (file-socket-connect fd remote-filename)
+  (make-file-socket-stream fd :format format :eol eol :class class))
+
+
 (defun make-tcp-stream (fd &key format eol (class 'tcp-stream)  &allow-other-keys)
   (declare (ignore eol))		;???
   (let ((element-type (ecase format
@@ -348,13 +543,31 @@
 		    :direction :io
 		    :element-type element-type)))
 
+(defun make-file-socket-stream (fd &key format eol (class 'file-socket-stream)  &allow-other-keys)
+  (declare (ignore eol))		;???
+  (let ((element-type (ecase format
+			((nil :text) 'character)
+			((:binary :bivalent) '(unsigned-byte 8)))))
+    ;; TODO: check out fd-stream-advance, -listen, -eofp, -force-output, -close
+    ;; See if should specialize any of 'em.
+    (make-fd-stream fd
+		    :class class
+		    :direction :io
+		    :element-type element-type)))
+
 (defun make-tcp-listener-socket (fd &rest keys &key backlog &allow-other-keys)
   (socket-call "listen" (c_listen fd (or backlog 5)))
   (make-instance 'listener-socket
 		 :device fd
 		 :keys keys))
 
-(defun inet-accept (fd wait)
+(defun make-file-listener-socket (fd &rest keys &key backlog &allow-other-keys)
+  (socket-call "listen" (c_listen fd (or backlog 5)))
+  (make-instance 'file-listener-socket
+		 :device fd
+		 :keys keys))
+
+(defun socket-accept (fd wait)
   (flet ((_accept (fd async)
 	   (let ((res (c_accept fd (%null-ptr) (%null-ptr))))
 	     (declare (fixnum res))
@@ -386,20 +599,26 @@
 		    (_accept fd t))
 		(socket-call "fcntl" (fd-set-flags fd old))))))))
 
-(defmethod ACCEPT-CONNECTION ((socket listener-socket) &key (wait t))
+(defun accept-socket-connection (socket wait stream-create-function)
   (let ((listen-fd (socket-device socket))
 	(fd -1))
     (unwind-protect
       (progn
-	(setq fd (inet-accept listen-fd wait))
+	(setq fd (socket-accept listen-fd wait))
 	(cond ((>= fd 0)
-	       (prog1 (apply #'make-tcp-stream fd (socket-keys socket))
+	       (prog1 (apply stream-create-function fd (socket-keys socket))
 		 (setq fd -1)))
 	      ((eql fd (- #$EAGAIN)) nil)
 	      (t (socket-error "accept" fd))))
       (when (>= fd 0)
 	(fd-close fd)))))
 
+(defmethod ACCEPT-CONNECTION ((socket listener-socket) &key (wait t))
+  (accept-socket-connection socket wait #'make-tcp-stream))
+
+(defmethod ACCEPT-CONNECTION ((socket file-listener-socket) &key (wait t))
+  (accept-socket-connection socket wait #'make-file-socket-stream))
+
 (defun verify-socket-buffer (buf offset size)
   (unless offset (setq offset 0))
   (unless (<= (+ offset size) (length buf))
@@ -507,7 +726,7 @@
 		     (_inet_aton host))
 		(multiple-value-bind (addr err) (c_gethostbyname host)
 		  (or addr
-		      (socket-error "gethostbyname" err)))))))
+		      (socket-error "gethostbyname" err t)))))))
 
 
 (defun DOTTED-TO-IPADDR (name &key (errorp t))
@@ -531,7 +750,7 @@
 (defun IPADDR-TO-HOSTNAME (ipaddr &key ignore-cache)
   (declare (ignore ignore-cache))
   (multiple-value-bind (name err) (c_gethostbyaddr (#_htonl ipaddr))
-    (or name (socket-error "gethostbyaddr" err))))
+    (or name (socket-error "gethostbyaddr" err t))))
   
 
 (defun int-getsockopt (socket level optname)
@@ -550,6 +769,8 @@
     (socket-call "setsockopt"
       (c_setsockopt socket level optname valptr (record-length :signed)))))
 
+(defloadvar *h-errno* (foreign-symbol-address #+darwinppc-target "_h_errno"
+                                              #-darwinppc-target "h_errno"))
 #+darwinppc-target
 (defun c_gethostbyaddr (addr)
   (rlet ((addrp :unsigned))
@@ -559,8 +780,8 @@
        (declare (dynamic-extent hp))
        (if (not (%null-ptr-p hp))
 	 (%get-cstring (pref hp :hostent.h_name))
-	 (values nil -1))))))		; screw: find h_errno
-      
+	 (values nil (pref *h-errno* :signed)))))))
+
 #+linuxppc-target
 (defun c_gethostbyaddr (addr)
   (rlet ((hostent :hostent)
@@ -589,7 +810,7 @@
        (if (not (%null-ptr-p hp))
 	 (%get-unsigned-long
 	  (%get-ptr (pref hp :hostent.h_addr_list)))
-	 (values nil -1))))))
+	 (values nil (pref *h-errno* :signed)))))))
 
 #+linuxppc-target
 (defun c_gethostbyname (name)
@@ -618,6 +839,7 @@
       (unless (%null-ptr-p servent-ptr)
 	(pref servent-ptr :servent.s_port)))))
 
+#+linuxppc-target
 (defun _inet_ntoa (addr)
   (rlet ((addrp :unsigned))
     (setf (pref addrp :unsigned) addr)
@@ -625,6 +847,15 @@
       (%setf-macptr p (#_inet_ntoa addrp))
       (unless (%null-ptr-p p) (%get-cstring p)))))
 
+#+darwinppc-target
+(defun _inet_ntoa (addr)
+  (with-macptrs ((p))
+    (%setf-macptr p (external-call "_inet_ntoa"
+				   :unsigned-fullword addr
+				   :address))
+    (unless (%null-ptr-p p) (%get-cstring p))))				   
+
+
 (defun _inet_aton (string)
   (with-cstrs ((name string))
     (rlet ((addr :in_addr))
@@ -642,9 +873,39 @@
           (%get-long params 8) protocol)
     (syscall os::socketcall 1 params)))
 
+(defun init-unix-sockaddr (addr path)
+  (macrolet ((sockaddr_un-path-len ()
+               (/ (ensure-foreign-type-bits
+                    (foreign-record-field-type 
+                     (%find-foreign-record-type-field
+                      (parse-foreign-type '(:struct :sockaddr_un)) :sun_path)))
+                  8)))
+    (let* ((name (native-translated-namestring path))
+           (namelen (length name))
+           (pathlen (sockaddr_un-path-len))
+           (copylen (min (1- pathlen) namelen)))
+        (setf (pref addr :sockaddr_un.sun_family) #$AF_LOCAL)
+        (%copy-ivector-to-ptr name 0
+                              (pref addr :sockaddr_un.sun_path) 0
+                              copylen))))
+
+(defun bind-unix-socket (socketfd path)
+  (rletz ((addr :sockaddr_un))
+    (init-unix-sockaddr addr path)
+    (socket-call
+     "bind"
+     (c_bind socketfd
+             addr
+             (+ 2
+                (#_strlen
+                 (pref addr :sockaddr_un.sun_path)))))))
+      
+
 (defun c_bind (sockfd sockaddr addrlen)
   #+darwinppc-target
-  (syscall os::bind sockfd sockaddr addrlen)
+  (progn
+    (setf (pref sockaddr :sockaddr_in.sin_len) addrlen)
+    (syscall os::bind sockfd sockaddr addrlen))
   #+linuxppc-target
   (%stack-block ((params 12))
     (setf (%get-long params 0) sockfd
@@ -812,3 +1073,103 @@
 	  (%get-ptr params 4) msghdrp
 	  (%get-long params 8) flags)
     (syscall os::socketcall 17 params)))
+
+;;; Return a list of currently configured interfaces, a la ifconfig.
+(defstruct ip-interface
+  name
+  addr
+  netmask
+  flags
+  address-family)
+
+(defun dump-buffer (p n)
+  (dotimes (i n (progn (terpri) (terpri)))
+    (unless (logtest i 15)
+      (format t "~&~8,'0x: " (%ptr-to-int (%inc-ptr p i))))
+    (format t " ~2,'0x" (%get-byte p i))))
+
+(defun %get-ip-interfaces ()
+  (let* ((buffsize 4000))
+    (%stack-block ((buff buffsize))
+      (rlet ((conf :ifconf :ifc_len buffsize))
+	(setf (pref conf :ifconf.ifc_ifcu.ifcu_buf) buff)
+        (let* ((fd (c_socket #$AF_INET #$SOCK_DGRAM 0)))
+	  (if (< fd 0)
+	    (error "Error creating socket."))
+	  (if (< (#_ioctl fd #$SIOCGIFCONF :address conf) 0)
+	    (error "Can't get interfaces configuration"))
+	  (do* ((n (pref conf :ifconf.ifc_len))
+		(offset 0)
+		(req (pref conf :ifconf.ifc_ifcu.ifcu_req))
+		(res ()))
+	       ((>= offset n) (progn (fd-close fd) (nreverse res)))
+	    (declare (fixnum offset n))
+	    (let* ((sa_len #+darwinppc-target (pref (pref req :ifreq.ifr_ifru.ifru_addr) :sockaddr_in.sin_len)
+			   #+linuxppc-target
+			   (external-call "__libc_sa_len"
+					  :unsigned-halfword
+					  (pref (pref req :ifreq.ifr_ifru.ifru_addr) :sockaddr_in.sin_family)
+					  :signed-fullword))
+		   (delta
+		    (max (+ 16 sa_len)
+			 (%foreign-type-or-record-size :ifreq :bytes))))
+	      (declare (fixnum sa_len delta))
+	      ;(dump-buffer req delta)
+	      (let* ((name (%get-cstring (pref req
+					       #+darwinppc-target :ifreq.ifr_name
+					       #+linuxppc-target :ifreq.ifr_ifrn.ifrn_name
+					       ))))
+		(unless (member name res
+				:test #'string=
+				:key #'ip-interface-name)
+		  (when (zerop (#_ioctl fd #$SIOCGIFADDR :address req))
+		    (let* ((addr (pref (pref req :ifreq.ifr_ifru.ifru_addr)
+				       :sockaddr_in.sin_addr.s_addr))
+			   (af (pref (pref req :ifreq.ifr_ifru.ifru_addr)
+				     :sockaddr_in.sin_family)))
+		      (when (zerop (#_ioctl fd #$SIOCGIFFLAGS :address req))
+			(let* ((flags (pref req :ifreq.ifr_ifru.ifru_flags)))
+			  (when (zerop (#_ioctl fd #$SIOCGIFNETMASK :address req))
+			    (push (make-ip-interface
+				   :name name
+				   :addr addr
+				   :netmask (pref (pref req :ifreq.ifr_ifru.ifru_addr)
+						  :sockaddr_in.sin_addr.s_addr)
+				   :flags flags
+				   :address-family af)
+				  res))))))))
+	      (%incf-ptr req delta)
+	      (incf offset delta))))))))
+
+
+(defloadvar *ip-interfaces* ())
+
+(defun ip-interfaces ()
+  (or *ip-interfaces*
+      (setq *ip-interfaces* (%get-ip-interfaces))))
+
+;;; This should presumably happen after a configuration change.
+;;; How do we detect a configuration change ?
+(defun %reset-ip-interfaces ()
+  (setq *ip-interfaces* ()))
+
+;;; Return the first non-loopback interface that's up and whose address
+;;; family is #$AF_INET.  If no such interface exists, return
+;;; the loopback interface.
+(defun primary-ip-interface ()
+  (let* ((ifaces (ip-interfaces)))
+    (or (find-if #'(lambda (i)
+		     (and (eq #$AF_INET (ip-interface-address-family i))
+			  (let* ((flags (ip-interface-flags i)))
+			    (and (not (logtest #$IFF_LOOPBACK flags))
+				 (logtest #$IFF_UP flags)))))
+		 ifaces)
+	(car ifaces))))
+
+(defun primary-ip-interface-address ()
+  (let* ((iface (primary-ip-interface)))
+    (if iface
+      (ip-interface-addr iface)
+      (error "Can't determine primary IP interface"))))
+	  
+	  
