@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;; L1-stack-groups.lisp
 
@@ -291,12 +287,17 @@
        (%free-younger-cs-areas (sg.cs-area sg))
        ))))
 
+(defparameter *default-control-stack-size* (ash 1 20))
+(defparameter *default-value-stack-size* (ash 1 20))
+(defparameter *default-temp-stack-size* (ash 1 18))
 
 ; This makes a stack group only.
 ; Allocating the stacks and creating the thread is done by stack-group-preset
 ; now we have 3 optional args stack-size args
-(defun make-stack-group (name &optional (stack-size 16384)(vstack-size stack-size)
-                              (tstack-size stack-size))       ; default is 16K for each stack
+(defun make-stack-group (name &optional
+			      (stack-size *default-control-stack-size*)
+			      (vstack-size *default-value-stack-size*)
+                              (tstack-size *default-temp-stack-size*))
   (setq name (require-type name 'string))
   (unless (and (fixnump stack-size) (> (the fixnum stack-size) 0))
     (setq stack-size (require-type stack-size '(and fixnum (integer 0 *)))))
@@ -345,8 +346,6 @@
 
 
 
-(defparameter *initial-tsp-stack-segment-size* (* 64 1024))
-(defparameter *initial-vsp-stack-segment-size* (* 64 1024))
 
 (defmacro with-area-macptr ((var area) &body body)
   `(with-macptrs (,var)
@@ -395,8 +394,8 @@
                (fn.args (cheap-cons function (cheap-copy-list args))))
           (unwind-protect
             (progn
-              (setq vs-area (allocate-vstack *initial-vsp-stack-segment-size*))
-              (setq ts-area (allocate-tstack *initial-tsp-stack-segment-size*))
+              (setq vs-area (allocate-vstack (sg.vs-size sg)))
+              (setq ts-area (allocate-tstack (sg.ts-size sg)))
               (setq cs-area (new-cstack-area))
               (setq thread-id (new-stack-group-thread (sg.cs-size sg)))
               (setf (gc-area.threadid cs-area) thread-id)
@@ -501,37 +500,12 @@
          (ash sb arch::fixnum-shift)
          (ash was arch::fixnum-shift)))
 
-; the size of the active portion of a control stack segment
-(defparameter *cs-segment-size* (* 64 1024))
 
 
 
-; the size of the active portion of a value stack segment
-(defparameter *vs-segment-size* (* 64 1024))
-
-
-
-; the size of the active portion of a temp stack segment
-(defparameter *ts-segment-size* (* 64 1024))
-
-
-
-; How much space to leave for overflow on the control stack
-(defparameter *cs-hard-overflow-size* 4096)
-(defparameter *cs-soft-overflow-size* 4096)
-
-
-(declaim (fixnum *cs-segment-size* *cs-hard-overflow-size* 
-                 *cs-soft-overflow-size* *vs-soft-overflow-size* *ts-soft-overflow-size*))
-
-(declaim (fixnum *cs-segment-size* *cs-hard-overflow-size* 
-                 *cs-soft-overflow-size* *vs-soft-overflow-size* *ts-soft-overflow-size*))
-
 ; Create a new Thread Manager thread
 (defun new-stack-group-thread (cs-size)
-  (let ((stack-size (+ (logand (+ cs-size 4095) -4096)          ; round up to 4K multiple
-                       *cs-hard-overflow-size*
-                       *cs-soft-overflow-size*)))
+  (let ((stack-size (logand (+ cs-size 4095) -4096)))          ; round up to 4K multiple
     (rlet ((threadMade :long))
       (ff-call
        (%kernel-import arch::kernel-import-newthread)
@@ -616,7 +590,8 @@
     (when resumer
       (stack-group-resume resumer value))))
 
-
+(defconstant cstack-hardprot (ash 100 10))
+(defconstant cstack-softprot (ash 100 10))
 
 
 (defun initialize-sg-cs-area (sg sp)
@@ -631,8 +606,8 @@
           (logand #xfc (%get-byte stack-space-ptr)))    ; ensure tagged as fixnum
     (let* ((stack-space (%get-object stack-space-ptr 0))
            (limit (- sp stack-space))
-           (hard-limit (+ limit (the fixnum (ash *cs-hard-overflow-size* -2))))
-           (soft-limit (+ hard-limit (the fixnum (ash *cs-soft-overflow-size* -2))))
+           (hard-limit (+ limit (the fixnum (ash cstack-hardprot -2))))
+           (soft-limit (+ hard-limit (the fixnum (ash cstack-softprot -2))))
            (cs-area (sg.cs-area sg))
            (ndwords (ash stack-space -1)))      ; longwords -> doublewords
       (declare (fixnum stack-space limit hard-limit soft-limit))
@@ -1318,6 +1293,16 @@
   (let ((loc (nth-value-in-frame-loc sp n sg nil nil child-frame vsp parent-vsp)))
     (setf (%fixnum-ref loc) new-value)))
 
+(defun nth-raw-frame (n start-frame sg)
+  (declare (fixnum n))
+  (do* ((p start-frame (parent-frame p sg))
+	(i 0 (1+ i))
+	(q (last-frame-ptr sg)))
+       ((or (null p) (eq p q) (%stack< q p sg)))
+    (declare (fixnum i))
+    (if (= i n)
+      (return p))))
+
 ; True if the object is in one of the heap areas
 (defun %in-consing-area-p (x area)
   (declare (fixnum x))                  ; lie
