@@ -2,22 +2,18 @@
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
 ;;;   Portions copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (eval-when (:compile-toplevel)
   #+linuxppc-target
@@ -826,7 +822,7 @@
     (let* ((ch (%ioblock-tyi ioblock)))
       (if (eq ch :eof)
 	(return (- i start)))
-      (setf (uvref vector i) (char-code ch))
+      (setf (aref vector i) (char-code ch))
       (incf i)
       (decf need)
       (let* ((idx (io-buffer-idx in))
@@ -1665,7 +1661,7 @@
 	(string-input-stream-start s))
   nil)
 
-(defmethod stream-positiion ((s string-input-stream) &optional newpos)
+(defmethod stream-position ((s string-input-stream) &optional newpos)
   (let* ((start (string-input-stream-start s))
 	 (end (string-input-stream-end s))
 	 (len (- end start)))
@@ -2043,15 +2039,46 @@
                                  :address fdset
                                  :unsigned-fullword)))))
 
+;;; Both PROCESS-INPUT-WAIT and PROCESS-OUTPUT-WAIT will enqueue
+;;; requests (handled by the scheduler) if I/O isn't immediately
+;;; possible.  Because of the way that PROCESS-INTERRUPT works,
+;;; it's possible for the PROCESS-WAIT on the request to return
+;;; a false positive, so both functions loop until they're as
+;;; sure as they can be that I/O won't block.
 (defun process-input-wait (fd)
   #+proxy-scheduler (%proxy-wait-for-input fd)
   #-proxy-scheduler
-  (process-wait "input-wait" #'fd-input-available-p fd 1))
+  (let* ((req (%cons-process-io-request fd nil)))
+    (declare (dynamic-extent req))
+    (loop
+	(when (fd-input-available-p fd 0)
+	  (return t))
+	(unwind-protect
+	     (progn
+	       (enqueue-io-request req)
+	       (process-wait "input-wait"
+			     #'(lambda (r)
+				 (process-io-request-ready r)) req))
+	  (when (dll-node-succ req)
+	    (remove-dll-node req))))))
+
 
 (defun process-output-wait (fd)
   #+proxy-scheduler (%proxy-wait-for-output fd)
   #-proxy-scheduler
-  (process-wait "output-wait" #'fd-ready-for-output-p fd 1))
+  (let* ((req (%cons-process-io-request fd t)))
+    (declare (dynamic-extent req))
+    (loop
+	(when (fd-ready-for-output-p fd 0)
+	  (return t))
+	(unwind-protect
+	     (progn
+	       (enqueue-io-request req)
+	       (process-wait "output-wait"
+			     #'(lambda (r)
+				 (process-io-request-ready r)) req))
+	  (when (dll-node-succ req)
+	    (remove-dll-node req))))))
 
   
 ;; Use this when it's possible that the fd might be in
@@ -2077,7 +2104,7 @@
 
 (defun ticks-to-timeval (ticks tv)
   (when ticks
-    (let* ((total-us (* ticks (/ 100000 *ticks-per-second*))))
+    (let* ((total-us (* ticks (/ 1000000 *ticks-per-second*))))
       (multiple-value-bind (seconds us) (floor total-us 1000000)
 	(setf (pref tv :timeval.tv_sec) seconds
 	      (pref tv :timeval.tv_usec) us)))))
@@ -2379,7 +2406,6 @@
                       (external-format :default)
 		      (class *default-file-stream-class*)
                       (elements-per-buffer *elements-per-buffer*))
-  (declare (ignore external-format))
   (loop
     (restart-case
       (return
@@ -2389,7 +2415,8 @@
 			  if-exists
 			  if-does-not-exist
 			  elements-per-buffer
-			  class))
+			  class
+			  external-format))
       (retry-open ()
                   :report (lambda (stream) (format stream "Retry opening ~s" filename))
                   nil))))
