@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 (declaim (special %all-packages%))
@@ -80,17 +76,12 @@
         (return-from %string= nil)))
     t))
 
-(defun symbol-or-string-arg (thing)
-  (if (symbolp thing) 
-    (symbol-name thing)
-    (if (stringp thing)
-      thing
-      (report-bad-arg thing '(or string symbol)))))
+
 
 (defun find-package (name)
   (if (packagep name) 
     name
-    (%find-pkg (symbol-or-string-arg name))))
+    (%find-pkg (string name))))
 
 (defun set-package (name &aux (pkg (find-package name)))
   (if pkg
@@ -106,8 +97,10 @@
     (progn
       (dolist (sym sym-or-syms)
         (unless (symbolp sym) (return (setq sym-or-syms  (mapcar #'(lambda (s) (require-type s 'symbol)) sym-or-syms)))))
-      ; First, see if any packages used by the package being "exported from" already contain a
-      ; distinct non-shadowing symbol that conflicts with one of those that we're trying to export.
+      ;; First, see if any packages used by the package being
+      ;; "exported from" already contain a distinct non-shadowing
+      ;; symbol that conflicts with one of those that we're trying to
+      ;; export.
       (let* ((conflicts (check-export-conflicts sym-or-syms package)))
         (if conflicts
           (progn 
@@ -122,8 +115,9 @@
                     (push s need-import)))))
             (when missing
               (cerror "Import missing symbols before exporting them from ~S."
-                      "The following symbols need to be imported to ~S before they can be exported ~
-                       from that package:~%~S" package missing)
+                      'export-requires-import
+                      :package  package
+                      :to-be-imported missing)
               (import missing package))
             (if need-import (import need-import package))
             ; Can't lose now: symbols are all directly present in package.
@@ -161,7 +155,7 @@
 
 (defun find-all-symbols (name)
   (let* ((syms ())
-         (pname (symbol-or-string-arg name))
+         (pname (string name))
          (len (length pname)))
     (dolist (p %all-packages% syms)
       (multiple-value-bind (sym foundp) (%find-package-symbol pname p len)
@@ -172,7 +166,7 @@
 
 (defun rename-package (package new-name &optional new-nicknames)
   (setq package (pkg-arg package)
-        new-name (ensure-simple-string (symbol-or-string-arg new-name)))
+        new-name (ensure-simple-string (string new-name)))
   (let* ((names (pkg.names package)))
     (declare (type cons names))
     (rplaca names (new-package-name new-name package))
@@ -185,7 +179,7 @@
                         (use nil use-p) 
                         (internal-size 60)
                         (external-size 10))
-  (let ((pkg (find-package (setq name (symbol-or-string-arg name)))))
+  (let ((pkg (find-package (setq name (string name)))))
     (if pkg
       (progn
         (use-package use pkg)
@@ -227,7 +221,7 @@
 
 (defun new-package-name (name &optional package)
   (do* ((prompt "Enter package name to use instead of ~S ."))
-       ((let* ((found (find-package (setq name (if (symbolp name) (symbol-name name) (ensure-simple-string name))))))
+       ((let* ((found (find-package (setq name (string name)))))
           (or (not found)
               (eq package found)))
         name)
@@ -242,8 +236,8 @@
                 (if new-name (setq name new-name))))))
        
 (defun new-package-nickname (name package)
-  (if (symbolp name) (setq name (symbol-name name)))
-  (let* ((other (find-package (setq name (if (symbolp name) (symbol-name name) (ensure-simple-string name)))))
+  (setq name (string name))
+  (let* ((other (find-package name))
          (prompt "Enter package name to use instead of ~S ."))
     (if other
       (unless (eq other package)
@@ -307,12 +301,13 @@
   (multiple-value-bind (foundsym table index) (%find-package-symbol (symbol-name symbol) package)
     (when (and table (eq symbol foundsym))
       (when (memq symbol (pkg.shadowed package))
-        ; A conflict is possible if more than one similarly-named external symbol 
-        ; exists in the packages used by this one.
-        ; Grovel around looking for such conflicts; if any are found, signal an
-        ; error (via %kernel-restart) which offers to either shadowing-import one
-        ; of the conflicting symbols into the current package or abandon the attempt
-        ; to unintern in the first place.
+        ;; A conflict is possible if more than one distinct
+        ;; similarly-named external symbols exist in the packages used
+        ;; by this one.  Grovel around looking for such conflicts; if
+        ;; any are found, signal an error (via %kernel-restart) which
+        ;; offers to either shadowing-import one of the conflicting
+        ;; symbols into the current package or abandon the attempt to
+        ;; unintern in the first place.
         (let* ((first nil)
                (first-p nil)
                (name (symbol-name symbol))
@@ -324,7 +319,9 @@
              (multiple-value-bind (found conflicting-sym) (%get-htab-symbol name len (pkg.etab pkg))
                (when found
                  (if first-p
-                   (push conflicting-sym others)
+                   (unless (or (eq conflicting-sym first)
+                               (memq conflicting-sym others))
+                     (push conflicting-sym others))
                    (setq first-p t first conflicting-sym))))))
           (when others
             ; If this returns, it will have somehow fixed things.
@@ -384,7 +381,7 @@
   t)
 
 (defun shadow-1 (package sym)
-  (let* ((pname (ensure-simple-string (symbol-or-string-arg sym)))
+  (let* ((pname (ensure-simple-string (string sym)))
          (len (length pname)))
     (without-interrupts
      (multiple-value-bind (symbol where internal-idx external-idx) (%find-symbol pname len package)
@@ -414,6 +411,10 @@
     (error "Can't unexport ~S from ~S ." sym package))
   (multiple-value-bind (foundsym foundp internal-offset external-offset)
                        (%findsym (symbol-name sym) package)
+    (unless foundp
+      (error 'symbol-name-not-accessible
+             :symbol-name (symbol-name sym)
+             :package package))
     (when (eq foundp :external)
       (let* ((evec (car (pkg.etab package)))
              (itab (pkg.itab package))
@@ -588,12 +589,13 @@
   (unless (packagep package)
     (setq package (or (find-package package)
                       (progn
-                        (cerror "Do nothing" (%rsc-string $xnopkg) package)
+                        (cerror "Do nothing" 'no-such-package :package package)
                         (return-from delete-package nil)))))
   (unless (memq package %all-packages%)
     (return-from delete-package nil))
   (when (pkg.used-by package)
-    (cerror "unuse ~S" "~S is used by ~S" package (pkg.used-by package)))
+    (cerror "unuse ~S" 'package-is-used-by :package package
+            :using-packages (pkg.used-by package)))
   (while (pkg.used-by package)
     (unuse-package package (car (pkg.used-by package))))
   (while (pkg.used package)
@@ -714,15 +716,15 @@
               (push sym syms)))
           (export syms pkg)))
       (when (and doc *save-doc-strings*)
-	(setf (documentation pkg t) doc))
+        (set-documentation pkg t doc))
       pkg)))
 
 ; The guts of with-package-iterator
 (defun %start-with-package-iterator (p)
   (let ((pkgs (pkg-iter.pkgs p)))
     (if (listp pkgs)
-      (every #'find-package pkgs)
-      (find-package pkgs)))
+      (every #'pkg-arg pkgs)
+      (pkg-arg pkgs)))
   (%pkg-iter-next-package p))
 
 (defun %pkg-iter-next-package (p)
@@ -736,11 +738,12 @@
         (setq pkg (pop pkgs))
         (setq pkg pkgs
               pkgs nil))
-      (setf (pkg-iter.pkg p) (setq pkg (find-package pkg))
+      (setf (pkg-iter.pkg p) (setq pkg (pkg-arg pkg))
             (pkg-iter.pkgs p) pkgs)
-      (cond ((logbitp $pkg-iter-external types) (%start-pkg-iter-externals p))
-            ((logbitp $pkg-iter-internal types) (%start-pkg-iter-internals p))
-            ((logbitp $pkg-iter-inherited types) (%start-pkg-iter-inherited p))))))
+      (when pkg
+        (cond ((logbitp $pkg-iter-external types) (%start-pkg-iter-externals p))
+              ((logbitp $pkg-iter-internal types) (%start-pkg-iter-internals p))
+              ((logbitp $pkg-iter-inherited types) (%start-pkg-iter-inherited p)))))))
 
 (defun %start-pkg-iter-externals (p)
   (let ((tbl (car (uvref (pkg-iter.pkg p) pkg.etab))))
@@ -761,12 +764,13 @@
          (pkg (pop pkgs))
          tbl)
     (if pkg
-      (setf (pkg-iter.state p) #'%get-pkg-iter-inherited
-            (pkg-iter.used p) pkgs
-            (pkg-iter.tbl p) (setq tbl (car (uvref pkg pkg.etab)))
-            (pkg-iter.index p) (length tbl))
-      (setf (pkg-iter.state p) nil)))
-  (%get-pkg-iter-inherited p))
+      (progn
+        (setf (pkg-iter.state p) #'%get-pkg-iter-inherited
+              (pkg-iter.used p) pkgs
+              (pkg-iter.tbl p) (setq tbl (car (uvref pkg pkg.etab)))
+              (pkg-iter.index p) (length tbl))
+        (%get-pkg-iter-inherited p))
+      (setf (pkg-iter.state p) nil))))
 
 (defun %next-pkg-iter-symbol (tbl index)
   (declare (fixnum index))
@@ -872,7 +876,7 @@
     (when import-from-p
       (let ((from-pkg (pop import-from)))
         (dolist (name import-from)
-          (multiple-value-bind (sym win) (find-symbol (symbol-or-string-arg name) from-pkg)
+          (multiple-value-bind (sym win) (find-symbol (string name) from-pkg)
             (when win (import-1 pkg sym))))))
     (when import-p (import import pkg))
     (when export-p
