@@ -1,26 +1,23 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (defstruct (file-ioblock (:include ioblock))
   (octet-pos 0 :type fixnum)		; current io position in octets
-  (fileeof 0 :type fixnum))		; file length in elements
+  (fileeof 0 :type fixnum)		; file length in elements
+)
 
 
 ;;; The file-ioblock-octet-pos field is the (octet) position
@@ -37,6 +34,58 @@
   (setf (file-ioblock-octet-pos file-ioblock)
 	(file-octet-filepos file-ioblock)))
 
+(defun translate-cr-to-lf (file-ioblock)
+  (let* ((inbuf (file-ioblock-inbuf file-ioblock))
+	 (string (io-buffer-buffer inbuf))
+	 (n (io-buffer-count inbuf)))
+    (declare (simple-base-string string)
+	     (fixnum n))
+    (dotimes (i n n)
+      (if (eq (schar string i) #\Return)
+	(setf (schar string i) #\Linefeed)))))
+
+(defun translate-lf-to-cr (file-ioblock n)
+  (declare (fixnum n))
+  (let* ((outbuf (file-ioblock-outbuf file-ioblock))
+	 (string (io-buffer-buffer outbuf)))
+    (declare (simple-base-string string))
+    (dotimes (i n n)
+      (if (eq (schar string i) #\Linefeed)
+	(setf (schar string i) #\Return)))))
+
+(defun infer-external-format (file-stream)
+  (with-stream-ioblock-input (ioblock file-stream :speedy t)
+    (setf (file-stream-external-format file-stream)
+	  (if (eq (%ioblock-peek-char ioblock) :eof)
+	    :unix
+	    (let* ((inbuf (ioblock-inbuf ioblock))
+		   (string (io-buffer-buffer inbuf))
+		   (n (io-buffer-count inbuf)))
+	      (declare (simple-base-string string)
+		       (fixnum n))
+	      (dotimes (i n :unix)
+		(let* ((ch (schar string i)))
+		  (if (eq ch #\Linefeed)
+		    (return :unix))
+		  (when (eq ch #\Return)
+		    (translate-cr-to-lf ioblock)
+		    (return :macos)))))))))
+
+(defvar *default-external-format* :unix)
+
+(defparameter *external-format-translations*
+  '((:unix nil nil)
+    (:macos translate-cr-to-lf translate-lf-to-cr))
+  "an alist: external-format-name, input-translation-function (or NIL),
+   output-translation-function (or NIL)")
+
+(defun file-stream-force-output (stream ioblock count finish-p)
+  (let* ((filter (caddr (assoc (file-stream-external-format stream)
+			       *external-format-translations*
+			       :test #'eq))))
+    (when filter
+      (funcall filter ioblock count))
+    (fd-stream-force-output stream ioblock count finish-p)))
 
 ;;; Establish a new position for the specified file-stream.
 (defun file-ioblock-seek (file-ioblock newoctetpos)
@@ -98,10 +147,14 @@
 	    (setf (io-buffer-idx outbuf) newidx))
 	  (progn
 	    (when (file-ioblock-dirty file-ioblock)
-	      (fd-stream-force-output (file-ioblock-stream file-ioblock)
-				      file-ioblock
-				      (io-buffer-count outbuf)
-				      nil))
+	      (file-stream-force-output (file-ioblock-stream file-ioblock)
+					file-ioblock
+					(io-buffer-count outbuf)
+					nil)
+	      ;; May have just extended the file; may need to update
+	      ;; fileeof.
+	      (when (> maxpos (file-ioblock-fileeof file-ioblock))
+		(setf (file-ioblock-fileeof file-ioblock) maxpos)))
 	    (file-ioblock-seek-and-reset file-ioblock
 					 (ioblock-elements-to-octets
 					  file-ioblock
@@ -123,8 +176,14 @@
 	 (curidx (io-buffer-idx outbuf))
 	 (curpos (+ element-base curidx)))
     (if (null newpos)
-      curpos
+      (if (file-ioblock-untyi-char file-ioblock)
+	(1- curpos)
+	curpos)
       (let* ((incount (io-buffer-count outbuf)))
+	(when (file-ioblock-untyi-char file-ioblock)
+	  (setf (file-ioblock-untyi-char file-ioblock) nil)
+	  (if (> curidx 0)
+	    (decf curpos)))
 	(cond 
 	  ((and (>= newpos element-base)
 		(<= newpos curpos))
@@ -137,12 +196,15 @@
 		(< newpos (+ element-base incount)))
 	   (setf (io-buffer-idx outbuf) (- newpos element-base)))
 	  (t
+	   (let* ((maxpos (+ element-base (io-buffer-count outbuf))))
+	     (when (> maxpos (file-ioblock-fileeof file-ioblock))
+	       (setf (file-ioblock-fileeof file-ioblock) maxpos)))
 	   (when (file-ioblock-dirty file-ioblock)
-             (file-ioblock-seek file-ioblock octet-base)
-	     (fd-stream-force-output (file-ioblock-stream file-ioblock)
-				     file-ioblock
-				     (io-buffer-count outbuf)
-				     nil))
+	     (file-ioblock-seek file-ioblock octet-base)
+	     (file-stream-force-output (file-ioblock-stream file-ioblock)
+				       file-ioblock
+				       (io-buffer-count outbuf)
+				       nil))
 	   (file-ioblock-seek-and-reset file-ioblock
 					(ioblock-elements-to-octets
 					 file-ioblock newpos))))
@@ -155,7 +217,7 @@
 (defun %ioblock-input-file-length (file-ioblock newlen)
   (unless newlen
     (file-ioblock-fileeof file-ioblock)))
-
+ 
 (defun %ioblock-output-file-length (file-ioblock newlen)
   (let* ((octet-base (file-ioblock-octet-pos file-ioblock))
 	 (element-base (ioblock-octets-to-elements file-ioblock octet-base))
@@ -206,7 +268,10 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (defclass file-stream (fd-stream)
     ((filename :initform nil :initarg :filename :accessor file-stream-filename)
-     (original-name :initform nil :initarg :original-name)))
+     (original-name :initform nil :initarg :original-name)
+     (external-format :initform :default :initarg :external-format
+		      :accessor file-stream-external-format)))
+  
 
 (defmethod stream-filename ((s file-stream))
   (file-stream-filename s))
@@ -288,6 +353,16 @@
     (synch-file-octet-filepos file-ioblock)
     nil))
 
+;;; Fill the input buffer, possibly doing newline translation.
+(defun file-stream-advance (stream file-ioblock read-p)
+  (let* ((n (fd-stream-advance stream file-ioblock read-p))
+	   (filter (cadr (assoc (stream-external-format stream)
+				*external-format-translations*
+				:test #'eq))))
+      (if filter
+	(funcall filter file-ioblock))
+      n))
+  
 ;;; If we've been reading, the file position where we're going
 ;;; to read this time is (+ where-it-was-last-time what-we-read-last-time.)
 (defun input-file-ioblock-advance (stream file-ioblock read-p)
@@ -297,8 +372,7 @@
       (break "Expected newpos to be ~d, fd is at ~d"
 	     newpos (file-octet-filepos file-ioblock)))
     (setf (file-ioblock-octet-pos file-ioblock) newpos)
-    ;; (call-next-method)
-    (fd-stream-advance stream file-ioblock read-p)))
+    (file-stream-advance stream file-ioblock read-p)))
 
 ;;; If the buffer's dirty, we have to back up and rewrite it before
 ;;; reading in a new buffer.
@@ -309,13 +383,12 @@
 		    (ioblock-elements-to-octets file-ioblock count))))
     (when (ioblock-dirty file-ioblock)
       (file-ioblock-seek file-ioblock curpos)
-      (fd-stream-force-output stream file-ioblock count nil))
+      (file-stream-force-output stream file-ioblock count nil))
     (unless (eql newpos (file-octet-filepos file-ioblock))
       (break "Expected newpos to be ~d, fd is at ~d"
 	     newpos (file-octet-filepos file-ioblock)))
     (setf (file-ioblock-octet-pos file-ioblock) newpos)
-    ;; (call-next-method)
-    (fd-stream-advance stream file-ioblock read-p)))
+    (file-stream-advance stream file-ioblock read-p)))
 
 		    
 (defun output-file-force-output (stream file-ioblock count finish-p)
@@ -324,7 +397,7 @@
     (unless (eql curpos (file-octet-filepos file-ioblock))
       (break "Expected newpos to be ~d, fd is at ~d"
 	     curpos (file-octet-filepos file-ioblock)))
-    (let* ((n (fd-stream-force-output stream file-ioblock count finish-p)))
+    (let* ((n (file-stream-force-output stream file-ioblock count finish-p)))
       (incf (file-ioblock-octet-pos file-ioblock) (or n 0))
       n)))
 
@@ -393,7 +466,7 @@
 	  (progn
 	    (setf (ioblock-dirty ioblock) nil)
 	    (fd-stream-close s ioblock)
-	    (rename-file original-name filename))
+	    (rename-file original-name filename :if-exists :overwrite))
 	  (delete-file original-name)))
       (setq *open-file-streams* (nremove s *open-file-streams*))
       (call-next-method))))
@@ -421,7 +494,8 @@
 			 if-exists
 			 if-does-not-exist
 			 elements-per-buffer
-			 class)
+			 class
+			 external-format)
   (let* ((temp-name nil)
 	 (pathname (pathname filename)))
     (block open
@@ -445,12 +519,12 @@
 	    (if (eq direction :probe)
 	      (return-from open nil)
 	      (signal-file-error (- #$EISDIR)  filename))
-	    (if (setq filename (if-exists if-exists native-truename "Open ..."))
+	    (if (setq filename (if-exists if-exists filename "Open ..."))
 	      (progn
 		(multiple-value-setq (native-truename kind) (probe-file-x filename))
 		(cond 
 		  ((not native-truename)
-		   (setq native-truename (create-file filename)))
+		   (setq native-truename (%create-file filename)))
 		  ((memq direction '(:output :io))
 		   #|			;
 					; this prevents us from writing a file that is open for anything            
@@ -459,12 +533,13 @@
 		   (signal-file-error EBUSY filename))
 		   |#
 		   (when (eq if-exists :supersede)
-		     (setq temp-name (gen-file-name native-truename))
-		     (rename-file native-truename temp-name)
-		     (create-file native-truename)))))
+		     (let ((truename (native-to-pathname native-truename)))
+		       (setq temp-name (gen-file-name truename))
+		       (rename-file truename temp-name :if-exists :overwrite)
+		       (%create-file native-truename))))))
 	      (return-from open nil)))
 	  (if (setq filename (if-does-not-exist if-does-not-exist filename))
-	    (setq native-truename (create-file filename))
+	    (setq native-truename (%create-file filename))
 	    (return-from open nil)))
 	(let* ((fd (fd-open native-truename (case direction
 					      ((:probe :input) #$O_RDONLY)
@@ -476,6 +551,20 @@
 		 (io-p (eq direction :io))
 		 (char-p (or (eq element-type 'character)
 			     (subtypep element-type 'character)))
+		 (infer nil)
+		 (real-external-format
+		  (if (and char-p in-p)
+		    (progn
+		      (if (eq external-format :default)
+			(setq external-format *default-external-format*))
+		      (if (eq external-format :inferred)
+			(setq infer t external-format :unix)
+			(unless (assoc external-format
+				       *external-format-translations*
+				       :test #'eq)
+			  (setq external-format :unix)))
+		      external-format)
+		    :binary))
 		 (fstream (make-ioblock-stream
 			   (select-stream-class class in-p out-p char-p)
 			   :insize (if in-p elements-per-buffer)
@@ -498,12 +587,15 @@
 			     'io-file-force-output
 			     (if out-p
 			       'output-file-force-output))
-			   :device fd))
+			   :device fd
+			   :external-format real-external-format))
 		 (ioblock (stream-ioblock fstream)))
 	    (setf (stream-filename fstream) (namestring pathname)
 		  (stream-original-name fstream) temp-name)
 	    (setf (file-ioblock-fileeof ioblock)
 		  (ioblock-octets-to-elements ioblock (fd-size fd)))
+	    (if infer
+	      (infer-external-format fstream))
 	    (cond ((eq if-exists :append)
 		   (file-position fstream :end))
 		  ((and (memq direction '(:io :output))
@@ -514,6 +606,10 @@
 	      (push fstream *open-file-streams*))
 	    fstream))))))
 
+(defun stream-external-format (stream)
+  (require-type stream 'file-stream)
+  (file-stream-external-format stream))
+
 ;;; Under the circumstances, this is a very slow way of saying
 ;;; "we don't support EXTENDED-CHARs".
 (defun file-string-length (stream object)
