@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 
@@ -144,11 +140,12 @@
                    (setq symbol tem)
                    (progn (setq symbol (cadr symbol))
                           (setq setf-p t))))))
-					; assoc-pair is e.g. (function file1 ...) or (class . file)
-					; or (method (method-object  file1 ...) ...) or (method (method-object . file) ...)
-        (when (symbolp symbol)		; avoid boot problems - you thought          
+        ;; assoc-pair is e.g. (function file1 ...)  or (class . file)
+        ;; or (method (method-object file1 ...) ...) or (method
+        ;; (method-object . file) ...)
+        (when (symbolp symbol)		; avoid boot problems - you thought 
           (multiple-value-bind (source-files-info assoc-pair files)
-	      (source-file-or-files symbol def-type setf-p method)                                             
+	      (source-file-or-files symbol def-type setf-p method) 
             (setq old-file 
                   (cond ((consp files)
                          (if (consp (cdr files)) (cadr files) (cdr files)))
@@ -208,11 +205,11 @@
   (memq (if (symbolp parent) (find-class parent nil) parent)
         (%inited-class-cpl (class-of ob))))
 
-; returns new plist with value spliced in or key, value consed on.
+;;; returns new plist with value spliced in or key, value consed on.
 (defun setprop (plist key value &aux loc)
- (if (setq loc (pl-search plist key))
-  (progn (%rplaca (%cdr loc) value) plist)
-  (cons key (cons value plist))))
+  (if (setq loc (pl-search plist key))
+    (progn (%rplaca (%cdr loc) value) plist)
+    (cons key (cons value plist))))
 
 (defun getf-test (place indicator test &optional default)
   (loop
@@ -304,12 +301,12 @@
 (%fhave 'set-macro-function #'%macro-have)   ; redefined in sysutils.
 
 ; Define special forms.
-(dolist (sym '(block catch compiler-let declare eval-when
+(dolist (sym '(block catch compiler-let eval-when
                flet function go if labels let let* macrolet
                multiple-value-call multiple-value-prog1
                progn progv quote return-from setq tagbody
                the throw unwind-protect locally load-time-value
-	       symbol-macrolet 
+	       symbol-macrolet
 ; These are implementation-specific special forms :
 	       nfunction without-interrupts
                macro-bind debind 
@@ -332,7 +329,7 @@
     (record-source-file name 'function)
     (set-macro-function name named-fn)
     (when (and doc *save-doc-strings*)
-      (setf (documentation name 'function) doc))
+      (set-documentation name 'function doc))
     (when body-pos
       (setf (assq name *fred-special-indent-alist*) body-pos))
     (when arglist
@@ -340,18 +337,14 @@
     (when *fasload-print* (format t "~&~S~%" name))
     name))
 
-#+ignore                                ; moved to "ccl:level-0;l0-symbol.lisp"
-(defun boundp (sym)
-  (not (eq (%sym-value sym) (%unbound-marker)))) ; undefinedp
-
 
 (defun %defvar (var &optional doc)
   "Returns boundp"
   (%proclaim-special var)
   (record-source-file var 'variable)
+  (when (and doc *save-doc-strings*)
+    (set-documentation var 'variable doc))
   (cond ((not (boundp var))
-         (when (and doc *save-doc-strings*)
-           (setf (documentation var 'variable) doc))
          (when *fasload-print* (format t "~&~S~%" var))
          nil)
         (t t)))
@@ -360,7 +353,7 @@
   (%proclaim-special var)
   (record-source-file var 'variable)
   (when (and doc *save-doc-strings*)
-    (setf (documentation var 'variable) doc))
+    (set-documentation var 'variable doc))
   (when *fasload-print* (format t "~&~S~%" var))
   (set var value)
   var)
@@ -604,7 +597,7 @@
   (record-source-file var 'constant)
   (define-constant var value)
   (when (and doc *save-doc-strings*)
-    (setf (documentation var 'variable) doc))
+    (set-documentation var 'variable doc))
   (when *fasload-print* (format t "~&~S~%" var))
   var)
 
@@ -807,10 +800,13 @@ vector
            (keywordp form)))
   )
 
-(defun constantp (form)
+(defun constantp (form &optional env)
    (or (self-evaluating-p form)
        (quoted-form-p form)
-       (constant-symbol-p form)))
+       (constant-symbol-p form)
+       (and env
+	    (symbolp form)
+	    (eq :constant (variable-information form env)))))
 
 (defun eval-constant (form)
   (if (quoted-form-p form) (%cadr form)
@@ -843,8 +839,6 @@ vector
         (qlfun bootstrapping-%setf-method (name)
           (get name 'bootstrapping-setf-method)))
 
-(%fhave 'function-spec-p #'symbolp)     ; redefined later
-
 
 ;;; Lisp Development System/Application Module Loading
 
@@ -1005,16 +999,14 @@ vector
     
 ;;;;;;;;; VALUE BINDING Functions
 
-(defun gensym (&optional string-or-integer)
+(defun gensym (&optional (string-or-integer nil string-or-integer-p))
   "Behaves just like Common Lisp. Imagine that."
   (let ((prefix "G")
         (counter nil))
-    (if (integerp string-or-integer)
-      (setq counter string-or-integer) ; & emit-style-warning
-      (if (stringp string-or-integer)
-        (setq prefix (ensure-simple-string string-or-integer))
-        (if string-or-integer ;not string or index or NIL
-          (report-bad-arg string-or-integer '(or string integer (eql nil))))))
+    (when string-or-integer-p
+      (etypecase string-or-integer
+        (integer (setq counter string-or-integer)) ; & emit-style-warning
+        (string (setq prefix (ensure-simple-string string-or-integer)))))
     (unless counter
       (setq *gensym-counter* (1+ (setq counter *gensym-counter*))))
     (make-symbol (%str-cat prefix (%integer-to-string counter)))))
@@ -1184,42 +1176,6 @@ vector
   arg)
 
 
-
-
-; start and end are character positions
-; maxsize is a byte count.
-(defun %put-string-segment-contents (p str start end &optional maxsize script)
-  (declare (ignore script))
-  (multiple-value-setq (str start end)(string-start-end str start end))
-  (flet ((err () (error "String too big to fit in record")))
-    (let ((size (- end start)))
-      (if (and maxsize (%i> size maxsize))
-        (err)
-            (cond
-             ((simple-base-string-p str)
-              (%copy-ivector-to-ptr str start p 0 size)
-              size)
-             (t (let (;(table (get-char-byte-table script))
-                  (j 0))
-              (when (not maxsize)(setq  maxsize most-positive-fixnum))
-              (dotimes (i size)                
-                (let ((c (%scharcode str i)))                  
-                  ;(cerror "b" "foo ~s ~s ~S" i j C)
-                  (if (and #|table|# (%i> c #xff) #|(eq (svref table (%ilsr 8 c)) 1)|#)
-                    (progn 
-                      (when  (%i>= j (%i- maxsize 1))(err))
-                      (%put-word p c j)
-                      (setq j (%i+ j 2)))
-                    (progn
-                      (when (%i>= j maxsize)(err))
-                      (%put-byte p c j)
-                      (setq j (%i+ j 1))))))
-              j)))))))
-
-(defun %set-string (pointer offset &optional (string (prog1 offset (setq offset 0))))
-  (%put-string pointer string offset)
-  string)
-
 (defun %put-cstring (ptr str &optional (offset 0))
   (do* ((len (length str))
 	(i 0 (1+ i)))
@@ -1254,39 +1210,6 @@ vector
   (%set-double-float macptr offset value))
 
 
-  
-(defun %get-single-float (macptr &optional (offset 0) (res (%make-sfloat)))
-  (unless (macptrp macptr)
-    (setq macptr (require-type macptr 'macptr)))
-  (unless (fixnump offset)
-    (setq offset (require-type offset 'fixnum)))
-  (unless (typep res 'short-float)
-    (setq res (require-type res 'short-float)))
-  (locally (declare (fixnum offset)) 
-    (with-macptrs ((single (%inc-ptr macptr offset)))
-      (%ref-ieee-single-float single res))))
-
-
-
-(defun %set-single-float (macptr offset &optional (value nil value-p))
-  (unless value-p
-    (setq value offset
-          offset 0))
-  (unless (macptrp macptr)
-    (setq macptr (require-type macptr 'macptr)))
-  (unless (fixnump offset)
-    (setq offset (require-type offset 'fixnum)))
-  (locally (declare (fixnum offset))
-    (cond ((typep value 'short-float)
-           (with-macptrs ((single (%inc-ptr macptr offset)))
-             (%set-ieee-single-float value single)))
-          ((typep value 'double-float)
-           (with-macptrs ((single (%inc-ptr macptr offset)))
-             (%set-ieee-single-float-from-double value single)))
-          (t (return-from %set-single-float
-               (%set-single-float
-                macptr offset (require-type value 'short-float))))))
-  value)
 
 
 
@@ -1368,38 +1291,6 @@ vector
    (stf %f2 (%imm0)))
 
 
-; Copy an IEEE-single (aka "short") float from a macptr to a lisp short float
-#+ppc-target
-(defppclapfunction %ref-ieee-single-float ((macptr arg_y) (dest arg_z))
-  (check-nargs 2)
-  (lwz imm0 arch::macptr.address macptr)
-  (lfs fp1 0 imm0)
-  (put-single-float fp1 dest)
-  (blr))
-
-#+sparc-target
-(defsparclapfunction %ref-ieee-single-float ((macptr %arg_y) (dest %arg_z))
-  (check-nargs 2)
-  (ld (macptr arch::macptr.address) %imm0)
-  (ldf (%imm0) %f2)
-  (retl)
-   (put-single-float %f2 dest))
-
-#+ppc-target
-(defppclapfunction %set-ieee-single-float ((src arg_y) (macptr arg_z))
-  (check-nargs 2)
-  (lwz imm0 arch::macptr.address macptr)
-  (get-single-float fp1 src)
-  (stfs fp1 0 imm0)
-  (blr))
-
-#+sparc-target
-(defsparclapfunction %set-ieee-single-float ((src %arg_y) (macptr %arg_z))
-  (check-nargs 2)
-  (ld (macptr arch::macptr.address) %imm0)
-  (get-single-float src %f2)
-  (retl)
-    (stf %f2 (%imm0)))
 
 #+ppc-target
 (defppclapfunction %set-ieee-single-float-from-double ((src arg_y) (macptr arg_z))
@@ -1473,12 +1364,7 @@ vector
 
 
 
-(defun point-string (point)
-  (%str-cat "#@("
-            (%integer-to-string (point-h point))
-            " "
-            (%integer-to-string (point-v point))
-            ")"))
+
 
 
 (defun position (item sequence &rest ignored-keys)
