@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (eval-when (:compile-toplevel :execute)
   #+linuxppc-target
@@ -65,8 +61,12 @@
     (syscall os::chdir dirname)))
 
 (defun %mkdir (name mode)
-  (with-cstrs ((name name))
-    (syscall os::mkdir name mode)))
+  (let* ((last (1- (length name))))
+    (with-cstrs ((name name))
+      (when (and (>= last 0)
+		 (eql (%get-byte name last) (char-code #\/)))
+	(setf (%get-byte name last) 0))
+    (syscall os::mkdir name mode))))
 
 (defun getenv (key)
   (with-cstrs ((key (string key)))
@@ -219,7 +219,7 @@
 
 (defun get-internal-real-time ()
   (rlet ((tv :timeval))
-    (syscall os::gettimeofday tv (%null-ptr))
+    (#_gettimeofday tv (%null-ptr))
     (timeval->milliseconds tv)))
 
 (defun %add-timevals (result a b)
@@ -341,46 +341,78 @@
 	      (shlib.base lib) nil
 	      (shlib.map lib) nil)
 	    (unload-library-entrypoints lib)))))))
-	    
+
+#+darwinppc-target
+;; completely specifies whether to remove it totally from our list
+(defun close-shared-library (lib &key (completely nil))
+  (let* ((lib (if (typep lib 'string)
+		  (or (shared-library-with-name lib)
+		      (error "Shared library ~s not found." lib))
+		(require-type lib 'shlib))))
+    ;; no possible danger closing libsystem since dylibs can't be closed
+    (cond
+     ((or (not (shlib.map lib)) (not (shlib.base lib)))
+      (error "Shared library ~s uninitialized." (shlib.soname lib)))
+     ((and (not (%null-ptr-p (shlib.map lib)))
+	   (%null-ptr-p (shlib.base lib)))
+      (warn "Dynamic libraries cannot be closed on Darwin."))
+     ((and (%null-ptr-p (shlib.map lib))
+	   (not (%null-ptr-p (shlib.base lib))))
+      ;; we have a bundle type library
+      ;; not sure what to do with the completely flag
+      ;; when we open the same bundle more than once, Darwin gives back
+      ;; a new module address, so we have multiple entries on *shared-libraries*
+      ;; the best we can do is unlink the module asked for (or our best guess based on name)
+      ;; and invalidate any entries which refer to this container
+      (if (= 0 (#_NSUnLinkModule (shlib.base lib) #$NSUNLINKMODULE_OPTION_NONE))
+	  (error "Unable to close shared library, NSUnlinkModule failed.")
+	(progn
+	  (setf (shlib.map lib) nil
+		(shlib.base lib) nil)
+	  (unload-library-entrypoints lib)
+	  (when completely
+	    (setq *shared-libraries* (delete lib *shared-libraries*)))))))))
+
 
 
 ;;; Foreign (unix) processes.
 
 (defun call-with-string-vector (function strings)
-  (with-cstrs ((name (car strings)))
-    (%stack-block
-     ((buf (reduce #'+ strings
-                   :key #'(lambda (s) (1+ (length (string s))))))
-      (argv (ash (1+ (length strings)) 2)))
-     (do* ((strings strings (cdr strings))
-           (s (string (car strings)) (string (car strings)))
-           (p 0 (+ p len 1))
-           (len (length s) (length s))
-           (argp argv (%inc-ptr argp 4)))
-          ((null strings) (setf (%get-ptr argp) (%null-ptr)))
-       (declare (fixnum len p) (type macptr argp))
-       (setf (%get-ptr argp) (%inc-ptr buf p))
-       (%copy-ivector-to-ptr s 0 buf p len)
-       (setf (%get-byte buf (+ p len)) 0))
-     (funcall function argv))))
+  (let ((bufsize (reduce #'+ strings
+			 :key #'(lambda (s) (1+ (length (string s))))))
+	(argvsize (ash (1+ (length strings)) 2))
+	(bufpos 0)
+	(argvpos 0))
+    (%stack-block ((buf bufsize) (argv argvsize))
+      (flet ((init (s)
+	     (multiple-value-bind (sstr start end) (get-sstring s)
+	       (let ((len (- end start)))
+		 (%copy-ivector-to-ptr sstr start buf bufpos len)
+		 (setf (%get-byte buf (%i+ bufpos len)) 0)
+		 (setf (%get-ptr argv argvpos) (%inc-ptr buf bufpos))
+		 (setq bufpos (%i+ bufpos len 1))
+		 (setq argvpos (%i+ argvpos 4))))))
+	(declare (dynamic-extent #'init))
+	(map nil #'init strings))
+      (setf (%get-ptr argv argvpos) (%null-ptr))
+      (funcall function argv))))
 
 (defmacro with-string-vector ((var &rest strings) &body body)
   `(call-with-string-vector #'(lambda (,var) ,@body) ,@strings))
 
 (defloadvar *max-os-open-files* (#_getdtablesize))
 
-(defun %execvp (strings)
-  (call-with-string-vector #'(lambda (argv) (#_execvp (%get-ptr argv) argv))
-                           strings)
+(defun %execvp (argv)
+  (#_execvp (%get-ptr argv) argv)
   (#_exit #$EX_OSERR))
 
-(defun exec-with-io-redirection (new-in new-out new-err strings)
+(defun exec-with-io-redirection (new-in new-out new-err argv)
   (#_setpgid 0 0)
   (if new-in (#_dup2 new-in 0))
   (if new-out (#_dup2 new-out 1))
   (if new-err (#_dup2 new-err 2))
   (do* ((fd 3 (1+ fd)))
-       ((= fd *max-os-open-files*) (%execvp strings))
+       ((= fd *max-os-open-files*) (%execvp argv))
     (declare (fixnum fd))
     (#_close fd)))
 
@@ -395,14 +427,20 @@
 
 
 
-
+#+linuxppc-target
 (defun pipe ()
   (%stack-block ((pipes 8))
     (let* ((status (syscall os::pipe pipes)))
-      (if (eql 0 status)
+      (if (= 0 status)
 	(values (%get-long pipes 0) (%get-long pipes 4))
 	(%errno-disp status)))))
 
+#+darwinppc-target
+(defun pipe ()
+  (let* ((ans (syscall os::pipe)))
+    (if (< ans 0)
+      (%errno-disp ans nil)
+      (values (ldb (byte 32 32) ans) (ldb (byte 32 0) ans)))))
 
 (defstruct external-process
   pid
@@ -445,15 +483,15 @@
        (case direction
 	 (:input
 	  (values read-pipe
-		  (make-fd-stream read-pipe
-				  :direction :input
+		  (make-fd-stream write-pipe
+				  :direction :output
 				  :interactive nil)
 		  (cons read-pipe close-in-parent)
 		  (cons write-pipe close-on-error)))
 	 (:output
 	  (values write-pipe
-		  (make-fd-stream write-pipe
-				  :direction :output
+		  (make-fd-stream read-pipe
+				  :direction :input
 				  :interactive nil)
 		  (cons write-pipe close-in-parent)
 		  (cons read-pipe close-on-error)))
@@ -506,6 +544,8 @@
 (defloadvar *external-processes* nil)
 (defloadvar *watched-fd-handlers* nil)
 
+(defvar *external-process-watchdog*)
+
 (defun watch-file-descriptors ()
   (dolist (handler *watched-fd-handlers*) (funcall handler)))
 
@@ -570,23 +610,30 @@
 	       (get-descriptor-for error token close-in-parent close-on-error
 				   :direction :output
 				   :if-exists if-error-exists)))
-	   (let* ((child-pid (#_fork)))
-	     (declare (fixnum child-pid))
-	     (cond ((zerop child-pid)
-		    ;; Running in the child; do an exec
-		    (exec-with-io-redirection in-fd out-fd error-fd args))
-		   ((> child-pid 0)
-		    ;; Running in the parent: success
-		    (setq proc
-			  (make-external-process
-			   :pid child-pid
-			   :%status :running
-			   :input in-stream
-			   :output out-stream
-			   :error error-stream
-			   :token token
-			   :status-hook status-hook))
-		    (push proc *external-processes*)))))
+	   (call-with-string-vector
+	    #'(lambda (argv)
+		(let* ((child-pid (#_fork)))
+		  (declare (fixnum child-pid))
+		  (cond ((zerop child-pid)
+			 ;; Running in the child; do an exec
+			 (without-interrupts
+			  (exec-with-io-redirection
+			   in-fd out-fd error-fd argv)))
+			((> child-pid 0)
+			 ;; Running in the parent: success
+			 (setq proc
+			       (make-external-process
+				:pid child-pid
+				:%status :running
+				:input in-stream
+				:output out-stream
+				:error error-stream
+				:token token
+				:status-hook status-hook))
+			 (push proc *external-processes*)))))
+	    args))
+      (unless (process-active-p *external-process-watchdog*)
+	(process-reset-and-enable *external-process-watchdog*))
       (dolist (fd close-in-parent) (fd-close fd))
       (unless proc
 	(dolist (fd close-on-error) (fd-close fd)))
@@ -697,6 +744,7 @@
 			(delete p *external-processes*))))))))))
 
 
+
 (defun external-process-wait (proc &optional check-stopped)
   (process-wait "external-process-wait"
 		#'(lambda ()
@@ -737,3 +785,8 @@
     (or (eql error 0)
 	(%errno-disp error))))
 
+;;; EOF on a TTY is transient, but I'm less sure of other cases.
+(defun eof-transient-p (fd)
+  (case (%unix-fd-kind fd)
+    (:tty t)
+    (t nil)))
