@@ -1,22 +1,18 @@
 ;;; ppc-trap-support
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 ;; Copyright 1995 Digitool, Inc. The 'tool rules!
 ;;
 ;; Support for PPC traps, this includes the event-poll trap
@@ -268,14 +264,15 @@
 ;; Some of these could probably call %err-disp instead of error,
 ;; but I was too lazy to look them up.
 
-(defcallback cmain (:address xp 
-                             :unsigned-fullword fn-reg 
-                             :address pc-or-index 
-                             :unsigned-fullword the-trap
-                             :signed-fullword 
-			     #+proxy-scheduler trap-level
-			     #-proxy-scheduler ignore-0
-                             :signed-fullword ignore-1)
+(defcallback cmain (:without-interrupts t
+					:address xp 
+					:unsigned-fullword fn-reg 
+					:address pc-or-index 
+					:unsigned-fullword the-trap
+					:signed-fullword 
+					#+proxy-scheduler trap-level
+					#-proxy-scheduler ignore-0
+					:signed-fullword ignore-1)
   (declare (ignore ignore-1 #-proxy-scheduler ignore-0))
   ;; twgti nargs,0
   ;; time for event polling.
@@ -286,7 +283,7 @@
         (unwind-protect
           (progn
             (handle-gc-hooks)
-            (setq *interrupt-level* 0)
+            ;(setq *interrupt-level* 0)
 	    #+proxy-scheduler (cmain (not (eql 0 trap-level)))
 	    #-proxy-scheduler (cmain))
           ; Set the first binding of *interrupt-level* to >= 0
@@ -642,7 +639,7 @@
   (blr))
   
 
-(defun ff-call (entry &rest specs-and-vals)
+(defun %ff-call (entry &rest specs-and-vals)
   (declare (dynamic-extent specs-and-vals))
   (let* ((len (length specs-and-vals))
          (other-offset 8)
@@ -663,7 +660,7 @@
              (fpr-reloads (make-array 8 :element-type '(unsigned-byte 8))))
         (declare (fixnum nargs) (dynamic-extent fpr-reloads))
         (do* ((i 0 (1+ i))
-              (specs specs-and-vals (cddr specs-and-vals))
+              (specs specs-and-vals (cddr specs))
               (spec (car specs) (car specs)))
              ((= i nargs))
           (declare (fixnum i))
@@ -808,7 +805,7 @@
 ;;; arguments are aligned on word boundaries.
 #+poweropen-target
 (progn
-(defun ff-call (entry &rest specs-and-vals)
+(defun %ff-call (entry &rest specs-and-vals)
   (declare (dynamic-extent specs-and-vals))
   (let* ((len (length specs-and-vals))
 	 (total-words 0))
@@ -821,22 +818,25 @@
 	     (fpr-reload-offsets (make-array 13 :element-type '(unsigned-byte 16))))
         (declare (fixnum nargs) (dynamic-extent fpr-reload-sizes fpr-reload-offsets))
         (do* ((i 0 (1+ i))
-              (specs specs-and-vals (cddr specs-and-vals))
+              (specs specs-and-vals (cddr specs))
               (spec (car specs) (car specs)))
              ((= i nargs))
           (declare (fixnum i))
-          (ecase spec
+          (case spec
 	    ((:double-float :signed-doubleword :unsigned-doubleword)
 	     (incf total-words 2))
 	    ((:single-float :signed-byte :unsigned-byte :signed-halfword
 			    :unsigned-halfword :signed-fullword
 			    :unsigned-fullword :address)
-	     (incf total-words))))
+	     (incf total-words))
+	    (t (if (typep spec 'unsigned-byte)
+		 (incf total-words spec)
+		 (error "Invalid argument spec ~s" spec)))))
 	(%stack-block ((buf (ash (logand (lognot 1) (1+ (max 6  total-words))) 2)))
 	  (do* ((i 0 (1+ i))
 		(fpr 0)
 		(offset 0 (+ offset 4))
-		(specs specs-and-vals (cddr specs-and-vals))
+		(specs specs-and-vals (cddr specs))
 		(spec (car specs) (car specs))
 		(val (cadr specs) (cadr specs)))
 	       ((= i nargs))
@@ -864,7 +864,12 @@
 	      (:address
 	       (setf (%get-ptr buf offset) val))
 	      (t
-	       (setf (%get-long buf offset) val))))
+	       (if (typep spec 'unsigned-byte)
+		 (dotimes (i spec (decf offset 4))
+		   (setf (%get-ptr buf offset)
+			 (%get-ptr val (* i 4)))
+		   (incf offset 4))
+		 (setf (%get-long buf offset) val)))))
 	  (let* ((frame-size (if (<= total-words 8)
 			       (- (ash
 				   (+ ppc::c-frame.size ppc::lisp-frame.size)
@@ -878,7 +883,7 @@
 	    (%%ff-call
 	     fpr-reload-sizes
 	     fpr-reload-offsets
-	     frame-size
+	     (logandc2 (+ frame-size 15) 15)
 	     total-words
 	     buf
 	     entry))
