@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;; sysutils.lisp - things which have outgrown l1-utils
@@ -32,7 +28,7 @@
 
 ;;; things might be clearer if this stuff were in l1-typesys?
 ;;; Translation from type keywords to specific predicates.
-(eval-when (eval compile)
+(eval-when (:execute :compile-toplevel)
 
 (defconstant type-pred-pairs
   '((array . arrayp)
@@ -83,7 +79,6 @@
     (structure . structurep)
     (structure-object . structurep)
     (symbol . symbolp)
-    (symbol-locative . symbol-locative-p) ; not cl
     (t . true)
     (unsigned-byte . unsigned-byte-p) ;unsigned-byte-p is not cl.
     (vector . vectorp)
@@ -284,7 +279,8 @@
                                  rational float short-float single-float
                                  double-float long-float real member))))))
 
-(defun typep (object type)
+(defun typep (object type &optional env)
+  (declare (ignore env))
   (let* ((pred (if (symbolp type) (type-predicate type))))
     (if pred
       (funcall pred object)
@@ -317,7 +313,8 @@
 
 ; Subtypep.
 
-(defun subtypep (type1 type2)
+(defun subtypep (type1 type2 &optional env)
+  (declare (ignore env))
   "Return two values indicating the relationship between type1 and type2:
   T and T: type1 definitely is a subtype of type2.
   NIL and T: type1 definitely is not a subtype of type2.
@@ -344,17 +341,11 @@
 (defun copy-into-float (src dest)
   (%copy-double-float src dest))
 
-
- 
-
 (queue-fixup
  (defun fmakunbound (name)
-   (when (consp name)
-     (unless (eq (%car name) 'setf)
-       (return-from fmakunbound (funcall (function-spec-handler name) name nil)))
-     (setq name (setf-function-name (cadr name))))
-   (remhash name %structure-refs%)
-   (%unfhave name)
+   (let* ((fname (validate-function-name name)))
+     (remhash fname %structure-refs%)
+     (%unfhave fname))
    name))
 
 (defun frozen-definition-p (name)
@@ -370,7 +361,7 @@
                  (let ((pkg (symbol-package (if (consp name) (cadr name) name))))
                    (or (eq *common-lisp-package* pkg) (eq *ccl-package* pkg)))))
     (cerror "Replace the definition of ~S."
-            "The function ~S is already defined in the MCL kernel." name)
+            "The function ~S is predefined in OpenMCL." name)
     (unless (consp name)
       (proclaim-inline nil name))))
 
