@@ -1,22 +1,18 @@
 ; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 ;; chars.lisp
 
 
@@ -269,22 +265,10 @@
              (let ((str (make-string 2 :element-type 'base-char)))
                (declare (simple-base-string str))
                (setf (schar str 0) #\^)
-               (setf (schar str 1)(code-char (%ilogxor code #x40))))))
+               (setf (schar str 1)(code-char (%ilogxor code #x40)))
+               str)))
           ((and (< code #x100)(graphic-char-p c)) nil)
-          (t (let* ((n (if (< code #o1000)
-                                       3
-                                       (if (< code #o10000)
-                                         4
-                                         (if (< code #o100000) 5 6))))
-                    (str (make-string n :element-type 'base-char)))
-               (declare (fixnum n)(simple-base-string str))
-               (do ((i (1- n)(1- i)))
-                   ((< i 0))
-                 (declare (fixnum i))
-                 (setf (schar str i)
-                       (code-char (+ (char-code #\0)(%ilogand code 7))))
-                 (setq code (lsh code -3)))
-               str)))))
+          (t nil))))
 
 
 (defun string-downcase (string &key start end)
@@ -398,31 +382,40 @@
   string)
 
   
-  ;; returns nil when non = and non ascii, else keeps chugging
 (defun string-compare (string1 start1 end1 string2 start2 end2)
-  (if (and (simple-string-p string1)(null start1)(null end1))
+  (let ((istart1 (or start1 0)))
+    (if (and (typep string1 'simple-string)(null start1)(null end1))
       (setq start1 0 end1 (length string1))
       (multiple-value-setq (string1 start1 end1)(string-start-end string1 start1 end1)))
-  (if (and (simple-string-p string2)(null start2)(null end2))
+    (if (and (typep string2 'simple-string)(null start2)(null end2))
       (setq start2 0 end2 (length string2))
       (multiple-value-setq (string2 start2 end2)(string-start-end string2 start2 end2)))
-  (let* ((len1 (- end1 start1))
-         (len2 (- end2 start2)))
-    (declare (fixnum len1 len2))
-    (dotimes (i (the fixnum (min len1 len2)) (if (eq len1 len2)
-                                               (values t len1)
-                                               (values (if (< len1 len2) -1 1)
-                                                       (min len1 len2))))
-      (let* ((c1 (%scharcode string1 (%i+ start1 i)))
-             (c2 (%scharcode string2 (%i+ start2 i))))
-        (declare (fixnum c1 c2))
-        (unless (eq c1 c2)
-          (if (and (>= c1 (char-code #\a))(<= c1 (char-code #\z)))
-            (setq c1 (- c1 #.(- (char-code #\a)(char-code #\A)))))
-          (if (and (>= c2 (char-code #\a))(<= c2 (char-code #\z)))
-            (setq c2 (- c2 #.(- (char-code #\a)(char-code #\A)))))
-          (unless (eq c1 c2)
-            (return (values (if (> c1 c2) 1 -1) i))))))))
+    (setq istart1 (%i- start1 istart1))        
+    (let* ((val t))
+      (declare (optimize (speed 3)(safety 0)))
+      (do* ((i start1 (%i+ 1 i))
+            (j start2 (%i+ 1 j)))
+           ()
+        (when (eq i end1)
+          (when (neq j end2)(setq val -1))
+          (return))
+        (when (eq j end2)
+          (setq end1 i)
+          (setq val 1)(return))
+        (let ((code1 (%scharcode string1 i))
+              (code2 (%scharcode string2 j)))
+          (declare (fixnum code1 code2))
+          (if (and (>= code1 (char-code #\a))
+                   (<= code1 (char-code #\z)))
+            (setq code1 (- code1 (- (char-code #\a) (char-code #\A)))))
+          (if (and (>= code2 (char-code #\a))
+                   (<= code2 (char-code #\z)))
+            (setq code2 (- code2 (- (char-code #\a) (char-code #\A)))))
+          (unless (= code1 code2)            
+            (setq val (if (%i< code1 code2) -1 1))
+            (setq end1 i)
+            (return))))
+      (values val (%i- end1 istart1)))))
 
 
 (defun string-greaterp (string1 string2 &key start1 end1 start2 end2)
