@@ -1,22 +1,18 @@
 ; -*- Mode:Lisp; Package:INSPECTOR -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (defpackage "INSPECTOR"
   (:use "CL" "CCL"))
@@ -886,8 +882,6 @@
 (defmethod line-n ((num integer) n)
   (if (and (>= n 7) (not (< 0 num 4000))) (incf n))   ; maybe skip roman.
   (if (and (>= n 8) (not (<= 0 num 255))) (incf n))   ; maybe skip character.
-  (if (and (>= n 9) (not (<= #x-1000 (ash num -16) #xfff)))
-               (incf n))                ; maybe skip point
   (let* ((type :static)
          (neg? (< num 0))
          (norm (if neg? 
@@ -1234,7 +1228,7 @@
 (defun disassembly-line-n (f n)
   (let* ((line (svref (disasm-info f) n))
          (value (disasm-line-immediate line)))
-    (values value line (if value :static (disassembly-comment-type)))))
+    (values value line (if value :static :comment))))
 
 (defun set-disassembly-line-n (f n new-value &optional 
                                  (function (inspector-object f)))
