@@ -1,28 +1,31 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 ;;; This is a slightly-watered-down version of CMUCL's ALIEN-TYPE system.
 
 (in-package "CCL")
 
-(defstruct (interface-dir (:include dll-node))
+(defstruct (interface-dir
+	     (:include dll-node)
+	     (:print-object
+	      (lambda (d stream)
+		(print-unreadable-object (d stream :type t :identity t)
+		  (format stream "~s ~s"
+			  (interface-dir-name d)
+			  (interface-dir-subdir d))))))
   (name)
   (subdir)
   (constants-interface-db-file)
@@ -30,9 +33,6 @@
   (records-interface-db-file)
   (types-interface-db-file))
 
-(defmethod print-object ((d interface-dir) stream)
-  (print-unreadable-object (d stream :type t :identity t)
-    (format stream "~s ~s" (interface-dir-name d) (interface-dir-subdir d))))
   
 ;;; This is intended to try to encapsulate foreign type stuff, to
 ;;; ease cross-compilation (among other things.)
@@ -54,7 +54,9 @@
    #+darwinppc-target "DARWIN")
   (external-function-definitions (make-hash-table :test #'eq))
   (syscalls (make-hash-table :test #'eq))
-  (dirlist (make-dll-header)))
+  (dirlist (make-dll-header))
+  (attributes #+darwinppc-target '(:signed-char :struct-by-value)
+	      #+linuxppc-target ()))
 
 (defvar *host-ftd* (make-ftd))
 (defvar *target-ftd* *host-ftd*)
@@ -113,7 +115,11 @@
           ((> bits 1) 8)
           (t 1)))
 
-  (defstruct foreign-type-class
+  (defstruct (foreign-type-class
+	       (:print-object
+		(lambda (f out)
+		  (print-unreadable-object (f out :type t :identity t)
+		    (prin1 (foreign-type-class-name f) out)))))
     (name nil :type symbol)
     (include nil :type (or null foreign-type-class))
     (unparse nil :type (or null function))
@@ -129,10 +135,6 @@
     (result-tn nil :type (or null function))
     (subtypep nil :type (or null function)))
 
-  (defmethod print-object ((f foreign-type-class) out)
-    (print-unreadable-object (f out :type t :identity t)
-      (prin1 (foreign-type-class-name f) out)))
-
 
   (defvar *foreign-type-classes* (make-hash-table :test #'eq))
   
@@ -145,7 +147,7 @@
     (or (gethash (make-keyword x) (ftd-kind-info *target-ftd*)) :unknown))
   (defun (setf info-foreign-type-kind) (val x)
     (setf (gethash (make-keyword x) (ftd-kind-info *target-ftd*)) val))
-
+		   
   (defun info-foreign-type-definition (x)
     (gethash (make-keyword x) (ftd-definitions *target-ftd*)))
   (defun (setf info-foreign-type-definition) (val x)
@@ -275,7 +277,11 @@
   (find-or-create-foreign-type-class 'root nil))
 
 (defstruct (foreign-type
-	    (:constructor make-foreign-type (&key class bits alignment)))
+	    (:constructor make-foreign-type (&key class bits alignment))
+	    (:print-object
+	     (lambda (s out)
+	       (print-unreadable-object (s out :type t :identity t)
+		 (prin1 (unparse-foreign-type s) out)))))
   (class 'root :type symbol)
   (bits nil :type (or null unsigned-byte))
   (alignment (guess-alignment bits) :type (or null unsigned-byte)))
@@ -283,9 +289,7 @@
 (defmethod make-load-form ((s foreign-type) &optional env)
   (make-load-form-saving-slots s :environment env))
 
-(defmethod print-object ((s foreign-type) out)
-  (print-unreadable-object (s out :type t :identity t)
-    (prin1 (unparse-foreign-type s) out)))
+
 
 
 
@@ -332,7 +336,9 @@
           (or (info-foreign-type-definition type)
               (error "Definition missing for foreign type ~S?" type)))
       (:unknown
-       (load-foreign-type type)
+       (let* ((loaded (load-foreign-type type)))
+	 (if loaded
+	   (setq type loaded)))
        (or (info-foreign-type-definition type)
            (error "Unknown foreign type: ~S" type))))))
 
@@ -590,12 +596,18 @@
 (defvar *unsigned-integer-types*
   (let* ((a (make-array 33)))
     (dotimes (i 33 a)
-      (setf (svref a i) (make-foreign-integer-type :signed nil :bits i)))))
+      (setf (svref a i) (make-foreign-integer-type :signed nil
+						   :bits i
+						   :alignment
+						   (if (logtest 7 i) 1 i))))))
 
 (defvar *signed-integer-types*
   (let* ((a (make-array 33)))
     (dotimes (i 33 a)
-      (setf (svref a i) (make-foreign-integer-type :signed t :bits i)))))
+      (setf (svref a i) (make-foreign-integer-type :signed t
+						   :bits i
+						   :alignment
+ 						   (if (logtest 7 i) 1 i))))))
          
 (def-foreign-type-translator signed (&optional (bits 32))
   (if (<= bits 32)
@@ -653,7 +665,7 @@
 	    (16 `(%get-unsigned-word ,sap (/ ,offset 8)))
 	    (32 `(%get-unsigned-long ,sap (/ ,offset 8)))
 	    (64 `(%%get-unsigned-longlong ,sap (/ ,offset 8)))
-	    (t  `(%get-bitfield ,sap ,offset))))))
+	    (t  `(%get-bitfield ,sap ,offset ,(foreign-integer-type-bits type)))))))
     (or ref-form
 	(error "Cannot extract ~D bit integers."
 	       (foreign-integer-type-bits type)))))
@@ -894,19 +906,21 @@
 
 ;;;; The RECORD type.
 
-(defstruct foreign-record-field
+(defstruct (foreign-record-field
+	     (:print-object
+	      (lambda (field stream)
+		(print-unreadable-object (field stream :type t)
+		  (funcall (formatter "~S ~S~@[:~D~]")
+			   stream
+			   (foreign-record-field-type field)
+			   (foreign-record-field-name field)
+			   (foreign-record-field-bits field))))))
   (name () :type symbol)
   (type () :type foreign-type)
   (bits nil :type (or unsigned-byte null))
   (offset 0 :type unsigned-byte))
 
-(defmethod print-object ((field foreign-record-field) stream)
-  (print-unreadable-object (field stream :type t)
-    (funcall (formatter "~S ~S~@[:~D~]")
-	     stream
-	     (foreign-record-field-type field)
-	     (foreign-record-field-name field)
-	     (foreign-record-field-bits field))))
+
 
 (defmethod make-load-form ((f foreign-record-field) &optional env)
   (make-load-form-saving-slots f :environment env))
@@ -914,7 +928,11 @@
 (def-foreign-type-class (record :include mem-block)
   (kind :struct :type (member :struct :union))
   (name nil :type (or symbol null))
-  (fields nil :type list))
+  (fields nil :type list)
+  ;; For, e.g., records defined with #pragma options align=mac68k
+  ;; in effect.  When non-nil, this specifies the maximum alignment
+  ;; of record fields and the overall alignment of the record.
+  (alt-align nil :type (or unsigned-byte null)))
 
 (def-foreign-type-translator struct (name &rest fields)
   (parse-foreign-record-type :struct name fields))
@@ -950,16 +968,20 @@
 	   (type list fields))
   (let ((total-bits 0)
 	(overall-alignment 1)
-	(parsed-fields nil))
+	(parsed-fields nil)
+	(alt-alignment (foreign-record-type-alt-align result)))
     (dolist (field fields)
       (destructuring-bind (var type &optional bits) field
 	(declare (ignore bits))
 	(let* ((field-type (parse-foreign-type type))
 	       (bits (ensure-foreign-type-bits field-type))
-	       (alignment (foreign-type-alignment field-type))
+	       (natural-alignment (foreign-type-alignment field-type))
+	       (alignment (if alt-alignment
+			    (min natural-alignment alt-alignment)
+			    natural-alignment))
 	       (parsed-field
 		(make-foreign-record-field :type field-type
-					 :name var)))
+					   :name var)))
 	  (push parsed-field parsed-fields)
 	  (when (null bits)
 	    (error "Unknown size: ~S"
@@ -977,9 +999,10 @@
 	     (setf total-bits (max total-bits bits)))))))
     (let ((new (nreverse parsed-fields)))
       (setf (foreign-record-type-fields result) new))
-    (setf (foreign-record-type-alignment result) overall-alignment)
+    (setf (foreign-record-type-alignment result) (or alt-alignment
+						     overall-alignment))
     (setf (foreign-record-type-bits result)
-	  (align-offset total-bits overall-alignment))))
+	  (align-offset total-bits (or alt-alignment overall-alignment)))))
 
 (def-foreign-type-method (record :unparse) (type)
   `(,(case (foreign-record-type-kind type)
@@ -1120,7 +1143,14 @@
 ;;;
 ;;; Information describing a heap-allocated foreign.
 ;;; 
-(defstruct heap-foreign-info
+(defstruct (heap-foreign-info
+	     (:print-object
+	      (lambda (info stream)
+		(print-unreadable-object (info stream :type t)
+		  (funcall (formatter "~S ~S")
+			   stream
+			   (heap-foreign-info-sap-form info)
+			   (unparse-foreign-type (heap-foreign-info-type info)))))))
   ;; The type of this foreign.
   (type () :type foreign-type)
   ;; The form to evaluate to produce the SAP pointing to where in the heap
@@ -1129,12 +1159,7 @@
 
 
 ;;;
-(defmethod print-object ((info heap-foreign-info) stream)
-  (print-unreadable-object (info stream :type t)
-    (funcall (formatter "~S ~S")
-	     stream
-	     (heap-foreign-info-sap-form info)
-	     (unparse-foreign-type (heap-foreign-info-type info)))))
+
 
 (defmethod make-load-form ((h heap-foreign-info) &optional env)
   (make-load-form-saving-slots h :environment env))
@@ -1146,7 +1171,14 @@
 ;;; that local foreign is represented.
 ;;; 
 (defstruct (local-foreign-info
-	    (:constructor make-local-foreign-info (&key type force-to-memory-p)))
+	     (:constructor make-local-foreign-info (&key type force-to-memory-p))
+	     (:print-object
+	      (lambda (info stream)
+		(print-unreadable-object (info stream :type t)
+		  (funcall (formatter "~:[~;(forced to stack) ~]~S")
+			   stream
+			   (local-foreign-info-force-to-memory-p info)
+			   (unparse-foreign-type (local-foreign-info-type info)))))))
   ;; The type of the local foreign.
   (type () :type foreign-type)
   ;; T if this local foreign must be forced into memory.  Using the ADDR macro
@@ -1154,12 +1186,7 @@
   (force-to-memory-p (or (foreign-array-type-p type) (foreign-record-type-p type))
 		     :type (member t nil)))
 ;;;
-(defmethod print-object ((info local-foreign-info) stream)
-  (print-unreadable-object (info stream :type t)
-    (funcall (formatter "~:[~;(forced to stack) ~]~S")
-	     stream
-	     (local-foreign-info-force-to-memory-p info)
-	     (unparse-foreign-type (local-foreign-info-type info)))))
+
 
 (defmethod make-load-form ((l local-foreign-info) &optional env)
   (make-load-form-saving-slots l :environment env))
@@ -1235,8 +1262,6 @@
 				      (foreign-type-alignment element-type))
 			(reduce #'* dims))))))))
 
-
-
 (defun %foreign-type-or-record (type)
   (if (consp type)
     (%parse-foreign-type type)
@@ -1316,16 +1341,24 @@
 (def-foreign-type signed-char (signed 8))
 (def-foreign-type signed-byte (signed 8))
 (def-foreign-type short (signed 16))
+(def-foreign-type signed-halfword short)
 (def-foreign-type int (signed 32))
+(def-foreign-type signed-fullword int)
 (def-foreign-type long (integer 32))
 (def-foreign-type signed-short (signed 16))
 (def-foreign-type signed-int (signed 32))
 (def-foreign-type signed-long (signed 32))
-(def-foreign-type char (unsigned 8))
+(def-foreign-type signed-doubleword (signed 64))
+(def-foreign-type char #+linuxppc-target (unsigned 8)
+		  #+darwinppc-target (signed 8))
+(def-foreign-type unsigned-char (unsigned 8))
 (def-foreign-type unsigned-byte (unsigned 8))
 (def-foreign-type unsigned-short (unsigned 16))
+(def-foreign-type unsigned-halfword unsigned-short)
 (def-foreign-type unsigned-int (unsigned 32))
+(def-foreign-type unsigned-fullword unsigned-int)
 (def-foreign-type unsigned-long (unsigned 32))
+(def-foreign-type unsigned-doubleword (unsigned 64))
 
 (def-foreign-type float single-float)
 (def-foreign-type double double-float)
@@ -1333,6 +1366,7 @@
   (make-foreign-type :class 'root :bits 0 :alignment 0))
 
 (def-foreign-type void (root))
+(def-foreign-type address (* :void))
 
 (defmacro external (name)
   `(load-eep ,name))
@@ -1341,6 +1375,21 @@
   `(ff-call (%reference-external-entry-point
 	     (load-time-value (external ,name))) ,@args))
 
+(defmacro ff-call (entry &rest args)
+  (arch::collect ((representation nil))
+    (do* ((a args (cddr a)))
+	 ((null (cdr a))
+	  (if (null a) (representation :void)
+	    (let* ((rettype (car a)))
+	      (representation (foreign-type-to-representation-type
+			       rettype)))))
+      (let* ((spec (car a))
+	     (val (cadr a)))
+	(representation (foreign-type-to-representation-type spec))
+	(representation val)))
+    `(%ff-call ,entry ,@(representation))))
+	
+	  
 (make-built-in-class 'external-entry-point *istruct-class*)
 
 (defmethod make-load-form ((eep external-entry-point) &optional env)
@@ -1355,7 +1404,8 @@
       (if addr
 	(format out " (#x~8,'0x) " (ash addr 2))
 	(format out " {unresolved} "))
-      (when container
+      (when (and container (or (not (typep container 'macptr))
+				    (not (%null-ptr-p container))))
 	(format out "~a" (shlib.soname container))))))
 
 (make-built-in-class 'shlib *istruct-class*)
@@ -1370,11 +1420,17 @@
     (%setf-macptr p (#_dlerror))
     (unless (%null-ptr-p p) (%get-cstring p))))
 
-(defstruct (external-function-definition (:conc-name "EFD-"))
+(defstruct (external-function-definition (:conc-name "EFD-")
+                                         (:constructor
+                                          make-external-function-definition
+                                          (&key entry-name arg-specs
+                                                result-spec
+                                                (min-args (length arg-specs))))
+                                         )
   (entry-name "" :type string)
   (arg-specs () :type list)
   (result-spec nil :type symbol)
-  (min-args (length arg-specs) :type fixnum))
+  (min-args 0 :type fixnum))
 
 
 (defun %external-call-expander (whole env)
@@ -1454,3 +1510,52 @@
       ',name)))
 
 
+#+darwinppc-target
+(defun open-dylib (name)
+  (with-cstrs ((name name))
+    (#_NSAddImage name (logior #$NSADDIMAGE_OPTION_RETURN_ON_ERROR 
+			       #$NSADDIMAGE_OPTION_WITH_SEARCHING))))
+
+(defparameter *foreign-representation-type-keywords*
+  `(:signed-doubleword :signed-fullword :signed-halfword :signed-byte
+    :unsigned-doubleword :unsigned-fullword :unsigned-halfword :unsigned-byte
+    :address
+    :single-float :double-float
+    :void))
+
+(defun foreign-type-to-representation-type (f)
+  (if (or (member f *foreign-representation-type-keywords*)
+	  (typep f 'unsigned-byte))
+    f
+    (let* ((ftype (parse-foreign-type f)))
+      (or
+       (and (eq (foreign-type-class ftype) 'root) :void)	 
+       (typecase ftype
+	 (foreign-pointer-type :address)
+	 (foreign-double-float-type :double-float)
+	 (foreign-single-float-type :single-float)
+	 (foreign-integer-type
+	  (let* ((signed (foreign-integer-type-signed ftype))
+		 (bits (foreign-integer-type-bits ftype)))
+	    (if signed
+	      (if (<= bits 8)
+		:signed-byte
+		(if (<= bits 16)
+		  :signed-halfword
+		  (if (<= bits 32)
+		    :signed-fullword
+		    (if (<= bits 64)
+		      :signed-doubleword))))
+	      (if (<= bits 8)
+		:unsigned-byte
+		(if (<= bits 16)
+		  :unsigned-halfword
+		  (if (<= bits 32)
+		    :unsigned-fullword
+		    (if (<= bits 64)
+		      :unsigned-doubleword)))))))
+	 (foreign-record-type
+	  (let* ((bits (ensure-foreign-type-bits ftype)))
+	    (ceiling bits 32))))
+       (error "can't determine representation keyword for ~s" f)))))
+  
\ No newline at end of file
