@@ -1,22 +1,18 @@
 ;;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;;; Functions to implement FORMAT.
@@ -1604,7 +1600,7 @@
             (not (whitespacep (schar s i))))
         (setq *format-index* (1- i)))))
 
-(defformat  #\newline format-newline (stream colon atsign &rest parms)
+(defun format-newline (stream colon atsign &rest parms)
   (declare (dynamic-extent parms))
   (when parms
     (format-error "Parameters not allowed"))
@@ -1612,6 +1608,12 @@
          (when atsign (format-error "~:@<newline> is undefined")))
         (atsign (terpri stream) (format-eat-whitespace))
         (t (format-eat-whitespace))))
+  
+(defformat  #\newline format-newline (stream colon atsign &rest parms)
+  (apply #'format-newline stream colon atsign parms))
+
+(defformat #\return format-newline (stream colon atsign &rest parms)
+  (apply #'format-newline stream colon atsign parms))
 
 ;;; Indirection  ~?
 
