@@ -1,22 +1,20 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
+
+(in-package "CCL")
 
 (eval-when (eval compile)
   (require 'backquote)
@@ -203,8 +201,8 @@
 
 (defun ldiff (list object) 
   "Returns a new list, whose elements are those of List that appear before
-   Sublist.  If Sublist is not a tail of List, a copy of List is returned." 
-  (do* ((list list (cdr list)) 
+   Sublist.  If Sublist is not a tail of List, a copy of List is returned."
+  (do* ((list (require-type list 'list) (cdr list)) 
         (result (cons nil nil))
         (splice result))
        ((atom list) 
@@ -215,7 +213,7 @@
 	     (cons splice result))
     (if (eql list object) 
       (return (cdr result)) 
-      (setq splice (cdr (rplacd splice (list (car list)))))))) 
+      (setq splice (cdr (rplacd splice (list (car list))))))))
 
 
 ;;; Functions to alter list structure
@@ -240,8 +238,7 @@
 
 (eval-when (eval compile #-bccl load)
  (defmacro with-set-keys (funcall)
-   `(cond ((and testp notp) (test-not-error test test-not))
-          (notp ,(append funcall '(:key key :test-not test-not)))
+   `(cond (notp ,(append funcall '(:key key :test-not test-not)))
           (t ,(append funcall '(:key key :test test)))))
 
 ;;; Works with the above keylist.  We do three clauses so that if only test-not
@@ -255,39 +252,44 @@
 	    (not (funcall test-not (funcall key ,elt1) (funcall key ,elt2))))
        (eql (funcall key ,elt1) (funcall key ,elt2))))
 
-(defmacro satisfies-the-test (item elt)
-  `(or (and testp
-            (funcall test ,item (funcall key ,elt)))
-       (and notp
-            (not (funcall test-not ,item (funcall key ,elt))))
-       (funcall test ,item (funcall key ,elt))))
+
 
 )
 ;;; Substitution of expressions
 
 ;subst that doesn't call labels
-(defun subst (new old tree &key (key (function identity))
-		           (test (function eql) testp) (test-not nil notp))   
+(defun subst (new old tree &key key
+		           (test #'eql testp) (test-not nil notp))   
   "Substitutes new for subtrees matching old."
   (if (and testp notp)
     (test-not-error test test-not))
-  (subst-aux new old tree key test testp test-not notp))
+  (subst-aux new old tree key test test-not))
 
-(defun subst-aux (new old subtree key test testp test-not notp)
-  (cond ((satisfies-the-test old subtree) new)
-        ((atom subtree) subtree)
-        (t (let ((car (subst-aux new old (car subtree)
-                                 key test testp test-not notp))
-                 (cdr (subst-aux new old (cdr subtree)
-                                 key test testp test-not notp)))
-             (if (and (eq car (car subtree))
-                      (eq cdr (cdr subtree)))
-               subtree
-               (cons car cdr))))))
+(defun subst-aux (new old subtree key test test-not)
+  (flet ((satisfies-the-test (item elt)
+           (let* ((val (if key (funcall key elt) elt)))
+             (if test-not
+               (not (funcall test-not item val))
+               (funcall test item val)))))
+    (declare (inline satisfies-the-test))
+    (cond ((satisfies-the-test old subtree) new)
+          ((atom subtree) subtree)
+          (t (let ((car (subst-aux new old (car subtree)
+                                   key test test-not ))
+                   (cdr (subst-aux new old (cdr subtree)
+                                   key test test-not)))
+               (if (and (eq car (car subtree))
+                        (eq cdr (cdr subtree)))
+                 subtree
+                 (cons car cdr)))))))
 
-;subst-if without a call to labels
-(defun subst-if (new test tree &key (key #'identity))
+;;;subst-if without a call to labels
+;;; I've always wondered how those calls to a special operator
+;;; should best be avoided.  Clearly, the answer involves
+;;; lots of recursion.
+(defun subst-if (new test tree &key key)
   "Substitutes new for subtrees for which test is true."
+  (unless key (setq key #'identity))
   (cond ((funcall test (funcall key tree)) new)
         ((atom tree) tree)
         (t (let ((car (subst-if new test (car tree) :key key))
@@ -298,9 +300,10 @@
                (cons car cdr))))))
 
 ;subst-if-not without a call to labels
-(defun subst-if-not (new test tree &key (key #'identity))
+(defun subst-if-not (new test tree &key key)
   "Substitutes new for subtrees for which test is false."
   "replace with above def when labels works."
+  (unless key (setq key #'identity))
   (cond ((not (funcall test (funcall key tree))) new)
         ((atom tree) tree)
         (t (let ((car (subst-if-not new test (car tree) :key key))
@@ -310,32 +313,39 @@
                tree
                (cons car cdr))))))
 
-(defun nsubst (new old tree &key (key #'identity)
+(defun nsubst (new old tree &key key
                    (test #'eql testp) (test-not nil notp))
   "Substitutes new for subtrees matching old."
   "replace with above def when labels works"
   (if (and testp notp)
     (test-not-error test test-not))
-  (nsubst-aux new old tree key test testp test-not notp))
+  (nsubst-aux new old tree (or key #'identity) test test-not))
 
-(defun nsubst-aux (new old subtree key test testp test-not notp)   
-  (cond ((satisfies-the-test old subtree) new)
-        ((atom subtree) subtree)
-        (t (do* ((last nil subtree)
-                 (subtree subtree (cdr subtree)))
-                ((atom subtree)
-                 (if (satisfies-the-test old subtree)
-                   (set-cdr last new)))
-             (if (satisfies-the-test old subtree)
-               (return (set-cdr last new))
-               (set-car subtree 
-                        (nsubst-aux new old (car subtree)
-                                    key test testp test-not notp))))
-           subtree)))
+(defun nsubst-aux (new old subtree key test test-not)
+  (flet ((satisfies-the-test (item elt)
+           (let* ((val (if key (funcall key elt) elt)))
+             (if test-not
+               (not (funcall test-not item val))
+               (funcall test item val)))))
+    (declare (inline satisfies-the-test))
+    (cond ((satisfies-the-test old subtree) new)
+          ((atom subtree) subtree)
+          (t (do* ((last nil subtree)
+                   (subtree subtree (cdr subtree)))
+                  ((atom subtree)
+                   (if (satisfies-the-test old subtree)
+                     (set-cdr last new)))
+               (if (satisfies-the-test old subtree)
+                 (return (set-cdr last new))
+                 (set-car subtree 
+                          (nsubst-aux new old (car subtree)
+                                      key test test-not))))
+             subtree))))
 
-(defun nsubst-if (new test tree &key (key #'identity))
+(defun nsubst-if (new test tree &key key)
   "Substitutes new for subtrees of tree for which test is true."
-  "replace with above def when labels works." 
+  "replace with above def when labels works."
+  (unless key (setq key #'identity))
   (cond ((funcall test (funcall key tree)) new)
         ((atom tree) tree)
         (t (do* ((last nil tree)
@@ -349,9 +359,10 @@
                         (nsubst-if new test (car tree) :key key))))
            tree)))
 
-(defun nsubst-if-not (new test tree &key (key #'identity))
+(defun nsubst-if-not (new test tree &key key)
   "Substitutes new for subtrees of tree for which test is false."
   "Replace with above def when labels works."
+  (unless key (setq key #'identity))
   (cond ((not (funcall test (funcall key tree))) new)
         ((atom tree) tree)
         (t (do* ((last nil tree)
@@ -365,12 +376,12 @@
                         (nsubst-if-not new test (car tree) :key key))))
            tree)))
 
-(defun sublis (alist tree &key (key #'identity)
+(defun sublis (alist tree &key key
                      (test #'eql testp) (test-not nil notp))
   "Substitutes from alist into tree nondestructively."
   (if (and testp notp)
     (test-not-error test test-not))
-  (sublis-aux alist tree key test test-not notp))
+  (sublis-aux alist tree (or key #'identity) test test-not notp))
 
 (defun sublis-aux  (alist subtree key test test-not notp) 
   (let ((assoc (if notp
@@ -394,12 +405,12 @@
        (assoc (funcall key subtree) alist :test test)))
   )
 
-(defun nsublis (alist tree &key (key #'identity)
+(defun nsublis (alist tree &key key
                       (test #'eql testp) (test-not nil notp))
   "Substitutes new for subtrees matching old."
   (if (and testp notp)
     (test-not-error test test-not))
-  (nsublis-aux alist tree key test test-not notp))
+  (nsublis-aux alist tree (or key #'identity) test test-not notp))
 
 (defun nsublis-aux (alist subtree key test test-not notp &optional temp)
   (cond ((setq temp (nsublis-macro))
@@ -420,15 +431,17 @@
 ;;; Functions for using lists as sets
 
 
-(defun member-if (test list &key (key #'identity))
+(defun member-if (test list &key key )
   "Returns tail of list beginning with first element satisfying test(element)"
+  (unless key (setq key #'identity))
   (do ((list list (Cdr list)))
       ((endp list) nil)
     (if (funcall test (funcall key (car list)))
       (return list))))
 
-(defun member-if-not (test list &key (key #'identity))
+(defun member-if-not (test list &key key)
   "Returns tail of list beginning with first element not satisfying test(el)"
+  (unless key (setq key #'identity))
   (do ((list list (cdr list)))
       ((endp list) ())
     (if (not (funcall test (funcall key (car list))))
@@ -441,54 +454,24 @@
       (return t))))
 
 
-#| the below union is from slisp. it does not remove duplicates within
-   one of its args, which is permissible under cl, but obscure with common 
-   practice for using (union foo nil) to remove duplicates.
  
-(defun union (list1 list2  &key (key #'identity)
-                    (test #'eql testp) (test-not nil notp))
+(defun union (list1 list2  &key
+                    key
+                    (test #'eql testp)
+                    (test-not nil notp))
   "Returns the union of List1 and List2."
   (if (and testp notp)
     (test-not-error test test-not))
-  (let ((res list1))
-    (dolist (elt list2)
-      (print-db elt)
-      (if (not (with-set-keys (member (funcall key elt) list1)))
+  (unless key (setq key #'identity))
+  (let ((res list2))
+    (dolist (elt list1)
+      (if (not (with-set-keys (member (funcall key elt) list2)))
         (push elt res)))
     res))
 
-|#
 
-#| Apparently Gold Hill and Franz don't remove-duplicates so
-CCL decided not to either. That way the user doesn't pay the performance
-penalty if they don't want that functionality, but can always call
-remove-duplicates by hand.
-In this algorthm, duplicates are effectively removed from the First list.
-Thus (union '(1 2 1) nil) => (2 1)
-|#
 
-(defun union (list1 list2  &key (key #'identity)
-                    (test #'eql testp) (test-not nil notp) &aux result)
-  "Returns the union of List1 and List2. Elements are passed to 
-  the test in the order in which they appear in the args.
-  Ex: if (union '(1 2) '(3 4) :test #'<), then the test is (< 1 3),
-  not (< 3 1). "
-  ; Note that when using adjoin, it test (< elt list-item) instead of
-  ; (< list-item elt)
-  (if (and testp notp)
-    (test-not-error test test-not))
-  (setq result list2) 
-  (cond (test-not        
-         (dolist (elt list1)
-           (setq result (adjoin elt result :key key :test-not test-not)))
-         ;(remove-duplicates result :key key :test-not test-not)
-         )
-        (t
-         (dolist (elt list1)
-           (setq result (adjoin elt result :key key :test test)))
-         ;(remove-duplicates result :key key :test test)
-         ))
-  result)
+
 
 
 (eval-when (eval compile #-bccl load)
@@ -496,66 +479,45 @@ Thus (union '(1 2 1) nil) => (2 1)
 ;;; Sets the source to the cdr, and "conses" the 1st elt of 
 ;;; source to destination.
 (defmacro steve-splice (source destination)
-  "" "won't work until setf does. called by 
-     nunion, nintersection, nset-difference."
   `(let ((temp ,source))
      (setf ,source (cdr ,source)
            (cdr temp) ,destination
            ,destination temp)))
 )
 
-#| The old spice def which didn't work for (nunion '(3 4) '(1 2) :test #'<)
-(defun nunion (list1 list2 &key (key #'identity)
+(defun nunion (list1 list2 &key key
                      (test #'eql testp) (test-not nil notp))
   (if (and testp notp)
     (test-not-error test test-not))
-  (let ((res list1))
-    (do () ((endp list2))
-      (if (not (with-set-keys (member (funcall key (car list2)) list1)))
-        (steve-splice list2 res)
-        (setq list2 (cdr list2))))
+  (unless key (setq key #'identity))
+  (let ((res list2))
+    (do ()
+        ((endp list1))
+      (if (not (with-set-keys (member (funcall key (car list1)) list2)))
+        (steve-splice list1 res)
+        (setq list1 (cdr list1))))
     res))
-|#
 
-;exactly the same as the UNION def above.
-(defun nunion (list1 list2  &key (key #'identity)
-                    (test #'eql testp) (test-not nil notp) &aux result)
-  "Returns the union of List1 and List2. Elements are passed to 
-  the test in the order in which they appear in the args.
-  Ex: if (union '(1 2) '(3 4) :test #'<), then the test is (< 1 3),
-  not (< 3 1). "
-  ; Note that when using adjoin, it test (< elt list-item) instead of
-  ; (< list-item elt)
-  (if (and testp notp)
-    (test-not-error test test-not))
-  (setq result list2) 
-  (cond (test-not        
-         (dolist (elt list1)
-           (setq result (adjoin elt result :key key :test-not test-not)))
-         ;(remove-duplicates result :key key :test-not test-not)
-         )
-        (t
-         (dolist (elt list1)
-           (setq result (adjoin elt result :key key :test test)))
-         ;(remove-duplicates result :key key :test test)
-         ))
-  result)
 
-(defun intersection (list1 list2  &key (key #'identity)
+
+
+(defun intersection (list1 list2  &key key
                            (test #'eql testp) (test-not nil notp))
-  "Returns the union of List1 and List2."
+  "Returns the intersection of List1 and List2."
   (if (and testp notp)
     (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (let ((res nil))
     (dolist (elt list1)
       (if (with-set-keys (member (funcall key elt) list2))
         (push elt res)))
     res))
 
-(defun nintersection (list1 list2 &key (key #'identity)
+(defun nintersection (list1 list2 &key key
                             (test #'eql testp) (test-not nil notp))
   (if (and testp notp)
     (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (let ((res nil))
     (do () ((endp list1))
       (if (with-set-keys (member (funcall key (car list1)) list2))
@@ -563,21 +525,23 @@ Thus (union '(1 2 1) nil) => (2 1)
         (setq list1 (Cdr list1))))
     res))
 
-(defun set-difference (list1 list2 &key (key #'identity)
+(defun set-difference (list1 list2 &key key
                              (test #'eql testp) (test-not nil notp))
   "Returns a lsit of the elements in LIST1 which are not in LIST2."
   (if (and testp notp)
     (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (let ((res nil))
     (dolist (elt list1)
       (if (not (with-set-keys (member (funcall key elt) list2)))
         (push elt res)))
     res))
 
-(defun nset-difference (list1 list2 &key (key #'identity)
+(defun nset-difference (list1 list2 &key key
                               (test #'eql testp) (test-not nil notp))
   (if (and testp notp)
     (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (let ((res nil))
     (do () ((endp list1))
       (if (not (with-set-keys (member (funcall key (car list1)) list2)))
@@ -601,11 +565,12 @@ Thus (union '(1 2 1) nil) => (2 1)
     result))
 |#
 
-(defun set-exclusive-or (list1 list2 &key (key #'identity)
+(defun set-exclusive-or (list1 list2 &key key
                                (test #'eql testp) (test-not nil notp)
                                &aux result elt1-compare elt2-compare)
   (if (and testp notp)
     (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (dolist (elt1 list1)
     (setq elt1-compare (funcall key elt1))
     (if (if notp
@@ -668,17 +633,21 @@ on the example (nset-exclusive-or (list 1 1) (list 1))
             (t (setq splicey y))))))
 |#
 
-(defun nset-exclusive-or (list1 list2 &key (key #'identity)
+(defun nset-exclusive-or (list1 list2 &key key
                                (test #'eql testp) (test-not nil notp))
    (if (and testp notp)
      (test-not-error test test-not))
+   (unless key (setq key #'identity))
    (if notp
      (set-exclusive-or list1 list2 :key key :test-not test-not)
      (set-exclusive-or list1 list2 :key key :test test)
      ))
  
-(defun subsetp (list1 list2 &key (key #'identity)
+(defun subsetp (list1 list2 &key key
                       (test #'eql testp) (test-not nil notp))
+  (if (and testp notp)
+    (test-not-error test test-not))
+  (unless key (setq key #'identity))
   (dolist (elt list1)
     (unless (with-set-keys (member (funcall key elt) list2))
       (return-from subsetp nil)))
