@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 (eval-when (eval compile)
@@ -26,10 +22,7 @@
 (defun long-site-name   () (or *long-site-name* "unspecified"))
 
 (defun machine-instance ()
-  "fix machine-instance")
-
-
-
+  (%uname 1))
 
 
 (defun machine-type ()
@@ -50,21 +43,10 @@
   (%uname 2))
 
 
-(defvar %documentation
-  (make-hash-table :weak t :size 100 :test 'eq :rehash-threshold .95))
 
-(defun %get-documentation (object doc-id)
-  (cdr (assoc doc-id (gethash object %documentation))))
 
-(defun (setf %get-documentation) (new object doc-id)
-  (let* ((alist (gethash object %documentation))
-	 (pair (assoc object alist)))
-    (if pair
-      (setf (cdr pair) new)
-      (progn
-	(setf (gethash object %documentation)
-	      (cons (cons doc-id new) alist))
-	new))))
+
+
 
 ;;; Yawn.
 
@@ -73,31 +55,31 @@
 (defmethod documentation (thing doc-id)
   (%get-documentation thing doc-id))
 
-(unless (standard-generic-function-p #'(setf documentation))
-  (fmakunbound '(setf documentation)))
+(defun set-documentation (thing doc-id new)
+  (setf (documentation thing doc-id) new))
 
 (defmethod (setf documentation) (new thing doc-id)
-  (setf (%get-documentation thing doc-id) new))
+  (%put-documentation thing doc-id new))
 
 
 (defmethod documentation ((symbol symbol) (doc-type (eql 'function)))
   (let* ((def (fboundp symbol)))	; FBOUNDP returns info about definition
-    (%get-documentation
-     (if (special-operator-p symbol)
-       symbol
-       (or (macro-function symbol) def))
-     doc-type)))
+    (when def
+      (if (special-operator-p symbol)
+        (call-next-method)
+        (%get-documentation (or (macro-function symbol) def) t)))))
 
 (defmethod (setf documentation) ((new t)
 				 (symbol symbol)
 				 (doc-type (eql 'function)))
   (let* ((def (fboundp symbol)))	; FBOUNDP returns info about definition
-    (setf (%get-documentation 
-	   (if (special-operator-p symbol)
-	     symbol
-	     (or (macro-function symbol) def))
-	   doc-type)
-	  new)))
+    (when def
+      (if (special-operator-p symbol)
+        (call-next-method)
+        (%put-documentation (or (macro-function symbol) def)
+                            t
+                            new)))
+    new))
 
 (defmethod documentation ((symbol symbol) (doc-type (eql 'setf)))
   (call-next-method))
@@ -170,18 +152,18 @@
   (call-next-method))
 
 (defmethod documentation ((f function) (doc-type (eql 't)))
-  (documentation f 'function))
+  (call-next-method))
 
 (defmethod (setf documentation) ((new t) (f function) (doc-type (eql 't)))
-  (setf (documentation f 'function) new))
+  (call-next-method))
 
 (defmethod documentation ((f function) (doc-type (eql 'function)))
-  (call-next-method))
+  (documentation f t))
 
 (defmethod (setf documentation) ((new t)
 				 (f function)
 				 (doc-type (eql 'function)))
-  (call-next-method))
+  (setf (documentation f t) new))
 
 (defmethod documentation ((l list) (doc-type (eql 'function)))
   (documentation (setf-function-spec-name l) doc-type))
@@ -261,7 +243,7 @@
 (defun set-function-info (symbol info)
   (let* ((doc-string (if (consp info) (car info) info)))
     (if (and *save-doc-strings* (stringp doc-string))
-      (setf (documentation  symbol 'function) doc-string)))
+      (set-documentation  symbol 'function doc-string)))
   (let* ((cons (assq symbol *nx-globally-inline*))
          (lambda-expression (if (consp info) (cdr info))))
     (if (and (proclaimed-inline-p symbol)
@@ -371,39 +353,6 @@
 
 
 
-(defvar *function-specs* ())
-
-(defun function-spec-p (spec)
-  (or (symbolp spec)
-      (and (consp spec)
-           (symbolp (setq spec (%car spec)))
-           (neq spec 'lambda)
-           (getf *function-specs* spec))))
-
-(defun function-spec-handler (spec)
-  (or (function-spec-p spec)
-      (error "~s isn't a valid function name." spec)))
-
-(defmacro def-function-spec-handler (keyword arglist &body body)
-  `(setf (getf *function-specs* ',keyword)
-         (nfunction ,keyword (lambda ,arglist ,@body))))
-
-;(def-function-spec-handler :property (spec &optional (def nil def-p))
-;  (debind (keyword sym property) spec
-;    (declare (ignore keyword))
-;    (if def-p
-;      (put sym property def)
-;      (get sym property))))
-
-;This is often handled directly in critical sections, so it's not enough
-;to just change this definition...
-(def-function-spec-handler setf (spec &optional (def nil def-p))
-  (debind (keyword sym) spec
-    (declare (ignore keyword))
-    (if def-p
-      (fset spec def)
-      (fboundp (setf-function-name sym)))))
-
 ; ed is a CL fn so it must get loaded before tools, but it
 ; should be able to take advantage of tools if tools are loaded. - what tools?
 
@@ -604,6 +553,9 @@
 	  (format *query-io* "~&Type of ~s is not one of ~{~a~}"
 		  namestring file-types)))))
 
+(defparameter *overwrite-dialog-hook* nil)
 (defun overwrite-dialog (filename prompt)
-  )
+  (if *overwrite-dialog-hook*
+    (funcall *overwrite-dialog-hook* filename prompt)
+    t))
 
