@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;; :lib:nfcomp.lisp - New fasl compiler.
@@ -46,18 +42,18 @@
 
 ;File compiler options.  Not all of these need to be exported/documented, but
 ;they should be in the product just in case we need them for patches....
-(defvar *fasl-save-local-symbols* nil)
+(defvar *fasl-save-local-symbols* t)
 (defvar *fasl-deferred-warnings* nil)
 (defvar *fasl-non-style-warnings-signalled-p* nil)
 (defvar *fasl-warnings-signalled-p* nil)
 (defvar *compile-verbose* nil) ; Might wind up getting called *compile-FILE-verbose*
-(defvar *fasl-save-doc-strings* #-bccl nil #+bccl t)
+(defvar *fasl-save-doc-strings*  t)
 (defvar *fasl-save-definitions* nil)
 (defvar *compile-file-pathname* nil) ; pathname of src arg to COMPILE-FILE
 (defvar *compile-file-truename* nil) ; truename ...
 (defvar *fasl-target* (backend-name *host-backend*))
 (defvar *fasl-backend* *host-backend*)
-
+(defvar *fcomp-external-format* :default)
 
 (defvar *compile-print* nil) ; Might wind up getting called *compile-FILE-print*
 
@@ -99,6 +95,7 @@ Will differ from *compiling-file* during an INCLUDE")
                          (save-local-symbols *fasl-save-local-symbols*)
                          (save-doc-strings *fasl-save-doc-strings*)
                          (save-definitions *fasl-save-definitions*)
+			 (external-format :default)
                          force)
   (let* ((backend *host-backend*))
     (when (and target-p (not (setq backend (find-backend target))))
@@ -107,7 +104,7 @@ Will differ from *compiling-file* during an INCLUDE")
     (loop
 	(restart-case
 	 (return (%compile-file src output-file verbose print load features
-				save-local-symbols save-doc-strings save-definitions force backend))
+				save-local-symbols save-doc-strings save-definitions force backend external-format))
 	 (retry-compile-file ()
 			     :report (lambda (stream) (format stream "Retry compiling ~s" src))
 			     nil)
@@ -117,7 +114,7 @@ Will differ from *compiling-file* during an INCLUDE")
 
 
 (defun %compile-file (src output-file verbose print load features
-                          save-local-symbols save-doc-strings save-definitions force target-backend
+                          save-local-symbols save-doc-strings save-definitions force target-backend external-format
 			  &aux orig-src)
 
   (setq orig-src (merge-pathnames src))
@@ -163,7 +160,8 @@ Will differ from *compiling-file* during an INCLUDE")
 	     (*fasl-backend* target-backend)
 	     (*target-ftd* (backend-target-foreign-type-data target-backend))
              (defenv (new-definition-environment))
-             (lexenv (new-lexical-environment defenv)))
+             (lexenv (new-lexical-environment defenv))
+	     (*fcomp-external-format* external-format))
         (let ((forms nil))
           (let* ((*outstanding-deferred-warnings* (%defer-warnings nil)))
             (rplacd (defenv.type defenv) *outstanding-deferred-warnings*)
@@ -345,12 +343,14 @@ Will differ from *compiling-file* during an INCLUDE")
 ; orig-file is back-translated when from fcomp-file
 ; when from fcomp-include it's included filename merged with *compiling-file*
 ; which is not back translated
-(defun fcomp-read-loop (filename orig-file env processing-mode )
+(defun fcomp-read-loop (filename orig-file env processing-mode)
   (when *compile-verbose*
     (format t "~&;~A ~S..."
             (if (eq filename *compiling-file*) "Compiling" " Including")
             filename))
-  (with-open-file (stream filename :element-type 'base-char)
+  (with-open-file (stream filename
+			  :element-type 'base-char
+			  :external-format *fcomp-external-format*)
     (let* ((old-file (and (neq filename *compiling-file*) *fasl-source-file*))           
            (*fasl-source-file* filename)
            (*fcomp-toplevel-forms* nil)
@@ -636,6 +636,13 @@ Will differ from *compiling-file* during an INCLUDE")
             (defenv.functions definition-env)))
     name))
 
+(defun define-compile-time-symbol-macro (name expansion env)
+  (let* ((definition-env (definition-environment env)))
+    (if definition-env
+      (push (cons name expansion) (defenv.symbol-macros definition-env)))
+    name))
+
+
 (defun fcomp-proclaim-type (type syms)
   (dolist (sym syms)
     (if (symbolp sym)
@@ -861,7 +868,7 @@ Will differ from *compiling-file* during an INCLUDE")
 ;;;;
 ;These should be constants, but it's too much trouble when need to change 'em.
 (defparameter FASL-FILE-ID #xFF00)  ;Overall file format, shouldn't change much
-(defparameter FASL-VERSION #xFF32)  ;Fasl block format.
+(defparameter FASL-VERSION #xFF34)  ;Fasl block format.
 
 (defvar *fasdump-hash*)
 (defvar *fasdump-read-package*)
@@ -1056,10 +1063,9 @@ Will differ from *compiling-file* during an INCLUDE")
         (finished? nil))
     (unwind-protect
       (with-open-file (*fasdump-stream* filename :direction :output
-                                        :element-type 'base-char ; huh
+                                        :element-type '(unsigned-byte 8)
                                         :if-exists :supersede
-                                        :if-does-not-exist :create
-                                        :external-format (if (eq *fasl-target* :m68k) :fasl :pfsl))
+                                        :if-does-not-exist :create)
         (setq opened? t)
         (fasl-set-filepos 0)
         (fasl-out-word 0)             ;Will become the ID word
@@ -1283,7 +1289,13 @@ Will differ from *compiling-file* during an INCLUDE")
     (fasl-out-opcode $fasl-cons cons)
     (progn
       (fasl-out-opcode (if end $fasl-list* $fasl-list) cons)
-      (fasl-out-word cdr-len)))
+      ;; If the length doesn't fit in a 16-bit word, output
+      ;; a word of 0 followed by a 32-bit length
+      (if (> cdr-len #xffff)
+	(progn
+	  (fasl-out-word 0)
+	  (fasl-out-long cdr-len))
+	(fasl-out-word cdr-len))))
   (dotimes (i (the fixnum (1+ cdr-len)))
     (fasl-dump-form (%car cons))
     (setq cons (%cdr cons)))
@@ -1365,7 +1377,7 @@ Will differ from *compiling-file* during an INCLUDE")
   (fasl-out-byte word))
 
 (defun fasl-out-byte (byte)
-  (tyo (%code-char (%ilogand2 byte #xFF)) *fasdump-stream*))
+  (write-byte (%ilogand2 byte #xFF) *fasdump-stream*))
 
 (defun fasl-filepos ()
   (file-position *fasdump-stream*))
@@ -1375,6 +1387,111 @@ Will differ from *compiling-file* during an INCLUDE")
   #-bccl (unless (eq (file-position *fasdump-stream*) pos)
            (error "Unable to set file position to ~S" pos)))
 
+;;; Concatenate fasl files.
+
+;;; Format of a fasl file as expected by the fasloader.
+;;;
+;;; #xFF00         2 bytes - File version
+;;; Block Count    2 bytes - Number of blocks in the file
+;;; addr[0]        4 bytes - address of 0th block
+;;; length[0]      4 bytes - length of 0th block
+;;; addr[1]        4 bytes - address of 1st block
+;;; length[1]      4 bytes - length of 1st block
+;;; ...
+;;; addr[n-1]      4 bytes
+;;; length[n-1]    4 bytes
+;;; length[0] + length[1] + ... + length [n-1] bytes of data
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; (fasl-concatenate out-file fasl-files &key :if-exists)
+;;
+;; out-file     name of file in which to store the concatenation
+;; fasl-files   list of names of fasl files to concatenate
+;; if-exists    as for OPEN, defaults to :error
+;;
+;; function result: pathname to the output file.
+;; It works to use the output of one invocation of fasl-concatenate
+;; as an input of another invocation.
+;;
+(defun fasl-concatenate (out-file fasl-files &key (if-exists :error))
+  (%fasl-concatenate out-file fasl-files if-exists (pathname-type *.fasl-pathname*)))
+
+(defun %fasl-concatenate (out-file fasl-files if-exists file-ext)
+  (let ((count 0)
+        (created? nil)
+        (finished? nil)
+	(ext-pathname (make-pathname :type file-ext)))
+    (declare (fixnum count))
+    (flet ((fasl-read-halfword (f)
+	     (dpb (read-byte f) (byte 8 8) (read-byte f)))
+	   (fasl-write-halfword (h f)
+	     (write-byte (ldb (byte 8 8) h) f)
+	     (write-byte (ldb (byte 8 0) h) f)
+	     h))
+      (flet ((fasl-read-fullword (f)
+	       (dpb (fasl-read-halfword f) (byte 16 16) (fasl-read-halfword f)))
+	     (fasl-write-fullword (w f)
+	       (fasl-write-halfword (ldb (byte 16 16) w) f)
+	       (fasl-write-halfword (ldb (byte 16 0) w) f)
+	       w))
+	(dolist (file fasl-files)
+	  (setq file (merge-pathnames file ext-pathname))
+	  (unless (equal (pathname-type file) file-ext)
+	    (error "Not a ~A file: ~s" file-ext file))
+	  (with-open-file (instream file :element-type '(unsigned-byte 8))
+	    (unless (eql fasl-file-id (fasl-read-halfword instream))
+	      (error "Bad ~A file ID in ~s" file-ext file))
+	    (incf count (fasl-read-halfword instream))))
+	(unwind-protect
+	     (with-open-file (outstream
+			      (setq out-file (merge-pathnames out-file ext-pathname))
+			      :element-type '(unsigned-byte 8)
+			      :direction :output
+			      :if-does-not-exist :create
+			      :if-exists if-exists)
+	       (setq created? t)
+	       (let ((addr-address 4)
+		     (data-address (+ 4 (* count 8))))
+		 (fasl-write-halfword 0 outstream) ;  will be $fasl-id
+		 (fasl-write-halfword count outstream)
+		 (dotimes (i (* 2 count))
+		   (fasl-write-fullword 0 outstream)) ; for addresses/lengths
+		 (dolist (file fasl-files)
+		   (with-open-file (instream (merge-pathnames file ext-pathname)
+					     :element-type '(unsigned-byte 8))
+		     (fasl-read-halfword instream) ; skip ID
+		     (let* ((fasl-count (fasl-read-halfword instream))
+			    (addrs (make-array fasl-count))
+			    (sizes (make-array fasl-count))
+			    addr0)
+		       (declare (fixnum fasl-count)
+				(dynamic-extent addrs sizes))
+		       (dotimes (i fasl-count)
+			 (setf (svref addrs i) (fasl-read-fullword instream)
+			       (svref sizes i) (fasl-read-fullword instream)))
+		       (setq addr0 (svref addrs 0))
+		       (file-position outstream addr-address)
+		       (dotimes (i fasl-count)
+			 (fasl-write-fullword
+			  (+ data-address (- (svref addrs i) addr0))
+			  outstream)
+			 (fasl-write-fullword (svref sizes i) outstream)
+			 (incf addr-address 8))
+		       (file-position outstream data-address)
+		       (dotimes (i fasl-count)
+			 (file-position instream (svref addrs i))
+			 (let ((fasl-length (svref sizes i)))
+			   (dotimes (j fasl-length)
+			     (write-byte (read-byte instream) outstream))
+			   (incf data-address fasl-length))))))
+		 (stream-length outstream data-address)
+		 (file-position outstream 0)
+		 (fasl-write-halfword fasl-file-id outstream)
+		 (setq finished? t)))
+	  (when (and created? (not finished?))
+	    (delete-file out-file))))
+      out-file)))
 
 (provide 'nfcomp)
 
