@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;; Lib;numbers.lisp - Lisp arithmetic code.
@@ -41,10 +37,10 @@
                     ((and first (eq c #\+)))
                     ((and first (eq c #\-))
                      (setq sign -1))
-                    ((memq c '(#\s #\S))
+                    ((memq c '(#\s #\f #\S #\F))
                      (setq type 'short-float)
                      (return integer))
-                    ((memq c '(#\d #\f #\l  #\D #\F #\L))
+                    ((memq c '(#\d #\l  #\D  #\L))
                      (setq type 'double-float)
                      (return integer))
                     ((memq c '(#\e #\E))
@@ -116,7 +112,7 @@
 
 (defun float-5-to-e (e)
   (if (> e 22)  ; shouldnt happen
-    (expt 5.0 e)
+    (expt 5.0d0 e)
     (svref float-powers-of-5 e)))
 
 (defparameter a-short-float nil)
@@ -649,7 +645,7 @@
   (setq float-powers-of-5 (make-array 23))
   (let ((array float-powers-of-5))
     (dotimes (i 23)
-      (setf (svref array i)  (float (expt 5 i)))))
+      (setf (svref array i)  (float (expt 5 i) 0.0d0))))
   (setq integer-powers-of-5 (make-array (+ 12 (floor 324 12))))
   (let ((array integer-powers-of-5))
     (dotimes (i 12)
