@@ -2,22 +2,18 @@
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
 ;;;   Portions copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;;pathnames.lisp Pathnames for Coral Common LISP
@@ -92,7 +88,7 @@
       (declare (fixnum res))
       (if (zerop res)
         (values t nil)
-        (values nil res)))))
+        (values nil (%get-errno))))))
 
 (defun rename-file (file new-name &key (if-exists :error))
   "Rename File to have the specified New-Name.  If file is a stream open to a
@@ -112,12 +108,31 @@
 	  (error "Failed to rename ~A to ~A: ~A"
 		 original new-name error))
 	(when (streamp file)
-	  (setf (file-stream-filename file) new-namestring))
+	  (setf (file-stream-filename file)
+		(namestring (native-to-pathname new-namestring))))
 	(values new-name original (truename new-name))))))
 
 
-(defun copy-file (old-path new-path &key (if-exists :error))
-  (break "Copy-file ? ~s ~s ~s" old-path new-path if-exists))
+(defun copy-file (source-path dest-path &key (if-exists :error)
+			      (preserve-attributes nil))
+  (let* ((original (truename source-path))
+	 (original-namestring (native-translated-namestring original))
+	 (new-name (merge-pathnames dest-path original))
+	 (new-namestring (native-translated-namestring new-name))
+	 (flags (if preserve-attributes "-pf" "-f")))
+    (unless new-namestring
+      (error "~S can't be created." new-name))
+    (unless (and (probe-file new-name)
+		 (not (if-exists if-exists new-name)))
+      (let* ((proc (run-program "/bin/cp"
+				`(,flags ,original-namestring ,new-namestring)
+				:wait t))
+	     (exit-code (external-process-%exit-code proc)))
+	(unless (zerop exit-code)
+	  (error "Error copying ~s to ~s: ~a"
+		 source-path dest-path (%strerror exit-code)))
+	(values new-name original (truename new-name))))))
+
 
 (defun lock-file (path)
   (break "lock-file ? ~s" path))
@@ -128,7 +143,12 @@
 (defun create-directory (path &key (mode #o777))
   (let* ((pathname (translate-logical-pathname (merge-pathnames path)))
 	 (created-p nil)
-	 (parent-dirs (pathname-directory pathname))
+	 (parent-dirs (let* ((pd (pathname-directory pathname)))
+			(if (eq (car pd) :relative)
+			  (pathname-directory (merge-pathnames
+					       pathname
+					       (mac-default-directory)))
+			  pd)))
 	 (nparents (length parent-dirs)))
     (when (wild-pathname-p pathname)
       (error 'file-error :error-type "Inappropriate use of wild pathname ~s"
@@ -136,16 +156,18 @@
     (do* ((i 1 (1+ i)))
 	 ((> i nparents) (values pathname created-p))
       (declare (fixnum i))
-      (let* ((parent (make-pathname :host (pathname-host pathname)
-				    :device (pathname-device pathname)
-				    :directory
-				    (subseq parent-dirs 0 i)))
+      (let* ((parent (make-pathname
+		      :name :unspecific
+		      :type :unspecific
+		      :host (pathname-host pathname)
+		      :device (pathname-device pathname)
+		      :directory (subseq parent-dirs 0 i)))
 	     (parent-name (native-translated-namestring parent))
 	     (parent-kind (%unix-file-kind parent-name)))
 
 	(if parent-kind
 	  (unless (eq parent-kind :directory)
-	    (error 'file-error
+	    (error 'simple-file-error
 		   :error-type "Can't create directory ~s, since file ~a exists and is not a directory"
 		   :pathname pathname
 		   :format-arguments (list parent-name)))
@@ -155,19 +177,26 @@
 	      (signal-file-error result parent-name)
 	      (setq created-p t))))))))
 
+
 (defun ensure-directories-exist (pathspec &key verbose (mode #o777))
   "Tests whether the directories containing the specified file
   actually exist, and attempts to create them if they do not.
   Portable programs should avoid using the :MODE keyword argument."
-  (let* ((pathname (translate-logical-pathname (merge-pathnames pathspec)))
+  (let* ((pathname (make-directory-pathname :directory (pathname-directory (translate-logical-pathname (merge-pathnames pathspec)))))
 	 (created-p nil))
     (when (wild-pathname-p pathname)
       (error 'file-error
 	     :error-type "Inappropriate use of wild pathname ~s"
 	     :pathname pathname))
     (let ((dir (pathname-directory pathname)))
+      (if (eq (car dir) :relative)
+	(setq dir (pathname-directory (merge-pathnames
+				       pathname
+				       (mac-default-directory)))))
       (loop for i from 1 upto (length dir)
 	    do (let ((newpath (make-pathname
+			       :name :unspecific
+			       :type :unspecific
 			       :host (pathname-host pathname)
 			       :device (pathname-device pathname)
 			       :directory (subseq dir 0 i))))
@@ -177,10 +206,10 @@
 		       (format *standard-output* "~&Creating directory: ~A~%"
 			       namestring))
 		     (%mkdir namestring mode)
-		     (unless (probe-file namestring)
+		     (unless (probe-file newpath)
 		       (error 'file-error
 			      :pathname namestring
-			      :error-type "Can't create directory ~A."))
+			      :error-type "Can't create directory ~S."))
 		     (setf created-p t)))))
       (values pathname created-p))))
 
@@ -262,6 +291,10 @@
 	 (path (full-pathname (merge-pathnames path) :no-error nil))
 	 (dir (directory-namestring path)))
     (declare (dynamic-extent keys))
+    (if (null (pathname-directory path))
+      (setq dir (directory-namestring (setq path
+					    (merge-pathnames path
+							     (mac-default-directory))))))
     (assert (eq (car (pathname-directory path)) :absolute) ()
 	    "full-pathname returned relative path ~s??" path)
     ;; return sorted in alphabetical order, target-Xload-level-0 depends
@@ -338,7 +371,7 @@
 		      (multiple-value-bind (name type) (%std-name-and-type sub)
 			(%cons-pathname (or dir-list (setq dir-list (reverse so-far))) name type)))))
 	    (when (and ans (or (null test) (funcall test ans)))
-	      (push (if follow-links (truename ans) ans) result))))))
+	      (push (if follow-links (or (probe-file ans) ans) ans) result))))))
     result))
 
 ; now for samson:**:*c*:**: we get samson:ccl:crap:barf: twice because
@@ -436,219 +469,4 @@
 	       ((null type-pat) T)
 	       (t (%path-pstr*= type-pat (or type "")))))))
 
-#|
-(defmacro enumerate-matches ((var pathname &optional result
-				  &key (verify-existance t))
-			     &body body)
-  (let ((body-name (gensym)))
-    `(block nil
-       (flet ((,body-name (,var)
-		,@body))
-	 (%enumerate-matches (pathname ,pathname)
-			     ,verify-existance
-			     #',body-name)
-	 ,result))))
-
-(defun %enumerate-matches (pathname verify-existance function)
-  (when (pathname-type pathname)
-    (unless (pathname-name pathname)
-      (error "Cannot supply a type without a name:~%  ~S" pathname)))
-  (when (and (integerp (pathname-version pathname))
-	     (member (pathname-type pathname) '(nil :unspecific)))
-    (error "Cannot supply a version without a type:~%  ~S" pathname))
-  (let ((directory (pathname-directory pathname)))
-    (if directory
-	(ecase (car directory)
-	  (:absolute
-	   (%enumerate-directories "/" (cdr directory) pathname
-				   verify-existance function))
-	  (:relative
-	   (%enumerate-directories "" (cdr directory) pathname
-				   verify-existance function)))
-	(%enumerate-files "" pathname verify-existance function))))
-
-(defun %enumerate-directories (head tail pathname verify-existance function)
-  (declare (simple-string head))
-  (if tail
-    (let ((piece (car tail)))
-      (etypecase piece
-        (simple-string
-         (%enumerate-directories (concatenate 'string head piece "/")
-                                 (cdr tail) pathname verify-existance
-                                 function))
-        ((member :wild)
-         (let ((dir (%open-dir head)))
-           (when dir
-             (unwind-protect
-                  (loop
-                      (let ((name (%read-dir dir)))
-                        (cond ((null name)
-                               (return))
-                              ((string= name "."))
-                              ((string= name ".."))
-                              (t
-                               (let ((subdir (concatenate 'string
-                                                          head name)))
-                                 ;; see below for health warning
-                                 (when (eq (%unix-file-kind subdir t)
-                                           :directory)
-                                   (%enumerate-directories
-                                    (concatenate 'string subdir "/")
-                                    (cdr tail) pathname verify-existance
-                                    function)))))))
-               (close-dir dir)))))	   
-        ((member :wild-inferiors)
-         (let ((dir (%open-dir head)))
-           (when dir
-             (unwind-protect
-                  (loop
-                      (let ((name (%read-dir dir)))
-                        (cond ((null name)
-                               (return))
-                              ((string= name "."))
-                              ((string= name ".."))
-                              (t        ; always match 
-                               ;; warning - this depends
-                               ;; on lstat behaving in the same way on
-                               ;; different platforms. Basically,
-                               ;; lstat("foo") -> :link, lstat("foo/")
-                               ;; -> directory on linux 2.2.
-                               (let ((subdir (concatenate 'string
-                                                          head name)))
-                                 (when (eq (%unix-file-kind subdir t)
-                                           :directory)
-                                   (%enumerate-directories
-                                    (concatenate 'string subdir "/")
-                                    tail pathname verify-existance
-                                    function)))))))
-               (close-dir dir))
-             (%enumerate-files head pathname verify-existance	function))))
-        (pattern
-         (let ((dir (%open-dir head)))
-           (when dir
-             (unwind-protect
-                  (loop
-                      (let ((name (%read-dir dir)))
-                        (cond ((null name)
-                               (return))
-                              ((string= name "."))
-                              ((string= name ".."))
-                              ((pattern-matches piece name)
-                               (let ((subdir (concatenate 'string
-                                                          head name "/")))
-                                 (when (eq (%unix-file-kind subdir)
-                                           :directory)
-                                   (%enumerate-directories
-                                    subdir (cdr tail) pathname verify-existance
-                                    function)))))))
-               (close-dir dir)))))
-        ((member :up)
-         (%enumerate-directories (concatenate 'string head "../")
-                                 (cdr tail) pathname verify-existance
-                                 function))))
-    (%enumerate-files head pathname verify-existance function)))
-
-(defun %enumerate-files (directory pathname verify-existance function)
-  (declare (simple-string directory))
-  (let ((name (%pathname-name pathname))
-	(type (%pathname-type pathname))
-	(version (pathname-version pathname)))
-    (cond ((member name '(nil :unspecific))
-	   (when (or (not verify-existance)
-		     (%unix-file-kind directory))
-	     (funcall function directory)))
-	  ((or (pattern-p name)
-	       (pattern-p type)
-	       (eq name :wild)
-	       (eq type :wild))
-	   (let ((dir (%open-dir directory)))
-	     (when dir
-	       (unwind-protect
-		   (loop
-		     (let ((file (%read-dir dir)))
-		       (if file
-			   (unless (or (string= file ".")
-				       (string= file ".."))
-			     (multiple-value-bind
-				 (file-name file-type file-version)
-				 (let ((*ignore-wildcards* t))
-				   (extract-name-type-and-version
-				    file 0 (length file)))
-			       (when (and (components-match file-name name)
-					  (components-match file-type type)
-					  (components-match file-version
-							    version))
-				 (funcall function
-					  (concatenate 'string
-						       directory
-						       file)))))
-			   (return))))
-		 (close-dir dir)))))
-	  (t
-	   (let ((file (concatenate 'string directory name)))
-	     (unless (or (null type) (eq type :unspecific))
-	       (setf file (concatenate 'string file "." type)))
-	     (unless (member version '(nil :newest :wild))
-	       (setf file (concatenate 'string file "."
-				       (quick-integer-to-string version))))
-	     (when (or (not verify-existance)
-		       (%unix-file-kind file t))
-	       (funcall function file)))))))
-
-(defun quick-integer-to-string (n)
-  (declare (type integer n))
-  (cond ((not (fixnump n))
-	 (write-to-string n :base 10 :radix nil))
-	((zerop n) "0")
-	((eql n 1) "1")
-	((minusp n)
-	 (concatenate 'simple-string "-"
-		      (the simple-string (quick-integer-to-string (- n)))))
-	(t
-	 (do* ((len (1+ (truncate (integer-length n) 3)))
-	       (res (make-string len))
-	       (i (1- len) (1- i))
-	       (q n)
-	       (r 0))
-	      ((zerop q)
-	       (incf i)
-	       (replace res res :start2 i :end2 len)
-	       (shrink-vector res (- len i)))
-	   (declare (simple-string res)
-		    (fixnum len i r q))
-	   (multiple-value-setq (q r) (truncate q 10))
-	   (setf (schar res i) (schar "0123456789" r))))))
-
-(defun directory (pathname &key (all t) (check-for-subdirs t)
-			   (follow-links t))
-  "Returns a list of pathnames, one for each file that matches the given
-   pathname.  Supplying :ALL as nil causes this to ignore Unix dot files.  This
-   never includes Unix dot and dot-dot in the result.  If :FOLLOW-LINKS is NIL,
-   then symblolic links in the result are not expanded.  This is not the
-   default because TRUENAME does follow links, and the result pathnames are
-   defined to be the TRUENAME of the pathname (the truename of a link may well
-   be in another directory.)"
-  (let ((results nil))
-    (enumerate-matches (name (translate-logical-pathname
-			      (merge-pathnames pathname
-					       (make-pathname :name :wild
-							      :type :wild
-							      :version :wild))))
-       (when (or all
-		 (let ((slash (position #\/ name :from-end t)))
-		   (or (null slash)
-		       (= (1+ slash) (length name))
-		       (char/= (schar name (1+ slash)) #\.))))
-	 (push name results)))
-    (let ((*ignore-wildcards* t))
-      (mapcar #'(lambda (name)
-		  (let ((name (if (and check-for-subdirs
-				       (eq (%unix-file-kind name)
-					   :directory))
-				  (concatenate 'string name "/")
-				  name)))
-		    (if follow-links (truename name) (pathname name))))
-	      (sort (delete-duplicates results :test #'string=) #'string<)))))
-|#
-
 (provide "PATHNAMES")
