@@ -1,24 +1,20 @@
 ;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
-
+(in-package "CCL")
 
 
 ;;
@@ -59,34 +55,15 @@
 (defun matchp2 (item elt test test-not key)
   (if key
     (setq elt (funcall key elt)))
-  (let ((res (if test (if (eq test 'funcall) (funcall item elt) (funcall test item elt))  (eq item elt))))
+  (let ((res (if test
+               (if (eq test 'funcall)
+                 (funcall item elt)
+                 (funcall test item elt))
+               (eq item elt))))
     (if test-not
       (not res)
       res)))
 
-#|
-(defun matchp1 (elt test notp key)
-  (if (funcall test (if key (funcall key elt) elt))
-    (if notp nil t)
-    (if notp t nil)))
-|#
-
-
-
-
-
-(eval-when (:execute :compile-toplevel)
-  
-  (defmacro type-specifier-atom (type)
-    "Returns the broad class of which TYPE is a specific subclass."
-    ; Hmm... That means that 17 is the broad class of which '(17 11) is a subclass.
-    ; I see.
-    ; Macros that are called by sort (or anything else) are more trouble than
-    ; they're worth, don't you think ?
-    `(if (atom ,type) ,type (car (type-expand ,type))))
-
-  
-  )
 
 
 (defun make-sequence (type length &key (initial-element nil initial-element-p))
@@ -95,11 +72,13 @@
   (setq length (require-type length 'fixnum))
   (let* ((ctype (specifier-type type)))
     (declare (fixnum length))
+    (if (< length 0) (report-bad-arg length '(and fixnum unsigned-byte)))
     (let ((tlength (array-ctype-length ctype)))
       (if (and tlength (neq tlength length))
-        (error "Length of ~s is not ~s" type length)))
-    (cond ((csubtypep ctype (specifier-type 'list))
-           (make-list length :initial-element initial-element))
+        (error 'invalid-subtype-error
+               :datum type
+               :expected-type `(vector ,(type-specifier (array-ctype-element-type ctype)) ,tlength))))
+    (cond 
           ((csubtypep ctype (specifier-type 'base-string))
            (if initial-element-p
              (make-string length 
@@ -123,7 +102,8 @@
           ((csubtypep ctype (specifier-type 'array))
            (let* ((dims (array-ctype-dimensions ctype)))
              (when (consp dims)
-               (when (not (null (cdr dims)))(error "~s is not a sequence type." type))))
+               (when (not (null (cdr dims)))
+                 (error 'invalid-subtype-error :datum  type :expected-type 'vector))))
            (let* ((element-type (type-specifier (array-ctype-element-type ctype))))
              (if (eq element-type '*) (setq element-type t))
              (if initial-element-p
@@ -132,7 +112,17 @@
                            :initial-element initial-element)
                (make-array (the fixnum length)
                            :element-type element-type))))           
-          (t (error "~S is a bad type specifier for sequences." type)))))
+          ((csubtypep ctype (specifier-type 'null))
+           (unless (zerop length)
+             (error 'invalid-subtype-error :datum type :expected-type 'cons)))
+          ((csubtypep ctype (specifier-type 'cons))
+           (if (zerop length)
+             (error 'invalid-subtype-error :datum type :expected-type 'null)
+             (make-list length :initial-element initial-element)))
+          ((csubtypep ctype (specifier-type 'list))
+           (make-list length :initial-element initial-element))
+          (t (error 'invalid-subtype-error :datum  type
+                    :expected-type 'sequence)))))
 
 
 
@@ -203,17 +193,7 @@
          (%typed-miscset subtype result i (%typed-miscref subtype src offset))
          (incf offset))))))
 
-(defun check-sequence-bounds (seq start end &optional length)
-  (if (not length) 
-    (setq length (length seq)))
-  (if (not end)
-    (setq end length))
-  (cond ((> end length)
-         (%err-disp $XACCESSNTH end seq))
-        ((> start end)
-         (report-bad-arg start `(integer 0 ,end)))
-        (t end)))
-  
+
 
 ;;; Fill:
 
@@ -323,12 +303,17 @@
                                      simple-base-string
                                      bit-vector simple-bit-vector))
       (setq output-type-spec (type-expand output-type-spec))))
-  (case (type-specifier-atom output-type-spec)
+  (case (if (atom output-type-spec) output-type-spec (car output-type-spec))
     (list (apply #'concat-to-list* sequences))
-    ((simple-vector simple-string simple-base-string base-string vector string array simple-array
+    ((simple-vector simple-string simple-base-string base-string vector string array
                     bit-vector simple-bit-vector)
      (apply #'concat-to-simple* output-type-spec sequences))
-    (t (error "~S: invalid output type specification." output-type-spec))))
+    (t
+     (if (subtypep output-type-spec 'vector)
+       (apply #'concat-to-simple* output-type-spec sequences)
+       (if (subtypep output-type-spec 'list)
+         (apply #'concat-to-list* sequences)
+         (error "~S: invalid output type specification." output-type-spec))))))
 
 ;;; Internal Frobs:
 
@@ -466,9 +451,10 @@
            (list sequences arg-slice cur-slice)
            (dynamic-extent sequences arg-slice))
   (dolist (seq sequences)
-    (when (vectorp seq)
-      (setq min-vector-length (min min-vector-length
-                                   (length seq)))))
+    (seq-dispatch seq
+                  nil
+                  (setq min-vector-length (min min-vector-length
+                                               (length seq)))))
   (dotimes (index min-vector-length)
     (dolist (one-seq sequences)
       (%rplaca cur-slice
@@ -711,19 +697,25 @@
         (coerce-to-compiled-function object))
        ((csubtypep type (specifier-type 'function))
         (coerce-to-function-1 object))
+       ((csubtypep type (specifier-type 'cons))
+        (if object
+          (coerce-to-list object)
+          (report-bad-arg object 'cons)))
        ((csubtypep type (specifier-type 'list))
         (coerce-to-list object))
        ((csubtypep type (specifier-type 'string))
         (let ((length (array-ctype-length type)))
           (if (and length (neq length (length object)))
-            (error "Length of ~s is not ~s" object length)))
+            (report-bad-arg (make-string length) `(string ,(length object)))))
         (coerce-to-uarray object (type-keyword-code :base-string)
                           t))
        ((csubtypep type (specifier-type 'vector))
-        (let ((length (array-ctype-length type)))
-          (if (and length (neq length (length object)))
-            (error "Length of ~s is not ~s." object length)))
-        (coerce-to-uarray object (element-type-subtype (type-specifier (array-ctype-element-type type))) t))
+        (let* ((element-type (type-specifier (array-ctype-element-type type))))
+          (let ((length (array-ctype-length type)))
+            (if (and length (neq length (length object)))
+              (report-bad-arg (make-array length :element-type element-type)
+                              `(vector ,(length object))))
+            (coerce-to-uarray object (element-type-subtype element-type) t))))
        ((csubtypep type (specifier-type 'array))
         (let* ((dims (array-ctype-dimensions type)))
           (when (consp dims)
@@ -1070,7 +1062,8 @@
 
 
 (defun remove (item sequence &key from-end test test-not (start 0)
-                    end (count most-positive-fixnum) key)
+                    end count key)
+  (setq count (check-count count))
   (seq-dispatch
    sequence
    (list-delete-1 item 
@@ -1097,6 +1090,7 @@
 
 (defun remove-if (test sequence &key from-end (start 0)
                          end count key)
+  (setq count (check-count count))
   (remove test sequence
           :test #'funcall
           :from-end from-end
@@ -1107,6 +1101,7 @@
 
 (defun remove-if-not (test sequence &key from-end (start 0)
                          end count key)
+  (setq count (check-count count))
   (remove test sequence
           :test-not #'funcall
           :from-end from-end
@@ -1127,7 +1122,7 @@
 ; for its test-not fn, so I special cased the call to member. --- cfry
 
 (defun remove-duplicates (sequence &key (test #'eql) test-not (start 0) 
-      from-end (end (length sequence)) (key #'identity))
+      from-end (end (length sequence)) key)
   (delete-duplicates (copy-seq sequence) :from-end from-end :test test
                      :test-not test-not :start start :end end :key key))
 
@@ -1219,12 +1214,13 @@
 		                           :end (if from-end jndex end) :test-not test-not)
               (setq jndex (1+ jndex)))))
 
-(defun delete-duplicates (sequence &key (test #'eql) test-not (start 0) from-end end (key #'identity))
+(defun delete-duplicates (sequence &key (test #'eql) test-not (start 0) from-end end key)
   "The elements of Sequence are examined, and if any two match, one is
    discarded.  The resulting sequence, which may be formed by destroying the
    given sequence, is returned.
    Sequences of type STR have a NEW str returned."
   (unless end (setq end (length sequence)))
+  (unless key (setq key #'identity))
   (seq-dispatch sequence
     (if sequence
 	      (list-delete-duplicates* sequence test test-not key from-end start end))
@@ -1585,46 +1581,105 @@
 
 ;;; Count:
 
-(defun count (item sequence &key from-end test test-not (start 0) end key)
+(defun vector-count-from-start (test item sequence start end key)
+  (declare (fixnum start end))
+  (do* ((index start (1+ index))
+        (count 0))
+       ((= index end) count)
+    (declare (fixnum index count limit))
+    (when (funcall test item  (funcall key (aref sequence index)))
+      (incf count))))
+
+(defun vector-count-from-end (test item sequence start end key)
+  (declare (fixnum start end))
+  (do* ((index (1- end) (1- index))
+        (count 0)
+        (limit (1- start)))
+       ((= index limit) count)
+    (declare (fixnum index count limit))
+    (when (funcall test item (funcall key (aref sequence index)))
+      (incf count))))
+
+(defun vector-count-not-p-from-start (test-not item sequence start end key)
+  (declare (fixnum start end))
+  (do* ((index start (1+ index))
+        (count 0))
+       ((= index end) count)
+    (declare (fixnum index count limit))
+    (unless (funcall test-not item (funcall key (aref sequence index)))
+      (incf count))))
+
+(defun vector-count-not-p-from-end (test-not item sequence start end key)
+  (declare (fixnum start end))
+  (do* ((index (1- end) (1- index))
+        (count 0)
+        (limit (1- start)))
+       ((= index limit) count)
+    (declare (fixnum index count limit))
+    (unless (funcall test-not item (funcall key (aref sequence index)))
+      (incf count))))
+
+(defun list-count-from-start (test item sequence start end key)
+  (declare (fixnum start end) (list sequence))
+  (do* ((seq (nthcdr start sequence) (cdr seq))
+        (element (car seq) (car seq))
+        (index start (1+ index))
+        (count 0))
+       ((or (= index end) (null seq)) count)
+    (declare (fixnum index count) (list seq))
+    (when (funcall test item (funcall key element))
+      (incf count))))
+
+(defun list-count-from-end (test item sequence start end key)
+  (declare (fixnum start end))
+  (let* ((len (length sequence)))
+    (declare (fixnum len))
+    (list-count-from-start test item (reverse sequence) (- len end) (- len start) key)))
+
+(defun list-count-not-p-from-start (test-not item sequence start end key)
+  (declare (fixnum start end) (list sequence))
+  (do* ((seq (nthcdr start sequence) (cdr seq))
+        (element (car seq) (car seq))
+        (index start (1+ index))
+        (count 0))
+       ((or (= index end) (null seq)) count)
+    (declare (fixnum index count) (list seq))
+    (unless (funcall test-not item  (funcall key element))
+      (incf count))))
+
+(defun list-count-not-p-from-end (test-not item sequence start end key)
+  (declare (fixnum start end))
+  (let* ((len (length sequence)))
+    (declare (fixnum len))
+    (list-count-not-p-from-start test-not item (reverse sequence) (- len end) (- len start) key)))
+
+(defun count (item sequence &key from-end (test #'eql testp)
+                   (test-not nil notp) (start 0) end key)
+  (if (and testp notp)
+    (test-not-error test test-not))
+  (unless key
+    (setq key #'identity))
+  (setq end (check-sequence-bounds sequence start end))
   (if sequence
     (seq-dispatch
      sequence
-     (list-count item sequence  :from-end from-end :test test 
-                 :test-not test-not :start start :end end :key key)
-     (vector-count item sequence :from-end from-end :test test
-                   :test-not test-not :start start :end end :key key))
+     (if notp
+       (if from-end
+         (list-count-not-p-from-end test-not item  sequence start end key)
+         (list-count-not-p-from-start test-not item sequence start end key))
+       (if from-end
+         (list-count-from-end test item sequence start end key)
+         (list-count-from-start test item sequence start end key)))
+     (if notp
+       (if from-end
+         (vector-count-not-p-from-end test-not item sequence start end key)
+         (vector-count-not-p-from-start test-not item sequence start end key))
+       (if from-end
+         (vector-count-from-end test item sequence start end key)
+         (vector-count-from-start test item sequence start end key))))
     0))
 
 
-(defun list-count (item list &key from-end test test-not (start 0) end key
-                        &aux (count 0) (pos 0))
-  (declare (ignore from-end))
-  (setq key (adjust-key key))
-  (multiple-value-setq (test test-not)
-                       (adjust-test-args item test test-not))
-  (setq end (check-sequence-bounds list start end most-positive-fixnum))
-  (loop
-    (if (and list (neq end pos))
-      (progn
-        (if (and (%i<= start pos)
-                 (matchp2 item (car list) test test-not key))
-          (setq count (%i+ count 1)))
-        (setq list (%cdr list) pos (%i+ pos 1)))
-      (return count))))
-
-(defun vector-count (item vector &key from-end test test-not (start 0) end key
-                          &aux (count 0) (pos start))
-  (declare (ignore from-end))
-  (setq key (adjust-key key))
-  (multiple-value-setq (test test-not)
-                       (adjust-test-args item test test-not))
-  (setq end (check-sequence-bounds vector start end))
-  (loop
-    (if (eq end pos) (return count))
-    (if (matchp2 item (aref vector pos) test test-not key)
-      (setq count (%i+ count 1)))
-    (setq pos (%i+ pos 1))))
-
 ;;; Count-if:
 
 (defun count-if (test sequence &key from-end (start 0) end key)
