@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;;; Sort
@@ -425,12 +421,13 @@
   a sequence of type Result-Type using the Predicate to order the elements.
   If result-type specifies an array, the returned array will not be
    a complex array. Usually, result-type is either LIST, ARRAY or STRING."
-  (cond ((not (sequencep sequence1))
-         (report-bad-arg sequence1 'sequence))
-        ((not (sequencep sequence2))
-         (report-bad-arg sequence2 'sequence)))
-  (let ((result-ctype (specifier-type result-type)))
-    (cond ((csubtypep result-ctype (specifier-type 'list))
+  (let* ((result-len (+ (length sequence1) (length sequence2)))
+         (result-ctype (specifier-type result-type)))
+    (cond ((csubtypep result-ctype (specifier-type 'null))
+           (unless (zerop result-len)
+             (error 'invalid-subtype-error :datum result-type
+                    :expected-type 'cons)))
+          ((csubtypep result-ctype (specifier-type 'list))
            (canonicalize-pred-and-key predicate key)
            (values                      ; For the terminally pedantic.
             (merge-lists* (if (listp sequence1)
@@ -449,7 +446,9 @@
                             sequence2)
                           predicate key
                           result-type))
-          (t (error "~s does not specify a sequence type." (type-specifier result-type))))))
+          (t (error 'invalid-subtype-error
+                    :datum result-type
+                    :expected-type 'sequence)))))
 
 (defun merge-vectors (vector-1 vector-2 pred key 
                                &optional (result-type 'vector))
@@ -483,17 +482,19 @@
 ;; OK, here goes the type parsing...
 (defun make-merge-vectors-result (result-type result-length vector-1 vector-2)
   (let* ((ctype (specifier-type result-type)))
-    (unless (csubtypep ctype (specifier-type 'vector))
-      (error "~a does not specify a vector." result-type))
     (let* ((size (array-ctype-length ctype))
            (elt-type (array-or-union-ctype-element-type ctype)))
-      (unless (or (eq size '*) (eq size result-length))
-        (error "~a specifies a vector of the wrong length." result-type))
       (if (eq elt-type '*)
         (let ((et1 (array-element-type vector-1))
               (et2 (array-element-type vector-2)))
           (setq elt-type (if (eq et1 et2) et1 `(or ,et1 ,et2)))))
-      (make-array result-length :element-type elt-type))))
+      (if (and size (not (eq size result-length)))
+        (error 'invalid-subtype-error
+               :datum result-type
+               :expected-type `(vector ,elt-type ,result-length))
+        (make-array (the fixnum (or size result-length))
+                    :element-type elt-type)))))
+        
 
 ;; Gee, that wasn't so bad after all.
 ;; Well, when you're building on the shoulders of giants,
