@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 
@@ -125,7 +121,7 @@
  (xfasload         "ccl:xdump;xfasload"       ("ccl:xdump;xfasload.lisp"))
  (xsparcfasload    "ccl:xdump;xsparcfasload"  ("ccl:xdump;xsparcfasload.lisp"))
  (xppcfasload      "ccl:xdump;xppcfasload"    ("ccl:xdump;xppcfasload.lisp"))
- (pef-file         "ccl:xdump;pef-file"       ("ccl:xdump;pef-file.lisp"))
+ (heap-image       "ccl:xdump;heap-image"     ("ccl:xdump;heap-image.lisp"))
  (xsym             "ccl:xdump;xsym"           ("ccl:xdump;xsym.lisp"))
  (number-macros "ccl:bin;number-macros"    ("ccl:lib;number-macros.lisp"))
  (number-case-macro  "ccl:bin;number-case-macro" ("ccl:lib;number-case-macro.lisp"))
@@ -170,22 +166,8 @@
  (mcl-compat       "ccl:bin;mcl-compat"       ("ccl:lib;mcl-compat.lisp"))
  (backtrace        "ccl:bin;backtrace"        ("ccl:lib;backtrace.lisp"))
  (backtrace-lds    "ccl:bin;backtrace-lds"    ("ccl:lib;backtrace-lds.lisp"))
- (edhardcopy       "ccl:bin;edhardcopy"       ("ccl:lib;edhardcopy.lisp"))
- (sanetraps        "ccl:bin;sanetraps"        ("ccl:lib;sanetraps.lisp"))
  (apropos          "ccl:bin;apropos"          ("ccl:lib;apropos.lisp"))
- (mac-file-io      "ccl:library;mac-file-io"  ("ccl:library;mac-file-io.lisp"))
- (boyer-moore      "ccl:bin;boyer-moore"      ("ccl:lib;boyer-moore.lisp"
-                                                    "ccl:library;mac-file-io.lisp"))
- (icon-dialog-item "ccl:library;icon-dialog-item"
-                                                   ("ccl:library;icon-dialog-item.lisp"))
- (hide-listener-support
-                   "ccl:library;hide-listener-support"
-                                                   ("ccl:library;hide-listener-support.lisp"
-                                                    "ccl:library;icon-dialog-item.lisp"))
- (save-application-dialog
-                   "ccl:library;save-application-dialog"
-                                                   ("ccl:library;save-application-dialog.lisp"
-                                                    "ccl:library;hide-listener-support.lisp"))
+
  (appgen-misc      "ccl:lib;appgen-misc"      ("ccl:lib;appgen-misc.lisp"))
  (build-application "ccl:lib;build-application" ("ccl:lib;build-application.lisp"
                                                       "ccl:lib;appgen-misc.lisp"))
@@ -195,7 +177,6 @@
                                                 "ccl:lib;aplink-macros.lisp"))
  (aplink-dump      "ccl:bin;aplink-dump"      ("ccl:lib;aplink-dump.lisp"
                                                 "ccl:lib;aplink-macros.lisp"))
- (disasm           "ccl:bin;disasm"           ("ccl:lib;disasm.lisp"))
  (defstruct        "ccl:bin;defstruct"        ("ccl:lib;defstruct.lisp"
                                                     "ccl:lib;defstruct-macros.lisp"))
  (defstruct-lds    "ccl:bin;defstruct-lds"    ("ccl:lib;defstruct-lds.lisp"
@@ -203,7 +184,6 @@
  (method-combination
                    "ccl:bin;method-combination"
                                                    ("ccl:lib;method-combination.lisp"))
- (meter            "ccl:bin;meter"            ("ccl:lib;meter.lisp"))
  (encapsulate      "ccl:bin;encapsulate"      ("ccl:lib;encapsulate.lisp"))
  (read             "ccl:bin;read"           ("ccl:lib;read.lisp"))
  (misc             "ccl:bin;misc"           ("ccl:lib;misc.lisp"))
@@ -219,64 +199,27 @@
  (time             "ccl:bin;time"           ("ccl:lib;time.lisp"))
  (print-db         "ccl:bin;print-db"       ("ccl:lib;print-db.lisp"))
  (eval             "ccl:bin;eval"           ("ccl:lib;eval.lisp"))
- (fred-additions   "ccl:bin;fred-additions" ("ccl:lib;fred-additions.lisp"))
- (fred-help        "ccl:bin;fred-help"      ("ccl:lib;fred-help.lisp"))
- (edit-definition  "ccl:bin;edit-definition" ("ccl:lib;edit-definition.lisp"))
- (list-definitions "ccl:bin;list-definitions" ("ccl:lib;list-definitions.lisp"))
 
  (arglist          "ccl:bin;arglist"          ("ccl:lib;arglist.lisp"))
- (dialog-macros    "ccl:library;dialog-macros"("ccl:library;dialog-macros.lisp"))
- (ccl-menus        "ccl:bin;ccl-menus"        ("ccl:lib;ccl-menus.lisp"))
- (ccl-menus-lds    "ccl:bin;ccl-menus-lds"    ("ccl:lib;ccl-menus-lds.lisp"
-                                                    "ccl:library;dialog-macros.lisp"))
- (inspect          "ccl:bin;inspect"          ("ccl:lib;inspect.lisp"))
- (apropos-dialog   "ccl:bin;apropos-dialog"   ("ccl:lib;apropos-dialog.lisp"
-                                                    "ccl:library;dialog-macros.lisp"))
- (views            "ccl:bin;views"          ("ccl:lib;views.lisp"))
- (windoids         "ccl:bin;windoids"       ("ccl:lib;windoids.lisp"))
- (scrolling-fred-view
-  		   "ccl:library;scrolling-fred-view"
-						 ("ccl:library;scrolling-fred-view.lisp"))
+
  (edit-callers	   "ccl:bin;edit-callers"   ("ccl:lib;edit-callers.lisp"))
  (step             "ccl:bin;step"           ("ccl:lib;step.lisp"))
- (disassemble      "ccl:bin;disassemble"      ("ccl:lib;disassemble.lisp"
-                                                "ccl:compiler;subprims8.lisp"))
- (subprims8        ()                          	("ccl:compiler;subprims8.lisp"))
  (ccl-export-syms  "ccl:bin;ccl-export-syms"  ("ccl:lib;ccl-export-syms.lisp"))
  (systems          "ccl:bin;systems"        ("ccl:lib;systems.lisp"))
  (compile-ccl      "ccl:bin;compile-ccl"    ("ccl:lib;compile-ccl.lisp"))
  (ppc-init-ccl     "ccl:bin;ppc-init-ccl"   ("ccl:lib;ppc-init-ccl.lisp"))
  (merge-ccl-update "ccl:bin;merge-ccl-update" ("ccl:lib;merge-ccl-update.lisp"))
- (ff               "ccl:library;ff"         ("ccl:lib;ff-source.lisp"))
  (distrib-inits    "ccl:bin;distrib-inits"  ("ccl:lib;distrib-inits.lisp"))
  (lisp-package     "ccl:library;lisp-package" ("ccl:library;lisp-package.lisp"))
- (help-manager     "ccl:library;help-manager" ("ccl:library;help-manager.lisp"))
- (resources        "ccl:library;resources"  ("ccl:library;resources.lisp"))
- (interfaces       "ccl:library;interfaces" ("ccl:library;interfaces.lisp"))
- (patchenv         "ccl:bin;patchenv"       ("ccl:lib;patchenv.lisp"))
- (resident-interfaces
-                   "ccl:bin;resident-interfaces"
-                                                   ("ccl:lib;resident-interfaces.lisp"))
- (old-file-dialogs "ccl:library;old-file-dialogs"
-                                                   ("ccl:library;old-file-dialogs.lisp"))
- (new-file-dialogs "ccl:library;new-file-dialogs"
-                                                   ("ccl:library;new-file-dialogs.lisp"))
  ; need to add swapping, xdump to CCL's *module-search-path*
- (boot-init        "ccl:swapping;boot-init"   ("ccl:swapping;boot-init.lisp"))
- (save             "ccl:swapping;save"        ("ccl:swapping;save.lisp"))
- (make-functions-resident
-                   "ccl:swapping;make-functions-resident"
-                                                   ("ccl:swapping;make-functions-resident.lisp"))
  (xdump            "ccl:xdump;xdump"          ("ccl:xdump;xdump.lisp"))
  (fasload          "ccl:xdump;fasload"        ("ccl:xdump;fasload.lisp"))
  (loop             "ccl:library;loop"         ("ccl:library;loop.lisp"))
- (ff-call-68k      "ccl:bin;ff-call-68k"      ("ccl:lib;ff-call-68k.lisp"))
- (font-menus       "ccl:library;font-menus"   ("ccl:library;font-menus.lisp"))
- (search-files     "ccl:bin;search-files"     ("ccl:lib;search-files.lisp"))
  (mcl-extensions   "ccl:bin;mcl-extensions"   ("ccl:lib;mcl-extensions.lisp"))
  (linux-files      "ccl:l1f;linux-files"      ("ccl:level-1;linux-files.lisp"))
  (solaris-files    "ccl:l1f;solaris-files"    ("ccl:level-1;solaris-files.lisp"))
  (solaris-records  "ccl:library;solaris-records" ("ccl:library;solaris-records.lisp"))
  (source-files     "ccl:bin;source-files"     ("ccl:lib;source-files.lisp"))
- ))
+ 
+ (prepare-mcl-environment "ccl:bin;prepare-mcl-environment" ("ccl:lib;prepare-mcl-environment.lisp"))))
 
