@@ -1,22 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 
 ;; LispEqu.lisp
@@ -500,6 +496,15 @@
   unknown-ctype-specifier
 )
 
+;;; CONS-ctype is a subclass of CTYPE
+(def-accessors (cons-ctype) %svref
+  nil                                   ; 'cons-ctype
+  nil                                   ; ctype-class-info
+  nil                                   ; ctype-enumerable
+  cons-ctype-car-ctype                  ; ctype of the car
+  cons-ctype-cdr-ctype                  ; ctype of the cdr
+  )
+
 ; NUMERIC-ctype is a subclass of CTYPE
 (def-accessors (numeric-ctype) %svref
   nil                                   ; numeric-ctype
@@ -570,14 +575,14 @@
   foreign-ctype-foreign-type
 )
   
-; Most "real" CLOS objects have one of these in their class-wrapper's 
-; %wrapper-type-info slot
+; Most "real" CLOS objects have one of these in their %class-ctype slot
 
 (def-accessors (class-ctype) %svref
   nil                                   ; 'class-ctype
   nil                                   ; ctype-class-info           
   nil                                   ; ctype-enumerable
   class-ctype-class                     ; backptr to class.
+  class-ctype-translation               ; ctype for some built-in-classes.
 )
 
 
@@ -629,10 +634,6 @@
                                (hash-index '(new-class-wrapper-hash-index)))
   `(%istruct 'class-wrapper ,hash-index ,class nil nil #| nil nil nil nil |#))
 
-#|
-  `(%gvector $v_istruct
-             'class-wrapper ,hash-index ,class nil nil #| nil nil nil nil |#))
-|#
 
 (defconstant %instance-class-wrapper 0)
 (defmacro %instance-class-wrapper (instance) `(%svref ,instance 0))
@@ -709,9 +710,6 @@
 (defmacro %cons-built-in-class (name)
   `(%instance-vector *built-in-class-wrapper* ,name nil nil nil nil nil nil))
 
-#|
-  `(%gvector $v_instance *built-in-class-wrapper* ,name nil nil nil nil nil nil))
-|#
 
 (defmacro %cons-standard-class (name &optional
                                      (metaclass-wrapper '*standard-class-wrapper*))
