@@ -133,7 +133,6 @@
   )
 
 
-
 ;;; The uses of this macro are retained in the CL version of loop, in
 ;;; case they are needed in a particular implementation.  Originally
 ;;; dating from the use of the Zetalisp COPYLIST* function, this is used
@@ -780,7 +779,7 @@ a LET-like macro, and a SETQ-like macro, which perform LOOP-style destructuring.
 		     epilogue
 		     &aux rbefore rafter flagvar)
   (unless (= (length before-loop) (length after-loop))
-    (error "LOOP-BODY called with non-synched before- and after-loop lists."))
+    (loop-error "LOOP-BODY called with non-synched before- and after-loop lists."))
   ;;All our work is done from these copies, working backwards from the end:
   (setq rbefore (reverse before-loop) rafter (reverse after-loop))
   (labels ((psimp (l)
@@ -957,7 +956,7 @@ a LET-like macro, and a SETQ-like macro, which perform LOOP-style destructuring.
 (defun loop-error (format-string &rest format-args)
   #+(or Genera CLOE) (declare (dbg:error-reporter))
   #+Genera (setq format-args (copy-list format-args))	;Don't ask.
-  (error "~?~%Current LOOP context:~{ ~S~}." format-string format-args (loop-context)))
+  (#+mcl ccl::signal-program-error #-mcl error "~?~%Current LOOP context:~{ ~S~}." format-string format-args (loop-context)))
 
 
 (defun loop-warn (format-string &rest format-args)
@@ -988,6 +987,23 @@ collected result will be returned as the value of the LOOP."
 
 
 
+(defun subst-gensyms-for-nil (tree)
+  (declare (special *ignores*))
+  (cond
+    ((null tree) (car (push (loop-gentemp) *ignores*)))
+    ((atom tree) tree)
+    (t (cons (subst-gensyms-for-nil (car tree))
+	     (subst-gensyms-for-nil (cdr tree))))))
+ 
+(defun loop-build-destructuring-bindings (crocks forms)
+  (if crocks
+      (let ((*ignores* ()))
+	(declare (special *ignores*))
+	`((destructuring-bind ,(subst-gensyms-for-nil (car crocks))
+	      ,(cadr crocks)
+	    (declare (ignore ,@*ignores*))
+	    ,@(loop-build-destructuring-bindings (cddr crocks) forms))))
+      forms))
 
 (defun loop-translate (*loop-source-code* *loop-macro-environment* *loop-universe*)
   (let ((*loop-original-source-code* *loop-source-code*)
@@ -1021,9 +1037,6 @@ collected result will be returned as the value of the LOOP."
 		     ,(nreverse *loop-body*)
 		     ,(nreverse *loop-after-body*)
 		     ,(nreconc *loop-epilogue* (nreverse *loop-after-epilogue*)))))
-      (do () (nil)
-	(setq answer `(block ,(pop *loop-names*) ,answer))
-	(unless *loop-names* (return nil)))
       (dolist (entry *loop-bind-stack*)
 	(let ((vars (first entry))
 	      (dcls (second entry))
@@ -1033,14 +1046,17 @@ collected result will be returned as the value of the LOOP."
 	    (setq answer (append w (list answer))))
 	  (when (or vars dcls crocks)
 	    (let ((forms (list answer)))
-	      (when crocks (push crocks forms))
+	      ;;(when crocks (push crocks forms))
 	      (when dcls (push `(declare ,@dcls) forms))
 	      (setq answer `(,(cond ((not vars) 'locally)
 				    (*loop-destructuring-hooks* (first *loop-destructuring-hooks*))
 				    (t 'let))
 			     ,vars
-			     ,@forms))))))
-      answer)))
+			     ,@(loop-build-destructuring-bindings crocks forms)))))))
+      (if *loop-names*
+	  (do () ((null (car *loop-names*)) answer)
+	    (setq answer `(block ,(pop *loop-names*) ,answer)))
+	  `(block nil ,answer)))))
 
 
 (defun loop-iteration-driver ()
@@ -1069,9 +1085,17 @@ collected result will be returned as the value of the LOOP."
       (loop-error "LOOP source code ran out when another token was expected.")))
 
 
+(defun loop-get-compound-form ()
+  (let ((form (loop-get-form)))
+    (unless (consp form)
+      (loop-error "Compound form expected, but found ~A." form))
+    form))
+
 (defun loop-get-progn ()
-  (do ((forms (list (loop-pop-source)) (cons (loop-pop-source) forms))
-       (nextform (car *loop-source-code*) (car *loop-source-code*)))
+  (do ((forms (list (loop-get-compound-form))
+              (cons (loop-get-compound-form) forms))
+       (nextform (car *loop-source-code*)
+                 (car *loop-source-code*)))
       ((atom nextform)
        (if (null (cdr forms)) (car forms) (cons 'progn (nreverse forms))))))
 
@@ -1094,8 +1118,9 @@ collected result will be returned as the value of the LOOP."
   (setq *loop-emitted-body* t)
   (loop-pseudo-body form))
 
-(defun loop-emit-final-value (form)
-  (push (loop-construct-return form) *loop-after-epilogue*)
+(defun loop-emit-final-value (&optional (form nil form-supplied-p))
+  (when form-supplied-p
+    (push (loop-construct-return form) *loop-after-epilogue*))
   (when *loop-final-value-culprit*
     (loop-warn "LOOP clause is providing a value for the iteration,~@
 	        however one was already established by a ~S clause."
@@ -1107,6 +1132,15 @@ collected result will be returned as the value of the LOOP."
   #+(or Genera CLOE) (declare (dbg:error-reporter))
   (when *loop-inside-conditional*
     (loop-error "~:[This LOOP~;The LOOP ~:*~S~] clause is not permitted inside a conditional." kwd)))
+
+(defun loop-disallow-anonymous-collectors ()
+  (when (find-if-not 'loop-collector-name *loop-collection-cruft*)
+    (loop-error "This LOOP clause is not permitted with anonymous collectors.")))
+
+(defun loop-disallow-aggregate-booleans ()
+  (when (loop-tmember *loop-final-value-culprit* '(always never thereis))
+    (loop-error "This anonymous collection LOOP clause is not permitted with aggregate booleans.")))
+
 
 
 ;;;; Loop Types
@@ -1130,6 +1164,7 @@ collected result will be returned as the value of the LOOP."
 		;; a common lisp type specifier or pattern (matching the variable) thereof.
 		(loop-pop-source)
 		(loop-pop-source))
+		      
 	       ((symbolp z)
 		;;This is the (sort of) "old" syntax, even though we didn't used to support all of
 		;; these type symbols.
@@ -1176,14 +1211,19 @@ collected result will be returned as the value of the LOOP."
 
 (defun loop-bind-block ()
   (when (or *loop-variables* *loop-declarations* *loop-wrappers*)
-    (push (list (nreverse *loop-variables*) *loop-declarations*
-                (loop-make-desetq *loop-desetq-crocks*) *loop-wrappers*)
+    (push (list (nreverse *loop-variables*) *loop-declarations* *loop-desetq-crocks* *loop-wrappers*)
 	  *loop-bind-stack*)
     (setq *loop-variables* nil
 	  *loop-declarations* nil
 	  *loop-desetq-crocks* nil
 	  *loop-wrappers* nil)))
 
+(defun loop-variable-p (name)
+  (do ((entry *loop-bind-stack* (cdr entry))) (nil)
+    (cond ((null entry)
+	   (return nil))
+	  ((assoc name (caar entry) :test #'eq)
+	   (return t)))))
 
 (defun loop-make-variable (name initialization dtype &optional iteration-variable-p)
   (cond ((null name)
@@ -1211,10 +1251,12 @@ collected result will be returned as the value of the LOOP."
 		(loop-declare-variable name dtype)
 		(push (list name initialization) *loop-variables*))
 	       (t (let ((newvar (loop-gentemp 'loop-destructure-)))
+		    (loop-declare-variable name dtype)
 		    (push (list newvar initialization) *loop-variables*)
 		    ;; *LOOP-DESETQ-CROCKS* gathered in reverse order.
 		    (setq *loop-desetq-crocks*
-			  (list* name newvar *loop-desetq-crocks*))
+		      (list* name newvar *loop-desetq-crocks*))
+		    #+ignore
 		    (loop-make-variable name nil dtype iteration-variable-p)))))
 	(t (let ((tcar nil) (tcdr nil))
 	     (if (atom dtype) (setq tcar (setq tcdr dtype))
@@ -1239,7 +1281,7 @@ collected result will be returned as the value of the LOOP."
 		(loop-declare-variable (cdr name) (cdr dtype)))
 	       (t (loop-declare-variable (car name) dtype)
 		  (loop-declare-variable (cdr name) dtype))))
-	(t (error "Invalid LOOP variable passed in: ~S." name))))
+	(t (loop-error "Invalid LOOP variable passed in: ~S." name))))
 
 
 (defun loop-maybe-bind-form (form data-type)
@@ -1250,7 +1292,10 @@ collected result will be returned as the value of the LOOP."
 
 
 (defun loop-do-if (for negatep)
-  (let ((form (loop-get-form)) (*loop-inside-conditional* t) (it-p nil))
+  (let ((form (loop-get-form))
+	(*loop-inside-conditional* t)
+	(it-p nil)
+	(first-clause-p t))
     (flet ((get-clause (for)
 	     (do ((body nil)) (nil)
 	       (let ((key (car *loop-source-code*)) (*loop-body* nil) data)
@@ -1260,7 +1305,8 @@ collected result will be returned as the value of the LOOP."
 			  key for))
 		       (t (setq *loop-source-context* *loop-source-code*)
 			  (loop-pop-source)
-			  (when (loop-tequal (car *loop-source-code*) 'it)
+			  (when (and (loop-tequal (car *loop-source-code*) 'it)
+				     first-clause-p)
 			    (setq *loop-source-code*
 				  (cons (or it-p (setq it-p (loop-when-it-variable)))
 					(cdr *loop-source-code*))))
@@ -1272,6 +1318,7 @@ collected result will be returned as the value of the LOOP."
 				   "~S does not introduce a LOOP clause that can follow ~S."
 				   key for))
 				(t (setq body (nreconc *loop-body* body)))))))
+	       (setq first-clause-p nil)
 	       (if (loop-tequal (car *loop-source-code*) :and)
 		   (loop-pop-source)
 		   (return (if (cdr body) `(progn ,@(nreverse body)) (car body)))))))
@@ -1294,32 +1341,21 @@ collected result will be returned as the value of the LOOP."
 
 (defun loop-do-finally ()
   (loop-disallow-conditional :finally)
-  (let ((keyword (car *loop-source-code*)))
-    (push (if (loop-tequal keyword 'return)
-            (progn (loop-pop-source)
-                   (loop-construct-return (loop-get-form)))
-            (progn
-              (when (or (loop-tequal keyword 'do)
-                        (loop-tequal keyword 'doing))
-                (loop-pop-source))
-              (loop-get-progn)))
-          *loop-epilogue*)))
+  (push (loop-get-progn) *loop-epilogue*))
 
 (defun loop-do-do ()
   (loop-emit-body (loop-get-progn)))
 
-(defun loop-do-named (restrictive)
-  (let ((name (loop-get-form)))
+(defun loop-do-named ()
+  (let ((name (loop-pop-source)))
     (unless (symbolp name)
-	    (loop-error "The argument ~S to the LOOP NAMED clause must be a symbol"
-			name))
-    (when restrictive
-	  (when *loop-names*
-		(loop-error "Can't name LOOP ~S as it is already named ~S"
-			    name (first *loop-names*)))
-	  (unless (eq *loop-source-code* (cddr *loop-original-source-code*))
-		  (loop-error "NAMED must be the first LOOP clause")))
-    (push name *loop-names*)))
+      (loop-error "~S is an invalid name for your LOOP." name))
+    (when (or *loop-before-loop* *loop-body* *loop-after-epilogue* *loop-inside-conditional*)
+      (loop-error "The NAMED ~S clause occurs too late." name))
+    (when *loop-names*
+      (loop-error "You may only use one NAMED clause in your loop: NAMED ~S ... NAMED ~S."
+		  (car *loop-names*) name))
+    (setq *loop-names* (list name nil))))
 
 (defun loop-do-return ()
   (loop-pseudo-body (loop-construct-return (loop-get-form))))
@@ -1347,11 +1383,15 @@ collected result will be returned as the value of the LOOP."
 		(loop-pop-source))))
     (when (not (symbolp name))
       (loop-error "Value accumulation recipient name, ~S, is not a symbol." name))
+    (unless name
+      (loop-disallow-aggregate-booleans))
     (unless dtype
       (setq dtype (or (loop-optional-type) default-type)))
     (let ((cruft (find (the symbol name) *loop-collection-cruft*
 		       :key #'loop-collector-name)))
       (cond ((not cruft)
+	     (when (and name (loop-variable-p name))
+	       (loop-error "Variable ~S cannot be used in INTO clause" name))
 	     (push (setq cruft (make-loop-collector
 				 :name name :class class
 				 :history (list collector) :dtype dtype))
@@ -1445,6 +1485,7 @@ collected result will be returned as the value of the LOOP."
 (defun loop-do-always (restrictive negate)
   (let ((form (loop-get-form)))
     (when restrictive (loop-disallow-conditional))
+    (loop-disallow-anonymous-collectors)
     (loop-emit-body `(,(if negate 'when 'unless) ,form
 		      ,(loop-construct-return nil)))
     (loop-emit-final-value t)))
@@ -1455,6 +1496,8 @@ collected result will be returned as the value of the LOOP."
 ;;; Under ANSI this is not permitted to appear under conditionalization.
 (defun loop-do-thereis (restrictive)
   (when restrictive (loop-disallow-conditional))
+  (loop-disallow-anonymous-collectors)
+  (loop-emit-final-value)
   (loop-emit-body `(when (setq ,(loop-when-it-variable) ,(loop-get-form))
 		     ,(loop-construct-return *loop-when-it-variable*))))
 
@@ -1473,6 +1516,8 @@ collected result will be returned as the value of the LOOP."
 		     (loop-pop-source)
 		     (loop-get-form))
 		    (t nil)))
+    (when (and var (loop-variable-p var))
+      (loop-error "Variable ~S has already been used" var))
     (loop-make-variable var val dtype)
     (if (loop-tequal (car *loop-source-code*) :and)
 	(loop-pop-source)
@@ -1555,20 +1600,21 @@ collected result will be returned as the value of the LOOP."
       (loop-error "~S is an unknown keyword in FOR or AS clause in LOOP." keyword))
     (apply (car tem) var first-arg data-type (cdr tem))))
 
-
 (defun loop-do-repeat ()
+  (loop-disallow-conditional :repeat)
   (let ((form (loop-get-form))
-	(type (loop-check-data-type (loop-optional-type) *loop-real-data-type*)))
-    (when (and (consp form) (eq (car form) 'the) (subtypep (second form) type))
-      (setq type (second form)))
-    (multiple-value-bind (number constantp value)
-	(loop-constant-fold-if-possible form type)
-      (cond ((and constantp (<= value 1)) `(t () () () ,(<= value 0) () () ()))
-	    (t (let ((var (loop-make-variable (loop-gentemp 'loop-repeat-) number type)))
-		 (if constantp
-		     `((not (plusp (setq ,var (1- ,var)))) () () () () () () ())
-		     `((minusp (setq ,var (1- ,var))) () () ()))))))))
-
+	(type 'real))
+    (let ((var (loop-make-variable (loop-gentemp) form type)))
+      (push `(when (minusp (decf ,var)) (go end-loop)) *loop-before-loop*)
+      (push `(when (minusp (decf ,var)) (go end-loop)) *loop-after-body*)
+      ;; FIXME: What should
+      ;;   (loop count t into a
+      ;;         repeat 3
+      ;;         count t into b
+      ;;         finally (return (list a b)))
+      ;; return: (3 3) or (4 3)? PUSHes above are for the former
+      ;; variant, L-P-B below for the latter.
+      #+nil (loop-pseudo-body `(when (minusp (decf ,var)) (go end-loop))))))
 
 (defun loop-when-it-variable ()
   (or *loop-when-it-variable*
@@ -1660,7 +1706,13 @@ collected result will be returned as the value of the LOOP."
       (multiple-value-bind (list-step step-function) (loop-list-step listvar)
 	(declare #+(and (not LOOP-Prefer-POP) (not CLOE)) (ignore step-function))
 	;;@@@@ The CLOE problem above has to do with bug in macroexpansion of multiple-value-bind.
-	(let* ((first-endtest (hide-variable-reference (eq var listvar) listvar `(endp ,listvar)))
+	(let* ((first-endtest
+		(hide-variable-reference
+		 (eq var listvar)
+		 listvar
+		 ;; the following should use `atom' instead of `endp', per
+		 ;; [bug2428]
+		 `(atom ,listvar)))
 	       (other-endtest first-endtest))
 	  (when (and constantp (listp list-value))
 	    (setq first-endtest (null list-value)))
@@ -1689,7 +1741,7 @@ collected result will be returned as the value of the LOOP."
       (loop-make-iteration-variable var nil data-type)
       (loop-make-variable listvar list 'list)
       (multiple-value-bind (list-step step-function) (loop-list-step listvar)
-        #-LOOP-Prefer-POP (declare (ignore step-function))
+	#-LOOP-Prefer-POP (declare (ignore step-function))
 	(let* ((first-endtest `(endp ,listvar))
 	       (other-endtest first-endtest)
 	       (step `(,var (car ,listvar)))
@@ -1825,12 +1877,6 @@ collected result will be returned as the value of the LOOP."
 	    ((and USING-allowed (loop-tequal token 'using))
 	     (loop-pop-source)
 	     (do ((z (loop-pop-source) (loop-pop-source)) (tem)) (nil)
-	       (when (or (atom z)
-			 (atom (cdr z))
-			 (not (null (cddr z)))
-			 (not (symbolp (car z)))
-			 (and (cadr z) (not (symbolp (cadr z)))))
-		 (loop-error "~S bad variable pair in path USING phrase." z))
 	       (when (cadr z)
 		 (if (setq tem (loop-tassoc (car z) *loop-named-variables*))
 		     (loop-error
@@ -2002,6 +2048,7 @@ collected result will be returned as the value of the LOOP."
       ;; into multiple-value-setq variable lists.
       #-Genera (setq other-p t
 		     dummy-predicate-var (loop-when-it-variable))
+      (setq variable (or variable (loop-gentemp 'ignore-)))
       (let ((key-var nil)
 	    (val-var nil)
 	    (bindings `((,variable nil ,data-type)
@@ -2029,16 +2076,20 @@ collected result will be returned as the value of the LOOP."
 
 
 (defun loop-package-symbols-iteration-path (variable data-type prep-phrases &key symbol-types)
-  (cond ((or (cdr prep-phrases) (not (member (caar prep-phrases) '(:in :of))))
+  (cond ((and prep-phrases (cdr prep-phrases))
 	 (loop-error "Too many prepositions!"))
-	((null prep-phrases) (loop-error "Missing OF or IN in ~S iteration path.")))
+	((and prep-phrases (not (member (caar prep-phrases) '(:in :of))))
+	 (loop-error "Unknown preposition ~S" (caar prep-phrases))))
   (unless (symbolp variable)
     (loop-error "Destructuring is not valid for package symbol iteration."))
   (let ((pkg-var (loop-gentemp 'loop-pkgsym-))
-	(next-fn (loop-gentemp 'loop-pkgsym-next-)))
+	(next-fn (loop-gentemp 'loop-pkgsym-next-))
+	(variable (or variable (loop-gentemp 'ignore-)))
+	(pkg (or (cadar prep-phrases) '*package*)))
     (push `(with-package-iterator (,next-fn ,pkg-var ,@symbol-types)) *loop-wrappers*)
     #+ccl-2 (push `(ignorable ,(loop-when-it-variable)) *loop-declarations*)
-    `(((,variable nil ,data-type) (,pkg-var ,(cadar prep-phrases)))
+    
+    `(((,variable nil ,data-type) (,pkg-var ,pkg))
       ()
       ()
       ()
@@ -2055,7 +2106,8 @@ collected result will be returned as the value of the LOOP."
 
 (defun make-ansi-loop-universe (extended-p)
   (let ((w (make-standard-loop-universe
-	     :keywords `((initially (loop-do-initially))
+	     :keywords `((named (loop-do-named))
+			 (initially (loop-do-initially))
 			 (finally (loop-do-finally))
 			 (do (loop-do-do))
 			 (doing (loop-do-do))
@@ -2064,7 +2116,6 @@ collected result will be returned as the value of the LOOP."
 			 (collecting (loop-list-collection list))
 			 (append (loop-list-collection append))
 			 (appending (loop-list-collection append))
-			 (named (loop-do-named t))
 			 (nconc (loop-list-collection nconc))
 			 (nconcing (loop-list-collection nconc))
 			 (count (loop-sum-collection count ,*loop-real-data-type* fixnum))
@@ -2083,7 +2134,8 @@ collected result will be returned as the value of the LOOP."
 			 (when (loop-do-if when nil))	; Normal, do when
 			 (if (loop-do-if if nil))	; synonymous
 			 (unless (loop-do-if unless t))	; Negate the test on when
-			 (with (loop-do-with)))
+			 (with (loop-do-with))
+			 (repeat (loop-do-repeat)))
 	     :for-keywords '((= (loop-ansi-for-equals))
 			     (across (loop-for-across))
 			     (in (loop-for-in))
@@ -2092,20 +2144,22 @@ collected result will be returned as the value of the LOOP."
 			     (downfrom (loop-for-arithmetic :downfrom))
 			     (upfrom (loop-for-arithmetic :upfrom))
 			     (below (loop-for-arithmetic :below))
+			     (above (loop-for-arithmetic :above))
+			     (by (loop-for-arithmetic :by))
 			     (to (loop-for-arithmetic :to))
 			     (upto (loop-for-arithmetic :upto))
+                             (downto (loop-for-arithmetic :downto))
 			     (being (loop-for-being)))
 	     :iteration-keywords '((for (loop-do-for))
-				   (as (loop-do-for))
-				   (repeat (loop-do-repeat)))
-	     :type-symbols '(array atom bignum bit bit-vector character #| common |# compiled-function
+				   (as (loop-do-for)))
+	     :type-symbols '(array atom bignum bit bit-vector character compiled-function
 				   complex cons double-float fixnum float
 				   function hash-table integer keyword list long-float
 				   nil null number package pathname random-state
 				   ratio rational readtable sequence short-float
 				   simple-array simple-bit-vector simple-string
 				   simple-vector single-float standard-char
-				   stream string string-char
+				   stream string base-char
 				   symbol t vector)
 	     :type-keywords nil
 	     :ansi (if extended-p :extended t))))
@@ -2128,7 +2182,7 @@ collected result will be returned as the value of the LOOP."
     (add-loop-path '(present-symbol present-symbols) 'loop-package-symbols-iteration-path w
 		   :preposition-groups '((:of :in))
 		   :inclusive-permitted nil
-		   :user-data '(:symbol-types (:internal)))
+		   :user-data '(:symbol-types (:internal :external)))
     w))
 
 
@@ -2153,4 +2207,4 @@ collected result will be returned as the value of the LOOP."
   (loop-standard-expansion keywords-and-forms env *loop-ansi-universe*))
 
 #+ccl-2
-(ccl:provide "LOOP")
+(cl:provide "LOOP")
