@@ -1,26 +1,22 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 2001 Clozure Associates
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 
 (in-package "CCL")
 
-
+(defvar *parse-ffi-target-ftd* *target-ftd*)
 (defvar *ffi-lisp-readtable* (copy-readtable nil))
 (defvar *ffi-ordinal* -1)
 (defpackage "C" (:use))
@@ -54,9 +50,14 @@
 (defvar *ffi-db-records* nil)
 (defvar *ffi-db-functions* nil)
 (defvar *ffi-typedefs*)
+(defvar *ffi-global-typedefs* nil)
 (defvar *ffi-unions*)
+(defvar *ffi-global-unions* nil)
 (defvar *ffi-structs*)
+(defvar *ffi-global-structs* nil)
 (defvar *ffi-functions*)
+(defvar *ffi-global-functions* nil)
+(defvar *ffi-global-constants* nil)
 (defvar *ffi-macros*)
 (defvar *ffi-vars*)
 (defvar *ffi-out* t)
@@ -69,20 +70,20 @@
       (setf (gethash string *ffi-structs*)
             (make-ffi-struct :string string
                              :name (unless (digit-char-p (schar string 0))
-                                     (make-keyword (string-upcase string)))))))
+				     (escape-foreign-name string))))))
 
 (defun find-or-create-ffi-union (string)
   (or (gethash string *ffi-unions*)
       (setf (gethash string *ffi-unions*)
             (make-ffi-union :string string
                             :name (unless (digit-char-p (schar string 0))
-                                    (make-keyword (string-upcase string)))))))  
+				    (escape-foreign-name string))))))  
 
 (defun find-or-create-ffi-typedef (string)
   (or (gethash string *ffi-typedefs*)
       (setf (gethash string *ffi-typedefs*)
             (make-ffi-typedef :string string
-                              :name (make-keyword (string-upcase string))))))
+                              :name (escape-foreign-name string)))))
 
 (defun eval-complex-c-expression (string constant-alist)
   (declare (ignore string constant-alist)))
@@ -189,7 +190,7 @@
            (or (ignore-errors (parse-foreign-type spec))
                *the-ffi-void-type*))
          (make-type-name (name)
-           (intern (string-upcase name) *keyword-package*)))
+	   (escape-foreign-name (string name))))
     (cond ((eq (car (last spec)) 'c::*) *the-ffi-pointer-type*)
           ((member (car spec) '(c::|struct| c::|union|))
            (parse-it-or-lose (mapcar #'make-type-name spec)))
@@ -278,7 +279,8 @@
                      (when bits
                        (ash (+ bits 7) -3))))
                   (t
-                   (break "~s" expression)))))
+                   ;(break "~s" expression)
+		   nil))))
              (2
               (let* ((a (car operands))
                      (b (cadr operands)))
@@ -292,9 +294,12 @@
                   (c::\| (logior a b))
                   (c::\& (logand a b))
                   (c::cast (if (foreign-typep b a) b))
-                  (t (break "binary op = ~s ~s ~s" operator a b)))))
+                  (t 
+		   ;(break "binary op = ~s ~s ~s" operator a b)
+		   nil))))
              (t
-              (break "expression = ~s" expression))))))))))
+              ;(break "expression = ~s" expression)
+	      nil)))))))))
 
 (defun eval-c-expression (macro constant-alist macro-table)
   (let* ((string (ffi-macro-expansion macro))
@@ -355,8 +360,12 @@
     (t
      (list :primitive
            (ecase (car spec)
+	     (:char (if (member :signed-char
+			       (ftd-attributes *parse-ffi-target-ftd*))
+		      '(:signed 8)
+		      '(:unsigned 8)))
              (:signed-char  '(:signed 8))
-             ((:char :unsigned-char) '(:unsigned 8))
+             (:unsigned-char '(:unsigned 8))
              (:short '(:signed 16))
              (:unsigned-short '(:unsigned 16))
              ((:int :long)  '(:signed 32))
@@ -376,7 +385,7 @@
 (defun process-ffi-fieldlist (fields)
   (let* ((parsed-fields ()))
     (dolist (field fields (nreverse parsed-fields))
-      (let* ((field-name (make-keyword (string-upcase (car field))))
+      (let* ((field-name (escape-foreign-name (car field)))
              (field-descr (cadr field)))
         (push (list field-name
                     (ecase (car field-descr)
@@ -386,24 +395,30 @@
               parsed-fields)))))
 
 (defun process-ffi-union (form)
-  (let* ((string (caddr form))
-         (fields (cadddr form))
-         (union (find-or-create-ffi-union string)))
-    (setf (ffi-union-ordinal union) (incf *ffi-ordinal*))
-    (unless (ffi-union-fields union)
-      (setf (ffi-union-fields union)
-            (process-ffi-fieldlist fields)))
-    union))
+  (destructuring-bind (source-info string fields &optional alignform)
+      (cdr form)
+    (declare (ignore source-info))
+    (let* ((union (find-or-create-ffi-union string)))
+      (setf (ffi-union-ordinal union) (incf *ffi-ordinal*))
+      (when alignform
+	(setf (ffi-union-alt-alignment-bits union) (cadr alignform)))
+      (unless (ffi-union-fields union)
+	(setf (ffi-union-fields union)
+	      (process-ffi-fieldlist fields)))
+      union)))
 
 (defun process-ffi-struct (form)
-  (let* ((string (caddr form))
-         (fields (cadddr form))
-         (struct (find-or-create-ffi-struct string)))
-    (setf (ffi-struct-ordinal struct) (incf *ffi-ordinal*))
-    (unless (ffi-struct-fields struct)
-      (setf (ffi-struct-fields struct)
-            (process-ffi-fieldlist fields)))
-    struct))
+  (destructuring-bind (source-info string fields &optional alignform)
+      (cdr form)
+    (declare (ignore source-info))
+    (let* ((struct (find-or-create-ffi-struct string)))
+      (setf (ffi-struct-ordinal struct) (incf *ffi-ordinal*))
+      (when alignform
+	(setf (ffi-struct-alt-alignment-bits struct) (cadr alignform)))
+      (unless (ffi-struct-fields struct)
+	(setf (ffi-struct-fields struct)
+	      (process-ffi-fieldlist fields)))
+      struct)))
 
 (defun process-ffi-typedef (form)
   (let* ((string (caddr form))
@@ -489,12 +504,19 @@
     (format *ffi-out* "~&~vt(:union ~s " *ffi-indent* (ffi-union-name u))
     (emit-fieldlist (ffi-union-fields u))
     (format *ffi-out* ")")))
-  
+
+(defun emit-global-union (u)
+  (when *ffi-db-records*
+    (save-ffi-union *ffi-db-records* u)))
+
+(defun record-global-union (u)
+  (when *ffi-global-unions*
+    (setf (gethash (ffi-union-reference u) *ffi-global-unions*) u)))
+
 (defun define-union (u)
   (unless (ffi-union-defined u)
     (setf (ffi-union-defined u) t)
-    (when *ffi-db-records*
-      (save-ffi-union *ffi-db-records* u))
+    (record-global-union u)
     (when (ffi-union-name u)
       (let* ((fields (ffi-union-fields u)))
         (ensure-fields-defined fields)
@@ -516,11 +538,18 @@
     (emit-fieldlist (ffi-struct-fields s))
     (format *ffi-out* ")")))
 
+(defun emit-global-struct (s)
+  (when *ffi-db-records*
+    (save-ffi-struct *ffi-db-records* s)))
+
+(defun record-global-struct (s)
+  (when *ffi-global-structs*
+    (setf (gethash (ffi-struct-reference s) *ffi-global-structs*) s)))
+
 (defun define-struct (s)
   (unless (ffi-struct-defined s)
     (setf (ffi-struct-defined s) t)
-    (when *ffi-db-records*
-      (save-ffi-struct *ffi-db-records* s))
+    (record-global-struct s)
     (when (typep (ffi-struct-name s) 'keyword)
       (let* ((fields (ffi-struct-fields s)))
         (ensure-fields-defined fields)
@@ -536,11 +565,18 @@
       (define-struct s)
       (ensure-fields-defined (ffi-struct-fields s)))))
 
+(defun emit-global-typedef (def)
+  (when *ffi-db-types*
+    (save-ffi-typedef *ffi-db-types* def)))
+
+(defun record-global-typedef (def)
+  (when *ffi-global-typedefs*
+    (setf (gethash (ffi-typedef-string def) *ffi-global-typedefs*) def)))
+  
 (defun define-typedef (def)
   (unless (ffi-typedef-defined def)
     (setf (ffi-typedef-defined def) t)
-    (when *ffi-db-types*
-      (save-ffi-typedef *ffi-db-types* def))
+    (record-global-typedef def)
     (let* ((target (ffi-typedef-type def)))
       (unless (and (consp target)
 		   (member (car target) '(:struct :union :primitive)))
@@ -549,10 +585,17 @@
 	(format *ffi-out* "~&(ccl::def-foreign-type ~s " (ffi-typedef-name def))
 	(emit-type-reference target)
 	(format *ffi-out* ")~&")))))
+
+(defun record-global-constant (name val)
+  (when *ffi-global-constants*
+    (setf (gethash name *ffi-global-constants*) val)))
+
+(defun emit-global-constant (name val)
+  (when *ffi-db-constants*
+    (db-define-constant *ffi-db-constants* name val)))
       
 (defun emit-ffi-constant (name val)
-  (when *ffi-db-constants*
-    (db-define-constant *ffi-db-constants* (string-upcase name) val))
+  (record-global-constant name val)
   (when *ffi-out*
     (format *ffi-out* "~&(cl:defconstant ~a " name)
     (format *ffi-out*
@@ -602,6 +645,15 @@
     (:typedef (ffi-record-type-p (ffi-typedef-type (cadr typeref))))
     (t nil)))
 
+(defun record-global-function (ffi-function)
+  (when *ffi-global-functions*
+    (setf (gethash (ffi-function-string ffi-function) *ffi-global-functions*)
+	  ffi-function)))
+
+(defun emit-global-function (ffi-function)
+  (when *ffi-db-functions*
+    (save-ffi-function *ffi-db-functions* ffi-function)))
+    
 (defun emit-function-decl (ffi-function)
   (let* ((args (ffi-function-arglist ffi-function))
          (retval (ffi-function-return-value ffi-function)))
@@ -609,13 +661,12 @@
       (setq args (butlast args)))
     (when (ffi-record-type-p retval)
       (push retval args)
-      (push retval (ffi-function-arglist ffi-function))
+      (push `(:pointer ,retval) (ffi-function-arglist ffi-function))
       (setf (ffi-function-return-value ffi-function) *ffi-void-reference*)
       (setq retval *ffi-void-reference*))
     (dolist (arg args) (ensure-referenced-type-defined arg))
     (ensure-referenced-type-defined retval)
-    (when *ffi-db-functions*
-      (save-ffi-function *ffi-db-functions* ffi-function))
+    (record-global-function ffi-function)
     (when *ffi-out*
       (format *ffi-out* "~&(ccl::define-external-function ")
       (emit-ffi-function-name (ffi-function-string ffi-function))
@@ -647,7 +698,7 @@
                                (args (ffi-macro-args m)))
                           (if args
                             (setf (gethash (string (ffi-macro-name m)) argument-macros) args)
-                          (push m defined-macros))))
+			    (push m defined-macros))))
                 (:type (push (process-ffi-typedef form) defined-types))
                 (:var (process-ffi-var form))
                 (:enum-ident (push (process-ffi-enum-ident form) defined-constants))
@@ -655,35 +706,35 @@
                 (:union (push (process-ffi-union form) defined-types)))))
           (multiple-value-bind (new-constants new-macros)
               (process-defined-macros defined-macros (reverse defined-constants) argument-macros)
-          ;;; If we're really lucky, we might be able to turn some C macros
-          ;;; into lisp macros.  We can probably turn some C macros into
-          ;;; lisp constants.
+	    ;; If we're really lucky, we might be able to turn some C macros
+	    ;; into lisp macros.  We can probably turn some C macros into
+	    ;; lisp constants.
             (declare (ignore new-macros))
 	    (let* ((*ffi-out* (when outpath
 				(open outpath
-				  :direction :output
-				  :if-exists :supersede
-				  :if-does-not-exist :create))))
+				      :direction :output
+				      :if-exists :supersede
+				      :if-does-not-exist :create))))
 	    
-	    (unwind-protect
-              (let* ((*print-case* :downcase))
-		(when *ffi-out*
-		  (format *ffi-out* "~%~%(cl::in-package ~a)~%~%" package-name))
-                (dolist (x (reverse new-constants))
-                  (emit-ffi-constant (car x) (cdr x)))
-		(when *ffi-out*
-		  (terpri *ffi-out*)
-		  (terpri *ffi-out*))
-                (dolist (x (sort defined-types #'< :key #'ffi-type-ordinal))
-                  (typecase x
-                    (ffi-struct (define-struct x))
-                    (ffi-union (define-union x))
-                    (ffi-typedef (define-typedef x))))
-		(when *ffi-out*
-		  (terpri *ffi-out*)
-		  (terpri *ffi-out*))
-                (dolist (f defined-functions) (emit-function-decl f)))
-	      (when *ffi-out* (close *ffi-out*))))
+	      (unwind-protect
+		   (let* ((*print-case* :downcase))
+		     (when *ffi-out*
+		       (format *ffi-out* "~%~%(cl::in-package ~a)~%~%" package-name))
+		     (dolist (x (reverse new-constants))
+		       (emit-ffi-constant (car x) (cdr x)))
+		     (when *ffi-out*
+		       (terpri *ffi-out*)
+		       (terpri *ffi-out*))
+		     (dolist (x (sort defined-types #'< :key #'ffi-type-ordinal))
+		       (typecase x
+			 (ffi-struct (define-struct x))
+			 (ffi-union (define-union x))
+			 (ffi-typedef (define-typedef x))))
+		     (when *ffi-out*
+		       (terpri *ffi-out*)
+		       (terpri *ffi-out*))
+		     (dolist (f defined-functions) (emit-function-decl f)))
+		(when *ffi-out* (close *ffi-out*))))
             outpath))))))
 
 (defun lisp-pathname-from-ffi-pathname (ffi-pathname)
@@ -696,35 +747,64 @@
                    :name (pathname-name inpath)
                    :type "lisp")))
 
+
 (defun parse-standard-ffi-files (dirname &key
 					 (ftd *target-ftd*)
 					 (write-lisp-file nil)
-					 (prepend-underscores nil))
-  (let* ((d (use-interface-dir dirname ftd))
+					 (prepend-underscores
+					  #+(or darwinppc-target) t
+					  #-(or darwinppc-target) nil))
+  (let* ((*parse-ffi-target-ftd* ftd)
+	 (d (use-interface-dir dirname ftd))
 	 (interface-dir (merge-pathnames
 			 (interface-dir-subdir d)
 			 (ftd-interface-db-directory ftd)))
 	 (*prepend-underscores-to-ffi-function-names* prepend-underscores)
-	 (package-name (ftd-interface-package-name ftd)))
+	 (package-name (ftd-interface-package-name ftd))
+	 (*ffi-global-typedefs* (make-hash-table :test 'string= :hash-function 'sxhash))
+	 (*ffi-global-unions* (make-hash-table :test 'string= :hash-function 'sxhash))
+	 (*ffi-global-structs* (make-hash-table :test 'string= :hash-function 'sxhash))
+	 (*ffi-global-functions* (make-hash-table :test 'string= :hash-function 'sxhash))
+	 (*ffi-global-constants* (make-hash-table :test 'string= :hash-function 'sxhash)))
+         
+    (dolist (f (directory (merge-pathnames ";C;**;*.ffi"
+					   interface-dir)))
+      (format t "~&~s ..." f)
+      (format t " ~s" (parse-ffi f
+				 (if write-lisp-file
+				   (lisp-pathname-from-ffi-pathname f))
+				 :package-name package-name )))
     (with-new-db-file (*ffi-db-constants* (merge-pathnames
-					   "new-constants.db"
+					   "new-constants.cdb"
 					   interface-dir))
-      (with-new-db-file (*ffi-db-types* (merge-pathnames
-					 "new-types.db"
+      (maphash #'(lambda (name def)
+		   (emit-global-constant name def))
+	       *ffi-global-constants*))
+    (with-new-db-file (*ffi-db-types* (merge-pathnames
+				       "new-types.cdb"
+				       interface-dir))
+      (maphash #'(lambda (name def)
+		   (declare (ignore name))
+		   (emit-global-typedef def))
+	       *ffi-global-typedefs*))
+    (with-new-db-file (*ffi-db-records* (merge-pathnames
+					 "new-records.cdb"
 					 interface-dir))
-	(with-new-db-file (*ffi-db-records* (merge-pathnames
-					     "new-records.db"
-					     interface-dir))
-	  (with-new-db-file (*ffi-db-functions* (merge-pathnames
-						 "new-functions.db"
-						 interface-dir))
-	    (dolist (f (directory (merge-pathnames ";C;**;*.ffi"
-						   interface-dir)))
-	      (format t "~&~s ..." f)
-	      (format t " ~s" (parse-ffi f
-					 (if write-lisp-file
-					   (lisp-pathname-from-ffi-pathname f))
-					:package-name package-name )))))))
+      (maphash #'(lambda (name def)
+		   (declare (ignore name))
+		   (emit-global-union def))
+	       *ffi-global-unions*)
+      (maphash #'(lambda (name def)
+		   (declare (ignore name))
+		   (emit-global-struct def))
+	       *ffi-global-structs*))
+    (with-new-db-file (*ffi-db-functions* (merge-pathnames
+					   "new-functions.cdb"
+					   interface-dir))
+      (maphash #'(lambda (name def)
+		   (declare (ignore name))
+		   (emit-global-function def))
+	       *ffi-global-functions*))
     (install-new-db-files ftd d)))
 
 (defvar *c-readtable* (copy-readtable nil))
@@ -1284,35 +1364,34 @@
 	     (let* ((path (merge-pathnames path dir))
 		    (newpath (merge-pathnames newpath dir)))
 	       (when was-open
-		 (%db-close was-open))
+		 (cdb-close was-open))
 	       (when (probe-file path)
 		 (rename-file path
 			      (concatenate 'string (namestring path) "-BAK")
 			      :if-exists :supersede))
-	       (compact-db-file newpath path)
-	       (delete-file newpath)
+	       (rename-file newpath path)
 	       (when was-open
-		 (open-input-db-file path)))))
+		 (cdb-open path)))))
       (without-interrupts
        (setf (interface-dir-constants-interface-db-file d)
 	     (rename-and-reopen
 	      (interface-dir-constants-interface-db-file d)
-	      "constants.db"
-	      "new-constants.db"))
+	      "constants.cdb"
+	      "new-constants.cdb"))
        (setf (interface-dir-functions-interface-db-file d)
 	     (rename-and-reopen
 	      (interface-dir-functions-interface-db-file d)
-	      "functions.db"
-	      "new-functions.db"))
+	      "functions.cdb"
+	      "new-functions.cdb"))
        (setf (interface-dir-records-interface-db-file d)
 	     (rename-and-reopen
 	      (interface-dir-records-interface-db-file d)
-	      "records.db"
-	      "new-records.db"))
+	      "records.cdb"
+	      "new-records.cdb"))
        (setf (interface-dir-types-interface-db-file d)
 	     (rename-and-reopen
 	      (interface-dir-types-interface-db-file d)
-	      "types.db"
-	      "new-types.db")))))
+	      "types.cdb"
+	      "new-types.cdb")))))
   t)
    
\ No newline at end of file
