@@ -1,21 +1,17 @@
 /*
    Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Opensourced MCL.
+   This file is part of OpenMCL.  
 
-   Opensourced MCL is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+   License , known as the LLGPL and distributed with OpenMCL as the
+   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+   which is distributed with OpenMCL as the file "LGPL".  Where these
+   conflict, the preamble takes precedence.  
 
-   Opensourced MCL is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   OpenMCL is referenced in the preamble as the "LIBRARY."
 
+   The LLGPL is also available online at
+   http://opensource.franz.com/preamble.html
 */
 
 
@@ -39,7 +35,6 @@
 #include "bits.h"
 #include "gc.h"
 #include "area.h"
-#include "pef.h"
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -90,7 +85,7 @@ add_area_before(area *new_area, area *before)
 void
 add_area(area *new_area)
 {
-  area *that = KG->all_areas;
+  area *that = all_areas;
   int
     thiscode = (int)(new_area->code),
     thatcode;
@@ -114,7 +109,7 @@ add_area(area *new_area)
 static area *
 find_area_forward(BytePtr addr, area_code mincode)
 {
-  area *p, *header = KG->all_areas;
+  area *p, *header = all_areas;
 
   for (p = header->succ; p != header; p = p->succ) {
     area_code pcode = p->code;
@@ -139,7 +134,7 @@ find_area_forward(BytePtr addr, area_code mincode)
 static area *
 find_area_backward(BytePtr addr, area_code maxcode)
 {
-  area *p, *header = KG->all_areas;
+  area *p, *header = all_areas;
 
   for (p = header->pred; p != header; p = p->pred) {
     area_code pcode = p->code;
@@ -350,12 +345,12 @@ tenure_to_area(area *target)
   BytePtr 
     curfree = a->active,
     target_low = target->low,
-    tenured_low = KG->tenured_area->low;
+    tenured_low = tenured_area->low;
   unsigned 
     dynamic_dwords = area_dword(curfree, a->low),
-    new_tenured_dwords = area_dword(curfree, KG->tenured_area->low);
+    new_tenured_dwords = area_dword(curfree, tenured_area->low);
   bitvector 
-    refbits = KG->tenured_area->refbits,
+    refbits = tenured_area->refbits,
     markbits = a->markbits,
     new_markbits;
 
@@ -372,7 +367,7 @@ tenure_to_area(area *target)
 
   new_markbits = refbits + ((new_tenured_dwords + 31) >> 5);
   
-  if (target == KG->tenured_area) {
+  if (target == tenured_area) {
     zero_bits(refbits, new_tenured_dwords);
     lisp_global(OLDEST_EPHEMERAL) = (LispObj) curfree;
   } else {
@@ -399,7 +394,7 @@ untenure_from_area(area *from)
   if (lisp_global(OLDEST_EPHEMERAL) != 0) {
     area *a = active_dynamic_area, *child;
     BytePtr curlow = from->low;
-    unsigned new_tenured_dwords = area_dword(curlow, KG->tenured_area->low);
+    unsigned new_tenured_dwords = area_dword(curlow, tenured_area->low);
     
     for (child = from; child != a; child = child->younger) {
       child->low = child->active = child->high = curlow;
@@ -409,8 +404,8 @@ untenure_from_area(area *from)
     a->low = curlow;
     a->ndwords = area_dword(a->high, curlow);
     
-    a->markbits = (KG->tenured_area->refbits) + ((new_tenured_dwords+31)>>5);
-    if (from == KG->tenured_area) {
+    a->markbits = (tenured_area->refbits) + ((new_tenured_dwords+31)>>5);
+    if (from == tenured_area) {
       /* Everything's in the dynamic area */
       lisp_global(OLDEST_EPHEMERAL) = 0;
     }
@@ -429,11 +424,11 @@ egc_control(Boolean activate, BytePtr curfree)
       a->active = curfree;
     }
     if (activate) {
-      a->older = KG->g1_area;
-      tenure_to_area(KG->tenured_area);
+      a->older = g1_area;
+      tenure_to_area(tenured_area);
       egc_is_active = true;
     } else {
-      untenure_from_area(KG->tenured_area);
+      untenure_from_area(tenured_area);
       a->older = NULL;
       egc_is_active = false;
     }
@@ -489,6 +484,7 @@ delete_protected_area(protected_area_ptr p)
 void
 condemn_area(area *a)
 {
+  void free_stack(void *);
   area *prev = a->pred, *next = a->succ;
   Handle h = a->h;
   protected_area_ptr p;
@@ -503,7 +499,7 @@ condemn_area(area *a)
 
   if (p) delete_protected_area(p);
 
-  if (h) deallocate(h);
+  if (h) free_stack(h);
   deallocate((Ptr)a);
 }
 
@@ -1216,6 +1212,7 @@ mark_cstack_area(area *a)
       Bug(NULL, "Child stack frame older than parent");
     }
 #endif
+    if (next == NULL) break;
     if (((next - current) == sizeof(lisp_frame)) &&
 	(((((lisp_frame *)current)->savefn) == 0) ||
 	 (fulltag_of(((lisp_frame *)current)->savefn) == fulltag_misc))) {
@@ -2072,6 +2069,7 @@ forward_cstack_area(area *a)
 
   for (current = a->active; (current >= low) && (current < limit); current = next) {
     next = *((BytePtr *)current);
+    if (next == NULL) break;
     if (((next - current) == sizeof(lisp_frame)) &&
 	(((((lisp_frame *)current)->savefn) == 0) ||
 	 (fulltag_of(((lisp_frame *)current)->savefn) == fulltag_misc))) {
@@ -2253,7 +2251,7 @@ compact_dynamic_heap()
       unsigned nbytes = (unsigned)dest - (unsigned)GCfirstunmarked;
 
       if (nbytes != 0) {
-        MakeDataExecutable((LogicalAddress)GCfirstunmarked, nbytes);
+        xMakeDataExecutable((LogicalAddress)GCfirstunmarked, nbytes);
       }
     }
   }
@@ -2264,7 +2262,7 @@ int
 find_memoized_areas( BytePtr *lowp, BytePtr *highp)
 {
   area *a = active_dynamic_area;
-  BytePtr high = a->low, low = KG->tenured_area->low;
+  BytePtr high = a->low, low = tenured_area->low;
   int n = 1;
 
   if (low == high) {
@@ -2292,14 +2290,14 @@ note_memoized_references(ExceptionInformation *xp,
   unsigned dword, ndwords;
   bitvector refbits;
 
-  if (KG->tenured_area && (find_memoized_areas(&low, &high))) {
+  if (tenured_area && (find_memoized_areas(&low, &high))) {
     /* Some areas have refbits.  The lowest memoized
        address is "low" & the highest is "high".  Bang
        through the memoization buffer; if the memoized
        address was in range, look to see if the address
        belongs to a memoized area and, if it does, set
        the corresponding doubleword bit. */
-    refbits = KG->tenured_area->refbits;
+    refbits = tenured_area->refbits;
     ndwords = area_dword(high, low);
     while (startp < endp) {
       next = *startp++;
@@ -2345,7 +2343,7 @@ memoize_binding_stack(special_binding *chain, BytePtr low, BytePtr high)
 {
   BytePtr x;
   unsigned dword, max = area_dword(high, low);
-  bitvector refbits = KG->tenured_area->refbits;
+  bitvector refbits = tenured_area->refbits;
 
   while (chain) {
     x = (BytePtr) (chain->sym) + ((offsetof(lispsymbol, vcell) - fulltag_misc));
@@ -2367,7 +2365,7 @@ memoize_special_bindings(BytePtr low, BytePtr high)
   memoize_binding_stack((special_binding *) lisp_global(DB_LINK), low, high);
 
   /* Do the same for other stack groups. */
-  for (a = KG->all_areas->pred; a->code < AREA_READONLY; a = a->pred) {
+  for (a = all_areas->pred; a->code < AREA_READONLY; a = a->pred) {
     if ((a->code == AREA_TSTACK) && 
         (a != ((area *) lisp_global(CURRENT_TS))) &&
         youngest_non_null_area_p(a)) {
@@ -2456,23 +2454,23 @@ gc(xframe_list *xframes)
 
   if (GCephemeral_low) {
     GCn_ephemeral_dwords = area_dword(oldfree, GCephemeral_low);
-    if ((oldfree-KG->g1_area->low) < KG->g1_area->threshold) {
-      to = KG->g1_area;
+    if ((oldfree-g1_area->low) < g1_area->threshold) {
+      to = g1_area;
       note = a;
       timeidx = 4;
-    } else if ((oldfree-KG->g2_area->low) < KG->g2_area->threshold) {
-      to = KG->g2_area;
-      from = KG->g1_area;
-      note = KG->g1_area;
+    } else if ((oldfree-g2_area->low) < g2_area->threshold) {
+      to = g2_area;
+      from = g1_area;
+      note = g1_area;
       timeidx = 3;
     } else {
-      to = KG->tenured_area;
-      from = KG->g2_area;
-      note = KG->g2_area;
+      to = tenured_area;
+      from = g2_area;
+      note = g2_area;
       timeidx = 2;
     } 
   } else {
-    note = KG->tenured_area;
+    note = tenured_area;
 #ifdef mac_os
 `    if (! (nrs_GC_EVENT_STATUS_BITS.vcell & gc_use_gc_cursor)) {
       if (gc_cursor) {
@@ -2555,7 +2553,7 @@ gc(xframe_list *xframes)
       case AREA_STATIC:
       case AREA_DYNAMIC:                  /* some heap that isn't "the" heap */
         /* In both of these cases, we -could- use the area's "markbits"
-           bitvector as a reference map.  It's safe (but slower) to
+           bitvector as a reference map.  It's safe (but slower) to
            ignore that map and process the entire area.
            */
         if ((memohigh == NULL) || (next_area->younger == NULL)) {
@@ -2570,7 +2568,7 @@ gc(xframe_list *xframes)
   }
   
   if (lisp_global(OLDEST_EPHEMERAL)) {
-    mark_memoized_area(KG->tenured_area, area_dword(a->low,KG->tenured_area->low));
+    mark_memoized_area(tenured_area, area_dword(a->low,tenured_area->low));
   }
 
   /* The xframes in other stack-groups have been marked by mark_stack_group_cframe */
@@ -2700,7 +2698,7 @@ gc(xframe_list *xframes)
   }
   
   if (GCephemeral_low) {
-    forward_memoized_area(KG->tenured_area, area_dword(a->low, KG->tenured_area->low));
+    forward_memoized_area(tenured_area, area_dword(a->low, tenured_area->low));
   }
 
   
@@ -2723,17 +2721,19 @@ gc(xframe_list *xframes)
      If this was a full GC and there's now room for the youngest generation,
      tenure everything.
      */
-  reprotect_dynamic_heap(xframes->curr, a->active, (GCephemeral_low == 0) ? lisp_heap_gc_threshold : 0 );
+  reprotect_dynamic_heap(a->active,
+			 (GCephemeral_low == 0) ? lisp_heap_gc_threshold : 0,
+			 (GCephemeral_low != 0));
 
   if (a->older != NULL) {
     unsigned nfree = (a->high - a->active);
 
 
     if (nfree < a->threshold) {
-      untenure_from_area(KG->tenured_area);
+      untenure_from_area(tenured_area);
     } else {
       if (GCephemeral_low == 0) {
-        tenure_to_area(KG->tenured_area);
+        tenure_to_area(tenured_area);
       }
     }
   }
@@ -2788,7 +2788,8 @@ gc(xframe_list *xframes)
 #endif
 }
 
-
+      
+    
 /*
   Total the (physical) byte sizes of all ivectors in the indicated memory range
 */
@@ -3044,6 +3045,7 @@ purify_cstack_area(area *a, BytePtr low, BytePtr high, area *to, int what)
 
   for (current = a->active; current != limit; current = next) {
     next = *((BytePtr *)current);
+    if (next == NULL) break;
     if (((next - current) == sizeof(lisp_frame)) && 
 	(((((lisp_frame *)current)->savefn) == 0) ||
 	 (fulltag_of(((lisp_frame *)current)->savefn) == fulltag_misc))) {
@@ -3154,27 +3156,23 @@ purify_areas(BytePtr low, BytePtr high, area *target, int what)
 void
 purify(ExceptionInformation *xp)
 {
+  extern area *extend_readonly_area(unsigned);
   xframe_list xframes, *x;
   area 
     *a = active_dynamic_area,
     *new_pure_area;
 
   unsigned max_pure_size;
-  Handle h;
   OSErr err;
+  BytePtr new_pure_start;
 
   xframes.prev = (xframe_list *) lisp_global(XFRAME);
   xframes.curr = xp;
 
   max_pure_size = unboxed_bytes_in_range((LispObj *)a->low, (LispObj *) a->active);
-  h = malloc(max_pure_size+7);
-  if (h) {
-    unsigned base;
-    base = (((unsigned ) h) +7) & ~7;
-    new_pure_area = new_area((BytePtr)base, (BytePtr)(base+max_pure_size), AREA_READONLY);
-    new_pure_area->active = new_pure_area->low;
-    new_pure_area->h = h;
-    add_area(new_pure_area);
+  new_pure_area = extend_readonly_area(max_pure_size);
+  if (new_pure_area) {
+    new_pure_start = new_pure_area->active;
     lisp_global(IN_GC) = (1<<fixnumshift);
 
     /* 
@@ -3187,7 +3185,7 @@ purify(ExceptionInformation *xp)
       Make the new_pure_area executable, just in case.
 
       Caller will typically GC again (and that should recover quite a bit of
-      the dynamic heap.
+      the dynamic heap.)
       */
 
     {
@@ -3234,308 +3232,22 @@ purify(ExceptionInformation *xp)
     purify_dohs(a->low, a->active, new_pure_area, COPY_CODE);
 
     {
-      unsigned puresize = (unsigned) (new_pure_area->active-new_pure_area->low);
+      unsigned puresize = (unsigned) (new_pure_area->active-new_pure_start);
       if (puresize != 0) {
-        MakeDataExecutable((LogicalAddress)(new_pure_area->low), puresize);
+        xMakeDataExecutable(new_pure_start, puresize);
+  
       }
     }
+    ProtectMemory(new_pure_area->low,
+		  align_to_power_of_2(new_pure_area->active-new_pure_area->low,
+				      12));
     lisp_global(IN_GC) = 0;
     just_purified_p = true;
   }
 }
 
 
-extern mutable_data_section_header **next_libP;
-extern unsigned next_libnum;
-
-Boolean
-get_lib_timestamp(mutable_data_section_header *libP)
-{
-  /* If the library's name ends in "@xxxxxxxx", where
-     each "x" is a hex digit, parse the hex string into
-     and use the result as a timestamp.  Otherwise, set
-     the timestamp from the current wall clock time. */
-
-  unsigned long libtime;
-  char *libname, *at_pos, *end_pos;
-  Boolean found_timestamp = false;
-
-  libname = libP->libname+1;
-  at_pos = strrchr(libname, '@');
-  if (at_pos) {
-    at_pos++;
-    if (strlen(at_pos) == 8) {
-      libtime = strtoul(at_pos, &end_pos, 16);
-      if (*end_pos == 0) {
-        found_timestamp = true;
-        /* 
-          We don't really need a version number
-          if the fragment name has a @xxxxxxxx suffix,
-          and library version numbers confuse GetSharedLibrary().
-          */
-        
-        libtime = 0;           
-      }
-    }
-  }
-  if (found_timestamp == false) {
-    time(&libtime);
-  }
-  libP->timestamp = libtime;
-  return found_timestamp;
-}
-
-OSErr
-lib_purify(ExceptionInformation *xp, char *libname, LispObj *start, LispObj *end)
-{
-  xframe_list xframes, *x;
-  area 
-    *a = active_dynamic_area,
-    *new_pure_area,
-    *staticlib_area,
-    *mutable_data_area;
-
-  int i;
-  BytePtr low = (BytePtr)start, high = (BytePtr) end;
-  LispObj *current;
-  unsigned elements, tag, subtag, lfbits;
-  mutable_data_section_header *libP;
-  unsigned max_pure_size;
-  Handle h;
-  OSErr err;
-  xframes.prev = (xframe_list *) lisp_global(XFRAME);
-  xframes.curr = xp;
-
-  max_pure_size = unboxed_bytes_in_range(start, end);
-
-  /* Make sure that we prepend a copy of the subprims to the readlonly section. */
-  {
-    LispObj *subprims = (LispObj *) lisp_global(READONLY_SECTION_START), header = *subprims;
-    unsigned subpbytes = ((header_element_count(header)<<2)+7)&~7;
-    
-    max_pure_size += subpbytes;
-
-    h = malloc(max_pure_size+7);
-    
-    if (h == NULL) {
-      return -1;
-    } else {
-      unsigned base;
-      base = (((unsigned) h) +7) & ~7;
-      new_pure_area = new_area((BytePtr)base, (BytePtr)(base+max_pure_size), AREA_READONLY);
-      new_pure_area->active = new_pure_area->low+subpbytes;
-      new_pure_area->h = h;
-      new_pure_area->owner = next_libnum;
-      add_area(new_pure_area);
-      memcpy(new_pure_area->low, (Ptr)subprims,  subpbytes);
-
-      h = allocate(15);
-      if (h == NULL) {
-        condemn_area(new_pure_area);
-        return -1;
-      }
-      base = (((unsigned) h) +7) & ~7;
-      staticlib_area = new_area((BytePtr)base, (BytePtr)(base+8), AREA_STATICLIB);
-      staticlib_area->h = h;
-      staticlib_area->owner = next_libnum;
-      add_area(staticlib_area);
-
-      h = malloc(11+sizeof(mutable_data_section_header));
-      if (h == NULL) {
-        condemn_area(staticlib_area);
-        condemn_area(new_pure_area);
-        return -1;
-      }
-      base = (((unsigned) h) +7) & ~7;
-      mutable_data_area = new_area((BytePtr)base, (BytePtr)(base+4+sizeof(mutable_data_section_header)), AREA_STATIC);
-      mutable_data_area->h = h;
-      mutable_data_area->owner = next_libnum;
-      add_area(mutable_data_area);
-      *(LispObj*)base = make_header(subtag_u32_vector, (sizeof(mutable_data_section_header)/sizeof(LispObj)));
-
-      libP = (mutable_data_section_header *) (base+(sizeof(LispObj)));
-      libP->next = NULL;
-      libP->flags = 0;
-      libP->mutable_low = mutable_data_area->low;
-      libP->mutable_high = mutable_data_area->active;
-      libP->immutable_low = staticlib_area->low;
-      libP->immutable_high = staticlib_area->active;
-      for (i=0; i < 64; i++) {
-        libP->libname[i] = 0;
-      }
-      strcpy(libP->libname, libname);
-      if (get_lib_timestamp(libP)) {
-        libP->flags |= kWeakImport;
-      }
-      *next_libP = libP;
-      next_libP = &(libP->next);
-      next_libnum++;
-
-      lisp_global(IN_GC) = (1<<fixnumshift);
-
-      /* Walk the dynamic heap between 'start' and 'end'.  If
-         we find a function whose lfun_bits indicate that its
-         code vector isn't shared (trampoline bit clear), copy
-         it to the readonly section if the code vector is also
-         in the indicated address range.  If we find a symbol
-         whose pname is in the address range, copy the pname
-         to the readonly section.  Then loop through all_areas,
-         forwarding references to copied objects.  We shouldn't
-         find any, but might find some PCs on the stacks or in
-         the xframe list.  Fixup the staticlib_area and return;
-         ready to call save_library.
-
-         Walk the dynamic heap twice: on the first pass, copy
-         code vectors; on the second, copy pnames.
-         */
-
-      
-      current = start;
-      while (current < end) {
-        header = *current;
-        if (header == forward_marker) {
-          current += 2;
-        } else {
-          tag = fulltag_of(header);
-          if (tag == fulltag_immheader) {
-            current = (LispObj *)skip_over_ivector((unsigned)current, header);
-          } else if (tag != fulltag_nodeheader) {
-            current += 2;
-          } else {
-            subtag = header_subtag(header);
-            elements = header_element_count(header);
-
-            if (subtag == subtag_function) {
-              lfbits = deref(current, elements);
-              if ((lfbits & lfbits_trampoline_mask) == 0) {
-                copy_ivector_reference(current+1, low, high, new_pure_area, COPY_CODE);
-              }
-            }
-            current += ((2+elements) & ~1);
-          }
-        }
-      }
-      
-      /* Same thing, only copy pnames this time. */
-
-      current = start;
-      while (current < end) {
-        header = *current;
-        if (header == forward_marker) {
-          current += 2;
-        } else {
-          tag = fulltag_of(header);
-          if (tag == fulltag_immheader) {
-            current = (LispObj *)skip_over_ivector((unsigned)current, header);
-          } else if (tag != fulltag_nodeheader) {
-            current += 2;
-          } else {
-            subtag = header_subtag(header);
-            elements = header_element_count(header);
-
-            if (subtag == subtag_symbol) {
-              copy_ivector_reference(current+1, low, high, new_pure_area, COPY_STRINGS);
-            }
-            current += ((2+elements) & ~1);
-          }
-        }
-      }
-      
-      ((BytePtr *)staticlib_area->low)[0] = new_pure_area->low;
-      ((BytePtr *)staticlib_area->low)[1] = new_pure_area->active;
-
-      purify_areas(low, high, new_pure_area, FORWARD_ONLY);
-    
-      /* Other stack-groups' xframe_lists have already been handled by purify_stack_group_cframe */
-      for (x = &xframes; x; x = x->prev) {
-        purify_xp(x->curr, low, high, new_pure_area, FORWARD_ONLY);
-      }
-
-      purify_dohs(a->low, a->active, new_pure_area, FORWARD_ONLY);
-
-      {
-        unsigned puresize = (unsigned) (new_pure_area->active-new_pure_area->low);
-        if (puresize != 0) {
-          MakeDataExecutable((LogicalAddress)(new_pure_area->low), puresize);
-        }
-      }
-      lisp_global(IN_GC) = 0;
-      just_purified_p = true;
-    }
-  }
-  return err;
-}
-
-/* This gets ff-called from lisp.
-   lib_purify created a readonly area for the library in which
-   the subprims (and all other code vectors) precede pnames.
-   Split this area in two, so that the code vectors get orphaned
-   and the pnames go in a new area (whose "owner" is the application.)
-   Unlink any areas owned by libnum from the library chain and orphan
-   them; decrement the "owner" fields of any areas > libnum, and
-   decrement next_libnum.
-
-   save_application will have to check for references to the orphaned
-   code vector area and turn them into references to %excised-code%.
-*/
-
-void
-excise_library(unsigned libnum)
-{
-  if ((libnum >= 2) && (libnum < next_libnum)) {
-    static_header *static_headerP = (static_header *) (&(lisp_global(STATIC_HEAP_START)));
-    mutable_data_section_header **prevP = &(static_headerP->next), *libP = *prevP;
-    unsigned i;
-    area *a, *old_lib_pure_area = NULL;
-
-    for (i = 2; i < libnum; i++, libP = *prevP) {
-      prevP = &(libP->next);
-    }
-    *prevP = (libP->next);
-    next_libnum--;
-    for (a = active_dynamic_area;a->code != AREA_VOID; a = a->succ) {
-      i = a->owner;
-
-      if (i >= libnum) {
-        if (i == libnum) {
-          a->owner = 0xffffffff;
-          if (a->code == AREA_READONLY) {
-            old_lib_pure_area = a;
-            a->owner = 1;       /* "owned" by application */
-          }
-        } else {
-          a->owner = i-1;
-        }
-      }
-    }
-
-    {
-      /* Split old_lib_pure_area in two. */
-      BytePtr 
-        code_start = old_lib_pure_area->low,
-        code_end,
-        pname_end = old_lib_pure_area->high;
-      LispObj 
-        *start = (LispObj *) code_start,
-        *end = (LispObj *) pname_end,
-        *current,
-        header;
 
 
-      for (current = start; 
-           current < end; 
-           current  = (LispObj *)skip_over_ivector((unsigned)current, header)) {
-        header = *current;
-        if (header_subtag(header) != subtag_code_vector) {
-          break;
-        }
-      }
-      code_end = (BytePtr) current;
-      old_lib_pure_area->low = code_end;
-      old_lib_pure_area->ndwords = area_dword(pname_end, code_end);
-      a = new_area(code_start, code_end, AREA_READONLY);
-      a->owner = 0xffffffff;
-      add_area(a);
-    }
-  }
-}
+
+
