@@ -1,21 +1,17 @@
 /*
    Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Opensourced MCL.
+   This file is part of OpenMCL.  
 
-   Opensourced MCL is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+   License , known as the LLGPL and distributed with OpenMCL as the
+   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+   which is distributed with OpenMCL as the file "LGPL".  Where these
+   conflict, the preamble takes precedence.  
 
-   Opensourced MCL is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   OpenMCL is referenced in the preamble as the "LIBRARY."
 
+   The LLGPL is also available online at
+   http://opensource.franz.com/preamble.html
 */
 
 #include "lisp-exceptions.h"
@@ -173,13 +169,13 @@ zero_cache_lines(BytePtr, size_t, size_t);
 void
 zero_page(BytePtr start)
 {
-  zero_cache_lines(start, (page_size/KG->cache_block_size), KG->cache_block_size);
+  zero_cache_lines(start, (page_size/cache_block_size), cache_block_size);
 }
 
 void
 zero_heap_segment(BytePtr start)
 {
-#if defined(LINUX) || defined(DARWIN)
+#if 0 && defined(LINUX) || defined(DARWIN)
   /* We can use mmap() to establish a new mapping with read/write access */
   if (start != mmap(start,
 		    heap_segment_size,
@@ -191,7 +187,7 @@ zero_heap_segment(BytePtr start)
     exit(1);
   }
 #else
-  zero_cache_lines(start, (heap_segment_size/KG->cache_block_size), KG->cache_block_size);
+  zero_cache_lines(start, (heap_segment_size/cache_block_size), cache_block_size);
 #endif
 }
 
@@ -224,10 +220,28 @@ signal_stack_soft_overflow(ExceptionInformation *xp, unsigned reg)
      control stack to do so until all stacks are well under their overflow
      limits. */
 
-  lisp_global(CS_OVERFLOW_LIMIT) = 0xfffffff8; /* force unsigned traps to fail */
+#if 0
+  lisp_global(CS_OVERFLOW_LIMIT) = CS_OVERFLOW_FORCE_LIMIT; /* force unsigned traps to fail */
+#endif
   handle_error(xp, error_stack_overflow, reg, 0, (unsigned) xpPC(xp));
 }
 
+void
+restore_soft_stack_limit(unsigned stkreg)
+{
+  area *a;
+
+  switch (stkreg) {
+  case sp:
+    a = (area *) lisp_global(CURRENT_CS);
+    lisp_global(CS_OVERFLOW_LIMIT) = (LispObj)a->softlimit;
+    break;
+  case vsp:
+    a = (area *) lisp_global(CURRENT_VS);
+    protect_area(a->softprot);
+    break;
+  }
+}
 extern LispObj (*start_lisp)(LispObj, LispObj);
 
 void
@@ -259,9 +273,9 @@ reset_lisp_process(ExceptionInformation *xp)
    and can give them back, do so.
 */
 void
-reprotect_dynamic_heap(ExceptionInformation *xp, 
-		       BytePtr newfree, 
-		       unsigned free_space_size)
+reprotect_dynamic_heap(BytePtr newfree, 
+		       unsigned free_space_size,
+		       Boolean for_egc)
 {
   unsigned protbytes;
   area *a = active_dynamic_area;
@@ -274,7 +288,7 @@ reprotect_dynamic_heap(ExceptionInformation *xp,
   heap_soft_area->start = heap_soft_area->end;
   unprotect_area(heap_hard_area);
   if (free_space_size) {
-    BytePtr lowptr = (BytePtr)(lisp_global(HEAP_START));
+    BytePtr lowptr = (BytePtr)lisp_global(HEAP_START);
     newlimit = lowptr + align_to_power_of_2(newfree-lowptr+free_space_size,
 					    log2_heap_segment_size);
     if (newlimit > heap_hard_area->end) {
@@ -284,11 +298,14 @@ reprotect_dynamic_heap(ExceptionInformation *xp,
     }
   }
 
+  protect_area(heap_hard_area);
   protbytes = (((heap_soft_area->start)-newfree)&~(heap_segment_size-1));
+  protptr = heap_soft_area->start-protbytes;
+  commit_pages(protptr, protbytes);
   if (protbytes) {
-    protect_area_prefix(heap_soft_area, protbytes);
+    protect_area_prefix(heap_soft_area, 
+			for_egc ? protbytes : heap_segment_size);
   }
-  protptr = heap_soft_area->start;
   protbytes = (protptr-newfree);
   while (protbytes >= page_size) {
     protptr -= page_size;
@@ -623,7 +640,7 @@ do_heap_hard_probe(ExceptionInformation *xp, protected_area_ptr p, BytePtr addr)
   have = (LispObj) (p->start) - curfreeptr;
   
   
-  untenure_from_area(KG->tenured_area); /* force a full GC */
+  untenure_from_area(tenured_area); /* force a full GC */
   freed = gc_from_xp(xp);
   got = have + freed;
 
@@ -799,9 +816,10 @@ sample_preserved_registers (ExceptionInformation *xp, preserved_registers *pr)
 /* Note: CURRENT_VS (CURRENT_TS) is always either the area containing
   the current value of VSP (TSP) or an older area.  */
 
+#ifdef EXTEND_VSTACK
+
 /* Find or create a new VSP segment. Copy the top-most stack frame to
   it */
-
 OSStatus
 do_vsp_overflow (ExceptionInformation *xp, BytePtr addr)
 {
@@ -858,8 +876,8 @@ do_vsp_overflow (ExceptionInformation *xp, BytePtr addr)
 
   frame_size = savevsp - vspPtr;
   size = align_to_power_of_2(512 + frame_size, 12);      /* Leave some space in the new segment */
-  if (STACK_SEGMENT_SIZE > size)
-    size = STACK_SEGMENT_SIZE;
+  if (MIN_VSTACK_SIZE > size)
+    size = MIN_VSTACK_SIZE;
 
   /* Look for an existing segment that's big enough.  If the register
      being stored through is not VSP, then we're in one of the
@@ -967,6 +985,17 @@ do_vsp_overflow (ExceptionInformation *xp, BytePtr addr)
   
   return 0;                     /* if we ever return. */
 }
+#else /* EXTEND_VSTACK */
+OSStatus
+do_vsp_overflow (ExceptionInformation *xp, BytePtr addr)
+{
+  area *a = (area *) lisp_global(CURRENT_VS);
+  protected_area_ptr vsp_soft = a->softprot;
+  unprotect_area(vsp_soft);
+  signal_stack_soft_overflow(xp,vsp);
+  return 0;
+}
+#endif
 
 OSStatus
 do_tsp_overflow (ExceptionInformation *xp, BytePtr addr)
@@ -988,7 +1017,7 @@ do_tsp_overflow (ExceptionInformation *xp, BytePtr addr)
     Bug(xp, "TSP overflow in non-lisp context");
   }
 
-  if ((frame_size > 4096) || (frame_size < 0)) {
+  if ((frame_size > TSTACK_SOFTPROT) || (frame_size < 0)) {
     Bug(xp, "TSP frame size out of range");
   };
 
@@ -1001,7 +1030,9 @@ do_tsp_overflow (ExceptionInformation *xp, BytePtr addr)
 
   /* If no existing segment, allocate a new one */
   if (! new_a) {
-    new_a = allocate_area_with_c_gc_context(xp, allocate_tstack, STACK_SEGMENT_SIZE);
+#ifdef EXTEND_VSTACK
+    new_a = allocate_area_with_c_gc_context(xp, allocate_tstack, MIN_TSTACK_SIZE);
+#endif
     if (! new_a)
       return do_hard_stack_overflow(xp, NULL, NULL);
     new_a->older = a;
@@ -1097,7 +1128,6 @@ handle_sigfpe(ExceptionInformation *xp)
 
 int
 altivec_present = 1;
-extern void altivec_probe(void);
 
 OSStatus
 PMCL_exception_handler(int xnum, ExceptionInformation *xp)
@@ -1120,10 +1150,6 @@ PMCL_exception_handler(int xnum, ExceptionInformation *xp)
       status = handle_uuo(xp, instruction, program_counter);
     } else if (is_conditional_trap(instruction)) {
       status = handle_trap(xp, instruction, program_counter);
-    } else if (program_counter == (pc)altivec_probe) {
-      altivec_present = 0;
-      adjust_exception_pc(xp, 4);
-      status = 0;
     }
   }
   xp->regs->gpr[PT_MQ] = oldMQ;    
@@ -1158,81 +1184,22 @@ handle_xalloc(ExceptionInformation *xp, unsigned rt, unsigned ra, unsigned rb)
     }
     gc_from_xp(xp);
     if (ra != rzero) {
-      if (ra == vsp) {          /* %save-library */
-        extern unsigned next_libnum;
-        OSErr err = -50;
-        char libname[64], *src;
-        int i, tag, subtag, elements, refnum = (int) (xpGPR(xp, rb));
-        LispObj 
-          *firstobj = (LispObj *) xpGPR(xp, arg_y), 
-          *lastobj = (LispObj *) xpGPR(xp, arg_z),
-          namestr = xpGPR(xp, arg_x),
-          header;
-        unsigned libnum = next_libnum;
-
-        tag = fulltag_of(namestr);
-        if (tag == fulltag_misc) {
-          header = deref(namestr, 0);
-          subtag = header_subtag(header);
-          elements = header_element_count(header);
-          if ((subtag == subtag_simple_base_string) &&
-              (elements < 64)) {
-            src = ((char *) namestr)+misc_data_offset;
-            for (i = 0; i < elements; i++) {
-              libname[i] = src[i];
-            }
-            libname[elements] = 0;
-            err = noErr;
-            if (firstobj == (LispObj *) lisp_nil) {
-              firstobj = (LispObj *) a->low;
-            } else {
-              firstobj = (LispObj *) untag((LispObj)firstobj);
-              if (firstobj < ((LispObj *) (a->low))) {
-                err = -50;
-              }
-            }
-            if (lastobj == (LispObj *)lisp_nil) {
-              lastobj = (LispObj *) a->active;
-            } else {
-              lastobj = (LispObj *) untag((LispObj)lastobj);
-              if (lastobj > ((LispObj *) (a->active))) {
-                err = -50;
-              }
-            }
-          }
-          if (err == noErr) {
-            err = lib_purify(xp, libname, firstobj, lastobj);
-            gc_from_xp(xp);
-            if (err == noErr) {
-              extern OSErr save_library(unsigned, unsigned, unsigned *);
-              unsigned version;
-
-              err = save_library(refnum, libnum, &version);
-              if (err == noErr) {
-                xpGPR(xp, imm1) = version;
-              }
-            }
-          }
-        }
-        xpGPR(xp, imm0) = (err << fixnumshift); 
-      } else {
-        purify(xp);
-        gc_from_xp(xp);
-        if (rb != 0) {
-          unsigned refnum = ((unsigned) (xpGPR(xp, rb))) >> fixnumshift;
-          OSErr err;
-          extern OSErr save_application(unsigned);
-          
-          err = save_application(refnum);
-          if (err == noErr) {
+      purify(xp);
+      gc_from_xp(xp);
+      if (rb != 0) {
+	unsigned refnum = ((unsigned) (xpGPR(xp, rb))) >> fixnumshift;
+	OSErr err;
+	extern OSErr save_application(unsigned, Boolean);
+	
+	err = save_application(refnum,(ra!=rnil));
+	if (err == noErr) {
 #ifdef VXWORKS
-	    fprintf(stderr, "lisp exiting\n");
-	    fflush(stderr);
+	  fprintf(stderr, "lisp exiting\n");
+	  fflush(stderr);
 #endif
-            lisp_exit(0);
-          }
-          fatal_oserr(": save_application", err);
-        }
+	  lisp_exit(0);
+	}
+	fatal_oserr(": save_application", err);
       }
     }
     if (egc_was_enabled) {
@@ -1243,11 +1210,13 @@ handle_xalloc(ExceptionInformation *xp, unsigned rt, unsigned ra, unsigned rb)
        in effect.  This may need to disable/reenable the EGC. */
     if ((ra == rnil) && (rb == rnil)) {
       a->active = (BytePtr)(xpGPR(xp, freeptr));
-      untenure_from_area(KG->tenured_area);
-      reprotect_dynamic_heap(xp, a->active, lisp_heap_gc_threshold);
+      untenure_from_area(tenured_area);
+      reprotect_dynamic_heap(a->active, 
+			     lisp_heap_gc_threshold, 
+			     egc_was_enabled);
       if (egc_was_enabled) {
 	if ((a->high - a->active) >= a->threshold) {
-	  tenure_to_area(KG->tenured_area);
+	  tenure_to_area(tenured_area);
 	}
       }
     }
@@ -1277,8 +1246,8 @@ handle_xalloc(ExceptionInformation *xp, unsigned rt, unsigned ra, unsigned rb)
       elements = xpGPR(xp,ra) >> fixnumshift, /* Typically arg_y */
       tag_n = tag_of(subtag);
 
-    if (KG->tenured_area) {
-      total = (a->high - KG->tenured_area->low);
+    if (tenured_area) {
+      total = (a->high - tenured_area->low);
     }
     if ((tag_n == fulltag_nodeheader) ||
         (subtag <= max_32_bit_ivector_subtag)) {
@@ -1296,7 +1265,7 @@ handle_xalloc(ExceptionInformation *xp, unsigned rt, unsigned ra, unsigned rb)
     if ((need >= have) && (need <= total)) {
       /* If we need to gc, force a full gc */
       a->active = curfree;
-      untenure_from_area(KG->tenured_area);
+      untenure_from_area(tenured_area);
       gc_from_xp(xp);           /* may retenure everything */
       curfree = a->active;
       have = a->high-curfree;
@@ -1307,11 +1276,11 @@ handle_xalloc(ExceptionInformation *xp, unsigned rt, unsigned ra, unsigned rb)
 
       if (avail > need) {
 	/* If EGC is active, deactivate it. */
-	untenure_from_area(KG->tenured_area);
-	reprotect_dynamic_heap(xp, curfree, have+need);
+	untenure_from_area(tenured_area);
+	reprotect_dynamic_heap(curfree, have+need, (a->older != NULL));
 	have = a->high-curfree;
 	if (a->older) {
-	  tenure_to_area(KG->tenured_area);
+	  tenure_to_area(tenured_area);
 	}
       }
     }
@@ -1566,10 +1535,11 @@ allocate_no_stack (unsigned size)
 }
 
 
-extern OSErr NewThread (ThreadStyle, ThreadEntryProcPtr, void *, Size, ThreadOptions, void **, ThreadID *);
-extern OSErr DisposeThread (ThreadID, void *, Boolean);
-extern OSErr YieldToThread (ThreadID);
+extern OSErr xNewThread (ThreadStyle, ThreadEntryProcPtr, void *, Size, ThreadOptions, void **, ThreadID *);
+extern OSErr xDisposeThread (ThreadID, void *, Boolean);
+extern OSErr xYieldToThread (ThreadID);
 
+#ifdef EXTEND_CSTACK
 /* Here is where we start up on the new thread after a control stack overflow. */
 voidPtr *cs_stack_switch_startup(void *threadParam)
 {
@@ -1626,11 +1596,12 @@ voidPtr *cs_stack_switch_startup(void *threadParam)
     lisp_global(CS_OVERFLOW_LIMIT) = (LispObj) a->softlimit;
     a->younger->active = a->younger->high;
     nrs_INTERRUPT_LEVEL.vcell = (LispObj) (interrupt_level + (2 << fixnumshift));
-    YieldToThread (area_ptr_threadID(a)); 
+    xYieldToThread (area_ptr_threadID(a)); 
 
   }
 }
 
+
 area *
 allocate_cstack (unsigned size)
 {
@@ -1639,17 +1610,17 @@ allocate_cstack (unsigned size)
   extern area *register_cstack(BytePtr, unsigned);
 
   size = (size+4095)&(~4095);           /* round up to multiple of 4096 */
-  if (noErr == NewThread(kCooperativeThread,          /* threadStyle */
-			 (ThreadEntryProcPtr) cs_stack_switch_startup,     /* threadEntry */
-			 NULL,                        /* threadParam */
-			 size,                        /* stackSize */
-			 kCreateIfNeeded,             /* options */
-			 NULL,                        /* threadResult */
-			 &threadMade))                /* threadMade */
+  if (noErr == xNewThread(kCooperativeThread,          /* threadStyle */
+			  (ThreadEntryProcPtr) cs_stack_switch_startup,     /* threadEntry */
+			  NULL,                        /* threadParam */
+			  size,                        /* stackSize */
+			  kCreateIfNeeded,             /* options */
+			  NULL,                        /* threadResult */
+			  &threadMade))                /* threadMade */
     {
     new_area = register_cstack(NULL, 0);             
     if (new_area == NULL) {
-      DisposeThread(threadMade, NULL, false);
+      xDisposeThread(threadMade, NULL, false);
       return(NULL);
     };
     area_ptr_threadID(new_area) = threadMade;
@@ -1658,6 +1629,8 @@ allocate_cstack (unsigned size)
     return NULL;
   };
 }
+#endif
+
 
 /* Cons up a new thread and a new control stack area.
    If successful, continue execution on the new thread,
@@ -1666,6 +1639,7 @@ allocate_cstack (unsigned size)
 Boolean
 continue_on_new_thread (ExceptionInformation *xp)
 {
+#ifdef EXTEND_CONTROL_STACK
   xframe_list xf;
   area *cs = (area *) lisp_global(CURRENT_CS);
   area *new_cs = cs->younger;
@@ -1699,7 +1673,7 @@ continue_on_new_thread (ExceptionInformation *xp)
 
     /* Switch to the new thread */
     if (new_cs != NULL) {
-      YieldToThread(area_ptr_threadID(new_cs));
+      xYieldToThread(area_ptr_threadID(new_cs));
     }
 
     /* Unlink the exception frame from the xframe list */
@@ -1707,6 +1681,9 @@ continue_on_new_thread (ExceptionInformation *xp)
 
     return (new_cs != NULL);
   }
+#else /* EXTEND_CONTROL_STACK */
+  return false;
+#endif
 }
 
 /* callback to (symbol-value cmain) if it is a macptr, 
@@ -1730,7 +1707,7 @@ handle_trap(ExceptionInformation *xp, opcode the_trap, pc where)
      control stack overflow check.  In that case:
      
      a) We're in "yellow zone" mode if the value of the lisp_global(CS_OVERFLOW_LIMIT)
-     is 0xfffffff8.  If we're not already in yellow zone mode, attempt to create
+     is CS_OVERFLOW_FORCE_LIMIT.  If we're not already in yellow zone mode, attempt to create
      a new thread and continue execution on its stack. If that fails, call
      signal_stack_soft_overflow to enter yellow zone mode and signal the condition to lisp.
      
@@ -1753,28 +1730,37 @@ handle_trap(ExceptionInformation *xp, opcode the_trap, pc where)
       (RA_field(the_trap) == sp) &&
       (TO_field(the_trap) == TO_LO)) {
     area 
-      *CS_area = (area *) lisp_global(CURRENT_CS);
+      *CS_area = (area *) lisp_global(CURRENT_CS),
+      *VS_area = (area *) lisp_global(CURRENT_VS);
       
     unsigned 
-      current_SP = xpGPR(xp,sp);
+      current_SP = xpGPR(xp,sp),
+      current_VSP = xpGPR(xp,vsp);
 
     if (current_SP  < (unsigned) (CS_area->hardlimit)) {
       /* If we're not in soft overflow mode yet, assume that the
          user has set the soft overflow size very small and try to
          continue on another thread before throwing to toplevel */
-      if (lisp_global(CS_OVERFLOW_LIMIT == 0xfffffff8) ||
+      if (lisp_global(CS_OVERFLOW_LIMIT == CS_OVERFLOW_FORCE_LIMIT) ||
           !continue_on_new_thread(xp)) {
         reset_lisp_process(xp);
       }
     } else {
-      if (lisp_global(CS_OVERFLOW_LIMIT) == 0xfffffff8) {
-        /* If the control stack is at least 4K away from its soft limit,
+      if (lisp_global(CS_OVERFLOW_LIMIT) == CS_OVERFLOW_FORCE_LIMIT) {
+        /* If the control stack pointer is at least 4K away from its soft limit
+	   and the value stack pointer is at least 4K away from its soft limit,
            stop trapping.  Else keep trapping. */
-        if (current_SP > (unsigned) ((CS_area->softlimit)+4096)) {
+        if ((current_SP > (unsigned) ((CS_area->softlimit)+4096)) &&
+	    (current_VSP > (unsigned) ((VS_area->softlimit)+4096))) {
+	  protected_area_ptr vs_soft = VS_area->softprot;
+	  if (vs_soft->nprot == 0) {
+	    protect_area(vs_soft);
+	  }
           lisp_global(CS_OVERFLOW_LIMIT) = (LispObj) (CS_area->softlimit);
         }
       } else {
         if (!continue_on_new_thread(xp)) {
+          lisp_global(CS_OVERFLOW_LIMIT) = (LispObj) (CS_area->hardlimit);	  
           signal_stack_soft_overflow(xp, sp);
         }
       }
@@ -1789,7 +1775,7 @@ handle_trap(ExceptionInformation *xp, opcode the_trap, pc where)
       if (message == NULL) {
 	message = "Lisp Breakpoint";
       }
-      Debugger(xp, message);
+      lisp_Debugger(xp, message);
       return noErr;
     }
     if ((fulltag_of(cmain) == fulltag_misc) &&
@@ -1798,10 +1784,9 @@ handle_trap(ExceptionInformation *xp, opcode the_trap, pc where)
       if (the_trap == TWI_instruction(TO_GT,nargs,0)) {
         /* Disable interrupts if we're about to process one */
         event_poll_p = true;    /* remember to turn interrupts back on */
-	old_interrupt_level = nrs_INTERRUPT_LEVEL.vcell;
         nrs_INTERRUPT_LEVEL.vcell = (-1 << fixnumshift);
       }
-      callback_for_trap(cmain, xp, (unsigned) where, (unsigned) the_trap, (event_poll_p && (old_interrupt_level == (2<<fixnumshift))) ? 1 : 0, 0);
+      callback_for_trap(cmain, xp, (unsigned) where, (unsigned) the_trap,  0, 0);
       if (event_poll_p) {
         nrs_INTERRUPT_LEVEL.vcell = 0;
       }
@@ -2148,8 +2133,8 @@ error_action( void )
 void
 signal_handler(int signum, ExceptionInformationPowerPC  *context)
 {
+  setr2(lisp_global(SAVE_TOC));
 
-  setr2(0);
   if (noErr != PMCL_exception_handler(signum, context)) {
     Bug(context, "Unhandled exception %d at 0x%08x, context->regs at #x%08x", signum, xpPC(context), (int)(context->regs));
   }
@@ -2260,7 +2245,7 @@ Bug(ExceptionInformation *xp, const char *format, ...)
     lisp_exit(1);
     break;
   case kDebugger:
-    Debugger(xp, "Bug() called");
+    lisp_Debugger(xp, "Bug() called");
     break;
   default:
     break;
