@@ -1,24 +1,25 @@
 /*
    Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Opensourced MCL.
+   This file is part of OpenMCL.  
 
-   Opensourced MCL is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+   License , known as the LLGPL and distributed with OpenMCL as the
+   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+   which is distributed with OpenMCL as the file "LGPL".  Where these
+   conflict, the preamble takes precedence.  
 
-   Opensourced MCL is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   OpenMCL is referenced in the preamble as the "LIBRARY."
 
+   The LLGPL is also available online at
+   http://opensource.franz.com/preamble.html
 */
 
-
+#ifdef DARWIN
+/*	dyld.h included here because something in "lisp.h" causes
+    a conflict (actually I think the problem is in "constants.h")
+*/
+#include <mach-o/dyld.h>
+#endif
 #include "lisp.h"
 #include "lisp_globals.h"
 #include "gc.h"
@@ -26,7 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include "lisp-exceptions.h"
-#include "loader.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/mman.h>
@@ -39,12 +39,21 @@
 #include <dirent.h>
 #include <dlfcn.h>
 #include <sys/time.h>
+#include <sys/resource.h>
+#include <link.h>
+#include <elf.h>
+#include <asm/cputable.h>
 #endif
 
 #ifdef DARWIN
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/mman.h>
+#include <sys/resource.h>
+#include <mach/mach_types.h>
+#include <mach/message.h>
+#include <mach/vm_region.h>
+#include <sys/sysctl.h>
 #endif
 
 #ifdef VXWORKS
@@ -63,9 +72,6 @@
 #endif
 
 
-/* This is kept outside of the kernel_globals structure
-   'cause it's referenced so often. */
-
 LispObj lisp_nil = (LispObj) 0;
 
 
@@ -94,7 +100,7 @@ align_to_power_of_2(unsigned n, unsigned power)
 
 
 void
-MakeDataExecutable(void *, unsigned);
+xMakeDataExecutable(void *, unsigned);
 
 void
 make_dynamic_heap_executable(LispObj *p, LispObj *q)
@@ -102,9 +108,33 @@ make_dynamic_heap_executable(LispObj *p, LispObj *q)
   void * cache_start = (void *) p;
   unsigned ncacheflush = (unsigned) q - (unsigned) p;
 
-  MakeDataExecutable(cache_start, ncacheflush);  
+  xMakeDataExecutable(cache_start, ncacheflush);  
 }
       
+size_t
+ensure_stack_limit(size_t stack_size)
+{
+  struct rlimit limits;
+  rlim_t cur_stack_limit, max_stack_limit;
+  
+  stack_size += (CSTACK_HARDPROT+CSTACK_SOFTPROT);
+  getrlimit(RLIMIT_STACK, &limits);
+  cur_stack_limit = limits.rlim_cur;
+  max_stack_limit = limits.rlim_max;
+  if (stack_size > max_stack_limit) {
+    stack_size = max_stack_limit;
+  }
+  if (cur_stack_limit < stack_size) {
+    limits.rlim_cur = stack_size;
+    errno = 0;
+    if (setrlimit(RLIMIT_STACK, &limits)) {
+      int e = errno;
+      fprintf(stderr, "errno = %d\n", e);
+      Fatal(": Stack resource limit too small", "");
+    }
+  }
+  return stack_size - (CSTACK_HARDPROT+CSTACK_SOFTPROT);
+}
 
 #ifdef DARWIN
 LispObj
@@ -136,7 +166,7 @@ darwin_remap_subprims(void *curloc)
        *dest++ = (BA_VAL) | target;
       }
       
-      MakeDataExecutable(ret, 1024);
+      xMakeDataExecutable(ret, 1024);
     }
   }
   return (LispObj)desired;
@@ -159,11 +189,13 @@ allocate_lisp_stack(unsigned useable,
                     protected_area_ptr *softp,
                     protected_area_ptr *hardp)
 {
+  void *allocate_stack(unsigned);
+  void free_stack(void *);
   unsigned size = useable+softsize+hardsize;
   unsigned overhead;
   BytePtr base, softlimit, hardlimit;
   OSErr err;
-  Ptr h = allocate(size+4095);
+  Ptr h = allocate_stack(size+4095);
   protected_area_ptr hprotp = NULL, sprotp;
 
   if (h == NULL) {
@@ -176,7 +208,7 @@ allocate_lisp_stack(unsigned useable,
 
   overhead = (base - (BytePtr) h);
   if (hardsize) {
-    hprotp = new_protected_area((BytePtr)base,hardlimit,hardkind, hardsize, false);
+    hprotp = new_protected_area((BytePtr)base,hardlimit,hardkind, hardsize, true);
     if (hprotp == NULL) {
       if (base_p) *base_p = NULL;
       if (h_p) *h_p = NULL;
@@ -192,7 +224,7 @@ allocate_lisp_stack(unsigned useable,
       if (h_p) *h_p = NULL;
       if (hardp) *hardp = NULL;
       if (hprotp) delete_protected_area(hprotp);
-      deallocate(h);
+      free_stack(h);
       return NULL;
     }
     if (softp) *softp = sprotp;
@@ -247,30 +279,32 @@ register_cstack(BytePtr bottom, unsigned size)
   BytePtr lowlimit = (BytePtr) (((((unsigned)bottom)-size)+4095)&~4095);
   area *a = new_area((BytePtr) bottom-size, bottom, AREA_CSTACK);
 
-  a->hardlimit = lowlimit+4096;
-  a->softlimit = lowlimit+8192;
+  a->hardlimit = lowlimit+CSTACK_HARDPROT;
+  a->softlimit = a->hardlimit+CSTACK_SOFTPROT;
   add_area(a);
   return a;
 }
   
 area*
-allocate_vstack(unsigned useable)
+allocate_vstack(unsigned usable)
 {
   return allocate_lisp_stack_area(AREA_VSTACK, 
-                                  useable, 
-                                  4096,
-                                  0,
+				  usable > MIN_VSTACK_SIZE ?
+				  usable : MIN_VSTACK_SIZE,
+                                  VSTACK_SOFTPROT,
+                                  VSTACK_HARDPROT,
                                   kVSPsoftguard,
                                   kVSPhardguard);
 }
 
 area *
-allocate_tstack(unsigned useable)
+allocate_tstack(unsigned usable)
 {
   return allocate_lisp_stack_area(AREA_TSTACK, 
-                                  useable, 
-                                  4096,
-                                  0,
+                                  usable > MIN_TSTACK_SIZE ?
+				  usable : MIN_TSTACK_SIZE,
+                                  TSTACK_SOFTPROT,
+                                  TSTACK_HARDPROT,
                                   kTSPsoftguard,
                                   kTSPhardguard);
 }
@@ -335,19 +369,9 @@ lisp_heap_partition = NULL;
 int
 reserved_area_size = (1<<30);
 
-void
-create_kernel_globals()
-{
-  KG = (kernel_globals *)allocate(sizeof(kernel_globals));
-  if (KG != NULL) {
-    bzero((void *)KG, sizeof(kernel_globals));
-    KG->cache_block_size = 32;	/* CPU-specific */
-  } else {
-    fprintf(stderr, "Can't create kernel globals structure\n");
-    fflush(stderr);
-    lisp_exit(-1);
-  }
-}
+area *nilreg_area=NULL, *tenured_area=NULL, *g2_area=NULL, *g1_area=NULL;
+area *all_areas=NULL;
+int cache_block_size=32;
 
 #ifdef VXWORKS
 Ptr
@@ -366,10 +390,15 @@ create_memory_partition(unsigned totalsize)
 #endif
 
 #define DEFAULT_LISP_HEAP_GC_THRESHOLD (4<<20)
+#define DEFAULT_INITIAL_STACK_SIZE (1<<20)
 
 unsigned
 lisp_heap_gc_threshold = DEFAULT_LISP_HEAP_GC_THRESHOLD;
 
+unsigned 
+initial_stack_size = DEFAULT_INITIAL_STACK_SIZE;
+
+
 /*
   'start' should be on a segment boundary; 'len' should be
   an integral number of segments.
@@ -399,6 +428,95 @@ uncommit_pages(void *start, unsigned len)
   /* UNLOCK_MMAP_LOCK(); */
 }
 
+void
+commit_pages(void *start, unsigned len)
+{
+  BytePtr p;
+
+  /* LOCK_MMAP_LOCK(); */
+
+  for (p = start; len; len -= heap_segment_size, p += heap_segment_size) {
+    munmap(p, heap_segment_size);
+    if (p != mmap(p,
+		  heap_segment_size,
+		  PROT_READ | PROT_WRITE | PROT_EXEC,
+		  MAP_PRIVATE | MAP_FIXED | MAP_ANON,
+		  -1,
+		  0)) {
+      Fatal("mmap error", "");
+    }
+  }
+  /* UNLOCK_MMAP_LOCK(); */
+}
+
+area *
+extend_readonly_area(unsigned more)
+{
+  area *a;
+  unsigned mask;
+  BytePtr new_start, new_end;
+  for (a = active_dynamic_area->succ; a != all_areas; a = a->succ) {
+    if (a->code == AREA_READONLY) {
+      if ((a->active + more) > a->high) {
+	return NULL;
+      }
+      mask = ((unsigned)a->active) & 4095;
+      if (mask) {
+	UnProtectMemory(a->active-mask, 4096);
+      }
+      new_start = (BytePtr)(align_to_power_of_2((unsigned)a->active,12));
+      new_end = (BytePtr)(align_to_power_of_2((unsigned)a->active+more,12));
+      if (mmap(new_start,
+	       new_end-new_start,
+	       PROT_READ | PROT_WRITE | PROT_EXEC,
+	       MAP_PRIVATE | MAP_ANON | MAP_FIXED,
+	       -1,
+	       0) != new_start) {
+	return NULL;
+      }
+      return a;
+    }
+  }
+  return NULL;
+}
+
+LispObj image_base=0;
+BytePtr pure_space_start, pure_space_active, pure_space_limit;
+BytePtr static_space_start, static_space_active, static_space_limit;
+
+#ifdef DARWIN
+/*
+  Check to see if the specified address is unmapped by trying to get
+  information about the mapped address at or beyond the target.  If
+  the difference between the target address and the next mapped address
+  is >= len, we can safely mmap len bytes at addr.
+*/
+Boolean
+address_unmapped_p(char *addr, unsigned len)
+{
+  vm_address_t vm_addr = (vm_address_t)addr;
+  vm_size_t vm_size;
+  vm_region_basic_info_data_t vm_info;
+  mach_msg_type_number_t vm_info_size = VM_REGION_BASIC_INFO_COUNT;
+  port_t vm_object_name = (port_t) 0;
+  kern_return_t kret;
+
+  kret = vm_region(mach_task_self(),
+		   &vm_addr,
+		   &vm_size,
+		   VM_REGION_BASIC_INFO,
+		   (vm_region_info_t)&vm_info,
+		   &vm_info_size,
+		   &vm_object_name);
+  if (kret != KERN_SUCCESS) {
+    return false;
+  }
+
+  return vm_addr >= (vm_address_t)(addr+len);
+}
+#endif
+
+
 
 
 area *
@@ -407,37 +525,85 @@ create_reserved_area(unsigned totalsize)
   OSErr err;
   Ptr h;
   unsigned base, n;
-  BytePtr  end, lastbyte, start, protstart, p;
+  BytePtr 
+    end, 
+    lastbyte, 
+    start, 
+    protstart, 
+    p, 
+    want = (BytePtr)IMAGE_BASE_ADDRESS,
+    try2;
   area *reserved;
   bitvector markbits;
+  Boolean fixed_map_ok = false;
 
+  /*
+    Through trial and error, we've found that IMAGE_BASE_ADDRESS is
+    likely to reside near the beginning of an unmapped block of memory
+    that's at least 1GB in size.  We'd like to load the heap image's
+    sections relative to IMAGE_BASE_ADDRESS; if we're able to do so,
+    that'd allow us to file-map those sections (and would enable us to
+    avoid having to relocate references in the data sections.)
 
+    In short, we'd like to reserve 1GB starting at IMAGE_BASE_ADDRESS
+    by creating an anonymous mapping with mmap().
+
+    If we try to insist that mmap() map a 1GB block at
+    IMAGE_BASE_ADDRESS exactly (by specifying the MAP_FIXED flag),
+    mmap() will gleefully clobber any mapped memory that's already
+    there.  (That region's empty at this writing, but some future
+    version of the OS might decide to put something there.)
+
+    If we don't specify MAP_FIXED, mmap() is free to treat the address
+    we give it as a hint; Linux seems to accept the hint if doing so
+    wouldn't cause a problem.  Naturally, that behavior's too useful
+    for Darwin (or perhaps too inconvenient for it): it'll often
+    return another address, even if the hint would have worked fine.
+
+    We call address_unmapped_p() to ask Mach whether using MAP_FIXED
+    would conflict with anything.  Until we discover a need to do 
+    otherwise, we'll assume that if Linux's mmap() fails to take the
+    hint, it's because of a legitimate conflict.
+
+    If Linux starts ignoring hints, we can parse /proc/<pid>/maps
+    to implement an address_unmapped_p() for Linux.
+  */
 
   totalsize = align_to_power_of_2(totalsize, log2_heap_segment_size);
-  start = mmap(NULL,
-	       totalsize+heap_segment_size,
+
+#ifdef DARWIN
+  fixed_map_ok = address_unmapped_p(want,totalsize);
+#endif
+  start = mmap((void *)want,
+	       totalsize + heap_segment_size,
 	       PROT_NONE,
-	       MAP_PRIVATE | MAP_ANON,
+	       MAP_PRIVATE | MAP_ANON | (fixed_map_ok ? MAP_FIXED : 0),
 	       -1,
 	       0);
   if (start == MAP_FAILED) {
     perror("Initial mmap");
     return NULL;
   }
-  munmap(start, totalsize+heap_segment_size);
-  start =  (void *)((((unsigned)start)+heap_segment_size) & ~(heap_segment_size-1));
-  for (p = start , n = totalsize; 
-       n ; 
-       n -= heap_segment_size, p += heap_segment_size) {
-    if(mmap(p, heap_segment_size, PROT_NONE, MAP_PRIVATE | MAP_ANON | MAP_FIXED, -1, 0) != p) {
+
+  if (start != want) {
+    munmap(start, totalsize+heap_segment_size);
+    start = (void *)((((unsigned)start)+heap_segment_size-1) & ~(heap_segment_size-1));
+    if(mmap(start, totalsize, PROT_NONE, MAP_PRIVATE | MAP_ANON | MAP_FIXED, -1, 0) != start) {
       return NULL;
     }
-    mprotect(p, heap_segment_size, PROT_NONE);
   }
+  mprotect(start, totalsize, PROT_NONE);
 
   h = (Ptr) start;
   base = (unsigned) start;
+  image_base = base;
   lastbyte = (BytePtr) (start+totalsize);
+  static_space_start = static_space_active = start;
+  static_space_limit = static_space_start + STATIC_RESERVE;
+  pure_space_start = pure_space_active = static_space_limit;
+  pure_space_limit = start + PURESPACE_RESERVE;
+  start = pure_space_limit;
+
   /*
     Allocate mark bits here.  They need to be 1/64 the size of the
      maximum useable area of the heap (+ 3 words for the EGC.)
@@ -451,7 +617,7 @@ create_reserved_area(unsigned totalsize)
   reserved = new_area(start, end, AREA_VOID);
   /* The root of all evil is initially linked to itself. */
   reserved->pred = reserved->succ = reserved;
-  KG->all_areas = reserved;
+  all_areas = reserved;
   reserved->markbits = markbits;
   return reserved;
 }
@@ -791,6 +957,16 @@ start_vbl()
 extern BytePtr
 current_stack_pointer(void);
 
+BytePtr
+initial_stack_bottom()
+{
+  extern char **environ;
+  char *p = *environ;
+  while (*p) {
+    p += (1+strlen(p));
+  }
+  return (BytePtr)((((unsigned) p) +4095) & ~4095);
+}
 
 
   
@@ -909,6 +1085,7 @@ usage_exit(char *herald, int exit_status, char* other_args)
   fprintf(stderr, "\t-R, --heap-reserve <n>: reserve <n> (default: %d)\n",
 	  reserved_area_size);
   fprintf(stderr, "\t\t bytes for heap expansion\n");
+  fprintf(stderr, "\t-S, --stack-size <n>: set size of initial stacks to <n> (default: %d)\n", initial_stack_size);
   fprintf(stderr, "\t-b, --batch: exit when EOF on *STANDARD-INPUT*\n");
   fprintf(stderr, "\t--no-sigtrap : obscure option for running under GDB\n");
   fprintf(stderr, "\t-I, --image-name <image-name>\n");
@@ -924,6 +1101,42 @@ int batch_flag = 0;
 
 #ifndef VXWORKS
 
+unsigned
+parse_numeric_option(char *arg, char *argname, unsigned default_val)
+{
+  char *tail;
+  unsigned val = 0;
+
+  val = strtoul(arg, &tail, 0);
+  switch(*tail) {
+  case '\0':
+    break;
+    
+  case 'M':
+  case 'm':
+    val = val << 20;
+    break;
+    
+  case 'K':
+  case 'k':
+    val = val << 10;
+    break;
+    
+  case 'G':
+  case 'g':
+    val = val << 30;
+    break;
+    
+  default:
+    fprintf(stderr, "couldn't parse %s argument %s", argname, arg);
+    val = default_val;
+    break;
+  }
+  return val;
+}
+  
+
+
 /* 
    The set of arguments recognized by the kernel is
    likely to remain pretty small and pretty simple.
@@ -936,6 +1149,9 @@ process_options(int argc, char *argv[])
 {
   int i, j, k, num_elide, flag, arg_error;
   char *arg, *val;
+#ifdef DARWIN
+  extern int NXArgc;
+#endif
 
   for (i = 1; i < argc;) {
     arg = argv[i];
@@ -962,9 +1178,8 @@ process_options(int argc, char *argv[])
 	  image_name = val;
 	}
       } else if ((flag = (strncmp(arg, "-R", 2) == 0)) ||
-	   (strcmp(arg, "--heap-reserve") == 0)) {
+		 (strcmp(arg, "--heap-reserve") == 0)) {
 	unsigned reserved_size;
-	char *tail;
 
 	if (flag && arg[2]) {
 	  val = arg+2;
@@ -979,37 +1194,42 @@ process_options(int argc, char *argv[])
 	}
 
 	if (val) {
-	  reserved_size = strtoul(val, &tail, 0);
-	  switch(*tail) {
-	  case '\0':
-	    break;
-
-	  case 'M':
-	  case 'm':
-	    reserved_size = reserved_size << 20;
-	    break;
-
-	  case 'K':
-	  case 'k':
-	    reserved_size = reserved_size << 10;
-	    break;
-
-	  case 'G':
-	  case 'g':
-	    reserved_size = reserved_size << 30;
-	    break;
-
-	  default:
-	    fprintf(stderr, "couldn't parse heap-reserve argument ~s", optarg);
-	    reserved_size = reserved_area_size;
-	    break;
-	  }
+	  reserved_size = parse_numeric_option(val, 
+					       "-R/--heap-reserve", 
+					       reserved_area_size);
 	}
 
 	if (reserved_size <= (1<< 30)) {
 	  reserved_area_size = reserved_size;
 	}
 
+      } else if ((flag = (strncmp(arg, "-S", 2) == 0)) ||
+		 (strcmp(arg, "--stack-size") == 0)) {
+	unsigned stack_size;
+
+	if (flag && arg[2]) {
+	  val = arg+2;
+	  num_elide = 1;
+	} else {
+	  if ((i+1) < argc) {
+	    val = argv[i+1];
+	    num_elide = 2;
+	  } else {
+	    arg_error = 1;
+	  }
+	}
+
+	if (val) {
+	  stack_size = parse_numeric_option(val, 
+					    "-S/--stack-size", 
+					    initial_stack_size);
+	  
+
+	  if (stack_size >= MIN_CSTACK_SIZE) {
+	    initial_stack_size = stack_size;
+	  }
+	}
+
       } else if (strcmp(arg, "--no-sigtrap") == 0) {
 	no_sigtrap = 1;
 	num_elide = 1;
@@ -1028,6 +1248,9 @@ process_options(int argc, char *argv[])
 	  argv[k] = argv[j];
 	}
 	argc -= num_elide;
+#ifdef DARWIN
+	NXArgc -= num_elide;
+#endif
 	argv[argc] = NULL;
       }
     }
@@ -1036,7 +1259,7 @@ process_options(int argc, char *argv[])
 
 
 
-main(int argc, char *argv[])
+main(int argc, char *argv[], char *envp[], void *aux)
 #endif
 #ifdef VXWORKS
 vxppcclMain()
@@ -1047,24 +1270,78 @@ vxppcclMain()
   char *argv[] = {"vxppccl", 0};
 #endif
   extern  set_fpscr(unsigned);
-  extern void altivec_probe(void);
+
   extern int altivec_present;
   extern LispObj load_image(char *);
   long resp;
-  BytePtr heap_start, heap_end, stack_end;
+  BytePtr stack_end;
   area *a;
-  BytePtr stack_base;
+  BytePtr stack_base, current_sp = current_stack_pointer();
+
+#ifdef LINUX
+  {
+    ElfW(auxv_t) *av = aux;
+    int hwcap, done = false;
+    
+    if (av) {
+      do {
+	switch (av->a_type) {
+	case AT_DCACHEBSIZE:
+	  cache_block_size = av->a_un.a_val;
+	  break;
+
+	case AT_HWCAP:
+	  hwcap = av->a_un.a_val;
+	  altivec_present = ((hwcap & PPC_FEATURE_HAS_ALTIVEC) != 0);
+	  break;
+
+	case AT_NULL:
+	  done = true;
+	  break;
+	}
+	av++;
+      } while (!done);
+    }
+  }
+#endif
+#ifdef DARWIN
+  {
+    unsigned value = 0;
+    size_t len = sizeof(value);
+    int mib[2];
+    
+    mib[0] = CTL_HW;
+    mib[1] = HW_CACHELINE;
+    if (sysctl(mib,2,&value,&len, NULL, 0) != -1) {
+      if (len == sizeof(value)) {
+	cache_block_size = value;
+      }
+    }
+    mib[1] = HW_VECTORUNIT;
+    value = 0;
+    len = sizeof(value);
+    if (sysctl(mib,2,&value,&len, NULL, 0) != -1) {
+      if (len == sizeof(value)) {
+	altivec_present = value;
+      }
+    }
+  }
+#endif
 
-  setr2(0);
 #if defined(LINUX) || defined(DARWIN)
 
   program_name = argv[0];
   if ((argc == 2) && (*argv[1] != '-')) {
+#ifdef DARWIN
+    extern int NXArgc;
+    NXArgc = 1;
+#endif
     image_name = argv[1];
     argv[1] = NULL;
   } else {
     process_options(argc,argv);
   }
+  initial_stack_size = ensure_stack_limit(initial_stack_size);
   if (image_name == NULL) {
     image_name = default_image_name(argv[0]);
   }
@@ -1077,7 +1354,6 @@ vxppcclMain()
     image_name = default_lisp_image_name;
 #endif
 
-  create_kernel_globals();
 #if 0
   fcntl(0, F_SETFL, O_NONBLOCK |  fcntl(0, F_GETFL));
 #endif
@@ -1101,6 +1377,12 @@ vxppcclMain()
 #endif
   create_reserved_area(reserved_area_size);
   set_nil(load_image(image_name));
+  lisp_global(SUBPRIMS_BASE) = subprims_base;
+  lisp_global(RET1VALN) = ret1valn;
+  lisp_global(LEXPR_RETURN) = lexpr_return;
+  lisp_global(LEXPR_RETURN1V) = lexpr_return1v;
+  lisp_global(ALL_AREAS) = (LispObj) (all_areas);
+
   exception_init();
 #ifdef PROXY_SCHEDULER
   puppet_init();
@@ -1110,10 +1392,6 @@ vxppcclMain()
     Fatal(": Couldn't load subprims library.", "");
   }
   
-  if (application_load_err != noErr) {
-    Fatal(": Error loading application data.", (load_error_message != NULL) ? load_error_message : (StringPtr) "Unknown.");
-  }
-
   set_fpscr(0xD0);
 
   lisp_global(IMAGE_NAME) = (LispObj) image_name;
@@ -1121,19 +1399,12 @@ vxppcclMain()
   lisp_global(KERNEL_IMPORTS) = (LispObj) import_ptrs_base;
   a = active_dynamic_area;
   lisp_global(SAVE_FREEPTR) = (LispObj) (a->active);
-  heap_start = a->low;
-  heap_end = a->high;
 
   lisp_global(METERING_INFO) = (LispObj) &lisp_metering;
 
   *(double *) &(lisp_global(DOUBLE_FLOAT_ONE)) = (double) 1.0;
   lisp_global(EXCEPTION_SAVED_REGISTERS) = (LispObj) 0;
 
-  
-  
-  lisp_global(HEAP_START) = (LispObj) heap_start;
-  lisp_global(HEAP_END) = (LispObj) heap_end;
-
   lisp_global(ARGV) = (LispObj) argv;
   lisp_global(ERRNO) = (LispObj) &errno;
   lisp_global(HOST_PLATFORM) = (LispObj)
@@ -1146,39 +1417,38 @@ vxppcclMain()
 #ifdef DARWIN
     3
 #endif
-    /*We'll get a syntax error here if nothing's defined. */
+    /* We'll get a syntax error here if nothing's defined. */
     << fixnumshift;
 
 
-#ifdef LINUX
   lisp_global(BATCH_FLAG) = (batch_flag << fixnumshift);
-#endif
-  if (KG->nilreg_area != NULL) {
+
+  if (nilreg_area != NULL) {
     BytePtr lowptr = (BytePtr) a->low;
 
     a = active_dynamic_area;
     /* Create these areas as AREA_STATIC, change them to AREA_DYNAMIC */
-    KG->g1_area = new_area(lowptr, lowptr, AREA_STATIC);
-    KG->g2_area = new_area(lowptr, lowptr, AREA_STATIC);
-    KG->tenured_area = new_area(lowptr, lowptr, AREA_STATIC);
-    add_area(KG->tenured_area);
-    add_area(KG->g2_area);
-    add_area(KG->g1_area);
+    g1_area = new_area(lowptr, lowptr, AREA_STATIC);
+    g2_area = new_area(lowptr, lowptr, AREA_STATIC);
+    tenured_area = new_area(lowptr, lowptr, AREA_STATIC);
+    add_area(tenured_area);
+    add_area(g2_area);
+    add_area(g1_area);
 
-    KG->g1_area->code = AREA_DYNAMIC;
-    KG->g2_area->code = AREA_DYNAMIC;
-    KG->tenured_area->code = AREA_DYNAMIC;
+    g1_area->code = AREA_DYNAMIC;
+    g2_area->code = AREA_DYNAMIC;
+    tenured_area->code = AREA_DYNAMIC;
 
 /*    a->older = g1_area; */ /* Not yet: this is what "enabling the EGC" does. */
-    KG->g1_area->younger = a;
-    KG->g1_area->older = KG->g2_area;
-    KG->g2_area->younger = KG->g1_area;
-    KG->g2_area->older = KG->tenured_area;
-    KG->tenured_area->younger = KG->g2_area;
-    KG->tenured_area->refbits = a->markbits;
-    lisp_global(TENURED_AREA) = (LispObj)(KG->tenured_area);
-    KG->g2_area->threshold = (1<<18); /* 256K */
-    KG->g1_area->threshold = (1<<17); /* 128K */
+    g1_area->younger = a;
+    g1_area->older = g2_area;
+    g2_area->younger = g1_area;
+    g2_area->older = tenured_area;
+    tenured_area->younger = g2_area;
+    tenured_area->refbits = a->markbits;
+    lisp_global(TENURED_AREA) = (LispObj)(tenured_area);
+    g2_area->threshold = (1<<18); /* 256K */
+    g1_area->threshold = (1<<17); /* 128K */
     a->threshold = (1<<16);     /* 64K */
   }
 
@@ -1186,18 +1456,18 @@ vxppcclMain()
     (BytePtr) allocate_lisp_stack(1<<15, 1<<12, 0, kMEMOprotect, kNotProtected, NULL, NULL, NULL, NULL);
   lisp_global(SAVE_MEMO) = (LispObj) memo_base;
       
-  a = allocate_vstack(STACK_SEGMENT_SIZE);
+  a = allocate_vstack(initial_stack_size);
   stack_end = a->high;
   lisp_global(SAVE_VSP) = (LispObj) stack_end;
   lisp_global(CURRENT_VS) = (LispObj) a;
 
-  a = allocate_tstack(STACK_SEGMENT_SIZE);
+  a = allocate_tstack(MIN_TSTACK_SIZE);
   stack_end = a->high;
   lisp_global(SAVE_TSP) = (LispObj) stack_end;
   lisp_global(CURRENT_TS) = (LispObj) a;
 
-  stack_base = current_stack_pointer()-StackSpace();
-  a = register_cstack(current_stack_pointer(), StackSpace());
+  stack_base = initial_stack_bottom()-xStackSpace();
+  a = register_cstack(current_sp, current_sp-stack_base);
 #ifndef MACOS
   init_mac_threads((void *)(stack_base));
 #endif
@@ -1206,8 +1476,6 @@ vxppcclMain()
 
   start_vbl();
 
-  altivec_present = 1;		/* assume no trap */
-  altivec_probe();		/* try to use vector unit */
   lisp_global(ALTIVEC_PRESENT) = altivec_present << fixnumshift;
 
 
@@ -1235,87 +1503,17 @@ unsigned next_libnum = 0;
 area *
 set_nil(LispObj r)
 {
-  static_header *static_headerP;
-  area *root, *a, *primary;
-  mutable_data_section_header *libP;
-  BytePtr low, high;
-  unsigned libnum = 2;
 
   if (lisp_nil == (LispObj)NULL) {
 
     lisp_nil = r;
-    lisp_global(SUBPRIMS_BASE) = subprims_base;
-    lisp_global(RET1VALN) = ret1valn;
-    lisp_global(LEXPR_RETURN) = lexpr_return;
-    lisp_global(LEXPR_RETURN1V) = lexpr_return1v;
-
-    static_headerP = (static_header *) (&(lisp_global(STATIC_HEAP_START)));
-    next_libP = &(static_headerP->next);
-
-    low = static_headerP->low;
-    high = static_headerP->high;
-    
-    primary = new_area(low, high, AREA_STATIC);
-    add_area(primary);
-
-    make_dynamic_heap_executable((LispObj *) (low),
-				 (LispObj *) (high));
-
-    for (libP = static_headerP->next; libP; libP = libP->next) {
-      BytePtr *lowP;
-      
-      /*      load_library_or_die(libP); */
-      low = libP->mutable_low;
-      a = new_area(low, libP->mutable_high, AREA_STATIC);
-      a->owner = libnum;
-      add_area(a);
-
-      low = libP->immutable_low;
-      lowP = (BytePtr *) low;
-      a = new_area(low, libP->immutable_high, AREA_STATICLIB);
-      a->owner = libnum;
-      add_area(a);
-
-      a = new_area(lowP[0], lowP[1], AREA_READONLY);
-      a->owner = libnum;
-      add_area(a);
-
-
-      libnum++;
-      next_libP = &(libP->next);
-    }
-
-    next_libnum = libnum;
-
-    static_headerP = (static_header *) (&(lisp_global(READONLY_SECTION_START)));
-    if (static_headerP->low) {
-      add_area(new_area(static_headerP->low, static_headerP->high, AREA_READONLY));
-    }
-
-    lisp_global(ALL_AREAS) = (LispObj) (KG->all_areas);
-    return primary;
   }
   return NULL;
 }
 
 
-LispObj init_code_vector = (LispObj)NULL;
 
 
-OSErr
-application_loader(int fd, long pos, LispObj r)
-{
-  area *a;
-  init_code_vector = deref(r,0);
-  deref(r, 0) = (LispObj)NULL;
-
-  KG->nilreg_area = set_nil(r);
-
-
-  a = load_application_data(&application_load_err, fd, pos);
-  return noErr;
-}
-
 
 
 #ifdef VXWORKS
@@ -1332,21 +1530,21 @@ char *strdup(char *src)
       
 #ifndef MACOS
 void
-MakeDataExecutable(void *start, unsigned nbytes)
+xMakeDataExecutable(void *start, unsigned nbytes)
 {
   extern void flush_cache_lines();
   unsigned ustart = (unsigned) start, base, end;
   
-  base = (ustart) & ~(KG->cache_block_size-1);
-  end = (ustart + nbytes + KG->cache_block_size - 1) & ~(KG->cache_block_size-1);
-  flush_cache_lines(base, (end-base)/KG->cache_block_size, KG->cache_block_size);
+  base = (ustart) & ~(cache_block_size-1);
+  end = (ustart + nbytes + cache_block_size - 1) & ~(cache_block_size-1);
+  flush_cache_lines(base, (end-base)/cache_block_size, cache_block_size);
 }
 
 int
-StackSpace()
+xStackSpace()
 {
 #if defined(LINUX) || defined(DARWIN)
-  return STACK_SEGMENT_SIZE;
+  return initial_stack_size+CSTACK_HARDPROT+CSTACK_SOFTPROT;
 #endif
 #ifdef VXWORKS
   WIND_TCB *tcb = taskTcb(0);
@@ -1355,23 +1553,109 @@ StackSpace()
 #endif
 }
 
+#ifndef DARWIN
 void *
-GetSharedLibrary(char *path, int mode)
+xGetSharedLibrary(char *path, int mode)
 {
 #ifdef VXWORKS
   return NULL;
-#else
-#ifdef DARWIN
-  return NULL;
 #else
   return dlopen(path, mode);
 #endif
-#endif
 }
+#else
+void *
+xGetSharedLibrary(char *path, int mode)
+{
+  NSObjectFileImageReturnCode code;
+  NSObjectFileImage	         moduleImage;
+  NSModule		         module;
+  const struct mach_header *     header;
+  const char *                   error;
+  int *                          resultType;
+  void *                         result;
+  /* not thread safe */
+  /*
+  static struct {
+    const struct mach_header  *header;
+    NSModule	              *module;
+    const char                *error;
+  } results;	
+  */
+  resultType = (int *)mode;
+  result = NULL;
+  error = NULL;
 
-
+  /* first try to open this as a bundle */
+  code = NSCreateObjectFileImageFromFile(path,&moduleImage);
+  if (code != NSObjectFileImageSuccess &&
+      code != NSObjectFileImageInappropriateFile &&
+      code != NSObjectFileImageAccess)
+    {
+      /* compute error strings */
+      switch (code)
+	{
+	case NSObjectFileImageFailure:
+	  error = "NSObjectFileImageFailure";
+	  break;
+	case NSObjectFileImageArch:
+	  error = "NSObjectFileImageArch";
+	  break;
+	case NSObjectFileImageFormat:
+	  error = "NSObjectFileImageFormat";
+	  break;
+	case NSObjectFileImageAccess:
+	  /* can't find the file */
+	  error = "NSObjectFileImageAccess";
+	  break;
+	default:
+	  error = "unknown error";
+	}
+      *resultType = 0;
+      return (void *)error;
+    }
+  if (code == NSObjectFileImageInappropriateFile ||
+      code == NSObjectFileImageAccess ) {
+    /* the pathname might be a partial pathane (hence the access error)
+       or it might be something other than a bundle, if so perhaps
+       it is a .dylib so now try to open it as a .dylib */
 
+    /* protect against redundant loads, Gary Byers noticed possible
+       heap corruption if this isn't done */
+    header = NSAddImage(path, NSADDIMAGE_OPTION_RETURN_ON_ERROR |
+			NSADDIMAGE_OPTION_WITH_SEARCHING |
+			NSADDIMAGE_OPTION_RETURN_ONLY_IF_LOADED);
+    if (!header)
+      header = NSAddImage(path, NSADDIMAGE_OPTION_RETURN_ON_ERROR |
+			  NSADDIMAGE_OPTION_WITH_SEARCHING);
+    result = (void *)header;
+    *resultType = 1;
+  }
+  else if (code == NSObjectFileImageSuccess) {
+    /* we have a sucessful module image
+       try to link it, don't bind symbols privately */
 
+    module = NSLinkModule(moduleImage, path,
+			  NSLINKMODULE_OPTION_RETURN_ON_ERROR | NSLINKMODULE_OPTION_BINDNOW);
+    NSDestroyObjectFileImage(moduleImage);	
+    result = (void *)module;
+    *resultType = 2;
+  }
+  if (!result)
+    {
+      /* compute error string */
+      NSLinkEditErrors ler;
+      int lerno;
+      const char* file;
+      NSLinkEditError(&ler,&lerno,&file,&error);
+      if (error) {
+	result = (void *)error;
+	*resultType = 0;
+      }
+    }
+  return result;
+}
+#endif
 
 
 
@@ -1411,54 +1695,6 @@ metering_control(int interval)
 }
 
 
-
-void
-set_nil_and_start()
-{
-}
-
-int
-ioctl_1(int fd, int selector, int *arg)
-{
-#ifdef VXWORKS
-  switch(selector) {
-  case 0x4004667f:
-    selector = FIONREAD;
-    break;
-  }
-#endif
-  return ioctl(fd, selector, (int)arg);
-}
-
-int
-xftruncate(int fd, int len)
-{
-#ifndef VXWORKS
-  return ftruncate(fd, len);
-#endif
-#ifdef VXWORKS
-  return -1;
-#endif
-}
-
-
-char *
-xgetcwd(char *buf, int len)
-{
-  char *p = getcwd(buf, len);
-#ifdef VXWORKS
-  if (*p != '/') {
-    char *q = strchr(p, ':');
-    
-    if ((q != NULL) &&
-	(*(++q) == '/')) {
-      strcpy(p, q);
-    }
-  }
-#endif
-  return p;
-}
-
 #ifdef VXWORKS
 #include <sys/stat.h>
 
@@ -1844,3 +2080,35 @@ do_fd_zero(fd_set *fdsetp)
 {
   FD_ZERO(fdsetp);
 }
+
+#include "image.h"
+
+LispObj
+load_image(char *path)
+{
+  int fd = open(path, O_RDONLY, 0666);
+  LispObj image_nil = 0;
+  if (fd > 0) {
+    openmcl_image_file_header ih;
+    image_nil = load_openmcl_image(fd, &ih);
+    /* We -were- using a duplicate fd to map the file; that
+       seems to confuse Darwin (doesn't everything ?), so
+       we'll instead keep the original file open.
+    */
+    if (!image_nil) {
+      close(fd);
+    }
+  }
+  if (image_nil == 0) {
+    fprintf(stderr, "Couldn't load lisp heap image from %s\n", path);
+    lisp_exit(-1);
+  }
+  return image_nil;
+}
+
+int
+set_errno(int val)
+{
+  errno = val;
+  return -1;
+}
