@@ -1,22 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
 ;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Opensourced MCL.
+;;;   This file is part of OpenMCL.  
 ;;;
-;;;   Opensourced MCL is free software; you can redistribute it and/or
-;;;   modify it under the terms of the GNU Lesser General Public
-;;;   License as published by the Free Software Foundation; either
-;;;   version 2.1 of the License, or (at your option) any later version.
+;;;   OpenMCL is licensed under the terms of the Lisp Lesser GNU Public
+;;;   License , known as the LLGPL and distributed with OpenMCL as the
+;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
+;;;   which is distributed with OpenMCL as the file "LGPL".  Where these
+;;;   conflict, the preamble takes precedence.  
 ;;;
-;;;   Opensourced MCL is distributed in the hope that it will be useful,
-;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;   Lesser General Public License for more details.
-;;;
-;;;   You should have received a copy of the GNU Lesser General Public
-;;;   License along with this library; if not, write to the Free Software
-;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+;;;   OpenMCL is referenced in the preamble as the "LIBRARY."
 ;;;
+;;;   The LLGPL is also available online at
+;;;   http://opensource.franz.com/preamble.html
 (in-package "CCL")
 
 
@@ -125,7 +121,6 @@
 ) ; end of #-ppc-target
 
 (defun untagged-addr (addr)
-  (declare (fixnum addr))
   (logand (lognot 7) addr))
 
 (defparameter *xload-spaces* nil)
@@ -138,25 +133,23 @@
   (vaddr 0)
   (size (ash 1 18))
   (lowptr 0)
-  (ref-map nil)
   (data nil)
-  (name ""))
+  (code 0))
 
 (defmethod print-object ((s xload-space) stream)
   (print-unreadable-object (s stream :type t)
-    (format stream "~a @#x~8,'0x len = ~d" (xload-space-name s) (xload-space-vaddr s) (xload-space-lowptr s))))
+    (format stream "~a @#x~8,'0x len = ~d" (xload-space-code s) (xload-space-vaddr s) (xload-space-lowptr s))))
 
 ; :constructor ... :constructor ... <gasp> ... must remember ... :constructor
 
-(defun init-xload-space (vaddr size name &optional relocs)
+(defun init-xload-space (vaddr size code)
   (let* ((nfullwords (ash (+ size 3) -2))
          (space (make-xload-space :vaddr vaddr
                                  :size size
-                                 :name name
-                                 :ref-map (if relocs (make-array nfullwords :element-type 'bit :initial-element 0))
                                  :data (make-array nfullwords
                                                    :element-type '(unsigned-byte 32)
-                                                   :initial-element 0))))
+                                                   :initial-element 0)
+				 :code code)))
     (push space *xload-spaces*)
     space))
 
@@ -170,25 +163,39 @@
         )
   "Symbols naming fixed-arg, single-valued functions")
         
-(defparameter *xload-nrs* (mapcar #'(lambda (s)
-                                     (or (assq s '((nil) (%pascal-functions%) (*all-metered-functions*)
-                                                   (*%dynvfp%* . 0) (*%dynvlimit%* . 0) (*background-sleep-ticks*)
-                                                   (*foreground-sleep-ticks*) (*window-update-wptr*)
-                                                   (*pre-gc-hook*) (*post-gc-hook*) (%handlers%) (%saved-method-var%)
-                                                   (%finalization-alist%) (*current-stack-group*) (%closure-code%)))
-                                         s))
-                                 arch::*ppc-nilreg-relative-symbols*))
+(defparameter *xload-nrs*
+  (mapcar
+   #'(lambda (s)
+       (or (assq s '((nil) (%pascal-functions%) (*all-metered-functions*)
+		     (*%dynvfp%* . 0) (*%dynvlimit%* . 0) (*background-sleep-ticks*)
+		     (*foreground-sleep-ticks*) (*window-update-wptr*)
+		     (*pre-gc-hook*) (*post-gc-hook*) (%handlers%) (%saved-method-var%)
+		     (%finalization-alist%) (*current-stack-group*) (%closure-code%)))
+	   s))
+   arch::*ppc-nilreg-relative-symbols*))
 
-(defparameter *xload-readonly-space-address* #x10000)
+;;; This should probably be a function of the target backend.
+(defparameter *xload-image-base-address*
+  #+darwinppc-target #x01000000
+  #+linuxppc-target #x31000000)
+
+(defparameter *xload-static-reserve* (ash 1 15))
+(defparameter *xload-purespace-reserve* #x04000000)
+(defparameter *xload-static-space-address* *xload-image-base-address*)
+(defparameter *xload-static-space-size* (ash 8 10))
+(defparameter *xload-readonly-space-address* (+ *xload-image-base-address*
+						*xload-static-reserve*))
 (defparameter *xload-readonly-space-size* (ash 1 18))
-(defparameter *xload-dynamic-space-address* (+ *xload-readonly-space-address* (ash 64 20 )))
+(defparameter *xload-dynamic-space-address* (+ *xload-image-base-address*
+					       *xload-purespace-reserve*))
 (defparameter *xload-dynamic-space-size* (ash 1 18))
-(defparameter *xload-nil* (+ *xload-dynamic-space-address* 16 (* 1024 4) arch::fulltag-nil))
+(defparameter *xload-nil* (+ *xload-static-space-address* 16 (* 1024 4) arch::fulltag-nil))
 (defparameter *xload-T* (+ *xload-nil* arch::t-offset))
 (defparameter *xload-nilsym* (+ *xload-T* arch::symbol.size))
-(defparameter %xload-unbound-function% (+ (- *xload-T* arch::fulltag-misc) (* (length *xload-nrs*) arch::symbol.size) arch::fulltag-misc))
+(defparameter %xload-unbound-function% (+ *xload-dynamic-space-address* arch::fulltag-misc))
 (defparameter *xload-dynamic-space* nil)
 (defparameter *xload-readonly-space* nil)
+(defparameter *xload-static-space* nil)
 (defparameter *xload-symbols* nil)
 (defparameter *xload-package-alist* nil)         ; maps real package to clone
 (defparameter *xload-aliased-package-addresses* nil)     ; cloned package to address
@@ -202,53 +209,6 @@
                                         ; pc-relative.
 
 
-; This is just a hint: if PROBE-FILE fails, do a CHOOSE-FILE-DIALOG to find the 
-; subprims library.
-
-(defparameter *xload-subprims-pathname* "ccl:pmcl;subprims")
-
-; This does a heap walk, starting at the indicated logical address.  Any word
-; tagged as a lisp pointer gets its ref-bit set.
-(defun xload-note-relocations (space start-dword-addr)
-  (multiple-value-bind (v byte-offset) (xload-lookup-address start-dword-addr)
-    (declare (fixnum byte-offset))
-    (let* ((word-offset (ash byte-offset -2))
-           (end-word (ash (xload-space-lowptr space) -2))
-           (bits (xload-space-ref-map space)))
-      (declare (fixnum word-offset end-word))
-      (flet ((note-if-relocatable (word offset bitmap)
-               (let* ((fulltag (logand word arch::fulltagmask)))
-                 (declare (fixnum fulltag))
-                 (if (or (= fulltag arch::fulltag-misc)
-                         (= fulltag arch::fulltag-cons)
-                         (= fulltag arch::fulltag-nil))
-                   (setf (sbit bitmap offset) 1)))))
-        (do* ()
-             ((>= word-offset end-word) (unless (= word-offset end-word) (error "Bug: walked past end of heap.")))
-          (let* ((w (aref v word-offset))
-                 (wtag (logand w arch::fulltagmask)))
-            (declare (fixnum wtag))
-            (if (= wtag arch::fulltag-immheader)
-              (progn                    ; skip this object
-                (let* ((subtag (logand w (1- (ash 1 arch::num-subtag-bits))))
-                       (element-count (ash w (- arch::num-subtag-bits)))
-                       (nwords element-count))
-                  (declare (fixnum nwords element-count subtag))
-                  (if (> subtag arch::max-32-bit-ivector-subtag)
-                    (if (<= subtag arch::max-8-bit-ivector-subtag)
-                      (setq nwords (ash (+ element-count 3) -2))
-                      (if (<= subtag arch::max-16-bit-ivector-subtag)
-                        (setq nwords (ash (1+ element-count) -1))
-                        (if (= subtag arch::subtag-bit-vector)
-                          (setq nwords (ash (+ element-count 31) -5))
-                          (setq nwords (ash element-count 2))))))
-                  (setq word-offset (logand (lognot 1) (+ word-offset (+ nwords 2))))))
-              (progn
-                (note-if-relocatable w word-offset bits)
-                (incf word-offset)
-                (note-if-relocatable (aref v word-offset) word-offset bits)
-                (incf word-offset)))))))))
-
         
 (defun xload-lookup-symbol (sym)
   (gethash (%symbol->symptr sym) *xload-symbols*))
@@ -257,13 +217,11 @@
   (setf (gethash (%symbol->symptr sym) *xload-symbols*) addr))
 
 (defun xload-lookup-address (address)
-  (declare (fixnum address))
   (dolist (space *xload-spaces* (error "Address #x~8,'0x not found in defined address spaces ." address))
     (let* ((vaddr (xload-space-vaddr space)))
-      (declare (fixnum vaddr))
       (if (and (<= vaddr address)
-               (< address (the fixnum (+ vaddr (the fixnum (xload-space-size space))))))
-        (return (values (xload-space-data space) (the fixnum (- address vaddr))))))))
+               (< address (+ vaddr (the fixnum (xload-space-size space)))))
+        (return (values (xload-space-data space) (- address vaddr)))))))
 
 (defun xload-u32-at-address (address)
   (multiple-value-bind (v o) (xload-lookup-address address)
@@ -297,41 +255,28 @@
     (ash imm arch::fixnumshift)
     (error "Bad idea.")))
 
-; "grow" the space: make a new data vector (and, if the space
-; has a ref-map, a new ref-map.) Copy old data (and ref-map
-; entries) to new data vector (& new ref-map).  Update size,
-; data, & ref-map fields.
-; Grow (arbitrarily) by 64K bytes, or as specified by caller.
+;;; "grow" the space: make a new data vector. Copy old data 
+;;;  to new data vector.  Update size and data fields.
+;;; Grow (arbitrarily) by 64K bytes, or as specified by caller.
 (defun xload-more-space (space &optional (delta (ash 1 16)))
   (declare (fixnum delta))
   (setq delta (logand (lognot 3) (the fixnum (+ delta 3))))
   (let* ((old-size (xload-space-size space))
          (old-data (xload-space-data space))
-         (old-ref-map (xload-space-ref-map space))
          (old-nfullwords (ash old-size -2))
          (delta-nfullwords (ash delta -2))
          (new-size (+ old-size delta))
          (new-nfullwords (+ old-nfullwords delta-nfullwords))
          (new-data (make-array (the fixnum new-nfullwords)
                                :element-type '(unsigned-byte 32)
-                               :initial-element 0))
-         (new-ref-map (if old-ref-map (make-array (the fixnum new-nfullwords)
-                                                  :element-type 'bit
-                                                  :initial-element 0))))
+                               :initial-element 0)))
     (declare (fixnum old-size old-nfullwords delta-nfullwords))
     (declare (type (simple-array (unsigned-byte 32) (*)) old-data new-data))
     (dotimes (i old-nfullwords)
       (declare (optimize (speed 3) (safety 0)))
       (setf (aref new-data i) (aref old-data i)))
-    (if new-ref-map
-      (locally
-        (declare (type simple-bit-vector old-ref-map new-ref-map))
-        (dotimes (i old-nfullwords)
-          (declare (optimize (speed 3) (safety 0)))
-          (setf (aref new-ref-map i) (aref old-ref-map i)))))
     (setf (xload-space-size space) new-size
-          (xload-space-data space) new-data
-          (xload-space-ref-map space) new-ref-map)
+          (xload-space-data space) new-data)
     new-size))
                                
 
@@ -339,14 +284,13 @@
   (declare (fixnum tag nbytes))
   (when (logtest 7 nbytes) (error "~d not a multiple of 8 ." nbytes))
   (let* ((free (xload-space-lowptr space)))
-    (declare (fixnum free))
     (if (> nbytes (the fixnum (- (the fixnum (xload-space-size space)) free)))
       (xload-more-space space (the fixnum (+ nbytes (ash 1 16)))))
     (setf (xload-space-lowptr space) (the fixnum (+ free nbytes)))
     (let* ((offset (+ free tag)))
       (declare (fixnum offset))
       (values 
-       (the fixnum (+ (the fixnum (xload-space-vaddr space)) offset))
+       (the fixnum (+ (xload-space-vaddr space) offset))
        (xload-space-data space)
        offset))))
 
@@ -367,9 +311,8 @@
   (xload-alloc space tag (logand (lognot 7) (+ 7 4 (ash (+ 7 nelements) -3)))))
 
 
-(defun xload-make-cons ( car cdr)
-  (multiple-value-bind (cell-addr data offset) (xload-alloc *xload-dynamic-space* arch::fulltag-cons arch::cons.size)
-    (declare (fixnum cell-addr offset))
+(defun xload-make-cons (car cdr &optional (space *xload-dynamic-space*))
+  (multiple-value-bind (cell-addr data offset) (xload-alloc space  arch::fulltag-cons arch::cons.size)
     (setf (u32-ref data (the fixnum (+ offset arch::cons.car))) car)
     (setf (u32-ref data (the fixnum (+ offset arch::cons.cdr))) cdr)
     cell-addr))
@@ -380,7 +323,14 @@
 (defun xload-make-gvector (subtag len)
   (declare (fixnum subtype len))
   (multiple-value-bind (cell-addr data offset) (xload-alloc-fullwords *xload-dynamic-space* arch::fulltag-misc len)
-    (declare (fixnum cell-addr offset))
+    (declare (fixnum offset))
+    (setf (u32-ref data (+ offset arch::misc-header-offset)) (make-xload-header len subtag))
+    cell-addr))
+
+(defun xload-make-word-ivector (subtag len space)
+  (declare (fixnum subtype len))
+  (multiple-value-bind (cell-addr data offset) (xload-alloc-fullwords space  arch::fulltag-misc len)
+    (declare (fixnum offset))
     (setf (u32-ref data (+ offset arch::misc-header-offset)) (make-xload-header len subtag))
     cell-addr))
 
@@ -396,8 +346,10 @@
                    *xload-package-alist*))
       (error "Package at address ~d not cloned ." a)))
 
-(defun xload-make-symbol (pname-address &optional (package-address *xload-nil*))
-  (multiple-value-bind (cell-addr data offset) (xload-alloc-fullwords *xload-dynamic-space* arch::fulltag-misc arch::symbol.element-count)
+(defun xload-make-symbol (pname-address &optional
+					(package-address *xload-nil*)
+					(space *xload-dynamic-space*))
+  (multiple-value-bind (cell-addr data offset) (xload-alloc-fullwords space arch::fulltag-misc arch::symbol.element-count)
     (declare (fixnum offset))
       (setf (u32-ref data (+ offset arch::symbol.header)) *xload-symbol-header*)
       (setf (u32-ref data (+ offset arch::symbol.flags)) 0)
@@ -449,7 +401,7 @@
     
 (defun xload-make-dfloat (space high low)
   (multiple-value-bind (dfloat-addr v o) (xload-alloc-fullwords space arch::fulltag-misc arch::double-float.element-count)
-    (declare (fixnum dfloat-addr o))
+    (declare (fixnum o))
     (setf (u32-ref v (the fixnum (+ o arch::double-float.header))) 
           (make-xload-header arch::double-float.element-count arch::subtag-double-float))
     (setf (u32-ref v (the fixnum (+ o arch::double-float.value))) high)
@@ -459,12 +411,12 @@
 (defun xload-make-ivector (space subtag nelements)
   (declare (fixnum subtype nelements))
   (multiple-value-bind (addr v o) (xload-alloc space arch::fulltag-misc (xload-dword-align (xload-subtag-bytes subtag nelements) t))
-    (declare (fixnum addr o))
+    (declare (fixnum o))
     (setf (u32-ref v (the fixnum (- o arch::fulltag-misc))) (make-xload-header nelements subtag))
     (values addr v o)))
 
 (defun xload-%svref (addr i)
-  (declare (fixnum addr i))
+  (declare (fixnum i))
   (if (= (the fixnum (logand addr arch::fulltagmask)) arch::fulltag-misc)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -472,7 +424,7 @@
     (error "Not a vector: #x~x" addr)))
 
 (defun (setf xload-%svref) (new addr i)
-  (declare (fixnum addr i))
+  (declare (fixnum i))
   (if (= (the fixnum (logand addr arch::fulltagmask)) arch::fulltag-misc)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -480,7 +432,6 @@
     (error "Not a vector: #x~x" addr)))
 
 (defun xload-car (addr)
-  (declare (fixnum addr))
   (if (= (the fixnum (logand addr arch::tagmask)) arch::tag-list)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -488,7 +439,6 @@
     (error "Not a list: #x~x" addr)))
 
 (defun (setf xload-car) (new addr)
-  (declare (fixnum addr))
   (if (= (the fixnum (logand addr arch::tagmask)) arch::tag-list)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -496,7 +446,6 @@
     (error "Not a list: #x~x" addr)))
 
 (defun xload-cdr (addr)
-  (declare (fixnum addr))
   (if (= (the fixnum (logand addr arch::tagmask)) arch::tag-list)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -504,7 +453,6 @@
     (error "Not a list: #x~x" addr)))
 
 (defun (setf xload-cdr) (new addr)
-  (declare (fixnum addr))
   (if (= (the fixnum (logand addr arch::tagmask)) arch::tag-list)
     (multiple-value-bind (v offset) (xload-lookup-address addr)
       (declare (fixnum offset))
@@ -512,13 +460,11 @@
     (error "Not a list: #x~x" addr)))
 
 (defun xload-symbol-value (addr)
-  (declare (fixnum addr))
   (if (= (xload-%svref addr -1) *xload-symbol-header*)
     (xload-%svref addr arch::symbol.vcell-cell)
     (error "Not a symbol: #x~x" addr)))
   
 (defun xload-symbol-name (addr)
-  (declare (fixnum addr))
   (if (= addr *xload-nil*) (incf addr (+ arch::t-offset arch::symbol.size)))
   (let* ((header (xload-%svref addr -1)))
     (if (= header *xload-symbol-header*)
@@ -547,10 +493,8 @@
     (error "Not a symbol: #x~x" addr)))
 
 (defun (setf xload-symbol-plist) (new addr)
-  (declare (fixnum new addr))
   (let* ((plist-addr (+ addr arch::symbol.package-plist))
          (package-plist (xload-u32-at-address plist-addr)))
-    (declare (fixnum plist-addr package-plist))
     (if (= arch::fulltag-cons (logand package-plist arch::fulltagmask))
       (let* ((pname-addr (+ addr arch::symbol.pname))
 	     (pname (xload-u32-at-address pname-addr))
@@ -568,22 +512,25 @@
     new))
       
   
-; This handles constants set to themselves.
-; Unless PRESERVE-CONSTANTNESS is true, the symbols $sym_vbit_const bit is cleared.
-;  (This is done because the kernel tries to call EQUALP if constants are "redefined", and
-;   EQUALP may not be defined early enough.)
-(defun xload-copy-symbol (symbol &optional (preserve-constantness (keywordp symbol)))
+;;; This handles constants set to themselves.  Unless
+;;; PRESERVE-CONSTANTNESS is true, the symbol's $sym_vbit_const bit is
+;;; cleared.  (This is done because the kernel tries to call EQUALP if
+;;; constants are "redefined", and EQUALP may not be defined early
+;;; enough.)
+(defun xload-copy-symbol (symbol &key
+				 (preserve-constantness (keywordp symbol))
+				 (space *xload-dynamic-space*))
   (or (xload-lookup-symbol symbol)
       (let* ((pname (symbol-name symbol))
              (home-package (symbol-package symbol))
              (addr (xload-make-symbol (xload-save-string pname nil (length pname))
                                      (if home-package 
                                        (xload-package->addr home-package)
-                                       *xload-nil*))))
-        (declare (fixnum addr))
+                                       *xload-nil*)
+				     space)))
         (xload-intern symbol)
         (let* ((bits (logandc2 (%symbol-bits symbol) (ash 1 $sym_vbit_typeppred))))
-          (setf (xload-u32-at-address (the fixnum (+ addr arch::symbol.flags)))
+          (setf (xload-u32-at-address (+ addr arch::symbol.flags))
                 (ash (if preserve-constantness
                        bits
                        (logand (lognot (ash 1 $sym_vbit_const)) bits)) 
@@ -594,9 +541,9 @@
         (setf (xload-lookup-symbol symbol) addr))))
 
 
-; Write a list to dynamic space.  No detection of circularity or structure sharing.
-; The cdr of the final cons can be nil (treated as *xload-nil*.)
-; All cars must be addresses.
+;;; Write a list to dynamic space.  No detection of circularity or
+;;; structure sharing.  The cdr of the final cons can be nil (treated
+;;; as *xload-nil*.)  All cars must be addresses.
 
 (defun xload-save-list (l)
   (if (atom l)
@@ -621,7 +568,7 @@
           (setf (u16-ref v p) (char-code (schar str i)))))
         addr)))
 
-; Read a string from fasl file, save it to readonly-space.
+;;; Read a string from fasl file, save it to readonly-space.
 (defun %xload-fasl-readstr (s extended-p)
   (multiple-value-bind (str n new-p) (%fasl-readstr s extended-p)
     (declare (fixnum n subtype))
@@ -673,12 +620,6 @@
                 (xload-save-list (mapcar #'xload-copy-symbol (pkg.shadowed p)))))))))
 
 
-(defun xload-init-spaces (nrs)
-  (dolist (sym nrs)
-    (let* ((pname (symbol-name sym)))
-      (setf (xload-lookup-symbol sym) 
-            (xload-make-symbol (xload-save-string pname nil (length pname)))))))
-
 
 (defun xload-fasload (pathnames)
   (dolist (path pathnames)
@@ -692,8 +633,8 @@
              (*xload-loading-file-source-file* nil)
              (*loading-file-source-file* (namestring source-file)))
         (when *load-verbose*
-            (format t "~&;Loading ~S..." *load-pathname*)
-            (force-output))
+	  (format t "~&;Loading ~S..." *load-pathname*)
+	  (force-output))
         (multiple-value-bind (winp err) (%fasload (native-translated-namestring path) *xload-fasl-dispatch-table*)
           (if (not winp) (%err-disp err)))))))
   
@@ -738,13 +679,12 @@
   (let* ((unbound-vcells nil)
          (undefined-fcells nil))
     (dolist (vcell *xload-vcell-refs*)
-      (declare (fixnum vcell))
-      (when (= (the fixnum (xload-u32-at-address (the fixnum (+ vcell arch::symbol.vcell))))
+      (when (=  (xload-u32-at-address (+ vcell arch::symbol.vcell))
                arch::unbound-marker)
         (push vcell unbound-vcells)))
     (dolist (fcell *xload-fcell-refs*)
       (declare (fixnum fcell))
-      (when (= (the fixnum (xload-u32-at-address (the fixnum (+ fcell arch::symbol.fcell))))
+      (when (= (xload-u32-at-address (+ fcell arch::symbol.fcell))
                %xload-unbound-function%)
         (push fcell undefined-fcells)))
     (flet ((xload-symbol-name-string (addr) (xload-get-string (xload-symbol-name addr))))
@@ -780,58 +720,42 @@
 (defun target-pmcl-kernel-version ()
   #x04008000)                           ; Get serious.
 
-(defparameter *pmcl-kernel-cfrg-name* "pmcl-kernel4.1")
 
 (defun xfasload (output-file &rest pathnames)
   (let* ((*xload-symbols* (make-hash-table :test #'eq))
          (*xload-spaces* nil)
-         (*xload-readonly-space* (init-xload-space *xload-readonly-space-address* *xload-readonly-space-size* ".text"))
-         (*xload-dynamic-space* (init-xload-space *xload-dynamic-space-address* *xload-dynamic-space-size* ".data" t))
+         (*xload-readonly-space* (init-xload-space *xload-readonly-space-address* *xload-readonly-space-size* arch::area-readonly))
+         (*xload-dynamic-space* (init-xload-space *xload-dynamic-space-address* *xload-dynamic-space-size* arch::area-dynamic))
+	 (*xload-static-space* (init-xload-space *xload-static-space-address* *xload-static-space-size* arch::area-static))
+						 
          (*xload-package-alist* (xload-clone-packages %all-packages%))
          (*xload-cold-load-functions* nil)
          (*xload-loading-file-source-file* nil)
          (*xload-vcell-refs* nil)
          (*xload-fcell-refs* nil)
-         (*xload-aliased-package-addresses* nil)
-	 (function-entry-code (backend-xload-info-application-entry-code
-			       *xload-target-backend*)))
-    ; The import descriptor is the first thing in dynamic space.
-    ; It's followed by the entrypoint transfer vector,
-    ;   which references the entrypoint code, which is the first thing in readonly space.
-    ; The first thing after the static subprims code, that is.
-    (funcall
-     (backend-xload-info-prepend-subprims-hook *xload-target-backend*)
-     *xload-readonly-space*)
-    (let* ((cv-len (uvsize function-entry-code)))
-      (multiple-value-bind (entry-address v offset) (xload-make-ivector *xload-readonly-space* arch::subtag-code-vector cv-len)
-        (dotimes (i cv-len) (setf (u32-ref v (+ offset arch::misc-data-offset (ash i 2))) (uvref function-entry-code i)))
-        (xload-make-cons 0 0)     ; cdr goes first in memory
-        (xload-make-cons *xload-nil* entry-address)))
-    
-    (incf (xload-space-lowptr *xload-dynamic-space*) 4096)
+         (*xload-aliased-package-addresses* nil))
+    (xload-make-word-ivector arch::subtag-u32-vector 1027 *xload-static-space*)
     ; Make NIL.  Note that NIL is sort of a misaligned cons (it straddles two doublewords.)
-    (xload-make-cons *xload-nil* 0)
-    (xload-make-cons 0 *xload-nil*)
-    ; Skip over the nilreg-relative symbols, create %unbound-function% and the package objects, then
-    ; go back and fill in the nilreg-relative symbols.  Then start consing ..
-    (let* ((nrs-lowptr (xload-space-lowptr *xload-dynamic-space*)))
-      (setf (xload-space-lowptr *xload-dynamic-space*) (+ nrs-lowptr (* (length *xload-nrs*) arch::symbol.size)))
-      ; The undefined-function object is a 1-element simple-vector (not a function vector).
-      ; The code-vector in its 0th element should report the appropriate error.
-      (let* ((udf-object (xload-make-gvector arch::subtag-simple-vector 1)))
-        (setf (xload-%svref udf-object 0) (xload-save-code-vector
-					   (backend-xload-info-udf-code
-					    *xload-target-backend*))))
-      (setq *xload-aliased-package-addresses* (xload-assign-aliased-package-addresses *xload-package-alist*))
-      (let* ((restore-lowptr (xload-space-lowptr *xload-dynamic-space*)))
-        (setf (xload-space-lowptr *xload-dynamic-space*) nrs-lowptr)
-        (dolist (pair *xload-nrs*)
-          (let* ((val-p (consp pair))
-                 (val (if val-p (or (cdr pair) *xload-nil*)))
-                 (sym (if val-p (car pair) pair)))
-            (xload-copy-symbol sym t)
-            (when val-p (xload-set sym val))))
-        (setf (xload-space-lowptr *xload-dynamic-space*) restore-lowptr)))
+    (xload-make-cons *xload-nil* 0 *xload-static-space*)
+    (xload-make-cons 0 *xload-nil* *xload-static-space*)
+    ;; Create %unbound-function% and the package objects in dynamic space,
+    ;; then fill in the nilreg-relative symbols in static space.
+    ;; Then start consing ..
+    ;; The undefined-function object is a 1-element simple-vector (not a function vector).
+    ;; The code-vector in its 0th element should report the appropriate error.
+    (let* ((udf-object (xload-make-gvector arch::subtag-simple-vector 1)))
+      (setf (xload-%svref udf-object 0) (xload-save-code-vector
+					 (backend-xload-info-udf-code
+					  *xload-target-backend*))))
+    (setq *xload-aliased-package-addresses* (xload-assign-aliased-package-addresses *xload-package-alist*))
+    (dolist (pair *xload-nrs*)
+      (let* ((val-p (consp pair))
+	     (val (if val-p (or (cdr pair) *xload-nil*)))
+	     (sym (if val-p (car pair) pair)))
+	(xload-copy-symbol sym
+			   :preserve-constantness t
+			   :space *xload-static-space*)
+	(when val-p (xload-set sym val))))
     ; This could be a little less ... procedural.
     (xload-set '*package* (xload-package->addr *ccl-package*))
     (xload-set '*keyword-package* (xload-package->addr *keyword-package*))
@@ -870,36 +794,27 @@
                                 (nreverse *xload-cold-load-functions*))))
     (xload-note-undefined-references)
     (xload-finalize-packages)
-    (xload-dump-image output-file 
-                     *xload-dynamic-space-address*
-                     (+ *xload-dynamic-space-address* 8)
-                     `((,*pmcl-kernel-cfrg-name* ,(target-pmcl-kernel-version) ,(target-pmcl-kernel-version) (:transfer-vector "set_nil_and_start" ,(+ *xload-dynamic-space-address* 4)))))))
+    (xload-dump-image output-file *xload-image-base-address*)))
+
+(defun xload-dump-image (output-file heap-start)
+  (declare (ftype (function (t t list)) write-image-file))
+  ;; Write the 0th and first elements of the kernel_globals vectors as
+  ;; untagged pointers to the beginning and end of "static space".
+  (setf (xload-u32-at-address (+ *xload-static-space-address* 16)) *xload-static-space-address*)
+  (setf (xload-u32-at-address (+ *xload-static-space-address* 20)) (+ *xload-static-space-address* (xload-space-lowptr *xload-static-space*)))
+
+  ;; The next word is supposed to be a pointer to some other static
+  ;; section in the application.  The following two words describe the
+  ;; start and end of the readonly section.
+  (setf (xload-u32-at-address (+ *xload-static-space-address* 28)) *xload-readonly-space-address*)
+  (setf (xload-u32-at-address (+ *xload-static-space-address* 32)) (+ *xload-readonly-space-address* (xload-space-lowptr *xload-readonly-space*)))
+  (write-image-file output-file
+		    heap-start
+		    (list *xload-readonly-space*
+			  *xload-static-space*
+			  *xload-dynamic-space*)))
+		    
 
-(defun xload-dump-image (output-file heap-start entry-address import-list)
-  (declare (ftype (function (t t t t t t) t) write-pef-file))
-  (xload-note-relocations *xload-dynamic-space* *xload-dynamic-space-address*)
-  (setf (xload-u32-at-address *xload-dynamic-space-address*) (arch::make-vheader (+ 1024 3) arch::subtag-u32-vector))
-  ; Write the 0th and first elements of the kernel_globals vectors as untagged pointers
-  ; to the beginning and end of "dynamic space" (this is so the kernel can do a heap walk
-  ; to relocate subprim calls.)  Mark these (dword-aligned) addresses as being relocatable.
-  (setf (xload-u32-at-address (+ *xload-dynamic-space-address* 16)) heap-start)
-  (setf (xload-u32-at-address (+ *xload-dynamic-space-address* 20)) (+ *xload-dynamic-space-address* (xload-space-lowptr *xload-dynamic-space*)))
-  (let* ((bits (xload-space-ref-map *xload-dynamic-space*)))
-    (setf (sbit bits (ash 16 -2)) 1)
-    (setf (sbit bits (ash 20 -2)) 1))
-  ; The next word is supposed to be a pointer to some other static section in the application.
-  ; The following two words describe the start and end of the readonly section.
-  (setf (xload-u32-at-address (+ *xload-dynamic-space-address* 28)) *xload-readonly-space-address*)
-  (setf (xload-u32-at-address (+ *xload-dynamic-space-address* 32)) (+ *xload-readonly-space-address* (xload-space-lowptr *xload-readonly-space*)))
-  (let* ((bits (xload-space-ref-map *xload-dynamic-space*)))
-    (setf (sbit bits (ash 28 -2)) 1)
-    (setf (sbit bits (ash 32 -2)) 1))
-  (write-pef-file output-file 
-                  0 
-                  *xload-readonly-space* 
-                  *xload-dynamic-space* 
-                  entry-address 
-                  import-list))
 
 
 
@@ -1073,12 +988,11 @@
       (setf (xload-%svref vector i) (%fasl-expr s)))))
 
 (defun xload-note-cell-ref (loc imm)
-  (declare (fixnum loc imm))
   (if (= loc arch::symbol.fcell)
     (pushnew imm *xload-fcell-refs*)
     (if (= loc arch::symbol.vcell)
       (pushnew imm *xload-vcell-refs*)))
-  (the fixnum (+ loc imm)))
+  (+ loc imm))
 
 
 (defun xload-lfun-name (lf)
@@ -1227,23 +1141,27 @@
   (let* ((backend (find-backend target))
 	 (any (not (null force)))
          (outpath (merge-pathnames dir (backend-target-fasl-pathname backend))))
-    (dolist (src (sort (directory (merge-pathnames dir "*.lisp"))
-		       #'string< :key #'namestring)
-	     any)
-      (let* ((fasl (merge-pathnames outpath  src)))
-        (when (or force
-                  (not (probe-file fasl))
-                  (> (file-write-date src)
-                     (file-write-date fasl)))
-	  (setq any t)
-          (compile-file src :target target
-                        :features *xcompile-features*
-                        :output-file  fasl 
-                        :verbose t))))))
+    (in-development-mode
+     (dolist (src (sort (directory (merge-pathnames dir "*.lisp"))
+			#'string< :key #'namestring)
+	      any)
+       (let* ((fasl (merge-pathnames outpath  src)))
+	 (when (or force
+		   (not (probe-file fasl))
+		   (> (file-write-date src)
+		      (file-write-date fasl)))
+	   (setq any t)
+	   (compile-file src :target target
+			 :features *xcompile-features*
+			 :output-file  fasl 
+			 :verbose t)))))))
 
 (defun target-xcompile-level-0 (target &optional force)
   (let* ((backend (or (find-xload-backend target)
 		      (error "Unknown xload backend: ~s" target)))
+         ;; Saving doc-strings doesn't work in level-0 (yet.)
+         (*save-doc-strings* nil)
+         (*fasl-save-doc-strings* nil)
 	 (a (target-xcompile-directory target "ccl:level-0;" force))
 	 (b (target-xcompile-directory target
 				       (backend-xload-info-subdir backend)
@@ -1263,24 +1181,25 @@
 	(delete-package user-pkg))
       (when (and lisp-pkg (not (eq lisp-pkg (find-package "CL"))))
 	(delete-package lisp-pkg)))
-    (when recompile
-      (target-Xcompile-level-0 target (eq recompile :force)))
-    (let* ((*load-verbose* t)
-	   (compiler-backend (find-backend
-			      (backend-xload-info-compiler-target-name
-			       *xload-target-backend*)))
-	   (wild-fasls (concatenate 'simple-base-string
-				    "*."
-				    (pathname-type
-				     (backend-target-fasl-pathname
-				      compiler-backend))))
-	   (wild-root (merge-pathnames "ccl:level-0;" wild-fasls))
-	   (wild-subdir (merge-pathnames
-			 (backend-xload-info-subdir *xload-target-backend*)
-			 wild-fasls))
-	   (*xload-image-file-name* (backend-xload-info-default-image-name *xload-target-backend*)))
-      (apply #'xfasload *xload-image-file-name*
-	     (append (sort (directory wild-subdir) #'string< :key #'namestring)
-		     (sort (directory wild-root) #'string< :key #'namestring))))))
+    (in-development-mode
+     (when recompile
+       (target-Xcompile-level-0 target (eq recompile :force)))
+     (let* ((*load-verbose* t)
+	    (compiler-backend (find-backend
+			       (backend-xload-info-compiler-target-name
+				*xload-target-backend*)))
+	    (wild-fasls (concatenate 'simple-base-string
+				     "*."
+				     (pathname-type
+				      (backend-target-fasl-pathname
+				       compiler-backend))))
+	    (wild-root (merge-pathnames "ccl:level-0;" wild-fasls))
+	    (wild-subdir (merge-pathnames
+			  (backend-xload-info-subdir *xload-target-backend*)
+			  wild-fasls))
+	    (*xload-image-file-name* (backend-xload-info-default-image-name *xload-target-backend*)))
+       (apply #'xfasload *xload-image-file-name*
+	      (append (sort (directory wild-subdir) #'string< :key #'namestring)
+		      (sort (directory wild-root) #'string< :key #'namestring)))))))
 
 (provide "XFASLOAD")
